/*
 * This file contains constants which might brake tests due to setup changes.
 * For the sake of stability make tests dependent only from setup
 * on individual workstation by
 * (a) fetch paths for tests from respective *.property OR
 * (b) (b) stay with paths within the local directory
 *
 * A string "exp_IsacCore_Tests_1b.xml" indicates a particular file,
 * which probably has disappeared.
 * TODO.WN111202: some constants indicate, that properties are still
 * insuffiient and should take these constants also for src/
 */
package isac;

import java.net.URL;

/**
 *
 * @author tomzi
 */
public class TestConstants {

    public static final class MINIBROWSER {
        //WN111104: clearer if left local to respective directory
        public static final URL TEST_HTML =
           getResource("/isac/gui/browser/minibrowser/test.html");
        public static final URL TEST2_HTML =
           getResource("/isac/gui/browser/minibrowser/test2.html");
    }

    public static final class KESTORE {

        public static final String XMLDATA_PATH = "../xmldata"; 
        //... get from KEStore.properties
        //public static final String KESTORE_DTD_PATH = "isac/src/xml/ke_store.dtd";
        //... WN111104 even not in proto2 (was java 1.5)
        public static final String KEYSTORE_SERVER_URL = "http://localhost:11030";
        //... WN111104 leave local or get from KEStore.properties

        /**
        * these names are fetched directly from KEStore.
        * So respective tests will break if KEStore is changed !
        */
        public static final class TEST_DATA {

            public static final String EXP_TESTS_1B = "exp_IsacCore_Tests_1b.xml";
            public static final String EXP_TESTS_7A = "exp_IsacCore_Tests_7a.xml";
            public static final String EXP_TEST_1C = "exp_IsacCore_Tests_1c.xml";
            public static final String EXP_TEST_1D = "exp_IsacCore_Tests_1d.xml";
            public static final String EXP_WHY_MATH_I = "exp_Why_learn_Math_I.xml";
            public static final String THY_FRAC_CONV = "thy_isac_Diff-thm-frac_conv.xml";
            public static final String THY_BUTLAST_CONS = "thy_scri_ListG-thm-butlast_Cons.xml";
        }
    }

    public static final class ISAC {

        public static final String ISAC_DTD_PATH = "src/xml/isac.dtd";
        //... WN111104: shift into BrideMain.properties
    }

    private static URL getResource(String resource) {
        
        return TestConstants.class.getResource(resource);
    }

    public static void main(String[] args) {

        System.out.println(MINIBROWSER.TEST_HTML);
    }
}
