/**
 * @author Walther Neuper
 * Created on Mar 30, 2017
 * (c) due to copyright terms
 */

package isac.bridge;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Paths;
import java.util.Properties;

import info.hupel.isabelle.api.Configuration;
import info.hupel.isabelle.api.Environment;
import info.hupel.isabelle.api.Version;
import info.hupel.isabelle.japi.JPlatform;
import info.hupel.isabelle.japi.JResources;
import info.hupel.isabelle.japi.JSetup;
import info.hupel.isabelle.japi.JSystem;
import info.hupel.isabelle.setup.Setup;

/*
 * Connect tests with Isac's mathematics engine.
 */
public class Isabelle_Isac {
  
  public static JSystem connect() {
    String isabelle_home_ = "";
    InputStream inputStream = null;
    try {
      Properties prop = new Properties();
      String propFileName = "./properties/BridgeMain.properties";
      inputStream = Isabelle_Isac.class.getClassLoader().getResourceAsStream(propFileName);
      if (inputStream != null) {
        prop.load(inputStream);
      } else {
        throw new FileNotFoundException("property file '" + propFileName + "' not found in the classpath");
      }
      // get the property value and print it out
      isabelle_home_ = prop.getProperty("ISABELLE_HOME");
        } catch (Exception e) {
        System.out.println("Exception: " + e);
      } finally {
        try { inputStream.close();
      } catch (IOException e) { e.printStackTrace(); }
      }
    JResources res = JResources.dumpIsabelleResources();
    Configuration config = Configuration.fromBuiltin("libisabelle_Isac");
    Environment env = JSetup.makeEnvironment(JSetup.defaultSetup(new Version.Stable("2018")), res);
    return JSystem.create(env, config);
  }

}
