/*
 * #############################################################################
 * ATTENTION: BridgeMain must be started because of BridgeRMI (design fault) !?!
 * #############################################################################
 * 
 * @author Walther Neuper, member of the ISAC-team, 
 * Copyright (c) 2004 by Walther Neuper 
 * created Aug 30, 2004, 11:58:35 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to license terms.
 */
package isac.bridge;

import isac.interfaces.ICalcElement;
import isac.interfaces.ICalcIterator;
import isac.interfaces.IToCalc;
import isac.interfaces.IToUser;
import isac.kestore.KEStoreServices;
import isac.util.CalcChanged;
import isac.util.CalcEvent;
import isac.util.CalcMessage;
import isac.util.Formalization;
import isac.util.formulae.CalcFormula;
import isac.util.formulae.Formula;
import isac.util.formulae.Position;
import isac.util.parser.FormalizationDigest;
import isac.TestConstants;

import java.util.Vector;

import org.apache.log4j.Logger;

import junit.framework.TestCase;

/**
 * @author Walther Neuper Aug 30, 2004, 11:58:35 AM
 */
public class TestBridgeInput extends TestCase {

    static Logger logger_ = Logger.getLogger(MathEngine.class.getName());

    /**
     * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
     * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
     * smltest/FE-interface/interface.sml "...enter} right"
     *
     * @see TestSolve#testAppendRight()
     */
    public void testAppendRight() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestBridgeInput#testAppendRight");

        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices(TestConstants.KESTORE.XMLDATA_PATH);
        Vector<String> vec = ks.loadContent("exp", TestConstants.KESTORE.TEST_DATA.EXP_TESTS_1B);
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization fmz = fd.parseFormalization((String) xmlstr);
        System.out.println(".end step_a-----");

        //start the calculation
        MathEngine.init("localhost");
        MathEngine me = MathEngine.getMathEngine();

        IToCalc ct = me.getCalcTree(fmz);
        IToUser user = new MockIToDialog();
        ct.addDataChangeListener(user);
        CalcChanged cce = null;
        System.out.println(".end step_a-----");
        //end step_a
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //do the first two steps
        ct.completeCalcHead();
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 = 2
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 + -1 * 2 = 0
        //		ICalcIterator ci = ct.getActiveFormula();
        //		Formula fa = (Formula) ci.getFormula();
        //		assertEquals("x + 1 + -1 * 2 = 0",fa.toSMLString());
        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //append "-1 + x = 0" which the kernel would have proposed, too
        CalcFormula in = new CalcFormula();
        in.setText("-1 + x = 0");
        ct.appendFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        //check (1): a formula has been appended
        cce = ((MockIToDialog) user).getCalcChangedEvent();
        assertEquals("unchanged", cce.getLastUnchangedFormula().toSMLString(),
                "([1],Res)");
        assertEquals("deleted", cce.getLastDeletedFormula().toSMLString(),
                "([1],Res)");
        assertEquals("generated", cce.getLastGeneratedFormula().toSMLString(),
                "([2],Res)");
        //check (2): no formulae have been inserted:
		Vector<CalcFormula> elems = cce.getLastUnchangedFormula().getFormulaeFromTo(
                cce.getLastGeneratedFormula(), new Integer(2), false);
        assertEquals("the input formula is returned:", ((CalcFormula) elems.firstElement()).toSMLString(), "-1 + x = 0");
        assertEquals("only _one_ formula returned", elems.size(), 1);

        cce.getLastUnchangedFormula().moveDown();
        assertEquals(cce.getLastUnchangedFormula().toSMLString(), cce.getLastGeneratedFormula().toSMLString());
        logger_.debug("---END isac.bridge.TestBridgeInput#testAppendRight");
    }
    /**
     * A regression test: without specific encoding for formulas input in isac-java,
     * e.g. appendFormula 1 "-4 * b ^ 2 / (a + b) + 4 * a ^ 2 / (a + b)"
     * PIDE multiplicated "^" to,
     * e.g. CalcMessage: syntax error in '-4 * b ^^^^^^^^^ 2 / (a + b) + 4 * a ^^^^^^^^^ 2 / (a + b)' -4 * b ^ 2 / (a + b) + 4 * a ^ 2 / (a + b)
     *  
     * See Isabelle/Isac (isabisac) 1ff126aa8445.
     */
    public void testAppendPower() throws Exception {
        System.out.println("/---BEGIN isac.bridge.TestBridgeInput#testAppendPower");

        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices(TestConstants.KESTORE.XMLDATA_PATH);
        Vector<String> vec = ks.loadContent("exp", "exp_IsacCore_Simp_Rat_Add_No-12.xml");
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization fmz = fd.parseFormalization((String) xmlstr);
        System.out.println(".end step_a-----");

        //start the calculation
        MathEngine.init("localhost");
        MathEngine me = MathEngine.getMathEngine();

        IToCalc ct = me.getCalcTree(fmz);
        IToUser user = new MockIToDialog();
        ct.addDataChangeListener(user);
        CalcChanged cce = null;
        System.out.println(".end step_a-----");
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //do the first step
        ct.completeCalcHead();
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// a^2/(a+b) + -4*b^2 / (a+b) + 3*a^2/(a+b)
        System.out.println();
        ICalcIterator ci = ct.getActiveFormula();
        Formula fa = (Formula) ci.getElement();
        assertEquals("a ^ 2 / (a + b) + -4 * b ^ 2 / (a + b) + 3 * a ^ 2 / (a + b)", fa.toSMLString());
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //append "-4 * b ^ 2 / (a + b) + 3 * a ^ 2 / (a + b)" which caused the error mentioned.
        CalcFormula in = new CalcFormula();
        in.setText("-4 * b ^ 2 / (a + b) + 4 * a ^ 2 / (a + b)"); //an obvious and simple input
        ct.appendFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        cce = ((MockIToDialog) user).getCalcChangedEvent();
        Vector<CalcFormula> elems = cce.getLastUnchangedFormula().getFormulaeFromTo(
                cce.getLastGeneratedFormula(), new Integer(1), false);
        System.out.println("###" + ((CalcFormula) elems.firstElement()).toSMLString());
        assertEquals("the input formula is returned:", 
                ((CalcFormula) elems.firstElement()).toSMLString(), "-4 * b ^ 2 / (a + b) + 4 * a ^ 2 / (a + b)");
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        logger_.debug("---END isac.bridge.TestBridgeInput#testAppendPower");
        System.out.println("\\---END isac.bridge.TestBridgeInput#testAppendPower");
    }

    /**
     * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
     * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
     * smltest/FE-interface/interface.sml "...enter} other"
     *
     * @see TestSolve#testAppendOther1
     */
    public void testAppendOther1() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestBridgeInput#testAppendOther1");

        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices(TestConstants.KESTORE.XMLDATA_PATH);
        Vector<String> vec = ks.loadContent("exp", TestConstants.KESTORE.TEST_DATA.EXP_TESTS_1B);
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization fmz = fd.parseFormalization((String) xmlstr);
        System.out.println(".end step_a-----");

        //start the calculation
        MathEngine.init("localhost");
        MathEngine me = MathEngine.getMathEngine();

        IToCalc ct = me.getCalcTree(fmz);
        IToUser user = new MockIToDialog();
        ct.addDataChangeListener(user);
        CalcChanged cce = null;
        System.out.println(".end step_a-----");
        //end step_a
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //do the first two steps
        ct.completeCalcHead();
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 = 2
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 + -1 * 2 = 0
        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //append "x - 1 = 0" which needs 11 steps (on same level) to justify
        CalcFormula in = new CalcFormula();
        in.setText("x - 1 = 0");
        ct.appendFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        //check (1): a formula has been appended
        cce = ((MockIToDialog) user).getCalcChangedEvent();
        assertEquals("unchanged", cce.getLastUnchangedFormula().toSMLString(),
                "([1],Res)");
        assertEquals("deleted", cce.getLastDeletedFormula().toSMLString(),
                "([1],Res)");
        assertEquals("generated", cce.getLastGeneratedFormula().toSMLString(),
                "([2],Res)");
        //check (2)
        Vector<CalcFormula> elems = cce.getLastUnchangedFormula().getFormulaeFromTo(
                cce.getLastGeneratedFormula(), new Integer(0), //not displayed
                false);
        assertEquals("the input formula is returned: ", ((CalcFormula) elems.firstElement()).toSMLString(), "x - 1 = 0");
        //check (3):
        assertEquals("only _one_ formula returned: ", elems.size(), 1);
        //check (4):
        elems = cce.getLastUnchangedFormula().getFormulaeFromTo(
                cce.getLastGeneratedFormula(), new Integer(2), // displayed
                false);
        assertEquals("there are 11 steps in between: ", elems.size(), 11);
        logger_.debug("---END isac.bridge.TestBridgeInput#testAppendOther1");
    }

    /**
     * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
     * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
     * smltest/FE-interface/interface.sml "...enter} oth 2"
     *
     * @see TestSolve#testAppendOther2
     */
    public void testAppendOther2() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestBridgeInput#testAppendOther2");

        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices(TestConstants.KESTORE.XMLDATA_PATH);
        Vector<String> vec = ks.loadContent("exp", TestConstants.KESTORE.TEST_DATA.EXP_TESTS_1B);
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization fmz = fd.parseFormalization((String) xmlstr);
        System.out.println(".end step_a-----");

        //start the calculation
        MathEngine.init("localhost");
        MathEngine me = MathEngine.getMathEngine();

        IToCalc ct = me.getCalcTree(fmz);
        IToUser user = new MockIToDialog();
        ct.addDataChangeListener(user);
        CalcChanged cce = null;
        System.out.println(".end step_a-----");
        //end step_a
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //do the first two steps
        ct.completeCalcHead();
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 = 2
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 + -1 * 2 = 0
        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //append "x = 1" which needs 6 steps (+ one level down) to justify
        CalcFormula in = new CalcFormula();
        in.setText("x = 1");
        ct.appendFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        //check (1): a formula has been appended
        cce = ((MockIToDialog) user).getCalcChangedEvent();
        assertEquals("unchanged", cce.getLastUnchangedFormula().toSMLString(),
                "([1],Res)");
        assertEquals("deleted", cce.getLastDeletedFormula().toSMLString(),
                "([1],Res)");
        assertEquals("generated", cce.getLastGeneratedFormula().toSMLString(),
                "([3,2],Res)");
        //check (2)
        String result = cce.getLastGeneratedFormula().getElement().toSMLString();
        assertEquals("input formula is returned: ", result, "x = 1");
       //check (3): the inserted formulae _MUST_ be displayed (but not on level 3 !):
        Vector<CalcFormula> elems = cce.getLastUnchangedFormula().getFormulaeFromTo(
                cce.getLastGeneratedFormula(), new Integer(3), false);
        ICalcElement fch = null;
        for (int i = 0; i < elems.size(); i++) {
            fch = (CalcFormula) elems.elementAt(i);
            System.out.println(".step " + i + ": "
                    + ((CalcFormula) fch).getPosition().toSMLString()
                    + "  " + fch.toSMLString());
        }
        assertEquals("CalcElements to be shown:", elems.size(), 8);
        logger_.debug("---END isac.bridge.TestBridgeInput#testAppendOther2");
    }

    /**
     * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
     * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
     * smltest/FE-interface/interface.sml "...enter} NOTok"
     *
     * @see TestSolve#testAppendNotOk
     */
    public void testAppendNotOk() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestBridgeInput#testAppendNotOk");

        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices(TestConstants.KESTORE.XMLDATA_PATH);
        Vector<String> vec = ks.loadContent("exp", TestConstants.KESTORE.TEST_DATA.EXP_TESTS_1B);
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization fmz = fd.parseFormalization((String) xmlstr);
        System.out.println(".end step_a-----");

        //start the calculation
        MathEngine.init("localhost");
        MathEngine me = MathEngine.getMathEngine();

        IToCalc ct = me.getCalcTree(fmz);
        IToUser user = new MockIToDialog();
        ct.addDataChangeListener(user);
        CalcEvent cce = null;
        System.out.println(".end step_a-----");
        //end step_a
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //do the first two steps
        ct.completeCalcHead();
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 = 2
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 + -1 * 2 = 0
        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //append "x = 4711" which cannot be derived
        CalcFormula in = new CalcFormula();
        in.setText("x = 4711");
        ct.appendFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        cce = ((MockIToDialog) user).getCalcMessageEvent();
        String msg = ((CalcMessage) cce).getText();
        assertEquals("CalcMessage:", msg, "no derivation found");

        logger_.debug("---END isac.bridge.TestBridgeInput#testAppendNotOk");
    }

    /**
     * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
     * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
     * smltest/FE-interface/interface.sml "...replace} right"
     *
     * @see TestSolve#testReplaceRight
     */
    public void testReplaceRight() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestBridgeInput#testReplaceRight");

        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices(TestConstants.KESTORE.XMLDATA_PATH);
        Vector<String> vec = ks.loadContent("exp", TestConstants.KESTORE.TEST_DATA.EXP_TESTS_1B);
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization fmz = fd.parseFormalization((String) xmlstr);
        System.out.println(".end step_a-----");

        //start the calculation
        MathEngine.init("localhost");
        MathEngine me = MathEngine.getMathEngine();

        IToCalc ct = me.getCalcTree(fmz);
        IToUser user = new MockIToDialog();
        ct.addDataChangeListener(user);
        CalcChanged cce = null;
        System.out.println(".end step_a-----");
        //end step_a
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 0);
        cce = ((MockIToDialog) user).getCalcChangedEvent();
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //replace "-1 + x = 0" by "-1 + x = 0", ie. no changes
        Position pos = new Position();//([2],Res)
        pos.addInt("2");
        pos.setKind("Res");
        ICalcIterator ci = new CalcIterator((CalcTree) ct, pos);
        ct.moveActiveFormula(ci);

        CalcFormula in = new CalcFormula();
        in.setText("-1 + x = 0");
        // WN050208 works only, if startet as only test in this JUnit (other
        // outcommented)
        // WN050408 ^^^^^^^^^^^^^^^^^^^^^^????????????????cannot reproduce!!!
        ct.replaceFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        // <ERROR> formula not changed </ERROR> thus ct _NOT_ changed !

        Vector<CalcFormula> elems = cce.getLastUnchangedFormula().getFormulaeFromTo(
                cce.getLastGeneratedFormula(), new Integer(3), false);
        assertEquals("calculation not affected; still elems.size():", elems.size(), 10);

        logger_.debug("---END isac.bridge.TestBridgeInput#testReplaceRight");
    }

    /**
     * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
     * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
     * smltest/FE-interface/interface.sml "...replace} other"
     *
     * @see TestSolve#testReplaceOther1
     */
    public void testReplaceOther1() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestBridgeInput#testReplaceOther1");

        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices(TestConstants.KESTORE.XMLDATA_PATH);
        Vector<String> vec = ks.loadContent("exp", TestConstants.KESTORE.TEST_DATA.EXP_TESTS_1B);
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization fmz = fd.parseFormalization((String) xmlstr);
        System.out.println(".end step_a-----");

        //start the calculation
        MathEngine.init("localhost");
        MathEngine me = MathEngine.getMathEngine();

        IToCalc ct = me.getCalcTree(fmz);
        IToUser user = new MockIToDialog();
        ct.addDataChangeListener(user);
        CalcChanged cce = null;
        System.out.println(".end step_a-----");
        //end step_a
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //autoCalculate
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 0);
        cce = ((MockIToDialog) user).getCalcChangedEvent();
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //replace "x - 1 = 0" which needs 11 steps (on same level) to justify
        Position pos = new Position();//([2],Res)
        pos.addInt("2");
        pos.setKind("Res");
        ICalcIterator ci = new CalcIterator((CalcTree) ct, pos);
        ct.moveActiveFormula(ci);

        CalcFormula in = new CalcFormula();
        in.setText("x - 1 = 0");
        ct.replaceFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        cce = ((MockIToDialog) user).getCalcChangedEvent();
        assertEquals("unchanged", cce.getLastUnchangedFormula().toSMLString(),
                "([1],Res)");
        assertEquals("deleted", cce.getLastDeletedFormula().toSMLString(),
                "([],Res)");
        assertEquals("generated", cce.getLastGeneratedFormula().toSMLString(),
                "([2],Res)");

        //the inserted formulae are _NOT_ displayed:
        Vector<CalcFormula> elems = cce.getLastUnchangedFormula().getFormulaeFromTo(
                cce.getLastGeneratedFormula(), new Integer(0), //not displayed
                false);
 System.out.println("elems= " + elems);
        assertEquals("the input formula is returned:", ((CalcFormula) elems.firstElement()).toSMLString(), "x - 1 = 0");
        assertEquals("only _one_ formula returned", elems.size(), 1);

        //but there are 11 steps in between
        elems = cce.getLastUnchangedFormula().getFormulaeFromTo(
                cce.getLastGeneratedFormula(), new Integer(2), // displayed
                false);
        assertEquals("elems could be displayed ", elems.size(), 11);
        //get_interval returns ([], Res) too much ! (see
        // smltest/FE-interface/interface.sml)

        logger_.debug("---END isac.bridge.TestBridgeInput#testReplaceOther1");
    }

    /**
     * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
     * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
     * smltest/FE-interface/interface.sml"...replace} other 2"
     *
     * @see TestSolve#testReplaceOther2
     */
    public void testReplaceOther2() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestBridgeInput#testReplaceOther2");

        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices(TestConstants.KESTORE.XMLDATA_PATH);
        Vector<String> vec = ks.loadContent("exp", TestConstants.KESTORE.TEST_DATA.EXP_TESTS_1B);
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization fmz = fd.parseFormalization((String) xmlstr);
        System.out.println(".end step_a-----");

        //start the calculation
        MathEngine.init("localhost");
        MathEngine me = MathEngine.getMathEngine();

        IToCalc ct = me.getCalcTree(fmz);
        IToUser user = new MockIToDialog();
        ct.addDataChangeListener(user);
        CalcChanged cce = null;
        System.out.println(".end step_a-----");
        //end step_a
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //autoCalculate
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 0);
        cce = ((MockIToDialog) user).getCalcChangedEvent();
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //replace "x = 1" which needs 6 steps (+ one level down) to justify
        Position pos = new Position();//([2],Res)
        pos.addInt("2");
        pos.setKind("Res");
        ICalcIterator ci = new CalcIterator((CalcTree) ct, pos);
        ct.moveActiveFormula(ci);

        CalcFormula in = new CalcFormula();
        in.setText("x = 1");
        ct.replaceFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        cce = ((MockIToDialog) user).getCalcChangedEvent();
        assertEquals("unchanged", cce.getLastUnchangedFormula().toSMLString(),
                "([1],Res)");
        assertEquals("deleted", cce.getLastDeletedFormula().toSMLString(),
                "([],Res)");
        assertEquals("generated", cce.getLastGeneratedFormula().toSMLString(),
                "([3,2],Res)");

        //the inserted formulae _MUST_ be displayed (but not on level 3 !):
        Vector<CalcFormula> elems = cce.getLastUnchangedFormula().getFormulaeFromTo(
                cce.getLastGeneratedFormula(), new Integer(3), false);
        CalcFormula fch = null;
        for (int i = 0; i < elems.size(); i++) {
            fch = (CalcFormula) elems.elementAt(i);
            System.out.println(".step " + i + ": "
                    + fch.getPosition().toSMLString()
                    + "  " + fch.toSMLString());
        }
        assertEquals("CalcElements to be shown:", elems.size(), 8);

        logger_.debug("---END isac.bridge.TestBridgeInput#testReplaceOther2");
    }

    /**
     * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
     * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
     * smltest/FE-interface/interface.sml "...replace} NOTok"
     *
     * @see TestSolve#testReplaceNotOk
     */
    public void testReplaceNotOk() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestBridgeInput#testReplaceNotOk");

        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices(TestConstants.KESTORE.XMLDATA_PATH);
        Vector<String> vec = ks.loadContent("exp", TestConstants.KESTORE.TEST_DATA.EXP_TESTS_1B);
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization fmz = fd.parseFormalization((String) xmlstr);
        System.out.println(".end step_a-----");

        //start the calculation
        MathEngine.init("localhost");
        MathEngine me = MathEngine.getMathEngine();

        IToCalc ct = me.getCalcTree(fmz);
        IToUser user = new MockIToDialog();
        ct.addDataChangeListener(user);
        CalcEvent cce = null;
        System.out.println(".end step_a-----");
        //end step_a
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //autoCalculate
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 0);
        cce = ((MockIToDialog) user).getCalcChangedEvent();
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //replace "x = 4711" which cannot be derived
        Position pos = new Position();//([2],Res)
        pos.addInt("2");
        pos.setKind("Res");
        ICalcIterator ci = new CalcIterator((CalcTree) ct, pos);
        ct.moveActiveFormula(ci);

        CalcFormula in = new CalcFormula();
        in.setText("x = 4711");
        ct.replaceFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        cce = ((MockIToDialog) user).getCalcMessageEvent();
        String msg = ((CalcMessage) cce).getText();
        assertEquals("CalcMessage:", msg, "no derivation found");

        logger_.debug("---END isac.bridge.TestBridgeInput#testReplaceNotOk");
    }
}