/*
 * @author Walther Neuper, member of the ISAC-team, 
 * Copyright (c) 2006 by Walther Neuper
 * created Jul 25, 2006 2:48:53 PM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.bridge;

import isac.TestConstants;
import isac.interfaces.IToCalc;
import isac.interfaces.IToUser;
import isac.kestore.KEStoreServices;
import isac.util.Formalization;
import isac.util.formulae.ContextMethod;
import isac.util.formulae.ContextProblem;
import isac.util.formulae.ContextTheory;
import isac.util.formulae.Position;
import isac.util.parser.FormalizationDigest;
import isac.wsdialog.IContextProvider.ContextType;

import java.util.Vector;

import junit.framework.TestCase;

public class TestContext extends TestCase {

    Formalization fmz_;

    MathEngine me_;

    /**
     * start the mini-subpbl
     * 
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
        KEStoreServices ks = new KEStoreServices(TestConstants.KESTORE.XMLDATA_PATH);
        Vector<String> vec = ks.loadContent("exp", TestConstants.KESTORE.TEST_DATA.EXP_TESTS_1B);

        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        fmz_ = fd.parseFormalization((String) xmlstr);

        MathEngine.init("localhost");
        me_ = MathEngine.getMathEngine();
    }

    public void testContext() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestContext#testContext");
        // start the calculation
        IToCalc ct = me_.getCalcTree(fmz_);
        IToUser user = new MockIToDialog();
        ct.addDataChangeListener(user);
        CalcIterator ci = null;
        // end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        // ME: autoCalculate 3 CompleteCalc;
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 0); // we drop CalcChanged
        // end step_b ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        // create intermediateSteps for various (Knowledge-)Context
        Position pos_Nil_Res = new Position();
        pos_Nil_Res.setKind("Res");
        ci = new CalcIterator((CalcTree) ct, pos_Nil_Res);
        //-->ISA: interSteps 1 ([],Res);  unfolds the calculation
        ct.intermediateSteps(ci); // we drop CalcChanged
        Position pos_3_Res = new Position();
        pos_3_Res.addInt("3");
        pos_3_Res.setKind("Res");
        ci = new CalcIterator((CalcTree) ct, pos_3_Res);
        //-->ISA: interSteps 1 ([3],Res); unfolds the subproblem
        ct.intermediateSteps(ci); // we drop CalcChanged
        Position pos_1_Frm = new Position();
        pos_1_Frm.addInt("1");
        pos_1_Frm.setKind("Frm");
        Position pos_1_Res = new Position();
        pos_1_Res.addInt("1");
        pos_1_Res.setKind("Res");
        Position pos_3_1_Res = new Position();
        pos_3_1_Res.addInt("3");
        pos_3_1_Res.addInt("1");
        pos_3_1_Res.setKind("Res");
        // end step_c ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        

        // check the various (Knowledge-)Context
        ContextTheory cont_thy;
        ContextProblem cont_pbl;
        ContextMethod cont_met;        
        //ContextExample needs not access the MathEngine
        
        //-->ISA: initContext 1 Thy_ ([1],Frm);
        ci = new CalcIterator((CalcTree) ct, pos_1_Frm);
        cont_thy = (ContextTheory) ci.initContext(ContextType.CONTEXT_THEORIES);
        assertEquals(cont_thy.getAppliedTo().toSMLString(), "x + 1 = 2");
        assertEquals(cont_thy.getResult().toSMLString(), "x + 1 + -1 * 2 = 0");

        //-->ISA: initContext 1 Thy_ ([1],Res);
        ci = new CalcIterator((CalcTree) ct, pos_1_Res);
        cont_thy = (ContextTheory) ci.initContext(ContextType.CONTEXT_THEORIES);
        assertEquals(cont_thy.getAppliedTo().toSMLString(), "x + 1 + -1 * 2 = 0");
        assertEquals(cont_thy.getResult().toSMLString(), "-1 + x = 0");

        //-->ISA: initContext 1 Pbl_ ([1],Res);
        ci = new CalcIterator((CalcTree) ct, pos_1_Res);
        cont_pbl = (ContextProblem) ci.initContext(ContextType.CONTEXT_PROBLEMS);
        assertEquals(cont_pbl.getHeadLine().toSMLString(), "solve (x + 1 = 2, x)");
        //-->ISA: initContext 1 Pbl_ ([3,1],Res);
        ci = new CalcIterator((CalcTree) ct, pos_3_1_Res);
        cont_pbl = (ContextProblem) ci.initContext(ContextType.CONTEXT_PROBLEMS);
        assertEquals(cont_pbl.getHeadLine().toSMLString(), "solve (-1 + x = 0, x)");

        //-->ISA: initContext 1 Met_ ([1],Res);
        ci = new CalcIterator((CalcTree) ct, pos_1_Res);
        cont_met = (ContextMethod) ci.initContext(ContextType.CONTEXT_METHODS);
        System.out.println("---------TestContext 5: getKEStoreKey = " + cont_met.getKEStoreKey());
        assertEquals(cont_met.getKEStoreKey(), "met_test_squ_sub");
        //-->ISA: initContext 1 Met_ ([3,1],Res);
        ci = new CalcIterator((CalcTree) ct, pos_3_1_Res);
        cont_met = (ContextMethod) ci.initContext(ContextType.CONTEXT_METHODS);
        System.out.println("---------TestContext 6: getModel = " + cont_met.getModel().toSMLString());
        assertEquals(cont_met.getModel().toSMLString(), 
          "Given [\"equality (-1 + x = 0)\", \"solveFor x\"], Find [\"solutions x_i\"], Relate []");
        // end step_d ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        System.out.println("---END isac.bridge.TestContext#testContext");
    }

}
