/*
 * @author Walther Neuper
 * Copyright (c) due to license terms
 * Created on Jul 14, 2015
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 */
package isac.bridge.xml

import isac.util.formulae.ModelItem
import isac.gui.mawen.scalaterm.Util

import info.hupel.isabelle._      // for Codec
import info.hupel.isabelle.api.XML
import info.hupel.isabelle.pure._ // DEFINES type Term

import java.util.Vector
import scala.collection.JavaConverters._
import scala.collection.JavaConversions._

/*
 * Compile test data for TestDataTypes.java.
 * 
 * This setup has been chosen after <Run As> <Scala JUnit Test> could not
 * made working with standard examples neither from within Eclipse
 * * http://scala-ide.org/docs/2.0.x/testingframeworks.html
 * * http://scala-ide.org/docs/current-user-doc/features/test-finder/index.html
 * nor with ScalaTest
 * * http://www.scalatest.org/download
 * * http://www.scalatest.org/user_guide/using_scalatest_with_eclipse
 */
object TestDataTypesDATA {
  
  //---------------------------------- for TestDataTypes.java#testCalcHead
  def create_REF_FORMULA_out (): XML.Tree = {
    val REF_FORMULA_out =
      XML.Elem(("REFFORMULA", Nil), List(
        XML.Elem(("CALCID", Nil), List(XML.Text ("1"))),
        XML.Elem(("CALCHEAD", List(("status", "incorrect"))), List(
          XML.Elem(("POSITION", Nil), List(
            XML.Elem(("INTLIST", Nil), Nil), 
            XML.Elem(("POS", Nil), List(XML.Text("Pbl"))))),
          XML.Elem(("HEAD", Nil), List(
            XML.Elem(("FORMULA", Nil), List(
              XML.Elem(("ISA", Nil), List(XML.Text("solve (x + 1 = 2, x)"))),
              XML.Elem(("TERM", Nil), List(Codec[Term].encode(TestsDATA.term_mini_subpbl_1())) ))) )),
          XML.Elem(("MODEL", Nil), List(
            XML.Elem(("GIVEN", Nil), Nil),
            XML.Elem(("WHERE", Nil), List(
              XML.Elem(("ITEM", List(("status", "false"))), List(
                XML.Elem(("MATHML", Nil), List(
                  XML.Elem(("ISA", Nil), List(XML.Text("precond_rootpbl v_v"))))))))),
            XML.Elem(("FIND", Nil), Nil),
            XML.Elem(("RELATE", Nil),  Nil))),
          XML.Elem(("BELONGSTO", Nil), List(XML.Text("Pbl"))),
          XML.Elem(("SPECIFICATION", Nil), List(
            XML.Elem(("THEORYID", Nil), List(XML.Text("e_domID"))),
            XML.Elem(("PROBLEMID", Nil), List(
              XML.Elem(("STRINGLIST", Nil), List(
                  XML.Elem(("STRING", Nil), List(XML.Text("e_pblID"))))))),
            XML.Elem(("METHODID", Nil), List(
              XML.Elem(("STRINGLIST", Nil), List(
                  XML.Elem(("STRING", Nil), List(XML.Text("e_metID")))))))))))));
    REF_FORMULA_out
  }
  def create_xml_model (REF_FORMULA_out: XML.Tree): XML.Tree = REF_FORMULA_out match {
    case
      XML.Elem(("REFFORMULA", Nil), List(
        XML.Elem(("CALCID", Nil), List(XML.Text ("1"))),
        XML.Elem(("CALCHEAD", List(("status", "incorrect"))), List(
          XML.Elem(("POSITION", Nil), List(
            XML.Elem(("INTLIST", Nil), Nil), 
            XML.Elem(("POS", Nil), List(XML.Text("Pbl"))))),
          XML.Elem(("HEAD", Nil), List(
            XML.Elem(("FORMULA", Nil), List(
              XML.Elem(("ISA", Nil), List(XML.Text("solve (x + 1 = 2, x)"))),
              XML.Elem(("TERM", Nil), _ ))) )),
          xml_model,
          XML.Elem(("BELONGSTO", Nil), List(XML.Text("Pbl"))),
          XML.Elem(("SPECIFICATION", Nil), List(
            XML.Elem(("THEORYID", Nil), List(XML.Text("e_domID"))),
            XML.Elem(("PROBLEMID", Nil), List(
              XML.Elem(("STRINGLIST", Nil), List(
                  XML.Elem(("STRING", Nil), List(XML.Text("e_pblID"))))))),
            XML.Elem(("METHODID", Nil), List(
              XML.Elem(("STRINGLIST", Nil), List(
                  XML.Elem(("STRING", Nil), List(XML.Text("e_metID")))))))))))))
       => xml_model
    case _ => throw new IllegalArgumentException("create_xml_model exn")    
  }
  def create_xml_given (xml_model: XML.Tree): XML.Tree = xml_model match {
    case
      XML.Elem(("MODEL", Nil), List(
        xml_given,
        XML.Elem(("WHERE", Nil), List(
          XML.Elem(("ITEM", List(("status", "false"))), List(
            XML.Elem(("MATHML", Nil), List(
              XML.Elem(("ISA", Nil), List(XML.Text("precond_rootpbl v_v"))))))))),
        XML.Elem(("FIND", Nil), Nil),
        XML.Elem(("RELATE", Nil),  Nil)))
      => xml_given
    case _ => throw new IllegalArgumentException("create_xml_given exn")    
  }
  def create_xml_where (xml_model: XML.Tree): XML.Tree = xml_model match {
    case
      XML.Elem(("MODEL", Nil), List(
        XML.Elem(("GIVEN", Nil), Nil),
        xml_where,
        XML.Elem(("FIND", Nil), Nil),
        XML.Elem(("RELATE", Nil),  Nil)))
      => xml_where
    case _ => throw new IllegalArgumentException("create_xml_where exn")    
  }
  def create_xml_find (xml_model: XML.Tree): XML.Tree = xml_model match {
    case
      XML.Elem(("MODEL", Nil), List(
        XML.Elem(("GIVEN", Nil), Nil),
        XML.Elem(("WHERE", Nil), List(
          XML.Elem(("ITEM", List(("status", "false"))), List(
            XML.Elem(("MATHML", Nil), List(
              XML.Elem(("ISA", Nil), List(XML.Text("precond_rootpbl v_v"))))))))),
        xml_find,
        XML.Elem(("RELATE", Nil),  Nil)))
      => xml_find
    case _ => throw new IllegalArgumentException("create_xml_find exn")    
  }
  def create_xml_relate (xml_model: XML.Tree): XML.Tree = xml_model match {
    case
      XML.Elem(("MODEL", Nil), List(
        XML.Elem(("GIVEN", Nil), Nil),
        XML.Elem(("WHERE", Nil), List(
          XML.Elem(("ITEM", List(("status", "false"))), List(
            XML.Elem(("MATHML", Nil), List(
              XML.Elem(("ISA", Nil), List(XML.Text("precond_rootpbl v_v"))))))))),
        XML.Elem(("FIND", Nil), Nil),
        xml_relate))
      => xml_relate
    case _ => throw new IllegalArgumentException("create_xml_relate exn")    
  }
  def create_xml_item (): XML.Tree = {
    XML.Elem(("ITEM", List(("status", "false"))), List(
      XML.Elem(("MATHML", Nil), List(
        XML.Elem(("ISA", Nil), List(XML.Text("precond_rootpbl v_v")))))))
  }
  def jVector2sList(ivect: Vector[ModelItem]): List[ModelItem] = {
    ivect.asScala.toList
  }
  def create_xml_calchead (REF_FORMULA_out: XML.Tree): XML.Tree = REF_FORMULA_out match {
    case
      XML.Elem(("REFFORMULA", Nil), List(
        XML.Elem(("CALCID", Nil), List(XML.Text ("1"))),
        xml_calchead)) => xml_calchead
    case _ => throw new IllegalArgumentException("create_xml_calchead exn")    
  }
  def create_xml_spec (REF_FORMULA_out: XML.Tree): XML.Tree = REF_FORMULA_out match {
    case
      XML.Elem(("REFFORMULA", Nil), List(
        XML.Elem(("CALCID", Nil), List(XML.Text ("1"))),
        XML.Elem(("CALCHEAD", List(("status", "incorrect"))), List(
          XML.Elem(("POSITION", Nil), List(
            XML.Elem(("INTLIST", Nil), Nil), 
            XML.Elem(("POS", Nil), List(XML.Text("Pbl"))))),
          XML.Elem(("HEAD", Nil), List(
            XML.Elem(("FORMULA", Nil), List(
              XML.Elem(("ISA", Nil), List(XML.Text("solve (x + 1 = 2, x)"))),
              XML.Elem(("TERM", Nil), _ ))) )),
          XML.Elem(("MODEL", Nil), List(
            XML.Elem(("GIVEN", Nil), Nil),
            XML.Elem(("WHERE", Nil), List(
              XML.Elem(("ITEM", List(("status", "false"))), List(
                XML.Elem(("MATHML", Nil), List(
                  XML.Elem(("ISA", Nil), List(XML.Text("precond_rootpbl v_v"))))))))),
            XML.Elem(("FIND", Nil), Nil),
            XML.Elem(("RELATE", Nil),  Nil))),
          XML.Elem(("BELONGSTO", Nil), List(XML.Text("Pbl"))),
          xml_spec ))))
      => xml_spec
    case _ => throw new IllegalArgumentException("create_xml_spec exn")    
  }
  def create_xml_thy (xml_spec: XML.Tree): XML.Tree = xml_spec match {
    case
      XML.Elem(("SPECIFICATION", Nil), List(
        XML.Elem(("THEORYID", Nil), List(xml_thy)),
        XML.Elem(("PROBLEMID", Nil), List(
          XML.Elem(("STRINGLIST", Nil), List(
              XML.Elem(("STRING", Nil), List(XML.Text("e_pblID"))))))),
        XML.Elem(("METHODID", Nil), List(
          XML.Elem(("STRINGLIST", Nil), List(
              XML.Elem(("STRING", Nil), List(XML.Text("e_metID")))))))))
      => xml_thy
    case _ => throw new IllegalArgumentException("create_xml_thy exn")    
  }
  def create_xml_pbl (xml_spec: XML.Tree): XML.Tree = xml_spec match {
    case
      XML.Elem(("SPECIFICATION", Nil), List(
        XML.Elem(("THEORYID", Nil), List(XML.Text("e_domID"))),
        XML.Elem(("PROBLEMID", Nil), List(xml_pbl)),
        XML.Elem(("METHODID", Nil), List(
          XML.Elem(("STRINGLIST", Nil), List(
              XML.Elem(("STRING", Nil), List(XML.Text("e_metID")))))))))
      => xml_pbl
    case _ => throw new IllegalArgumentException("create_xml_pbl exn")    
  }
  def create_xml_met (xml_spec: XML.Tree): XML.Tree = xml_spec match {
    case
      XML.Elem(("SPECIFICATION", Nil), List(
        XML.Elem(("THEORYID", Nil), List(XML.Text("e_domID"))),
        XML.Elem(("PROBLEMID", Nil), List(
          XML.Elem(("STRINGLIST", Nil), List(
              XML.Elem(("STRING", Nil), List(XML.Text("e_pblID"))))))),
        XML.Elem(("METHODID", Nil), List(xml_met))))
      => xml_met
    case _ => throw new IllegalArgumentException("create_xml_met exn")    
  }
  //TODO: vvvvvvvvvvvvvv should exist with another identifier
  def create_Formula_NEW (term: Term): XML.Tree = {
    XML.Elem(("FORMULA", Nil), List(
      XML.Elem(("ISA", Nil), List(XML.Text(Util.string_of(term)))),
      XML.Elem(("TERM", Nil), List(Codec[Term].encode(term)))))
  }
  
}
