/*
 * @author Walther Neuper
 * Copyright (c) due to license terms
 * Created on Aug 28, 2015
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 */
package isac.bridge.xml

import info.hupel.isabelle.api.XML
import info.hupel.isabelle.pure._ // DEFINES type Term

/*
 * Compile test data for TestDataTypes.java.
 * 
 * This setup has been chosen after <Run As> <Scala JUnit Test> could not
 * made working with standard examples neither from within Eclipse
 * * http://scala-ide.org/docs/2.0.x/testingframeworks.html
 * * http://scala-ide.org/docs/current-user-doc/features/test-finder/index.html
 * nor with ScalaTest
 * * http://www.scalatest.org/download
 * * http://www.scalatest.org/user_guide/using_scalatest_with_eclipse
 */
object TestsDATA {

  //---------------------------------- for TestXMLout.java#test_match
  def create_AAA (): XML.Tree =
    XML.Elem(("AAA", Nil), Nil)
  def create_BBB (): XML.Tree =
    XML.Elem(("BBB", Nil), Nil)
  def create_CCCddd (): XML.Tree =
    XML.Elem(("CCC", Nil), List(
      XML.Elem(("ddd", Nil), Nil)))
  def create_DDDeeeFFF (): XML.Tree =
    XML.Elem(("DDD", Nil), List(
      XML.Elem(("eee", Nil), List(XML.Text("FFF")))))
  def create_EEEfffGGGhhhIII (): XML.Tree =
    XML.Elem(("EEE", Nil), List(
      XML.Elem(("fff", Nil), List(XML.Text("GGG"))),
      XML.Elem(("hhh", Nil), List(XML.Text("III")))))

  def match_ABC(t: XML.Tree): java.lang.String = t match {
    case
      XML.Elem(("AAA", Nil), Nil) => "AAA"
    case
      XML.Elem(("BBB", Nil), Nil) => "BBB"
    case
      XML.Elem(("CCC", Nil), List(
        XML.Elem(("ddd", Nil), Nil))) => "CCCddd"
    case
      XML.Elem(("DDD", Nil), List(
        XML.Elem(("eee", Nil), List(XML.Text("FFF")))))
      => "DDDeeeFFF"
    case
      XML.Elem(("EEE", Nil), List(
        XML.Elem(("fff", Nil), List(XML.Text("GGG"))),
        XML.Elem(("hhh", Nil), List(XML.Text("III")))))
      => "EEEfffGGGhhhIII"
    case
      XML.Elem(("SYSERROR", Nil), List(
        XML.Elem(("CALCID", Nil), List(XML.Text(_))),
        XML.Elem(("ERROR", Nil), List(XML.Text(_)))))
      => "SYSERROR"
    case _ => "no match"
  }

  //---------------------------------- for TestXMLout.java#test_is_message
  def create_SYSERROR (): XML.Tree =
      XML.Elem(("SYSERROR", Nil), List(
        XML.Elem(("CALCID", Nil), List(XML.Text("1"))),
        XML.Elem(("ERROR", Nil), List(XML.Text("no Rewrite_Set...")))))
  /*
   * intermediateSteps xml_out = <SYSERROR><CALCID>1</CALCID><ERROR>no Rewrite_Set...</ERROR></SYSERROR>
   */

        
  //---------------------------------- for ../TestPIDE.java#testTermTransfer
  //----- see Isabelle_Isac/test/../mathml.sml--- create testdata for TestPIDE.java#testTermTransfer ---
  //      https://intra.ist.tugraz.at/hg/isa/rev/5ce72a592f7f
  // val t = @{term "aaa + bbb::real"}
  // ... gives in SML:
  //  ( ( Const ("Groups.plus_class.plus", "real ⇒ real ⇒ real") $ Free ("aaa", "real") ) $ Free ("bbb", "real") )
  // ... gives in SML with left-associativity:
  // val
  //  (*App*) (
  //    (*App*) ( 
  //      Const ("Groups.plus_class.plus", _) $ 
  //      Free ("aaa", _) ) $ 
  //    Free ("bbb", _)   ) = t
  // ... gives in SML with explicit typ:
  //  (*App*) (
  //    (*App*) (
  //        Const ("Groups.plus_class.plus", Type ("fun", [
  //                                           Type ("Real.real", []), 
  //                                           Type ("fun", [
  //                                             Type ("Real.real", []), 
  //                                             Type ("Real.real", [])])])) $ 
  //          Free ("aaa", Type ("Real.real", [])) ) $ 
  //          Free ("bbb", Type ("Real.real", [])) )
  def test_term (): Term = 
        App(
          App(
            Const ("Groups.plus_class.plus", Type("fun", List(
                                               Type("Real.real", List()), 
                                               Type("fun", List(
                                                 Type("Real.real", List()), 
                                                 Type("Real.real", List())))))),
              Free ("aaa", Type ("Real.real", List()))),
              Free ("bbb", Type ("Real.real", List())))

  /*
   * "solve (x + 1 = (2::real), (x::real))" parsed via <code>ScalaTermFromString</code> to
   * App(Const("Equation.solve",Type("fun",List(Type("Product_Type.prod",List(Type("HOL.bool",List()), Type("Real.real",List()))), Type(List.list,List(Type("HOL.bool",List())))))),App(App(Const("Product_Type.Pair",Type("fun",List(Type("HOL.bool",List()), Type("fun",List(Type("Real.real",List()), Type("Product_Type.prod",List(Type("HOL.bool",List()), Type("Real.real",List())))))))),App(App(Const("HOL.eq",Type("fun",List(Type("Real.real",List()), Type("fun",List(Type("Real.real",List()), Type("HOL.bool",List())))))),App(App(Const("Groups.plus_class.plus",Type("fun",List(Type("Real.real",List()), Type("fun",List(Type("Real.real",List()), Type("Real.real",List())))))),Free("x",Type("Real.real",List()))),Free("1",Type("Real.real",List())))),Free("2",Type(Real.real,List())))),Free("x",Type("Real.real",List()))))
   */
  def term_mini_subpbl_1 (): Term = {
    App(Const("Equation.solve",Type("fun",List(Type("Product_Type.prod",List(Type("HOL.bool",List()), Type("Real.real",List()))), Type("List.list",List(Type("HOL.bool",List())))))),App(App(Const("Product_Type.Pair",Type("fun",List(Type("HOL.bool",List()), Type("fun",List(Type("Real.real",List()), Type("Product_Type.prod",List(Type("HOL.bool",List()), Type("Real.real",List())))))))),App(App(Const("HOL.eq",Type("fun",List(Type("Real.real",List()), Type("fun",List(Type("Real.real",List()), Type("HOL.bool",List())))))),App(App(Const("Groups.plus_class.plus",Type("fun",List(Type("Real.real",List()), Type("fun",List(Type("Real.real",List()), Type("Real.real",List())))))),Free("x",Type("Real.real",List()))),Free("1",Type("Real.real",List())))),Free("2",Type("Real.real",List())))),Free("x",Type("Real.real",List()))))
  }
}