/****************************************************************** 
* Copyright (c) 2004, Alois Kirchsteiger, Student of 
* Software Engineering at FH-Hagenberg, Austria. 
* Project member of the isac team at the Institute for 
* Software-Technologie, Graz University of Technology, Austria. 
* 
* Use is subject to license terms.
******************************************************************/

package isac.gui;

import java.util.ArrayList;
import java.util.List;

import javax.swing.*;

import junit.extensions.jfcunit.*;
import junit.extensions.jfcunit.eventdata.*;
import junit.extensions.jfcunit.finder.*;

/**
 * @author Alois Kirchsteiger
 */
public class TestLoginScreen extends JFCTestCase {

    private LoginScreen loginScreen = null;
    private MockLoginListener loginListenerMock;

    public TestLoginScreen( String name ) {
        super( name );
    }

    protected void setUp( ) throws Exception {
        super.setUp( );
        setHelper( new JFCTestHelper( ) );
        loginScreen = new LoginScreen();
        loginScreen.setVisible( true );
        loginListenerMock = new MockLoginListener();
        loginScreen.addListener(loginListenerMock);
    }

    protected void tearDown( ) throws Exception {
        loginScreen = null;
        JFCTestHelper.cleanUp( this );
        super.tearDown( );
    }
    
    protected void disposeDialogs() {
    	DialogFinder dFinder =  new DialogFinder(null);
    	List<?> showingDialogs = dFinder.findAll();
    	
    	for (int i = 0; i < showingDialogs.size(); i++) {
        JFCTestHelper.disposeWindow((JDialog)showingDialogs.get(i), this);
    	}
    }
    
    
    public void testUserAndPasswordEmpty() {
        List<Object> showingDialogs = new ArrayList<Object>();
        
    	NamedComponentFinder finder = new NamedComponentFinder(JComponent.class, "Exit");
    	JButton exitButton = ( JButton ) finder.find( loginScreen, 0);
        assertNotNull( "Could not find the Exit button", exitButton );

        finder = new NamedComponentFinder(JComponent.class, "Login");
    	JButton loginButton = ( JButton ) finder.find( loginScreen, 0);
        assertNotNull( "Could not find the Enter button", loginButton );

        finder = new NamedComponentFinder(JComponent.class, "UserName");
     	JTextField userNameField = ( JTextField ) finder.find( loginScreen, 0);
        assertNotNull( "Could not find the userNameField", userNameField );
        assertEquals( "Username field is empty", "", userNameField.getText( ) );

        finder = new NamedComponentFinder(JComponent.class, "Password");
        JTextField passwordField = ( JTextField ) finder.find( loginScreen, 0 );
        assertNotNull( "Could not find the passwordField", passwordField );
        assertEquals( "Password field is not empty", "", passwordField.getText( ) );
        
        DialogFinder dFinder =  new DialogFinder(null);
        showingDialogs = dFinder.findAll();
        assertEquals( "Number of dialogs showing is wrong", 1, showingDialogs.size( ) );
        disposeDialogs();
    }
    
    public void testFireLoginEvent() {
    	NamedComponentFinder finder = new NamedComponentFinder(JComponent.class, "Login");
    	JButton loginButton = ( JButton ) finder.find( loginScreen, 0);
        assertNotNull( "Could not find the Enter button", loginButton );

        finder = new NamedComponentFinder(JComponent.class, "UserName");
     	JTextField userNameField = ( JTextField ) finder.find( loginScreen, 0);
        assertNotNull( "Could not find the userNameField", userNameField );
        String username = new String("username" + Math.random());
        getHelper().sendString( new StringEventData( this, userNameField, username ) );
        
        finder = new NamedComponentFinder(JComponent.class, "Password");
        JTextField passwordField = ( JTextField ) finder.find( loginScreen, 0 );
        assertNotNull( "Could not find the passwordField", passwordField );
        String password = new String("username" + Math.random());
        getHelper().sendString( new StringEventData( this, passwordField, password ) );
        
    	getHelper().enterClickAndLeave(new MouseEventData(this, loginButton));
    	LoginEvent loginEvent = loginListenerMock.getLastLoginEvent();
    	assertNotNull("no Login Event was fired!", loginListenerMock);
    	assertEquals("LoginEvent has the wrong type", loginEvent.getType(), LoginEvent.LOGIN_TYPE_TRY_TO_LOGIN);
    	assertEquals("LoginEvent did not return the entered username", username, loginEvent.getUsername());
    	assertEquals("LoginEvent did not return the entered password", password, loginEvent.getPassword());
    	disposeDialogs();
    }
    
    public void testFireExitEvent() {
    	NamedComponentFinder finder = new NamedComponentFinder(JComponent.class, "Exit");
    	JButton exitButton = ( JButton ) finder.find( loginScreen, 0);
        assertNotNull( "Could not find the Enter button", exitButton );

        finder = new NamedComponentFinder(JComponent.class, "UserName");
     	JTextField userNameField = ( JTextField ) finder.find( loginScreen, 0);
        assertNotNull( "Could not find the userNameField", userNameField );
        String username = new String("username" + Math.random());
        getHelper().sendString( new StringEventData( this, userNameField, username ) );
        
        finder = new NamedComponentFinder(JComponent.class, "Password");
        JTextField passwordField = ( JTextField ) finder.find( loginScreen, 0 );
        assertNotNull( "Could not find the passwordField", passwordField );
        String password = new String("username" + Math.random());
        getHelper().sendString( new StringEventData( this, passwordField, password ) );
        
    	getHelper().enterClickAndLeave(new MouseEventData(this, exitButton));
    	LoginEvent loginEvent = loginListenerMock.getLastLoginEvent();
    	assertNotNull("no Login Event was fired!", loginListenerMock);
    	assertEquals("LoginEvent has the wrong type", loginEvent.getType(), LoginEvent.LOGIN_TYPE_EXIT);
    	disposeDialogs();
    }
    
    public void testAddRemoveListeners() {
    	//find login button
    	NamedComponentFinder finder = new NamedComponentFinder(JComponent.class, "Login");
    	JButton loginButton = ( JButton ) finder.find( loginScreen, 0);
        assertNotNull( "Could not find the Enter button", loginButton );
    	
        //find exit button
        finder = new NamedComponentFinder(JComponent.class, "Exit");
    	JButton exitButton = ( JButton ) finder.find( loginScreen, 0);
        assertNotNull( "Could not find the Enter button", exitButton );
        
        try {
    		loginScreen.removeListener(loginListenerMock);
    		
    		MockLoginListener loginListenerMock1 = new MockLoginListener();
    		MockLoginListener loginListenerMock2 = new MockLoginListener();
    		
    		assertNull("loginListenerMock1 should have no latestLoginEvent", loginListenerMock1.getLastLoginEvent());
    		assertNull("loginListenerMock2 should have no latestLoginEvent", loginListenerMock2.getLastLoginEvent());
    		
    		//login button pressed but no listener registered
    		getHelper().enterClickAndLeave(new MouseEventData(this, loginButton));
    		assertNull("loginListenerMock1 should have no latestLoginEvent", loginListenerMock1.getLastLoginEvent());
    		assertNull("loginListenerMock2 should have no latestLoginEvent", loginListenerMock2.getLastLoginEvent());
    		
    		//register loginListenerMock1 & press login button
    		loginScreen.addListener(loginListenerMock1);
    		getHelper().enterClickAndLeave(new MouseEventData(this, loginButton));
    		assertNotNull("loginListenerMock1 should have an latestLoginEvent", loginListenerMock1.getLastLoginEvent());
    		assertEquals("tpye of loginEvent was wrong", loginListenerMock1.getLastLoginEvent().getType(), LoginEvent.LOGIN_TYPE_TRY_TO_LOGIN);
    		assertNull("loginListenerMock2 should have no latestLoginEvent", loginListenerMock2.getLastLoginEvent());
    		
    		//register loginListenerMock2 & press exit button
    		loginScreen.addListener(loginListenerMock2);
    		getHelper().enterClickAndLeave(new MouseEventData(this, exitButton));
    		assertNotNull("loginListenerMock1 should have an latestLoginEvent", loginListenerMock1.getLastLoginEvent());
    		assertNotNull("loginListenerMock2 should have an latestLoginEvent", loginListenerMock2.getLastLoginEvent());
    		assertEquals("tpye of loginEvent was wrong", loginListenerMock1.getLastLoginEvent().getType(), LoginEvent.LOGIN_TYPE_EXIT);
    		assertEquals("tpye of loginEvent was wrong", loginListenerMock2.getLastLoginEvent().getType(), LoginEvent.LOGIN_TYPE_EXIT);
    		
    		//unregister loginListenerMock1 & press login button
    		loginScreen.removeListener(loginListenerMock1);
    		getHelper().enterClickAndLeave(new MouseEventData(this, loginButton));
    		assertNotNull("loginListenerMock1 should have an latestLoginEvent", loginListenerMock1.getLastLoginEvent());
    		assertNotNull("loginListenerMock2 should have an latestLoginEvent", loginListenerMock2.getLastLoginEvent());
    		assertEquals("tpye of loginEvent was wrong", loginListenerMock1.getLastLoginEvent().getType(), LoginEvent.LOGIN_TYPE_EXIT);
    		assertEquals("tpye of loginEvent was wrong", loginListenerMock2.getLastLoginEvent().getType(), LoginEvent.LOGIN_TYPE_TRY_TO_LOGIN);
    		
    		loginScreen.removeListener(loginListenerMock2);
    		loginScreen.addListener(loginListenerMock);
    	} catch (Exception e) {
    		assertTrue("application entered a catch block", false);
    	}
    	disposeDialogs();
    }
}