/*
 * @author Walther Neuper, member of the ISAC-team, 
 * Copyright (c) 2007 by Christian Ropposch 
 * created Jun 13, 2007 5:53:44 PM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.gui;

import isac.browserdialog.IBrowserDialog;
import isac.interfaces.ICalcIterator;
import isac.interfaces.IToGUI;
import isac.interfaces.IToUser;
import isac.useractions.EUIContext;
import isac.useractions.EUIElement;
import isac.useractions.IUIAction;
import isac.useractions.IUserAction;
import isac.useractions.UIActionOnBrowserDialog;
import isac.useractions.UserAction;
import isac.useractions.UserActionOnLink;
import isac.users.UserManager;
import isac.util.CalcChanged;
import isac.util.CalcEvent;
import isac.util.ObjectManagerPaths;
import isac.util.WindowApplicationPaths;
import isac.util.formulae.CalcFormula;
import isac.wsdialog.IWorksheetDialog;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Vector;

import junit.framework.TestCase;

/**
 * requests all calculation steps from the MathEngine with minimal machinery in
 * the WindowApplication [brackets indicate source of code]
 * 
 * WN070718 stopped at an intermediate stage while trying to eliminate
 * MockObserver and keeping some useful stuff eg. UnicastRemoteObject
 */
public class TestWindowApplicationMini extends TestCase implements
        Serializable, IToGUI, /* IToWorksheetUser, */IToUser {

    private static final long serialVersionUID = 9004054700129514909L;

    private IBrowserDialog browser_dialog_;

    private IWorksheetDialog worksheet_dialog_;

    public void testWAmini() {
        try { // because TestWindowApplicationMini has already 1 extends
            UnicastRemoteObject.exportObject(this, 0);// 0 avoids _Stub error
        } catch (RemoteException e1) {
            e1.printStackTrace();
        }
        WindowApplicationPaths.loadPropertyFile();
        try {
            System.out.println("-----start rmi-connection--------------------");
            // [isac.gui.WindowApplication]
            System.out.println("-----login user x starts Session etc.--------");
            // [isac.gui.WindowApplication]
            browser_dialog_ = UserManager.getInstance().login("x", "x", null);  
            ObjectManagerPaths
                    .loadFromResource("/properties/ObjectManager.properties");

            MockObserver observer = new MockObserver();
            // ############### renamed: observer --> this doesn't work
            // browser-dialog needs browser-frame (BrowserDialog.java)
            browser_dialog_.registerBrowserFrame(observer);

            System.out.println("-----klick on start--------------------------");
            observer
                    .getBrowserDialog()
                    .notifyUserAction(
                            new UserActionOnLink(
                                    EUIElement.UI_MINIBROWSER_LINK,
                                    EUIContext.UI_CONTEXT_MINIBROWSER,
                                    "file://"
                                            + ObjectManagerPaths.SESSION_MANAGER_USER_PATH
                                            + "exp/"
                                            + "exp_IsacCore_Tests_1b.html#COMMAND_EXEC_EXAMPLE#exp_IsacCore_Tests_1b.xml"));

            System.out.println("-----WD has been set ----------------------");
            // WorksheetDialog has been set in observer#doUIAction
            worksheet_dialog_ = observer.getWorksheetDialog();
            // add our TestCase as listener to the worksheetdialog, to be
            // informed when data from the mathEngine changes
            worksheet_dialog_.addDataChangeListener(observer);
            // not necessary to get the data -> only to avoid null pointer
            // exception in the WorksheetDialog
            worksheet_dialog_.registerUIControlListener(observer);

            System.out.println("-----hit <AUTO> button ---------------------");
            // generate useraction for solving the example
            // (WorksheetDialog.java)
            UserAction userAction = new UserAction(
                    EUIElement.UI_SOLVE_CALCULATE_ALL,
                    EUIContext.UI_CONTEXT_WINDOW_APPLICATION);
            browser_dialog_.notifyUserAction(userAction);

            // The data change caused by the above UserAction in the MathEngine
            // are received and displayed by DataChangeListener(mockToGui)

        } catch (RemoteException e) {
            e.printStackTrace();
        }  catch (Exception e) {
            e.printStackTrace();
        }
    }

    private String createEmptyString(int count) {
        String ret = "";
        if (count > 0) {
            for (int i = 0; i < count; i++) {
                ret += "     ";
            }
        }
        return ret;
    }

    // -----IToGUI--------------------------------------------------
    public boolean doUIAction(IUserAction action) throws RemoteException {
        return false;
    }

    /**
     * only has to handle UI_OPEN_WORKSHEET + UI_OPEN_BROWSER_FRAME !
     */
    public void doUIAction(IUIAction action) throws RemoteException {

        System.out
                .println("In WindowApplication::doUIAction(IUIAction action) : Action is "
                        + action.getAction()
                        + ", Context is "
                        + action.getContext());

        // / if it is not our context go back
        if (action.getContext() != EUIContext.UI_CONTEXT_WINDOW_APPLICATION)
            return;

        switch (action.getAction()) {
        // case UI_OPEN_WORKSHEET: {
        // UIActionOnWSDialog uaowd = (UIActionOnWSDialog) action;
        // openNewWorksheet( uaowd.getWorksheetDialog() );
        // break;
        // }
        case UI_OPEN_BROWSER_FRAME:
            @SuppressWarnings("unused") // used in outcommented code
			UIActionOnBrowserDialog uaobd = (UIActionOnBrowserDialog) action;
            // openNewBrowserFrame( uaobd.getBrowserDialog(),
            // uaobd.getFrameName() );
            break;
        default: break;
        }
    }

    /**
     * creates a new browser frame, inserts it to the internal frames and
     * registeres it to the browser dialog
     * 
     * @see WindowApplication#openNewBrowserFrame(IBrowserDialog, String)
     */
    // private void openNewBrowserFrame( IBrowserDialog browser_dialog,
    // String frame_name ) {
    // try {
    // BrowserFrame browser_frame = new BrowserFrame( browser_dialog,
    // frame_name);
    // addInternalFrame(browser_frame);
    //            
    // num_browser_frames_++;
    //            
    // browser_frame.setSize( IGuiConstants.BROWSER_MIN_SIZE_X,
    // IGuiConstants.BROWSER_MIN_SIZE_Y );
    // browser_frame.setLocation( IGuiConstants.BROWSER_POS_X +
    // (num_browser_frames_ * 20),
    // IGuiConstants.BROWSER_POS_Y + ( num_browser_frames_ * 20 ) );
    //            
    // BrowserFrameRMI browser_framain_frame_rmi = new BrowserFrameRMI(
    // browser_frame);
    //
    // browser_dialog.registerBrowserFrame(browser_framain_frame_rmi);
    //
    //
    //            
    // } catch (RemoteException remote_exception) {
    // remote_exception.printStackTrace();
    // }
    // }
    // public void doUIAction(IUIAction action) throws RemoteException {
    // // catch action to get the browserdialog from the Session
    // // (WindowApplication.java)
    // if (action.getAction() == EUIElement.UI_OPEN_BROWSER_FRAME) {
    // if (action instanceof UIActionOnBrowserDialog) {
    // UIActionOnBrowserDialog actionOnBrowserDialog = (UIActionOnBrowserDialog)
    // action;
    // if (browser_dialog_ == null) {
    // browser_dialog_ = actionOnBrowserDialog.getBrowserDialog();
    // // the exp | thy | pbl | met -Dialog !
    // try {
    // browser_dialog_.registerBrowserFrame(this);
    // browser_dialog_.notifyUserAction(new UserAction(
    // EUIElement.UI_BROWSER_GOT_FOCUS,
    // EUIContext.UI_CONTEXT_BROWSER));
    // } catch (DialogProtocolException e) {
    // e.printStackTrace();
    // }
    // }
    // }
    // }
    // // catch action to get the worksheetdialog from the Session
    // // (WindowApplication.java)
    // if (action.getAction() == EUIElement.UI_OPEN_WORKSHEET) {
    // if (action instanceof UIActionOnWSDialog) {
    // UIActionOnWSDialog actionOnWsSDialog = (UIActionOnWSDialog) action;
    // if (worksheet_dialog_ == null) {
    // worksheet_dialog_ = actionOnWsSDialog.getWorksheetDialog();
    // }
    // }
    // }
    // }
    public void addUIElement(IUIAction action) throws RemoteException {
    }

    public boolean removeUIElement(IUIAction action) throws RemoteException {
        return false;
    }

    public boolean isUIRunning() throws RemoteException {
        return false;
    }

    // -----IToUser--------------------------------------------------
    // catch calcChanged to get the result of the calculation
    public void calcResponse(CalcEvent event) throws RemoteException {
        ICalcIterator from = ((CalcChanged) event).getLastUnchangedFormula();
        ICalcIterator to = ((CalcChanged) event).getLastGeneratedFormula();

		Vector<CalcFormula> calculation = from.getFormulaeFromTo(to, 3, false);
        for (int i = 0; i < calculation.size(); i++) {
            CalcFormula formula = (CalcFormula) calculation.elementAt(i);

            System.out.println(createEmptyString(formula.getPosition()
                    .getLevel())
                    + formula);
        }
    }

}
