/*
 * @author G"unther Gst"ottner and Christian Ropposch, members of the ISAC-team, 
 * Copyright (c) 2007 by G"unther Gst"ottner and Christian Ropposch 
 * created Jun 13, 2007 5:53:44 PM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.gui;

import isac.browserdialog.IBrowserDialog;
import isac.gui.browser.BrowserFrameRMI;
import isac.interfaces.ICalcIterator;
import isac.interfaces.IToGUI;
import isac.interfaces.IToUser;
import isac.useractions.EUIContext;
import isac.useractions.EUIElement;
import isac.useractions.IUIAction;
import isac.useractions.IUserAction;
import isac.useractions.UIActionOnBrowserDialog;
import isac.useractions.UIActionOnWSDialog;
import isac.useractions.UserAction;
import isac.useractions.UserActionOnLink;
import isac.users.UserManager;
import isac.util.CalcChanged;
import isac.util.CalcEvent;
import isac.util.ObjectManagerPaths;
import isac.util.WindowApplicationPaths;
import isac.util.formulae.CalcFormula;
import isac.wsdialog.IWorksheetDialog;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Vector;

import org.apache.log4j.Logger;

import junit.framework.TestCase;

/**
 * Requests all calculation steps from the MathEngine with minimal machinery in
 * the WindowApplication [brackets indicate source of code].
 * 
 * Calls methods more fine-grained than
 * java-tests/isac.gui.TestWindowApplication.
 * 
 * The MockBrowserFrame shall become BrowserFrameStub.
 * 
 */
public class TestWindowApplicationStub extends TestCase implements
        Serializable, IToGUI, /* IToWorksheetUser, */IToUser {

    private static final long serialVersionUID = 9004054700129514909L;

    private static final Logger logger = Logger
    .getLogger(TestWindowApplicationStub.class.getName());

    private IBrowserDialog browser_dialog_;// actually an ExampleDialog

    private int num_browser_frames_;

    private MockBrowserFrame example_browser_;

    @SuppressWarnings("unused") // is used below; deletion causes error
	private MockBrowserFrame theory_browser_;
    @SuppressWarnings("unused") // is used below; deletion causes error
	private MockBrowserFrame problem_browser_;
    @SuppressWarnings("unused") // is used below; deletion causes error
	private MockBrowserFrame method_browser_;

    //there should be 0..n Worksheets ?store WS or WSDialog?
    private IWorksheetDialog worksheet_dialog_;
    

    public void testMain() {
        try { // because TestWindowApplicationMini has already 1 extends
            UnicastRemoteObject.exportObject(this, 0);// 0 avoids _Stub error
        } catch (RemoteException e1) {
            e1.printStackTrace();
        }
        WindowApplicationPaths.loadPropertyFile();
        try {
            logger.info("----- start rmi-connection--------------------");
            // [isac.gui.WindowApplication]

            logger.info("----- login user x ---------------------------");
            // starts Session and WindowApplication [isac.users.UserManager]
            browser_dialog_ = UserManager.getInstance().login("x", "x", null);            
            ObjectManagerPaths
                    .loadFromResource("/properties/ObjectManager.properties");
            logger.info("----- instantiate BrowserFrame's ------------");
            //### GOON ((Session) browser_dialog_).initialiseWindowApplication();
            //via rmi no cast on implementation, only on interface

            logger.info("----- create new worksheet -------------------");
            MockTab worksheet = new MockTab();
            // browser-dialog needs browser-frame for starting example
            // (this is actually an ExampleDialog !) [BrowserDialog.java]
            browser_dialog_.registerBrowserFrame(worksheet);
            //##### should be done in new MockTab() ?!? ##############

            logger.info("----- klick on start--------------------------");
            // creates 
            // a Tab must be created (~WS) + including BrowserDialogs
            worksheet
                    .getBrowserDialog()
                    .notifyUserAction(
                            new UserActionOnLink(
                                    EUIElement.UI_MINIBROWSER_LINK,
                                    EUIContext.UI_CONTEXT_MINIBROWSER,
                                    "file://"
                                            + ObjectManagerPaths.SESSION_MANAGER_USER_PATH
                                            + "exp/"
                                            + "exp_IsacCore_Tests_1b.html#COMMAND_EXEC_EXAMPLE#exp_IsacCore_Tests_1b.xml"));

            logger.info("----- WorksheetDialog has been set -----------");
            // WorksheetDialog has been set in observer#doUIAction
            worksheet_dialog_ = worksheet.getWorksheetDialog();
            // add our TestCase as listener to the worksheetdialog, to be
            // informed when data from the mathEngine changes
            worksheet_dialog_.addDataChangeListener(worksheet);
            // not necessary to get the data -> only to avoid null pointer
            // exception in the WorksheetDialog
            worksheet_dialog_.registerUIControlListener(worksheet);

            System.out.println("----- hit <AUTO> button ----------------");
            // generate useraction for solving the example
            // (WorksheetDialog.java)
            UserAction userAction = new UserAction(
                    EUIElement.UI_SOLVE_CALCULATE_ALL,
                    EUIContext.UI_CONTEXT_WINDOW_APPLICATION);
            browser_dialog_.notifyUserAction(userAction);

            // The data change caused by the above UserAction in the MathEngine
            // are received and displayed by DataChangeListener(mockToGui)

        } catch (RemoteException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
        logger.fatal("----- end test---------------------");
    }

    private String createEmptyString(int count) {
        String ret = "";
        if (count > 0) {
            for (int i = 0; i < count; i++) {
                ret += "     ";
            }
        }
        return ret;
    }


    /**
     * creates a new browser frame, inserts it to the internal frames and
     * registeres it to the browser dialog
     * 
     * @see WindowApplication#openNewBrowserFrame(IBrowserDialog, String)
     */
    private void openNewBrowserFrame(IBrowserDialog browser_dialog,
            String frame_name) {
        try {
            MockBrowserFrame browser_frame = new MockBrowserFrame(browser_dialog,
                    frame_name);
            // addInternalFrame(browser_frame);

            num_browser_frames_++;
            // NEW: + ATTENTION see order in
            // isac.session.initialiseWindowApplication!
            switch (num_browser_frames_) {
            case 1:
                example_browser_ = browser_frame;
                TestCase
                .assertNotNull(
                        "TestWindowApplicationStub#openNewBrowserFrame() sets example_browser_",
                        example_browser_);
                  break;
            case 2:
                theory_browser_ = browser_frame;
                break;
            case 3:
                problem_browser_ = browser_frame;
                break;
            case 4:
                method_browser_ = browser_frame;
                break;
            }

            // browser_frame.setSize( IGuiConstants.BROWSER_MIN_SIZE_X,
            // IGuiConstants.BROWSER_MIN_SIZE_Y );
            // browser_frame.setLocation( IGuiConstants.BROWSER_POS_X +
            // (num_browser_frames_ * 20),
            // IGuiConstants.BROWSER_POS_Y + ( num_browser_frames_ * 20 ) );

            BrowserFrameRMI browser_framain_frame_rmi = new BrowserFrameRMI(
                    browser_frame);

            browser_dialog
                    .registerBrowserFrame((IToGUI) browser_framain_frame_rmi);

        } catch (RemoteException remote_exception) {
            remote_exception.printStackTrace();
        }
    }
   
    /**
     * Open a new Worksheet and start the modeling and specification phase.
     * 
     * @param fmz
     *            Formalization (can be empty see UC 2.5.4) that the math engine
     *            needs to start a calculation
     * @param source
     *            Source of the Formalization (e.g. ExampleHierarchy or from
     *            Scratch)
     * @param type
     *            Requested ViewType for the CalcHead
     * @throws RemoteException
     */
    private void openNewWorksheet(IWorksheetDialog dialog)
            throws RemoteException {

        // InternalFrame, holding the Frames for the Worksheet and the
        // corresponding CalcHeadPanel
        // JInternalFrame internalFrame = createInternalFrame("Worksheet", true,
        // true, false, false);
        // internalFrame.setResizable(true);
        // internalFrame.setLayout(new BorderLayout());

        // DesktopPane, added to internalFrame to be able to hae multiple
        // InternalFrames in this InternalFrame
        // final JDesktopPane desktop = new JDesktopPane();
        // desktop.setSize(WORKSHEET_DESKTOP_PANE_INITIAL_SIZE_X,
        // WORKSHEET_DESKTOP_PANE_INITIAL_SIZE_Y);

        // desktop.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);

        // internalFrame.setContentPane(desktop); // , BorderLayout.CENTER);

        // main_frame_.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

        @SuppressWarnings("unused") // used in outcommented code
		MockWorksheet ws = new MockWorksheet(null, dialog, null);
        // #############################################################################################################################################
        // ..INSTEAD Worksheet ws = new Worksheet(this, dialog, desktop);
        // new InternalFrame for the Worksheet
        // JInternalFrame ws_frame = new JInternalFrame();
        // ((BasicInternalFrameUI) ws_frame.getUI()).setNorthPane(null);
        // ws_frame.setSize(WORKSHEET_FRAME_INITIAL_SIZE_X, main_frame_
        // .getHeight()
        // - WORKSHEET_FRAME_INITIAL_OFFSET_Y);

        // Container c;
        // c = ws_frame.getContentPane();
        // c.setLayout(new BorderLayout());
        // c.add(ws, BorderLayout.CENTER);

        // NC060726: The Worksheet desktop manager must not have the
        // BorderLayout
        // desktop.setLayout(new BorderLayout());
        // desktop.add(ws_frame, BorderLayout.CENTER);
        // desktop.add(ws_frame);

        // try {
        // ws_frame.setMaximum(true);
        // } catch (PropertyVetoException e) {
        // e.printStackTrace();
        // }

        // ws_frame.setFocusable(false);
        // ws_frame.setVisible(true);

        // internalFrame.addInternalFrameListener(ws);

        // int width, height = 0;
        // height = internalFrame.getPreferredSize().height;
        // width = internalFrame.getPreferredSize().width;
        // make Frame a little smaller than the window
        // height = main_frame_.getHeight() - WORKSHEET_FRAME_INITIAL_OFFSET_Y;
        // width = width < 600 ? WORKSHEET_FRAME_INITIAL_SIZE_X : width;

        // JInternalFrame[] frames = worksheetDesktopPane.getAllFrames();
        // int num_frames = frames.length - 4;

        // this.addInternalFrame(internalFrame);

        // internalFrame.setLocation(num_frames * 20, num_frames * 20);
        // internalFrame.setPreferredSize(new Dimension(width, height));
        // internalFrame.pack();
        // internalFrame.setVisible(true);

        // main_frame_.setCursor(Cursor.getDefaultCursor());
    }

    // -----IToGUI--------------------------------------------------
    /** @deprecated in favour of doUIAction(IUIAction action)
     */
    public boolean doUIAction(IUserAction action) throws RemoteException {
        return false;
    }

    /**
     * only has to handle UI_OPEN_WORKSHEET + UI_OPEN_BROWSER_FRAME ! exact copy
     * from isac.gui.WindowApplication#doUIAction(IUIAction action)
     */
    public void doUIAction(IUIAction action) throws RemoteException {

        if (logger.isInfoEnabled())
            logger.info(("WA<-DG: TestWA#doUIAction : Action is "
                    + action.getAction() + ", Context is " + action
                    .getContext()));

        // / if it is not our context go back
        if (action.getContext() != EUIContext.UI_CONTEXT_WINDOW_APPLICATION)
            return;

        switch (action.getAction()) {
        case UI_OPEN_WORKSHEET: {
            UIActionOnWSDialog uaowd = (UIActionOnWSDialog) action;
            openNewWorksheet(uaowd.getWorksheetDialog());
            break;
        }
        case UI_OPEN_BROWSER_FRAME:
            UIActionOnBrowserDialog uaobd = (UIActionOnBrowserDialog) action;
            openNewBrowserFrame(uaobd.getBrowserDialog(), uaobd.getFrameName());
            break;
        default: break;
        }
    }

    public void addUIElement(IUIAction action) throws RemoteException {
    }

    public boolean removeUIElement(IUIAction action) throws RemoteException {
        return false;
    }

    public boolean isUIRunning() throws RemoteException {
        return false;
    }

    // -----IToUser--------------------------------------------------
    // catch calcChanged to get the result of the calculation
    public void calcResponse(CalcEvent event) throws RemoteException {
        ICalcIterator from = ((CalcChanged) event).getLastUnchangedFormula();
        ICalcIterator to = ((CalcChanged) event).getLastGeneratedFormula();

		Vector<CalcFormula> calculation = from.getFormulaeFromTo(to, 3, false);
        for (int i = 0; i < calculation.size(); i++) {
            CalcFormula formula = (CalcFormula) calculation.elementAt(i);

            System.out.println(createEmptyString(formula.getPosition()
                    .getLevel())
                    + formula);
        }
        
    }

}
