/*
 * Created on Jul 4, 2005
 *  
 */
package isac.gui.calcheadviews;

import isac.gui.CalcHeadPanel;
import isac.util.formulae.CalcHead;
import isac.util.formulae.CalcHeadSimpleID;
import isac.util.formulae.Formula;
import isac.util.formulae.HierarchyKey;
import isac.util.formulae.Model;
import isac.util.formulae.ModelItem;
import isac.util.formulae.ModelItemList;
import isac.util.formulae.Position;
import isac.util.formulae.ProblemID;
import isac.util.formulae.Specification;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;


/**
 * @author Manuel Koschuch
 * 
 * Manually create Calchead and display in a Calchead-Pane
 */

public class TestCalcheadView implements
        ActionListener {

    protected CalcHeadPanel chp_;

    private JPanel button_panel_;

    private JButton start_solving_;

    private JButton comp_calcHead_;

    private JButton check_calcHead_;

    private JButton reset_;

    private JFrame frame_;

    private CalcHead ch_;

    private CalcHead ch_new_;

    private JButton button_;

    private JDialog dialog_;

    private JButton next_;

    public static void main(String[] args) {
        TestCalcheadView tv = new TestCalcheadView();
        tv.testShowCalchead();

    }

    protected void setUp() throws Exception {
        frame_ = new JFrame("CalcheadTest");
        frame_.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    protected void tearDown() throws Exception {
          }

    private CalcHead setupEquationCalcHead() {
        CalcHead ch = new CalcHead();

        ch.setStatusString("correct");
        //----------------------the headline
        Formula f = new Formula();
        f.setText("solve (x + 1 = 2, x)");
        ch.setHeadLine(f);

        //----------------------the model
        Model m = new Model();
        ModelItemList its;
        ModelItem it1 = new ModelItem(), it2 = new ModelItem();

        it1.setText("equality (x+1=2)");
        it1.setItemStatus("correct");
        it2.setText("solveFor x");
        it2.setItemStatus("correct");
        its = new ModelItemList();
        its.addItem(it1);
        its.addItem(it2);
        m.setGiven(its);

        ModelItem it3 = new ModelItem(), it4 = new ModelItem();

        it3.setText("matches (?a = ?b) (x+1=2)");
        it3.setItemStatus("correct");
        its = new ModelItemList();
        its.addItem(it3);
        m.setWhere(its);

        it4.setText("solutions L");
        it4.setItemStatus("correct");
        its = new ModelItemList();
        its.addItem(it4);
        m.setFind(its);
        ch.setModel(m);

        //----------------------
        ch.setBelongsTo("Pbl");
        //ch.setBelongsTo("Met");
        
        //----------------------the specification
        Specification s = new Specification();
        CalcHeadSimpleID id;

        id = new CalcHeadSimpleID();
        id.setID("Test.thy");
        s.setTheory(id);

        HierarchyKey idl = new ProblemID();
        idl.addString("sqroot-test");
        idl.addString("univariate");
        idl.addString("equation");
        idl.addString("test");
        s.setProblem(idl);

        idl = new HierarchyKey();
        idl.addString("Test");
        idl.addString("squ-equ-test-subpbl1");
        s.setMethod(idl);

        ch.setSpecification(s);

        Position p = new Position();

        ch.setPosition(p);

        ch.setViewstyle(CalcHead.VIEWSTYLE_FULL);
        //ch.setViewstyle(CalcHead.VIEWSTYLE_MODEL);
        //ch.setViewstyle(CalcHead.VIEWSTYLE_SPECIFICATION);
        return ch;
    }

  
    private void buildCalcHeadPanelButtons() {
        button_panel_ = new JPanel();
        start_solving_ = new JButton("start Solving");
        comp_calcHead_ = new JButton("complete CalcHead");
        check_calcHead_ = new JButton("check CalcHead");
        next_ = new JButton("next");
        reset_ = new JButton("reset");
        start_solving_.addActionListener(this);
        comp_calcHead_.addActionListener(this);
        check_calcHead_.addActionListener(this);
        reset_.addActionListener(this);
        next_.addActionListener(this);
        button_panel_.add(check_calcHead_);
        button_panel_.add(comp_calcHead_);
        button_panel_.add(start_solving_);
        button_panel_.add(next_);
        button_panel_.add(reset_);
        dialog_.getContentPane().add(button_panel_, BorderLayout.SOUTH);
    }

    public void testShowCalchead() {
        ch_new_ = new CalcHead();
        frame_ = new JFrame("CalcheadTest");

        dialog_ = new JDialog(frame_, "CalcheadTest");
        button_ = new JButton("Calchead");
        button_.addActionListener(this);

        ch_ = setupEquationCalcHead();
        chp_ = new CalcHeadPanel(ch_);

               
        buildCalcHeadPanelButtons();

        frame_.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame_.getContentPane().add(button_);
        
        dialog_.getContentPane().add(chp_, BorderLayout.NORTH);
        dialog_.setSize(600, 300);
        dialog_.pack();

        frame_.setSize(600, 300);
        frame_.pack();
        frame_.setVisible(true);
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == comp_calcHead_) {
            System.out.println("Complete Calchead pressed!");
            chp_.fillTextFieldsFromCalcHead(ch_);
            dialog_.pack();
            System.out.println("After Calchead pressed!");
        } else if (e.getSource() == check_calcHead_) {
            System.out.println("Check Calchead pressed");
            chp_.fillCalcHeadFromTextFields(ch_new_);
            System.out.println("New Calchead:");
            System.out.println(ch_new_.toSMLString());
        } else if (e.getSource() == button_) {
            System.out.println("Button pressed");
            dialog_.setVisible(true);
        } else if (e.getSource() == reset_) {
            System.out.println("reset Button pressed");
            chp_.reset();
            dialog_.pack();
        } else if (e.getSource() == next_) {
            System.out.println("next Button pressed");
        } else if (e.getSource() == start_solving_) {
            ch_.setBelongsTo("Met");
            chp_.fillTextFieldsFromCalcHead(ch_);
            ch_.setBelongsTo("Pbl");
        }
    }

}

