/**
 * @author Walther Neuper
 * Created on Sep 25, 2015
 * (c) copyright due to license terms
 */

package isac.gui.mawen

import isac.gui.mawen.syntax.Ast._  //"._" simplifies "Ast.Ast" to "Ast"
import info.hupel.isabelle.api.XML

  /*
   * terms for tests
   */
object TestDATA {
  
  //---------------------------- for <code>TestXml</code>
  // the priorities of the operators are from ~~/src/HOL/Groups.thy, Fields.thy, Power.thy, ...
  
  // "1 + (2 * x * (y + 3)) / (4 * z * (y + 3)) + 5" ... Isabelle's + is left-associative:
  // "(1 + (2 * x * (y + 3)) / (4 * z * (y + 3))) + 5"
  def term_1(): XML.Tree = {
    XML.Elem(("OP", List(("id", "+"), ("args", "2"), ("prior", "65"), ("fix", "in"))), List(
      XML.Elem(("OP", List(("id", "+"), ("args", "2"), ("prior", "65"), ("fix", "in"))), List(
        XML.Elem(("NUM", Nil), List(XML.Text ("1"))),              
        XML.Elem(("OP", List(("id", "/"), ("args", "2"), ("prior", "70"), ("fix", "in"))), List(
          XML.Elem(("OP", List(("id", "*"), ("args", "2"), ("prior", "70"), ("fix", "in"))), List(
            XML.Elem(("OP", List(("id", "*"), ("args", "2"), ("prior", "70"), ("fix", "in"))), List(
              XML.Elem(("NUM", Nil), List(XML.Text ("2"))),              
              XML.Elem(("VAR", Nil), List(XML.Text ("x")))
              ) ),              
            XML.Elem(("OP", List(("id", "+"), ("args", "2"), ("prior", "65"), ("fix", "in"))), List(
              XML.Elem(("VAR", Nil), List(XML.Text ("y"))),              
              XML.Elem(("NUM", Nil), List(XML.Text ("3")))
              ) )
            ) ),              
          XML.Elem(("OP", List(("id", "*"), ("args", "2"), ("prior", "70"), ("fix", "in"))), List(
            XML.Elem(("OP", List(("id", "*"), ("args", "2"), ("prior", "70"), ("fix", "in"))), List(
              XML.Elem(("NUM", Nil), List(XML.Text ("4"))),              
              XML.Elem(("VAR", Nil), List(XML.Text ("y")))
              ) ),              
            XML.Elem(("OP", List(("id", "+"), ("args", "2"), ("prior", "65"), ("fix", "in"))), List(
              XML.Elem(("VAR", Nil), List(XML.Text ("y"))),              
              XML.Elem(("NUM", Nil), List(XML.Text ("3")))
              ) )
            ) )
          ) )
        ) ),              
      XML.Elem(("NUM", Nil), List(XML.Text ("5")))
      ) )
  }
  // the above term cancelled
  // "1 + (2 * x) / (4 * z) + 5" ... Isabelle's + is left-associative, thus () omitted:
  // "(1 + (2 * x) / (4 * z)) + 5"
  def term_2(): XML.Tree = {
    XML.Elem(("OP", List(("id", "+"), ("args", "2"), ("prior", "65"), ("fix", "in"))), List(
      XML.Elem(("OP", List(("id", "+"), ("args", "2"), ("prior", "65"), ("fix", "in"))), List(
        XML.Elem(("NUM", Nil), List(XML.Text ("1"))),              
        XML.Elem(("OP", List(("id", "/"), ("args", "2"), ("prior", "70"), ("fix", "in"))), List(
          XML.Elem(("OP", List(("id", "*"), ("args", "2"), ("prior", "70"), ("fix", "in"))), List(
            XML.Elem(("NUM", Nil), List(XML.Text ("2"))),              
            XML.Elem(("VAR", Nil), List(XML.Text ("x")))
            ) ),              
          XML.Elem(("OP", List(("id", "*"), ("args", "2"), ("prior", "70"), ("fix", "in"))), List(
            XML.Elem(("NUM", Nil), List(XML.Text ("4"))),              
            XML.Elem(("VAR", Nil), List(XML.Text ("y")))
            ) )
          ) )
        ) ),              
      XML.Elem(("NUM", Nil), List(XML.Text ("5")))
      ) )
  }
  
  //---------------------------- for <code>TestWorksheetForMawen</code>
  // "Diff (x + sin (x ^ 2), x)"
  def term_Nil_Frm(): XML.Tree = {
    XML.Elem(("CASCMD", List(("id", "Diff"), ("args", "2"))), List(
      XML.Elem(("OP", List(("id", "+"), ("args", "2"), ("prior", "65"), ("fix", "in"))), List(
        XML.Elem(("VAR", Nil), List(XML.Text ("x"))),
        XML.Elem(("OP", List(("id", "sin"), ("args", "1"), ("prior", "99"), ("fix", "pre"))), List(
          XML.Elem(("OP", List(("id", "^"), ("args", "2"), ("prior", "80"), ("fix", "in"))), List(
            XML.Elem(("VAR", Nil), List(XML.Text ("x"))),
            XML.Elem(("NUM", Nil), List(XML.Text ("2"))))))))),
      XML.Elem(("VAR", Nil), List(XML.Text ("x")))))
  }
  
  // "d_d x (x + sin (x ^ 2))" as it comes from Isabelle (as SMLString)
  def term_1_Frm_from_ISA(): XML.Tree = {                   // vvvvvvvvvvv--- enforces (term)
    XML.Elem(("OP", List(("id", "d_d"), ("args", "2"), ("prior", "99"), ("fix", "pre"))), List(
      XML.Elem(("VAR", Nil), List(XML.Text ("x"))),
      XML.Elem(("OP", List(("id", "+"), ("args", "2"), ("prior", "65"), ("fix", "in"))), List(
        XML.Elem(("VAR", Nil), List(XML.Text ("x"))),
        XML.Elem(("OP", List(("id", "sin"), ("args", "1"), ("prior", "99"), ("fix", "pre"))), List(
          XML.Elem(("OP", List(("id", "^"), ("args", "2"), ("prior", "80"), ("fix", "in"))), List(
            XML.Elem(("VAR", Nil), List(XML.Text ("x"))),
            XML.Elem(("NUM", Nil), List(XML.Text ("2")))))))))))
  }
  // "d_d x (x + sin (x ^ 2))" as presented to the gui ??????????????????????????????????????????????????
  // version a: involves not-clarified mixfix notation
  def term_1_Frm(): XML.Tree = {
    // ambiguous..: no division -------v             vvv             vvv--- like "=" in HOL.thy ?????????
    XML.Elem(("OP", List(("id", "/"), ("args", "3"), ("prior", "50"), ("fix", "mixfix"))), List(
      XML.Elem(("CONST", Nil), List(XML.Text ("d"))),  // CONST or OP or VAR ??????????????????????
      XML.Elem(("VAR", Nil), List(XML.Text ("x"))),
      XML.Elem(("OP", List(("id", "+"), ("args", "2"), ("prior", "65"), ("fix", "in"))), List(
        XML.Elem(("VAR", Nil), List(XML.Text ("x"))),
        XML.Elem(("OP", List(("id", "sin"), ("args", "1"), ("prior", "99"), ("fix", "pre"))), List(
            XML.Elem(("OP", List(("id", "^"), ("args", "2"), ("prior", "80"), ("fix", "in"))), List(
              XML.Elem(("VAR", Nil), List(XML.Text ("x"))),
              XML.Elem(("NUM", Nil), List(XML.Text ("2")))))))))))
  } // if ("args", "2") then we have no root for "/ d x" and "(x + sin (x ^ 2))" ????????????????????????
  // but we could treat this as fraction --------^^^^^^^ and ^^^^^^^^^^^^^^^^^^^ is dangling without root

  // "d_d x (x + sin (x ^ 2))" as presented to the gui ??????????????????????????????????????????????????
  // version b: allows to represent "d_d x" as a normal fraction, but introduces additional TERM
  def term_1_Frmb(): XML.Tree = {
    XML.Elem(("TERM", List(("args", "2"))), List(
      XML.Elem(("OP", List(("id", "/"), ("args", "2"), ("prior", "70"), ("fix", "in"))), List(
        XML.Elem(("CONST", Nil), List(XML.Text ("d"))),  // CONST or OP or VAR ????????????????????
        XML.Elem(("TERM", List(("args", "2"))), List(   // "d x" in the denominator have no root otherwise !!!!!!!!
          XML.Elem(("CONST", Nil), List(XML.Text ("d"))),
          XML.Elem(("VAR", Nil), List(XML.Text ("x"))))))),
      XML.Elem(("OP", List(("id", "+"), ("args", "2"), ("prior", "65"), ("fix", "in"))), List(
        XML.Elem(("VAR", Nil), List(XML.Text ("x"))),
        XML.Elem(("OP", List(("id", "sin"), ("args", "1"), ("prior", "99"), ("fix", "pre"))), List(
            XML.Elem(("OP", List(("id", "^"), ("args", "2"), ("prior", "80"), ("fix", "in"))), List(
              XML.Elem(("VAR", Nil), List(XML.Text ("x"))),
              XML.Elem(("NUM", Nil), List(XML.Text ("2")))))))))))
  }
  // CONCLUSION: either d_d as mixfix (with fraction?), or introduce TERM as root, or ???????????????????
  
  // TestData from TestUseCases.testUC_userguide()
  def ast_05() : Ast =
      Appl(List(
        Constant("Groups.plus_class.plus"),
        Appl(List(
          Constant("Diff.d_d"),
          Variable("x"),
          Variable("x"))),
        Appl(List(
          Constant("Diff.d_d"),
          Variable("x"),
          Appl(List(                 //added to above
            Constant("BOX.1"),       //added to above
            Appl(List(
              Constant("Transcendental.sin"),
              Appl(List(             //added to above
                Constant("BOX.2"),   //added to above
                Appl(List(
                  Constant("Power.power_class.power"),
                  Variable("x"),
                  Variable("2")))))))))))))
                  
    def ast_06() : Ast =
      Appl(List(
        Constant("HOL.eq"),
        Appl(List(
          Constant("Diff.d_d"),
          Variable("bdv"),
          Appl(List(                 //added to above
            Constant("BOX.1"),       //added to above
            Appl(List(
              Constant("Transcendental.sin"),
              Appl(List(             //added to above
                Constant("BOX.2"),   //added to above
                Variable("u"))))))))),
        Appl(List(
          Constant("Groups.times_class.times"),
          Appl(List(                 //added to above
            Constant("BOX.3"),       //added to above
            Appl(List(
              Constant("Transcendental.cos"),
              Variable("u"))))),
          Appl(List(
            Constant("Diff.d_d"),
            Variable("bdv"),
            Appl(List(               //added to above
              Constant("BOX.4"),     //added to above
              Variable("u")))))))))
              
    def ast_07() : Ast =
      Appl(List(
        Constant("Groups.plus_class.plus"),
        Appl(List(
          Constant("Diff.d_d"),
          Variable("x"),
          Variable("x"))),
        Appl(List(
          Constant("Groups.times_class.times"),
          Appl(List(                 //added to above
            Constant("BOX.3"),       //added to above
            Appl(List(
              Constant("Transcendental.cos"),
              Appl(List(
                Constant("Power.power_class.power"),
                Variable("x"),
                Variable("2"))))))),
          Appl(List(
            Constant("Diff.d_d"),
            Variable("x"),
            Appl(List(               //added to above
              Constant("BOX.4"),     //added to above
              Variable("GAP")))))))))

}