/*
 * @author Natalie Karl
 * Copyright (c) 2015 by Natalie Karl
 * FH Hagenberg, Software Engineering 
 * 
 * Use is subject to license terms.
 */
package isac.gui.mawen;

import static isac.gui.IGuiConstants.WINDOW_APPLICATION_INITIAL_SIZE_X;
import static isac.gui.IGuiConstants.WINDOW_APPLICATION_INITIAL_SIZE_Y;
import static isac.gui.IGuiConstants.WORKSHEET_DESKTOP_PANE_INITIAL_SIZE_X;
import static isac.gui.IGuiConstants.WORKSHEET_DESKTOP_PANE_INITIAL_SIZE_Y;
import static isac.gui.IGuiConstants.WORKSHEET_FRAME_INITIAL_OFFSET_Y;
import static isac.gui.IGuiConstants.WORKSHEET_FRAME_INITIAL_SIZE_X;
import isac.gui.IToWorksheetUser;
import isac.gui.Worksheet;
import isac.interfaces.ICalcIterator;
import isac.useractions.EUIElement;
import isac.useractions.IUIAction;
import isac.util.CalcChanged;
import isac.util.formulae.CalcFormula;
import isac.util.formulae.Position;
import isac.wsdialog.IWorksheetDialog;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.beans.PropertyVetoException;
import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
import javax.swing.JMenuBar;
import javax.swing.plaf.basic.BasicInternalFrameUI;
import javax.swing.SwingUtilities;

import junit.framework.TestCase;

/**
 * @author Natalie Karl
 * 
 *         With a minimum of code present the terms of the example for reference
 *         on screen.
 * 
 *         <code>Worksheet</code> is launched by
 *         <code>WindowApplication#openNewWorksheet</code>, which involves a
 *         <code>IWorksheetDialog</code>, which in turn involves RMI (the
 *         <code>WorksheetDialog</code> runs in the <code>ObjectManager</code>).
 * 
 *         We don't want RMI in this TestCase and modify code of
 *         <code>WindowApplication#openNewWorksheet</code> such that RMI is
 *         excluded.
 *         
 *         For running this Test see comment in testWorksheetIsac().
 * 
 */
public class TestWorksheetForMawen extends TestCase implements IToWorksheetUser 
{

	private static JFrame frame_;
	private JDesktopPane desktop_pane_;
	protected static Worksheet currentSession;

	@SuppressWarnings("unused") // is used below; deletion causes error
	private HashMap<JButton, EUIElement> map_buttons_actions_;
	@SuppressWarnings("unused") // is used below; deletion causes error
	private HashMap<EUIElement, JButton> map_actions_buttons_;
	private JMenuBar menu_bar_;
	@SuppressWarnings("unused") // is used below; deletion causes error
	private int num_browser_frames_;
	@SuppressWarnings("unused") // is used below; deletion causes error
	private boolean menu_separator_added_;

	// from "implements IToWorksheetUser"
	@Override
	public void addToMenuBar(IUIAction action) {
	}

	@Override
	public boolean removeFromMenuBar(IUIAction action) {
		return false;
	}

	@Override
	public void addCloseListener(Worksheet ws) {
	}

	@Override
	public void removeCloseListener(Worksheet ws) {
	}

	// ======= copies from constructor of WindowApplication
	private JFrame constructorWindowApplication() {
		menu_separator_added_ = false;

		frame_ = new JFrame();
		frame_.setSize(new Dimension(WINDOW_APPLICATION_INITIAL_SIZE_X,
				WINDOW_APPLICATION_INITIAL_SIZE_Y));
		// locate frame in the middle of the screen
		frame_.setLocationRelativeTo(null);

		// initialize hashMaps
		map_buttons_actions_ = new HashMap<JButton, EUIElement>();
		map_actions_buttons_ = new HashMap<EUIElement, JButton>();

		menu_bar_ = new JMenuBar();
		frame_.setJMenuBar(menu_bar_);
		num_browser_frames_ = 0;

		this.createWorksheetPane();
		frame_.add(desktop_pane_);
		JFrame.setDefaultLookAndFeelDecorated(true);

		frame_.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		frame_.setVisible(true);
		return frame_;
	}

	// ------ methods from WindowApplication
	private static WindowListener windowListener = new WindowAdapter() {

		public void windowClosing(WindowEvent w) {
			frame_.setVisible(false);
			frame_.dispose();
			System.exit(0);
		}
	};

	protected static void startApplication(/* final JFrame windowApplication */) {
		SwingUtilities.invokeLater(new Runnable() { // Start the application
					public void run() {
						frame_.addWindowListener(windowListener);
					}
				});
	}

	public void createWorksheetPane() {
		desktop_pane_ = new JDesktopPane();
		Dimension d = new Dimension();
		d.setSize(frame_.getWidth() * 0.60, 0.0);
		desktop_pane_.setMinimumSize(d);
		desktop_pane_.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
	}

	private void addInternalFrame(JInternalFrame frame) {
		desktop_pane_.add(frame);
		desktop_pane_.getDesktopManager().activateFrame(frame);
		frame.toFront();
		frame.grabFocus();
	}

	private JInternalFrame createInternalFrame(String title, boolean resizable,
			boolean closable, boolean maximizable, boolean iconifiable) {
		JInternalFrame internalFrame = new JInternalFrame(title, resizable,
				closable, maximizable, iconifiable);
		return internalFrame;
	}

	private void openNewWorksheet(IWorksheetDialog dialog)
			throws RemoteException {
		// InternalFrame, holding the Frames for the Worksheet and the
		// corresponding CalcHeadPanel
		JInternalFrame internalFrame = createInternalFrame("Worksheet", true,
				true, false, false);
		internalFrame.setResizable(true);
		internalFrame.setLayout(new BorderLayout());

		// DesktopPane, added to internalFrame to be able to handle multiple
		// InternalFrames in this InternalFrame
		final JDesktopPane desktop = new JDesktopPane();
		desktop.setSize(WORKSHEET_DESKTOP_PANE_INITIAL_SIZE_X,
				WORKSHEET_DESKTOP_PANE_INITIAL_SIZE_Y);

		desktop.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
		internalFrame.setContentPane(desktop); // , BorderLayout.CENTER);

		frame_.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

		Worksheet ws = new Worksheet((IToWorksheetUser) this, dialog, desktop);
		// new InternalFrame for the Worksheet
		JInternalFrame ws_frame = new JInternalFrame();
		((BasicInternalFrameUI) ws_frame.getUI()).setNorthPane(null);
		ws_frame.setSize(WORKSHEET_FRAME_INITIAL_SIZE_X, frame_.getHeight()
				- WORKSHEET_FRAME_INITIAL_OFFSET_Y);

		Container c;
		c = ws_frame.getContentPane();
		c.setLayout(new BorderLayout());
		c.add(ws, BorderLayout.CENTER);
		desktop.add(ws_frame);
		try {
			ws_frame.setMaximum(true);
		} catch (PropertyVetoException e) {
			e.printStackTrace();
		}
		ws_frame.setFocusable(false);
		ws_frame.setVisible(true);

		internalFrame.addInternalFrameListener(ws);

//		int width, height = 0;
//		height = internalFrame.getPreferredSize().height;
//		width = internalFrame.getPreferredSize().width;
//		// make Frame a little smaller than the window
//		height = frame_.getHeight() - WORKSHEET_FRAME_INITIAL_OFFSET_Y;
//		width = width < 600 ? WORKSHEET_FRAME_INITIAL_SIZE_X : width;
//
//		JInternalFrame[] frames = desktop_pane_.getAllFrames();
//		int num_frames = frames.length - 4;
		this.addInternalFrame(internalFrame);
//		internalFrame.setLocation(num_frames * 20, num_frames * 20);
//		internalFrame.setPreferredSize(new Dimension(width, height));
		internalFrame.setLocation(20, 10);
		internalFrame.setPreferredSize(new Dimension(500, 300));
		internalFrame.pack();
		internalFrame.setVisible(true);

		frame_.setCursor(Cursor.getDefaultCursor());
	}

	/* protocol of building testWorksheetIsac (below): getFormulaeFromTo */
	public void testModDialogIterator1() {
		System.out.println("/--BEGIN isac.gui.mawen.testModDialogIterator1");
		
		Vector<Integer> p = new Vector<Integer>(); p.add(1);
		Position pos = new Position(p, "Res");
		ICalcIterator from = new MockDialogIterator(pos);
		p = new Vector<Integer>(); p.add(2);
	    pos = new Position(p, "Res");
		ICalcIterator to = new MockDialogIterator(pos);
		
		Vector<CalcFormula> fs = null;
		try { // in the test NO rmi is involved
			fs = from.getFormulaeFromTo(to, 0, false);
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		assertEquals(fs.get(0).getPosition().toSMLString(), "([2],Res)");
		
		// -------------------------------------------------------------------
		p = new Vector<Integer>(); p.add(1);
		pos = new Position(p, "Frm");
		from = new MockDialogIterator(pos);
		p = new Vector<Integer>(); p.add(2);
	    pos = new Position(p, "Res");
		to = new MockDialogIterator(pos);
		
		fs = null;
		try { // in the test NO rmi is involved
			fs = from.getFormulaeFromTo(to, 0, false);
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		assertEquals(fs.get(0).getPosition().toSMLString(), "([1],Res)");
		assertEquals(fs.get(1).getPosition().toSMLString(), "([2],Res)");

		System.out.println("\\--END isac.gui.mawen.testModDialogIterator1");
	}

	/* protocol of building testWorksheetIsac (below): CalcChanged */
	public void testModDialogIterator2() {
		System.out.println("/--BEGIN isac.gui.mawen.testModDialogIterator2");
		
		Vector<Integer> p = new Vector<Integer>();
		Position pos = new Position(p, "Pbl");
		ICalcIterator ci = new MockDialogIterator(pos);

		CalcChanged e = new CalcChanged(4711, false, ci, ci, ci);
    	ICalcIterator from = e.getLastUnchangedFormula();
    	@SuppressWarnings("unused") // is used below; deletion causes error
    	ICalcIterator del = e.getLastDeletedFormula();
    	ICalcIterator to = e.getLastGeneratedFormula();
		
    	Vector<CalcFormula> fs = null;
		try {
			fs = from.getFormulaeFromTo(to, 0, false);
		} catch (RemoteException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		assertEquals(fs.size(), 1);
		assertEquals(fs.get(0).getPosition().toSMLString(), "([],Pbl)");
//		assertEquals(fs.get(0).getFormula().toSMLString(), "Simplify (1 + (2 * x * (y + 3)) / (4 * z * (y + 3)) + 5)");
		assertEquals(fs.get(0).getFormula().toSMLString(), "1+((2*x*(y+3)+((y^2+6*y+9)/(y+3)))/(4*z*(y+3)))+(5::real)");
		
		// -------------------------------------------------------------------
		p = new Vector<Integer>(); p.add(1);
	    pos = new Position(p, "Res");
	    ICalcIterator unc = new MockDialogIterator(pos);
		p = new Vector<Integer>(); p.add(2);
	    pos = new Position(p, "Res");
	    ICalcIterator gen = new MockDialogIterator(pos);
		
		e = new CalcChanged(4711, false, unc, unc, gen);
		from = e.getLastUnchangedFormula();
    	del = e.getLastDeletedFormula();
    	to = e.getLastGeneratedFormula();
		
    	fs = null;
		try {
			fs = from.getFormulaeFromTo(to, 0, false);
		} catch (RemoteException e1) {
			e1.printStackTrace();
		}
		System.out.println(fs.size());
		System.out.println(fs.get(0).getPosition().toSMLString());
		System.out.println(fs.get(0).getFormula().toSMLString());

		assertEquals(fs.size(), 1);
		assertEquals(fs.get(0).getPosition().toSMLString(), "([2],Res)");
//		assertEquals(fs.get(0).getFormula().toSMLString(), "1 + x / (2 * z) + 5");
		assertEquals(fs.get(0).getFormula().toSMLString(), "1+(((2*x+1)*(y+3))/(4*z*(y+3)))+(5::real)");

		
		System.out.println("\\--END isac.gui.mawen.testModDialogIterator2");
	}

	/* display example for reference in a Worksheet, preperations:
	 * 
	 * 1. In MockCalcTree define the formulas to be displayed by
	 * 1.1. writing the formulas into a methode build*
	 * 1.2. call this method in the constructor.
	 * 1.3. uncomment lines in testWorksheetIsac according to no. of formulas
	 * 
	 * 2. Run the test in the debugger (because some bug in the Mock* makes the test hang)
	 * 2.1. Set Breakpoint at isac.gui.treetable.CalculationModel.calcChanged() in line 90
	 * 2.2. hit <step over> down to the end of calcChanged()
	 * 2.3. repeat hits accordng to no. of formulas.
	 * 
	 */
	public void testWorksheetIsac() {
			System.out.println("/--BEGIN isac.gui.mawen.testWorksheetIsac");

		frame_ = constructorWindowApplication();
		startApplication(/* windowApplication */);
		MockWorksheetDialog wd = new MockWorksheetDialog();

		try {
			this.openNewWorksheet(wd); // <<<<<<<-----------------------------------
			
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		this.createWorksheetPane();
		
		Position from; Position to;
		//-->ISA: getFormulaeFromTo 1 ([],Pbl) ([],Pbl) 0 false;
		from = new Position(); from.setKind("Pbl");
		to = new Position(); to.setKind("Pbl");
		try {
      Thread.sleep(10);
    } catch (InterruptedException e1) {
      // TODO Auto-generated catch block
      e1.printStackTrace();
    }
		wd.sendFormulaToWorksheet (from, to); // the root-node in CalcModelHierarchy
		try {
      Thread.sleep(10);
    } catch (InterruptedException e1) {
      // TODO Auto-generated catch block
      e1.printStackTrace();
    }
		//-->ISA: getFormulaeFromTo 1 ([],Met) ([1],Frm) 0 false;
		from = new Position(); from.setKind("Met");
		to = new Position(); to.addInt("1"); to.setKind("Frm");
		try {
      Thread.sleep(100);
    } catch (InterruptedException e1) {
      // TODO Auto-generated catch block
      e1.printStackTrace();
    }
		wd.sendFormulaToWorksheet (from, to);
		try {
      Thread.sleep(100);
    } catch (InterruptedException e1) {
      // TODO Auto-generated catch block
      e1.printStackTrace();
    }
		//-->ISA: getFormulaeFromTo 1 ([1],Frm) ([1],Res) 0 false;
		from = new Position(); from.addInt("1"); from.setKind("Frm");
		to = new Position(); to.addInt("1"); to.setKind("Res");
		try {
      Thread.sleep(10);
    } catch (InterruptedException e1) {
      // TODO Auto-generated catch block
      e1.printStackTrace();
    }
		wd.sendFormulaToWorksheet (from, to);
		try {
      Thread.sleep(10);
    } catch (InterruptedException e1) {
      // TODO Auto-generated catch block
      e1.printStackTrace();
    }
//		//-->ISA: getFormulaeFromTo 1 ([1],Res) ([2],Res) 0 false;
//		from = new Position(); from.addInt("1"); from.setKind("Res");
//		to = new Position(); to.addInt("2"); to.setKind("Res");
//		try {
//      Thread.sleep(10);
//    } catch (InterruptedException e1) {
//      // TODO Auto-generated catch block
//      e1.printStackTrace();
//    }
//		wd.sendFormulaToWorksheet (from, to);
//		try {
//      Thread.sleep(10);
//    } catch (InterruptedException e1) {
//      // TODO Auto-generated catch block
//      e1.printStackTrace();
//    }
//		//-->ISA: getFormulaeFromTo 1 ([2],Res) ([3],Res) 0 false;
//		from = new Position(); from.addInt("2"); from.setKind("Res");
//		to = new Position(); to.addInt("3"); to.setKind("Res");
//		try {
//      Thread.sleep(10);
//    } catch (InterruptedException e1) {
//      // TODO Auto-generated catch block
//      e1.printStackTrace();
//    }
//		
//		wd.sendFormulaToWorksheet (from, to);
//		try {
//      Thread.sleep(10);
//    } catch (InterruptedException e1) {
//      // TODO Auto-generated catch block
//      e1.printStackTrace();
//    }
//		//ONLY SCEN 5-->ISA: getFormulaeFromTo 1 ([3],Res) ([4],Res) 0 false;
//	    from = new Position(); from.addInt("3"); from.setKind("Res");
//	    to = new Position(); to.addInt("4"); to.setKind("Res");
//	    try {
//	      Thread.sleep(10);
//	    } catch (InterruptedException e1) {
//	      // TODO Auto-generated catch block
//	      e1.printStackTrace();
//	    }
//	    wd.sendFormulaToWorksheet (from, to);
//	    try {
//	      Thread.sleep(10);
//	    } catch (InterruptedException e1) {
//	      // TODO Auto-generated catch block
//	      e1.printStackTrace();
//	    }
//	    from = new Position(); from.addInt("4"); from.setKind("Res");
//	    to = new Position(); to.addInt("4"); to.setKind("Res");
//	    
//	    try {
//	      Thread.sleep(10);
//	    } catch (InterruptedException e1) {
//	      // TODO Auto-generated catch block
//	      e1.printStackTrace();
//	    }
//	    wd.sendFormulaToWorksheet (from, to);
	    //END ONLY SCEN 5-->ISA: getFormulaeFromTo 1 ([3],Res) ([4],Res) 0 false;
		try { Thread.sleep(100000); } catch (InterruptedException e) { e.printStackTrace(); }
		System.out.println("\\--END isac.gui.mawen.testWorksheetIsac");
	}
}
