/**
 * @author Robert Koenighofer
 * created on Jul 10, 2006
 */
package isac.util;

import junit.framework.TestCase;

/**
 * This is a Junit test class for KEStoreCommunication, which tests the
 * loadHierarchy and the loadContent functionality for all items of the
 * knowledge base (problems, methods, theories and examples)
 * 
 * @author Robert Koenighofer
 * @see isac.util.KEStoreCommunication#loadHierarchy(String, String)
 * @see isac.util.KEStoreCommunication#loadContent(String, String, String)
 * 
 * Note WN170215: test* 2,4,6,.. herein are destroyed by @see isac.users.TestUserManager,
 * Thus @see isac.users.TestUserManager is outcommented in @see isac.users.Testall.
 */
public class TestKEStoreCommunication extends TestCase {

    /**
     * This method loads the problem hierarchy by use of the
     * KEStoreCommunication.loadHierarchy(). Some primitive checks are made to
     * ensure that the returned String is no nonsense.
     * 
     * @see isac.util.KEStoreCommunication#loadHierarchy(String, String)
     */
    public void testLoadHierarchyForProblems() {
        System.out.println("---BEGIN isac.util.TestKEStoreCommunication#testLoadHierarchyForProblems");
        
        String hierarchy_string = loadHirarchy("pbl");
        
        assertStartsWithXmlSignature(hierarchy_string);
        System.out.println("---END isac.util.TestKEStoreCommunication#testLoadHierarchyForProblems");
    }

    /**
     * This method loads an item of the problem hierarchy by use of the
     * KEStoreCommunication.loadContent(). The problem hierarchy is loaded
     * first, to get the name of an item to load. The names of hierarchy items
     * are encapsulated by <CONTENTREF> and </CONTENTREF> tags. Some primitive
     * checks are made to ensure that the returned String is no nonsense.
     * 
     * @see isac.util.KEStoreCommunication#loadContent(String, String, String)
     */
    public void testLoadContentProblemItem() {
        System.out.println("---BEGIN isac.util.TestKEStoreCommunication#testLoadContentProblemItem");
        
        String hierarchy_item = getHirarchyItemViaKeStore("pbl");
        String xml_item_string = loadXmlItemFromKeStore(hierarchy_item, "pbl");
        
        assertStartsWithXmlSignature(xml_item_string);
        System.out.println("---END isac.util.TestKEStoreCommunication#testLoadContentProblemItem");
    }

    /**
     * This method loads the method hierarchy by use of the
     * KEStoreCommunication.loadHierarchy(). Some primitive checks are made to
     * ensure that the returned String is no nonsense.
     * 
     * @see isac.util.KEStoreCommunication#loadHierarchy(String, String)
     */
    public void testLoadHierarchyForMethods() {
        System.out.println("---BEGIN isac.util.TestKEStoreCommunication#testLoadHierarchyForMethods");
        
        String hierarchy_string = loadHirarchy( "met");

        assertStartsWithXmlSignature(hierarchy_string);
        System.out.println("---END isac.util.TestKEStoreCommunication#testLoadHierarchyForMethods");
    }

    /**
     * This method loads an item of the method hierarchy by use of the
     * KEStoreCommunication.loadContent(). The method hierarchy is loaded first,
     * to get the name of an item to load. The names of hierarchy items are
     * encapsulated by <CONTENTREF> and </CONTENTREF> tags. Some primitive
     * checks are made to ensure that the returned String is no nonsense.
     * 
     * @see isac.util.KEStoreCommunication#loadContent(String, String, String)
     */
    public void testLoadContentMethodItem() {
        System.out.println("---BEGIN isac.util.TestKEStoreCommunication#testLoadContentMethodItem");

        String hierarchy_item = getHirarchyItemViaKeStore("met");
        String xml_item_string = loadXmlItemFromKeStore(hierarchy_item, "met");
        
        assertStartsWithXmlSignature(xml_item_string);
        System.out.println("---END isac.util.TestKEStoreCommunication#testLoadContentMethodItem");
    }

    /**
     * This method loads the theory hierarchy by use of the
     * KEStoreCommunication.loadHierarchy(). Some primitive checks are made to
     * ensure that the returned String is no nonsense.
     * 
     * @see isac.util.KEStoreCommunication#loadHierarchy(String, String)
     */
    public void testLoadHierarchyForTheory() {
        System.out.println("---BEGIN isac.util.TestKEStoreCommunication#testLoadHierarchyForTheory");
        
        String hierarchy_string = loadHirarchy( "thy");

        assertStartsWithXmlSignature(hierarchy_string);
        System.out.println("---END isac.util.TestKEStoreCommunication#testLoadHierarchyForTheory");
    }

    /**
     * This method loads an item of the theory hierarchy by use of the
     * KEStoreCommunication.loadContent(). The theory hierarchy itself is loaded
     * first, to get the name of an item to load. The names of hierarchy items
     * are encapsulated by <CONTENTREF> and </CONTENTREF> tags. Some primitive
     * checks are made to ensure that the returned String is no nonsense.
     * 
     * @see isac.util.KEStoreCommunication#loadContent(String, String, String)
     */
    public void testLoadContentTheoryItem() {
        System.out.println("---BEGIN isac.util.TestKEStoreCommunication#testLoadContentTheoryItem");

        String hierarchy_item = getHirarchyItemViaKeStore("thy");
        String xml_item_string = loadXmlItemFromKeStore(hierarchy_item, "thy");
        
        assertStartsWithXmlSignature(xml_item_string);
        System.out.println("---END isac.util.TestKEStoreCommunication#testLoadContentTheoryItem");
    }

    /**
     * This method loads the example hierarchy by use of the
     * KEStoreCommunication.loadHierarchy(). Some primitive checks are made to
     * ensure that the returned String is no nonsense.
     * 
     * @see isac.util.KEStoreCommunication#loadHierarchy(String, String)
     */
    public void testLoadHierarchyForExamples() {
        System.out.println("---BEGIN isac.util.TestKEStoreCommunication#testLoadHierarchyForExamples");
        
        String hierarchy_string = loadHirarchy("exp");

        assertStartsWithXmlSignature(hierarchy_string);
        System.out.println("---END isac.util.TestKEStoreCommunication#testLoadHierarchyForExamples");
    }

    /**
     * This method loads an item of the example hierarchy by use of the
     * KEStoreCommunication.loadContent(). The example hierarchy itself is
     * loaded first, to get the name of an item to load. The names of hierarchy
     * items are encapsulated by <CONTENTREF> and </CONTENTREF> tags. Some
     * primitive checks are made to ensure that the returned String is no
     * nonsense.
     * 
     * @see isac.util.KEStoreCommunication#loadContent(String, String, String)
     */
    public void testLoadContentExampleItem() {
        System.out.println("---BEGIN isac.util.TestKEStoreCommunication#testLoadContentExampleItem");
        
        String hierarchy_item = getHirarchyItemViaKeStore("exp");
        String xml_item_string = loadXmlItemFromKeStore(hierarchy_item, "exp");
        
        assertStartsWithXmlSignature(xml_item_string);
        System.out.println("---END isac.util.TestKEStoreCommunication#testLoadContentExampleItem");
    }
    
    private String getHirarchyItemViaKeStore(String type) {

        String hierarchy_string = KEStoreCommunication.loadHierarchy("", type);
        return getHirarchyItem(hierarchy_string);
    }

    private String getHirarchyItem(String hierarchy_string) {
        int first_item_start = hierarchy_string.indexOf("<CONTENTREF>");
        first_item_start += new String("<CONTENTREF>").length();
        int first_item_end = hierarchy_string.indexOf("</CONTENTREF>");
        String hierarchy_item = hierarchy_string.substring(first_item_start,
                first_item_end);
        hierarchy_item = hierarchy_item.trim();
        return hierarchy_item;
    }    
    
    private String loadHirarchy(String type) {
        String hierarchy_string = "";
        try {
            hierarchy_string = KEStoreCommunication.loadHierarchy("", type);
        } catch (Exception ex) {
            assertTrue("exception caught", false);
        }
        return hierarchy_string;
    }
    
    private String loadXmlItemFromKeStore(String hierarchy_item, String type) {
        String xml_item_string = "";
        try {
            xml_item_string = KEStoreCommunication.loadContent("", type,
                    hierarchy_item + ".xml");
        } catch (Exception ex) {
            assertTrue("exception caught", false);
        }
        return xml_item_string;
    }

    private void assertStartsWithXmlSignature(String xml) {
        assertNotNull("returned string is null", xml);
        assertFalse("returned string is empty", xml.equals(""));
        String expected_beginning = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>";
        assertTrue("returned string starts strange", xml.startsWith(expected_beginning));
    }
}
