/*
 * @author Walther Neuper, member of the ISAC-team, Copyright (c) 2004 by
 * Walther Neuper created Oct 7, 2004, 5:51:50 PM Institute for
 * Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to license terms.
 */
package isac.util.formulae;

import junit.framework.TestCase;

/**
 * @author Walther Neuper Oct 7, 2004, 5:51:50 PM
 * @version 1.0
 */
public class TestCalcHead extends TestCase {

    private CalcHead calchead_ = new CalcHead();

    protected void setUp() throws Exception {
        //the complete CalcHead for mini-subpbl
        calchead_.setStatusString("correct");
        //----------------------the headline
        Formula f = new Formula();
        f.setText("solve (x + 1 = 2, x)");
        calchead_.setHeadLine(f);

        //----------------------the model
        Model m = new Model();
        ModelItemList its;
        ModelItem it1 = new ModelItem(), it2 = new ModelItem();

        it1.setText("equality (x+1=2)");
        it1.setItemStatus("correct");
        it2.setText("solveFor x");
        it2.setItemStatus("correct");
        its = new ModelItemList();
        its.addItem(it1);
        its.addItem(it2);
        m.setGiven(its);//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

        //create 2 new modelItems, since they are passed to addItem just as
        // references,
        //setting new Text for it1 and it2 would affect given_ too

        ModelItem it3 = new ModelItem(), it4 = new ModelItem();
        it3.setText("matches (?a = ?b) (x+1=2)");
        it3.setItemStatus("correct");
        its = new ModelItemList();
        its.addItem(it3);
        m.setWhere(its);//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

        it4.setText("solutions L");
        it4.setItemStatus("correct");
        its = new ModelItemList();
        its.addItem(it4);
        m.setFind(its);//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        calchead_.setModel(m);

        //----------------------
        calchead_.setBelongsTo("Pbl");

        //----------------------the specification
        Specification s = new Specification();
        CalcHeadSimpleID id;

        id = new CalcHeadSimpleID();
        id.setID("Test.thy");
        s.setTheory(id);

        HierarchyKey idl = new ProblemID();
        idl.addString("sqroot-test");
        idl.addString("univariate");
        idl.addString("equation");
        idl.addString("test");
        s.setProblem(idl);

        idl = new HierarchyKey();
        idl.addString("Test");
        idl.addString("squ-equ-test-subpbl1");
        s.setMethod(idl);

        calchead_.setSpecification(s);

        Position p = new Position();

        calchead_.setPosition(p);

    }

    //	/**
    //	 * this method may be required by the WorksheetDialog in the future
    //	 */
    //	public void testUsesProblem() {
    //		System.out.println("---BEGIN isac.util.formulae.TestCalcHead");
    //		System.out.println("---BEGIN TestCalcHead#testUsesProblem");
    //
    //		ProblemID p = new ProblemID("[sqroot-test,univariate,equation,test]");
    //		assertTrue("pbl belongs to class defined by itself",ch.usesProblem(p));
    //		
    //		p = new ProblemID("[equation,test]");
    //		assertFalse("usual case: not member of class",ch.usesProblem(p));
    //		assertTrue("usual case: member of class",ch.usesProblemClass(p));
    //		
    //		p = new ProblemID("[]");
    //		assertTrue("[] is superClass for all pbls",ch.usesProblemClass(p));
    //
    //		System.out.println("---END TestCalcHead#testUsesProblem");
    //	}

    /**
     * create the string for input of the whole CalcHead to the SML stdin
     * 
     * @see {isac.functest.specify.TestModel#testEditItems}
     */
    public void testToSMLstring() {
        System.out.println("---BEGIN TestCalcHead#testToSMLstring");
        //copy from smltest/FE-interface/interface.sml-- modifyCalcHead --
        //		 modifyCalcHead 1 (([],Pbl),(*the position from refFormula*)
        //		  "solve (x+1=2, x)",(*the headline*)
        //		  [Given ["equality (x+1=2)", "solveFor x"],
        //		   Find ["solutions L"](*,Relate []*)],
        //		  Pbl,
        //		  ("Test.thy",
        //		   ["sqroot-test","univariate","equation","test"],
        //		   ["Test","squ-equ-test-subpbl1"]));

        String expected_string = "(([],Pbl), \"solve (x + 1 = 2, x)\"" + 
        	", [Given [\"equality (x+1=2)\", \"solveFor x\"], Find " + 
        	"[\"solutions L\"], Relate []], Pbl, (\"Test.thy\", " + 
        	"[\"sqroot-test\",\"univariate\",\"equation\",\"test\"]" + 
        	",[\"Test\",\"squ-equ-test-subpbl1\"]))";

        assertEquals(expected_string, calchead_.toSMLString());

        

        System.out.println("---END TestCalcHead#testToSMLstring");
        System.out.println("---END isac.util.formulae.TestCalcHead");
    }

}