/*
 * @author Robert Koenighofer, member of the ISAC-team, 
 * Copyright (c) 2006 by Robert Koenighofer
 * created Jul 24, 2006 10:09:13 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */

package isac.util.formulae;

import junit.framework.TestCase;

/**
 * This is a testclass for ContextProblem containing a test for the
 * toHTML method.
 * 
 * @author Robert Koenighofer
 * @see isac.util.formulae.ContextMethod
 */
public class TestContextMethod extends TestCase {
	
	/**
	 * This method tests the toHtml functionality of the ContextMethod.
	 * The attributes of the class are set to test values. The toHTML
	 * method is called and the resulting String is checked with a regular
	 * expression.
	 * 
	 * @see isac.util.formulae.ContextMethod#toHTML()
	 */
	public void testToHtml() {
		ContextMethod test_obj = new ContextMethod();
		KEStoreKey ke_store_key = new KEStoreKey("kestore_key");
		test_obj.setKEStoreKey(ke_store_key);	
		Formula script = new Formula();
		script.setText("script");
		test_obj.setScript(script);
		Model model = new Model();

		ModelItem given1 = new ModelItem();
		given1.setText("given1");
		given1.setItemStatus("correct");
		model.getGiven().addItem(given1);
		ModelItem given2 = new ModelItem();
		given2.setText("given2");
		given2.setItemStatus("false");
		model.getGiven().addItem(given2);
		ModelItem given3 = new ModelItem();
		given3.setText("given3");
		given3.setItemStatus("correct");
		model.getGiven().addItem(given3);
		
		ModelItem find1 = new ModelItem();
		find1.setText("find1");
		find1.setItemStatus("correct");
		model.getFind().addItem(find1);
		ModelItem find2 = new ModelItem();
		find2.setText("find2");
		find2.setItemStatus("false");
		model.getFind().addItem(find2);
		ModelItem find3 = new ModelItem();
		find3.setText("find3");
		find3.setItemStatus("correct");
		model.getFind().addItem(find3);
		
		ModelItem where1 = new ModelItem();
		where1.setText("where1");
		where1.setItemStatus("correct");
		model.getWhere().addItem(where1);
		ModelItem where2 = new ModelItem();
		where2.setText("where2");
		where2.setItemStatus("false");
		model.getWhere().addItem(where2);
		ModelItem where3 = new ModelItem();
		where3.setText("where3");
		where3.setItemStatus("correct");
		model.getWhere().addItem(where3);
		
		ModelItem relate1 = new ModelItem();
		relate1.setText("relate1");
		relate1.setItemStatus("correct");
		model.getRelate().addItem(relate1);
		ModelItem relate2 = new ModelItem();
		relate2.setText("relate2");
		relate2.setItemStatus("correct");
		model.getRelate().addItem(relate2);
		ModelItem relate3 = new ModelItem();
		relate3.setText("relate3");
		relate3.setItemStatus("correct");
		model.getRelate().addItem(relate3);
		
		test_obj.setModel(model);		
		
		//System.out.println(test_obj.toHTML());
		
		//The regular expression 'regex' is used to check the html-result of the
		//method. A character by character comparison with an expected
		//output is not done since small changes in the style (which lead
		//to additional whitespaces or something like that) would cause the tests
		//to fail. However if one of the main componets of the result is missing,
		//this regular expression will lead to a Failure.
		String any = "([\\S\\s]*)";
		String regex = any+"<div class=\"context\""+any;
		regex += "Given"+any+"given1"+any+"given2"+any+"given3"+any;
		regex += "Where"+any+"where1"+any+"where2"+any+"where3"+any;
		regex += "Find"+any+"find1"+any+"find2"+any+"find3"+any;
		regex += "Relate"+any+"relate1"+any+"relate2"+any+"relate3"+any;
		regex += "</table>"+any+"</div>"+any;
		
		String result = test_obj.toHTML();
		
		assertTrue("string does not match regular expression", result.matches(regex));
		
	}

}
