/*
 * @author Robert Koenighofer, member of the ISAC-team, 
 * Copyright (c) 2006 by Robert Koenighofer
 * created Jul 24, 2006 10:09:13 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.util.formulae;

import junit.framework.TestCase;
import isac.util.formulae.ContextTheory;

/**
 * This is a testclass for ContextProblem containing a test for the
 * toHTML method.
 * 
 * @author Robert Koenighofer
 * @see isac.util.formulae.ContextTheory
 */
public class TestContextTheory extends TestCase {
	
	/**
	 * This method tests the toHtml functionality of the ContextTheory.
	 * The attributes of the class are set to test values. The toHTML
	 * method is called and the resulting String is checked with a regular
	 * expression.
	 * 
	 * @see isac.util.formulae.ContextTheory#toHTML()
	 */
	public void testToHtml() {
			
		ContextTheory test_obj = new ContextTheory();
		KEStoreKey ke_store_key = new KEStoreKey("kestore_key");
		test_obj.setKEStoreKey(ke_store_key);	
		
		Formula f1 = (new Formula());
		f1.setText("test_sublist");
		test_obj.setSubsList(f1);
		
		Formula f2 = (new Formula());
		f2.setText("test_instanciated");
		test_obj.setInstanciatedTo(f2);
		
		Formula f3 = (new Formula());
		f3.setText("test_applied_to");
		test_obj.setAppliedTo(f3);
		
		Formula f4 = (new Formula());
		f4.setText("test_applied_at");
		test_obj.setAppliedAt(f4);
		
		test_obj.setOrder("TESTOrder");
		
		Formula pair_assumption = new Formula();
		pair_assumption.setText("test_assumptions1");
		FormulaPair f5 = (new FormulaPair());
		f5.setAssumtion(pair_assumption);
		f5.setValue("True");
		test_obj.setEvalAssumtion(f5);
		
		Formula pair_assumption2 = new Formula();
		pair_assumption2.setText("((could be 0..n assumptions))");
		FormulaPair f5b = (new FormulaPair());
		f5b.setAssumtion(pair_assumption2);
		f5b.setValue("false");
		test_obj.setEvalAssumtion(f5b);
		
		Formula f6 = (new Formula());
		f6.setText("test_lhs");
		test_obj.setLHS(f6);
		
		Formula f7 = (new Formula());
		f7.setText("test_lhs_instanciated");
		test_obj.setLHSInstanciated(f7);
		
		Formula f8 = (new Formula());
		f8.setText("test_rhs");
		test_obj.setRHS(f8);
		
		Formula f9 = (new Formula());
		f9.setText("test_rhs_instanciated");
		test_obj.setRHSInstanciated(f9);
		
		Formula f10 = (new Formula());
		f10.setText("test_result");
		test_obj.setResult(f10);
		
		test_obj.setAssumtions("test_assumptions");

		Reference ref = new Reference();
		ref.setTag("reftag");
		ref.setID("refID");
		KEStoreKey ke_store_key_ref = new KEStoreKey("pbl_kestore_key_ref");

		ref.setKEStoreKey(ke_store_key_ref);
		test_obj.setEvalRLS(ref);
		
		//System.out.println(test_obj.toHTML());
		
		String any = "([\\S\\s]*)";
		String regex = any+"<div class=\"context\""+any+"<table"+any;
		regex += "instantiated by"+any+"test_sublist"+any+"to"+any;
		regex += "test_instanciated"+any;
		regex += "applied to"+any+"test_applied_to"+any;
		regex += "at"+any+"test_applied_at"+any;
		regex += "under order:"+any+"TESTOrder"+any+"</table>"+any+"<table"+any;
		regex += "test_assumptions1"+any+"..evaluates"+any+"to.."+any+"true"+any;
		regex += "((could be 0..n assumptions))"+any+"..evaluates"+any+"to.."+any+"false"+any;
		regex += "test_lhs"+any+"..matches.."+any+"test_lhs_instanciated"+any;
		regex += "test_rhs"+any+"..instantiates"+any+"test_rhs_instanciated"+any;
		regex += "</table>"+any+"<table"+any+"results in:"+any;
		regex +="test_result"+any;
		regex += "test_assumptions"+any;
		regex += "evaluated by:"+any;
		regex += "<a href=\"../pbl/pbl_kestore_key_ref.html\">refID</a>"+any;
		regex += "</table>"+any+"</div>"+any;
		
		String result = test_obj.toHTML();
		
		assertTrue("string does not match regular expression", result.matches(regex));
	}

}
