/*
 * @author Walther Neuper, member of the ISAC-team, 
 * Copyright (c) 2004 by Walther Neuper 
 * created Sep 3, 2004, 3:07:00 PM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to license terms.
 */
package isac.util.parser;

import isac.TestConstants;
import isac.kestore.KEStoreServices;
import isac.util.Formalization;

import java.util.Vector;

import junit.framework.TestCase;

/**
 * @author Walther Neuper Sep 3, 2004, 3:07:00 PM
 */
public class TestFormalizationDigest extends TestCase {

    /**
     * parse the formalization of the 'minisubpbl' example
     * 
     * @see isac.kestore.TestKEStoreServices#testLoadContentMinisubpbl() for how
     *      an XML-file of an example is fetched
     * 
     * @see isac.bridge.TestBridge#testMinisubpblAutoCalc for how the
     *      Formalization is used to start an example
     *  
     */
    public void testParseExplMini() {
        System.out.println("---BEGIN isac.util.parser.TestFormalizationDigest");
        System.out.println("---BEGIN isac.util.parser.testParseExplMini");

        //see isac.kestore.TestKEStoreServices#testLoadContentMinisubpbl()
        KEStoreServices ks = new KEStoreServices(TestConstants.KESTORE.XMLDATA_PATH);
        Vector<String> vec = ks.loadContent("exp",  TestConstants.KESTORE.TEST_DATA.EXP_TESTS_1B);

        //parse the content
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization fmz = fd.parseFormalization((String) xmlstr);

        System.out.println("fmz = " + fmz.toSMLString());
        String test = "[([\"equality (x+1=(2::real))\", \"solveFor x\", \"solutions L\"],"
                + "(\"Test\","
                + "[\"sqroot-test\",\"univariate\",\"equation\",\"test\"],"
                + "[\"Test\",\"squ-equ-test-subpbl1\"]))]";
        String s1 = test;
        String s2 = fmz.toSMLString();
        char c1, c2;
        for (int i = 0; i < s2.length(); i++) {
            c1 = s1.charAt(i);
            c2 = s2.charAt(i);
            if (c1 == c2) {
                System.out.println("equal " + c1 + ' ' + ' ' + c2);
            } else {
                System.out.println("Unequ " + c1 + ' ' + ' ' + c2);
            }
        }
        assertEquals("xmlstr --> fmz", test, fmz.toSMLString());
        System.out.println("---END isac.util.parser.testParseExplMini");
    }

//WN060324 too sensible against authoring; would require \n in html !?!
//    /**
//     * fetch the formalization of a particular example (load from KEStore and
//     * parse). used for testing the html-format of the _description_:
//     * TODO.WN0510 parseFormalization is more restrictive than loadContent
//     */
//    public void testParseBiegelinie() {
//        System.out.println("---BEGIN isac.util.parser.testParseBiegelinie");
//
//        //see isac.kestore.TestKEStoreServices#testLoadContentMinisubpbl()
//        KEStoreServices ks = new KEStoreServices("isac/xmldata");
//        Vector<String> vec = ks.loadContent("exp",
//                "exp_Statics_Biegel_Timischl_7-27.xml");
//
//        //parse the content
//        String xmlstr = (String) vec.firstElement();
//        System.out.println(xmlstr);
//        FormalizationDigest fd = new FormalizationDigest();
//        Formalization fmz = fd.parseFormalization((String) xmlstr);
//        //parser-msgs:
//        //  "uuml" was referenced, but not declared. (??????? replace ue!)
//        //  "img" must be terminated by the matching end-tag "</img>"(add!)
//        //  "br" must be terminated by the matching end-tag "</br>" (add!)
//
//        System.out.println("fmz = " + fmz.toSMLString());
//        String test = "[([\"Traegerlaenge L\", \"Streckenlast q_0\", "
//                + "\"Biegelinie y\", "
//                + "\"RandbedingungenBiegung [y 0 = 0, y L = 0]\", "
//                + "\"RandbedingungenMoment [M_b 0 = 0, M_b L = 0]\", "
//                + "\"FunktionsVariable x\"],"
//                + "(\"Biegelinie.thy\",[\"MomentBestimmte\",\"Biegelinie\"],"
//                + "[\"IntegrierenUndKonstanteBestimmen\"]))]";
//        String s1 = test;
//        String s2 = fmz.toSMLString();
//        char c1, c2;
//        for (int i = 0; i < s2.length(); i++) {
//            c1 = s1.charAt(i);
//            c2 = s2.charAt(i);
//            if (c1 == c2) {
//                System.out.println("equal " + c1 + ' ' + ' ' + c2);
//            } else {
//                System.out.println("Unequ " + c1 + ' ' + ' ' + c2);
//            }
//        }
//        assertEquals("xmlstr --> fmz", test, fmz.toSMLString());
//        System.out.println("---END isac.util.parser.testParseBiegelinie");
//    }

}