/*
 * @author Walther Neuper, member of the ISAC-team, 
 * Copyright (c) 2004 by Walther Neuper 
 * created Nov 4, 2004, 2:31:27 PM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * MAKE saxParseError VISIBLE BY xml_parser_digest_.setValidating(true);
 * 
 * Use is subject to license terms.
 * 
 * ATTENTION WITH OUT-COMMENTING: code formatter destroys indentation in strings
 */
package isac.util.parser;

import isac.TestConstants;
import isac.bridge.CChanged;
import isac.bridge.CMessage;
import isac.util.ResponseWrapper;
import isac.util.formulae.Assumptions;
import isac.util.formulae.CalcElement;
import isac.util.formulae.CalcHead;
import isac.util.formulae.CalcFormula;
import isac.util.formulae.FormHeadsContainer;
import isac.util.formulae.Formula;
import isac.util.formulae.ModelItem;
import isac.util.formulae.Position;
import isac.util.tactics.Tactic;
import isac.util.tactics.TacticsContainer;

import java.rmi.RemoteException;
import java.util.Vector;

import junit.framework.TestCase;
import org.apache.log4j.Logger;

/**
 * test the xml-parser by Richard Gradischnegg
 * 
 * @author Walther Neuper Nov 4, 2004, 2:31:27 PM
 * @version 1.0
 */
public class TestXMLParserDigest extends TestCase {

    private XMLParserDigest xml_parser_digest_;
    ResponseWrapper wrapper_ = null;
    static Logger logger_ = Logger.getLogger(TestXMLParserDigest.class.getName());

    public void testParseRefFormula() {
        System.out.println("---BEGIN isac.util.parser.TestXMLParserDigest");
        logger_.debug("---testParseRefFormula begin@@@@@@@@@@@@@@@@@@@@@@@@@");
        xml_parser_digest_ = new XMLParserDigest(TestConstants.ISAC.ISAC_DTD_PATH);
        xml_parser_digest_.setValidating(true);
        String xmlString = "> @@@@@begin@@@@@" // ...........................
                + " 1 " // .........................................
                + "<REFFORMULA>" // ...............................
                + "  <CALCID> 1 </CALCID>" // .....................
                + "  <CALCFORMULA>" // .....................
                + "    <POSITION>" // ...............................
                + "      <INTLIST>" // ..............................
                + "        <INT> 1 </INT>" // .......................
                + "      </INTLIST>" // .............................
                + "      <POS> Frm </POS>" // .......................
                + "    </POSITION>" // ..............................
                + "    <FORMULA>" // ................................
                + "      <MATHML>" // ...............................
                + "        <ISA> x + 1 = 2 </ISA>" // ...............
                + "      </MATHML>" // ..............................
                + "    </FORMULA>" // ...............................
                + "  </CALCFORMULA>" // .....................
                + "</REFFORMULA>" // ..............................
                + "@@@@@end@@@@@";
        wrapper_ = xml_parser_digest_.parse(xmlString);
        int ci = wrapper_.getCalcID();
        CalcFormula form = (CalcFormula) wrapper_.getResponse();
        assertEquals(ci, 1);
        assertEquals(form.getPosition().toSMLString(), "([1],Frm)");
        assertEquals(form.toSMLString(), "x + 1 = 2");
    }

    public void testParseRefCalcHeadF() throws Exception {
        logger_.debug("---testParseRefFormula begin%%%%%%%%%%%%%%%%%%%%%%%%%");
        xml_parser_digest_ = new XMLParserDigest(TestConstants.ISAC.ISAC_DTD_PATH);
        xml_parser_digest_.setValidating(true);
        String xmlString = "> @@@@@begin@@@@@" // ...........................
                + " 1" // ..........................................
                + "<REFFORMULA>" // ...............................
                + "  <CALCID> 1 </CALCID>" // ......................
                + "  <CALCHEAD status = \"incorrect\">" // .........
                + "    <POSITION>" // ................................
                + "      <INTLIST>" // ...............................
                + "        <INT> 1 </INT>" // ........................
                + "      </INTLIST>" // ..............................
                + "      <POS> Pbl </POS>" // ........................
                + "    </POSITION>" // ...............................
                + "    <HEAD>" // ...................................
                // FIXXXME.WN050617 here should come <FORMULA> in HEAD
                + "      <MATHML>" // ...............................
                + "        <ISA> solve (1 + -1 * 2 + x = 0, x) </ISA>"
                + "      </MATHML>" // ..............................
                + "    </HEAD>" // ..................................
                + "    <MODEL>" // .................................
                + "      <GIVEN> </GIVEN>" // .....................
                + "      <WHERE>" // ...............................
                + "        <ITEM status = \"false\">" // ...........
                // FIXXXME.WN050617 here should come <FORMULA> in ITEM
                + "          <MATHML>" // ........................
                + "            <ISA> matches (v_ = 0) e_ </ISA>"
                + "          </MATHML>" // .......................
                + "        </ITEM>" // .............................
                + "      </WHERE>" // ..............................
                + "      <FIND> </FIND>" // .......................
                + "      <RELATE> </RELATE>" // ...................
                + "    </MODEL>" // ................................
                + "    <BELONGSTO> Pbl </BELONGSTO>" // ............
                + "    <SPECIFICATION>" // .........................
                + "      <THEORYID> e_domID </THEORYID>" // ........
                + "      <PROBLEMID>" // ...........................
                + "        <STRINGLIST>" // ........................
                + "          <STRING> e_pblID </STRING>" // ........
                + "        </STRINGLIST>" // .......................
                + "      </PROBLEMID>" // ..........................
                + "      <METHODID>" // ............................
                + "        <STRINGLIST>" // ........................
                + "          <STRING> e_metID </STRING>" // ........
                + "        </STRINGLIST>" // .......................
                + "      </METHODID>" // ...........................
                + "    </SPECIFICATION>" // ........................
                + "  </CALCHEAD>" // ...............................
                + "</REFFORMULA>" // ..............................
                + "@@@@@end@@@@@";
        wrapper_ = xml_parser_digest_.parse(xmlString);
        int ci = wrapper_.getCalcID();
        CalcHead chead = (CalcHead) wrapper_.getResponse();

        assertEquals(ci, 1);
        assertEquals(chead.getStatusString(), "incorrect");
        assertEquals(chead.getPosition().toSMLString(), "([1],Pbl)");
        assertEquals(chead.getHeadLine().toSMLString(),
                "solve (1 + -1 * 2 + x = 0, x)");

        assertEquals("false (matches (v_ = 0) e_)", chead.getModel().getWhere().getItems().elementAt(0).toString());
        assertEquals(((ModelItem) chead.getModel().getWhere().getItems().elementAt(0)).getStatus(), 5);// .."false"
    }

    public void testParseRefCalcHeadT() throws Exception {
        logger_.debug("---testParseRefFormula begin%%%%%%%%%%%%%%%%%%%%%%%%%%%");
        xml_parser_digest_ = new XMLParserDigest(TestConstants.ISAC.ISAC_DTD_PATH);
        xml_parser_digest_.setValidating(true);
        String xmlString = "> @@@@@begin@@@@@" // ...........................
                + " 1 " // ......................................................
                + "<REFFORMULA>" // ...............................
                + "    <CALCID> 1 </CALCID>" // ......................

                + "    <CALCHEAD status = \"correct\">" // ............................
                + "      <POSITION>" // ..............................................
                + "        <INTLIST>" // ..............................................
                + "          <INT> 3 </INT>" // .......................................
                + "        </INTLIST>" // .............................................
                + "        <POS> Pbl </POS>" // .......................................
                + "      </POSITION>" // .............................................
                + "      <HEAD>" // ..................................................
                + "        <MATHML>" // ..............................................
                + "          <ISA> solve (-1 + x = 0, x) </ISA>" // ..................
                + "        </MATHML>" // .............................................
                + "      </HEAD>" // .................................................
                + "      <MODEL>" // ................................................
                + "        <GIVEN>" // ..............................................
                + "          <ITEM status=\"correct\">" // ............................
                + "            <MATHML>" // .........................................
                + "              <ISA> equality (-1 + x = 0) </ISA>" // .............
                + "            </MATHML>" // ........................................
                + "          </ITEM>" // ............................................
                + "          <ITEM status=\"correct\">" // ............................
                + "            <MATHML>" // .........................................
                + "              <ISA> solveFor x </ISA>" // ........................
                + "            </MATHML>" // ........................................
                + "          </ITEM>" // ............................................
                + "        </GIVEN>" // .............................................
                + "        <WHERE>" // ..............................................
                + "          <ITEM status=\"correct\">" // ............................
                + "            <MATHML>" // .........................................
                + "              <ISA> matches (x = 0) (-1 + x = 0) |" // ............
                + "matches (?b * x = 0) (-1 + x = 0) |" // ............................
                + "matches (?a + x = 0) (-1 + x = 0) | " // ...........................
                + "matches (?a + ?b * x = 0) (-1 + x = 0) </ISA>" // ..................
                + "            </MATHML>" // ........................................
                + "          </ITEM>" // ............................................
                + "        </WHERE>" // .............................................
                + "        <FIND>" // ...............................................
                + "          <ITEM status=\"correct\">" // ............................
                + "            <MATHML>" // .........................................
                + "              <ISA> solutions x_i </ISA>" // .....................
                + "            </MATHML>" // ........................................
                + "          </ITEM>" // ............................................
                + "        </FIND>" // ..............................................
                + "        <RELATE>  </RELATE>" // ..................................
                + "      </MODEL>" // ...............................................
                + "      <BELONGSTO> Pbl </BELONGSTO>" // ...........................
                + "      <SPECIFICATION>" // ........................................
                + "        <THEORYID> Test.thy </THEORYID>" // ......................
                + "        <PROBLEMID>" // ..........................................
                + "          <STRINGLIST>" // .......................................
                + "            <STRING> linear </STRING>" // ........................
                + "            <STRING> univariate </STRING>" // ....................
                + "            <STRING> equation </STRING>" // ......................
                + "            <STRING> test </STRING>" // ..........................
                + "          </STRINGLIST>" // ......................................
                + "        </PROBLEMID>" // .........................................
                + "        <METHODID>" // ...........................................
                + "          <STRINGLIST>" // .......................................
                + "            <STRING> Test </STRING>" // ..........................
                + "            <STRING> solve_linear </STRING>" // ..................
                + "          </STRINGLIST>" // ......................................
                + "        </METHODID>" // ..........................................
                + "      </SPECIFICATION>" // .......................................
                + "    </CALCHEAD>" // ..............................................
                + "</REFFORMULA>" // ..............................
                + "@@@@@end@@@@@";
        wrapper_ = xml_parser_digest_.parse(xmlString);
        int ci = wrapper_.getCalcID();
        CalcHead chead = (CalcHead) wrapper_.getResponse();

        assertEquals(ci, 1);
        assertEquals(chead.getStatusString(), "correct");
        assertEquals(chead.getPosition().toSMLString(), "([3],Pbl)");
        assertEquals(chead.getHeadLine().toSMLString(), "solve (-1 + x = 0, x)");

        assertEquals("correct (equality (-1 + x = 0))", chead.getModel().getGiven().getItems().elementAt(0).toString());
        assertEquals("correct (solveFor x)", chead.getModel().getGiven().getItems().elementAt(1).toString());

        assertEquals(((ModelItem) chead.getModel().getGiven().getItems().elementAt(0)).getStatus(), 1);// .."correct"
        assertEquals(((ModelItem) chead.getModel().getGiven().getItems().elementAt(1)).getStatus(), 1);// .."correct"
    }

    public void testParseCalcChanged() throws RemoteException {
        logger_.debug("---testParseCalcChanged begin############################");
        xml_parser_digest_ = new XMLParserDigest(TestConstants.ISAC.ISAC_DTD_PATH);
        xml_parser_digest_.setValidating(true);
        String xmlString = "> @@@@@begin@@@@@" // ...........................
                + " 1 " // .........................................
                + "<AUTOCALC>" // ..................................
                + "  <CALCID> 1 </CALCID>" // ......................
                + "  <CALCCHANGED>" // .............................
                + "    <UNCHANGED>" // .............................
                + "      <INTLIST>" // .............................
                + "        <INT> 1 </INT>" // ......................
                + "      </INTLIST>" // ............................
                + "      <POS> Frm </POS>" // ......................
                + "    </UNCHANGED>" // ............................
                + "    <DELETED>" // ...............................
                + "      <INTLIST>" // .............................
                + "        <INT> 2 </INT>" // ......................
                + "      </INTLIST>" // ............................
                + "      <POS> Frm </POS>" // ......................
                + "    </DELETED>" // ..............................
                + "    <GENERATED>" // .............................
                + "      <INTLIST>" // .............................
                + "      </INTLIST>" // ............................
                + "      <POS> Res </POS>" // ......................
                + "    </GENERATED>" // ............................
                + "  </CALCCHANGED>" // ............................
                + "</AUTOCALC>" // .................................
                + "@@@@@end@@@@@";
        System.out.println("---testParseCalcChanged before parsing");
        wrapper_ = xml_parser_digest_.parse(xmlString);
        xml_parser_digest_.setValidating(true);
        System.out.println("---testParseCalcChanged after parsing");

        CChanged cc = (CChanged) wrapper_.getResponse();
        System.out.println("getLastUnchanged="
                + cc.getLastUnchanged().toSMLString());
        System.out.println("getLastDeleted="
                + cc.getLastDeleted().toSMLString());
        System.out.println("getLastGenerated="
                + cc.getLastGenerated().toSMLString());

        assertEquals(cc.getLastUnchanged().toSMLString(), "([1],Frm)");
        assertEquals(cc.getLastDeleted().toSMLString(), "([2],Frm)");
        assertEquals(cc.getLastGenerated().toSMLString(), "([],Res)");
    }

    public void testParseCalcMessage() throws RemoteException {
        logger_.debug("---testParseCalcMessage begin############################");
        xml_parser_digest_ = new XMLParserDigest(TestConstants.ISAC.ISAC_DTD_PATH);
        xml_parser_digest_.setValidating(true);
        String xmlString = "> @@@@@begin@@@@@" // ...........................
                // + " 1 " // .........................................
                // + "<APPENDFORMULA>" // ..................................
                // + " <CALCID> 1 </CALCID>" // ......................
                + "  <CALCMESSAGE> no derivation found </CALCMESSAGE>" // ...
                // + "</APPENDFORMULA>" // .................................
                + "@@@@@end@@@@@";
        xml_parser_digest_.setValidating(true);
        System.out.println("---testParseCalcMessage before parsing");
        wrapper_ = xml_parser_digest_.parse(xmlString);
        System.out.println("---testParseCalcMessage after parsing");

        CMessage cc = (CMessage) wrapper_.getResponse();
        System.out.println("text= " + cc.getMessage());

        assertEquals(cc.getMessage(), "no derivation found");
    }

    public void testParseCalcIterator() throws RemoteException {
        logger_.debug("---testParseCalcIterator begin###########################");
        xml_parser_digest_ = new XMLParserDigest(TestConstants.ISAC.ISAC_DTD_PATH);
        xml_parser_digest_.setValidating(true);
        String xmlString = "> @@@@@begin@@@@@" // ...........................
                + /**/ " 1 " // .............................................
                + /**/ "<CALCITERATOR>" // .................................
                + /**/ "  <CALCID> 1 </CALCID>" // ......................
                + /**/ "  <POSITION>" // .....................................
                + /**/ "    <INTLIST>" // .......................................
                + /**/ "      <INT> 1 </INT>" // ..................................
                + /**/ "    </INTLIST>" // ......................................
                + /**/ "    <POS> Frm </POS>" // ..................................
                + /**/ "  </POSITION>" // ......................................
                + /**/ "</CALCITERATOR>" // ..................................
                + "@@@@@end@@@@@";
        System.out.println("---testParseCalcIterator before parsing");
        wrapper_ = xml_parser_digest_.parse(xmlString);
        System.out.println("---testParseCalcIterator after parsing");

        Position pos = (Position) wrapper_.getResponse();
        assertEquals(pos.toSMLString(), "([1],Frm)");
    }

    public void testParseIteratorError() throws RemoteException {
        logger_.debug("---testParseIteratorError begin##########################");
        xml_parser_digest_ = new XMLParserDigest(TestConstants.ISAC.ISAC_DTD_PATH);
        xml_parser_digest_.setValidating(true);
        String xmlString = "> @@@@@begin@@@@@" // ...........................
                + " 1 " // ......................................................
                + "<CALCITERATOR>" // ...........................................
                + "  <CALCID> 1 </CALCID>" // ...................................
                + "  <ERROR> pos does not exist </ERROR>\n" // ..................
                + "</CALCITERATOR>" // ..........................................
                + "@@@@@end@@@@@";

        System.out.println("---testParseIteratorError before parsing");
        wrapper_ = xml_parser_digest_.parse(xmlString);
        System.out.println("---testParseIteratorError after parsing");

        String msg = ((CMessage) wrapper_.getResponse()).getMessage();
        assertEquals(msg, "pos does not exist");
    }

    public void testParseGetElementsFromTo() throws RemoteException {
        logger_.debug("---testParseIteratorError begin##########################");
        xml_parser_digest_ = new XMLParserDigest(TestConstants.ISAC.ISAC_DTD_PATH);
        xml_parser_digest_.setValidating(true);
        String xmlString = "> @@@@@begin@@@@@" // ...........................
                + " 1 " // ......................................................
                + "<GETELEMENTSFROMTO>" // ............................................
                + "  <CALCID> 1 </CALCID>" // .........................................
                + "  <FORMHEADS>" // ...............................................
                + "    <CALCFORMULA>" // ..................................................
                + "      <POSITION>" // ..............................................
                + "        <INTLIST>" // ..............................................
                + "          <INT> 2 </INT>" // .......................................
                + "        </INTLIST>" // .............................................
                + "        <POS> Res </POS>" // .......................................
                + "      </POSITION>" // .............................................
                + "      <FORMULA>" // ................................................
                + "        <MATHML>" // ...............................................
                + "          <ISA> -1 + x = 0 </ISA>" // ..............................
                + "        </MATHML>" // ..............................................
                + "      </FORMULA>" // ...............................................
                + "    </CALCFORMULA>" // .................................................
                // WN05?? here only <HEAD> listed for <CALCHEAD> any more
                + "    <CALCHEAD status = \"correct\">" // ............................
                + "      <POSITION>" // ..............................................
                + "        <INTLIST>" // ..............................................
                + "          <INT> 3 </INT>" // .......................................
                + "        </INTLIST>" // .............................................
                + "        <POS> Pbl </POS>" // .......................................
                + "      </POSITION>" // .............................................
                + "      <HEAD>" // ..................................................
                + "        <MATHML>" // ..............................................
                + "          <ISA> solve (-1 + x = 0, x) </ISA>" // ..................
                + "        </MATHML>" // .............................................
                + "      </HEAD>" // .................................................
                + "      <MODEL>" // ................................................
                + "        <GIVEN>" // ..............................................
                + "          <ITEM status=\"correct\">" // ............................
                + "            <MATHML>" // .........................................
                + "              <ISA> equality (-1 + x = 0) </ISA>" // .............
                + "            </MATHML>" // ........................................
                + "          </ITEM>" // ............................................
                + "          <ITEM status=\"correct\">" // ............................
                + "            <MATHML>" // .........................................
                + "              <ISA> solveFor x </ISA>" // ........................
                + "            </MATHML>" // ........................................
                + "          </ITEM>" // ............................................
                + "        </GIVEN>" // .............................................
                + "        <WHERE>" // ..............................................
                + "          <ITEM status=\"correct\">" // ............................
                + "            <MATHML>" // .........................................
                + "              <ISA> matches (x = 0) (-1 + x = 0) |" // ............
                + "matches (?b * x = 0) (-1 + x = 0) |" // ............................
                + "matches (?a + x = 0) (-1 + x = 0) | " // ...........................
                + "matches (?a + ?b * x = 0) (-1 + x = 0) </ISA>" // ..................
                + "            </MATHML>" // ........................................
                + "          </ITEM>" // ............................................
                + "        </WHERE>" // .............................................
                + "        <FIND>" // ...............................................
                + "          <ITEM status=\"correct\">" // ............................
                + "            <MATHML>" // .........................................
                + "              <ISA> solutions x_i </ISA>" // .....................
                + "            </MATHML>" // ........................................
                + "          </ITEM>" // ............................................
                + "        </FIND>" // ..............................................
                + "        <RELATE> </RELATE>" // ..................................
                + "      </MODEL>" // ...............................................
                + "      <BELONGSTO> Pbl </BELONGSTO>" // ...........................
                + "      <SPECIFICATION>" // ........................................
                + "        <THEORYID> Test.thy </THEORYID>" // ......................
                + "        <PROBLEMID>" // ..........................................
                + "          <STRINGLIST>" // .......................................
                + "            <STRING> linear </STRING>" // ........................
                + "            <STRING> univariate </STRING>" // ....................
                + "            <STRING> equation </STRING>" // ......................
                + "            <STRING> test </STRING>" // ..........................
                + "          </STRINGLIST>" // ......................................
                + "        </PROBLEMID>" // .........................................
                + "        <METHODID>" // ...........................................
                + "          <STRINGLIST>" // .......................................
                + "            <STRING> Test </STRING>" // ..........................
                + "            <STRING> solve_linear </STRING>" // ..................
                + "          </STRINGLIST>" // ......................................
                + "        </METHODID>" // ..........................................
                + "      </SPECIFICATION>" // .......................................
                + "    </CALCHEAD>" // ..............................................

                + "    <CALCFORMULA>" // ..................................................
                + "      <POSITION>" // ..............................................
                + "        <INTLIST>" // ..............................................
                + "          <INT> 3 </INT>" // .......................................
                + "          <INT> 1 </INT>" // .......................................
                + "        </INTLIST>" // .............................................
                + "        <POS> Frm </POS>" // .......................................
                + "      </POSITION>" // .............................................
                + "      <FORMULA>" // ................................................
                + "        <MATHML>" // ...............................................
                + "          <ISA> -1 + x = 0 </ISA>" // ..............................
                + "        </MATHML>" // ..............................................
                + "      </FORMULA>" // ...............................................
                + "    </CALCFORMULA>" // .................................................
                + "  </FORMHEADS>" // ...............................................
                + "</GETELEMENTSFROMTO>" // ...........................................
                + "@@@@@end@@@@@";

        System.out.println("---testParseGetElementsFromTo before parsing");
        wrapper_ = xml_parser_digest_.parse(xmlString);
        xml_parser_digest_.setValidating(true);
        int ci = wrapper_.getCalcID();
        CalcElement fch = (CalcElement) ((FormHeadsContainer) wrapper_.getResponse()).getElements().elementAt(2);
        assertEquals(ci, 1);
        assertEquals(((CalcFormula) fch).getPosition().toSMLString(),
                "([3,1],Frm)");
        assertEquals(fch.toSMLString(), "-1 + x = 0");
        System.out.println("---testParseGetElementsFromTo after parsing");
    }

    public void testParseGetTacticRewriteSet() throws RemoteException {
        logger_.debug("---testParseGetTactic begin##########################");
        xml_parser_digest_ = new XMLParserDigest(TestConstants.ISAC.ISAC_DTD_PATH);
        xml_parser_digest_.setValidating(true);
        String xmlString = "@@@@@begin@@@@@"// .................................
                + " 1"// ......................................................
                + " <GETTACTIC>"// ...........................................
                + "   <CALCID> 1 </CALCID>"// ..................................
                + "   <REWRITESETTACTIC name= \"Rewrite_Set\">"// .............
                + "     <RULESET> Test_simplify </RULESET>"// ....................
                + "   </REWRITESETTACTIC>"// ...................................
                + " </GETTACTIC>" // ........................................
                + " @@@@@end@@@@@";

        System.out.println("---testParseGetTacticRewriteSet before parsing");
        wrapper_ = xml_parser_digest_.parse(xmlString);
        System.out.println("---testParseGetTacticRewriteSet after parsing");

        Tactic tac = ((Tactic) wrapper_.getResponse());
        assertEquals(tac.toSMLString(), "Rewrite_Set \"Test_simplify\"");
    }

    public void testParseGetTacticRewrite() throws RemoteException {
        logger_.debug("---testParseGetTactic begin##########################");
        xml_parser_digest_ = new XMLParserDigest(TestConstants.ISAC.ISAC_DTD_PATH);
        xml_parser_digest_.setValidating(true);
        String xmlString = "@@@@@begin@@@@@"// .................................
                + " 1"// ......................................................
                + " <GETTACTIC>"// ...........................................
                + "   <CALCID> 1 </CALCID>"// ..................................
                + "   <REWRITETACTIC name=\"Rewrite\">"// .............
                + "     <THEOREM>"// .......................................
                + "       <ID> Belastung_Querkraft </ID>"// ..................
                + "       <FORMULA>"// .......................................
                + "         <MATHML>"// .......................................
                + "           <ISA> - q ?x = Q' ?x </ISA>"// ................
                + "         </MATHML>"// .......................................
                + "       </FORMULA>"// .......................................
                + "     </THEOREM>"// .......................................
                + "   </REWRITETACTIC>"// ...................................
                + " </GETTACTIC>" // ........................................
                + " @@@@@end@@@@@";

        System.out.println("---testParseGetTacticRewrite before parsing");
        wrapper_ = xml_parser_digest_.parse(xmlString);
        System.out.println("---testParseGetTacticRewrite after parsing");

        Tactic tac = ((Tactic) wrapper_.getResponse());
        assertEquals(tac.toSMLString(), "Rewrite(\"Belastung_Querkraft\",\"\")");
    }

    public void testParseFetchApplicableTactics() throws RemoteException {
        logger_.debug("---testParseFetchApplicableTactics begin###############");
        xml_parser_digest_ = new XMLParserDigest(TestConstants.ISAC.ISAC_DTD_PATH);
        xml_parser_digest_.setValidating(true);
        String xmlString = "> @@@@@begin@@@@@" // ...........................
                + " 1 " // ......................................................
                + "<APPLICABLETACTICS>" // .......................................
                + "  <CALCID> 1 </CALCID>" // ....................................
                + "  <TACLIST>" // ...............................................
                + "    <REWRITESETTACTIC name=\"Rewrite_Set\">"// .................
                + "      <RULESET> norm_equation </RULESET>"// ....................
                + "    </REWRITESETTACTIC>"// .....................................
                + "    <REWRITESETTACTIC name=\"Rewrite_Set\">"// .................
                + "      <RULESET> Test_simplify </RULESET>"// ....................
                + "    </REWRITESETTACTIC>"// .....................................
                + "    <SUBPROBLEMTACTIC name=\"Subproblem\">"// ..................
                + "      <THEORY> Test.thy </THEORY>"// ...........................
                + "      <PROBLEM>"// .............................................
                + "        <STRINGLIST>"// ........................................
                + "          <STRING> linear </STRING>"// .........................
                + "          <STRING> univariate </STRING>"// .....................
                + "          <STRING> equation </STRING>"// .......................
                + "          <STRING> test </STRING>"// ...........................
                + "        </STRINGLIST>"// .......................................
                + "      </PROBLEM>"// ............................................
                + "    </SUBPROBLEMTACTIC>"// .....................................
                + "    <SIMPLETACTIC name=\"Check_elementwise\">"// ...............
                + "      <MATHML>"// ............................................
                + "        <ISA> Assumptions </ISA>"// ..........................
                + "      </MATHML>"// ...........................................
                + "    </SIMPLETACTIC>"// .........................................
                + "  </TACLIST>"// ................................................
                + "</APPLICABLETACTICS>"// ........................................
                + "@@@@@end@@@@@";
        System.out.println("---testParseFetchApplicableTactics before parsing");
        wrapper_ = xml_parser_digest_.parse(xmlString);
        Tactic tac = (Tactic) ((TacticsContainer) wrapper_.getResponse()).getTactics().elementAt(3);
        assertEquals(tac.toSMLString(), "Check_elementwise \"Assumptions\"");

        System.out.println("---testParseFetchApplicableTactics after parsing");
    }

    public void testParseGetAssumptions() throws RemoteException {
        logger_.debug("---testParseGetAssumptions begin##########################");
        xml_parser_digest_ = new XMLParserDigest(TestConstants.ISAC.ISAC_DTD_PATH);
        xml_parser_digest_.setValidating(true);
        String xmlString = "@@@@@begin@@@@@" // ...............................
                + " 1" // ....................................................
                + " <GETASSUMPTIONS>"// .......................................
                + "   <CALCID> 1 </CALCID>"// ...............................
                + "   <ASMLIST>" // ...........................................
                + "     <FORMULA>"// .........................................
                + "       <MATHML>" // ..........................................
                + "         <ISA> x ~= 0 </ISA>" // ................................
                + "       </MATHML>" // .......................................
                + "     </FORMULA>"// .........................................
                + "     <FORMULA>"// ..........................................
                + "       <MATHML>" // .........................................
                + "         <ISA> 9 * x + (x ^^^ 3 + -6 * x ^^^ 2) ~= 0 </ISA>"
                + "       </MATHML>" // ..........................................
                + "     </FORMULA>"// .............................................
                + "   </ASMLIST>"// .............................................
                + " </GETASSUMPTIONS>"// ......................................
                + " @@@@@end@@@@@";

        System.out.println("---testParseGetAssumptions before parsing");
        wrapper_ = xml_parser_digest_.parse(xmlString);
        System.out.println("---testParseGetAssumptions after parsing");

        Vector<Formula> asms = ((Assumptions) wrapper_.getResponse()).getFormulae();
        assertEquals(((Formula) asms.elementAt(0)).toSMLString(), "x ~= 0");
        assertEquals(((Formula) asms.elementAt(1)).toSMLString(),
                "9 * x + (x ^^^ 3 + -6 * x ^^^ 2) ~= 0");

        System.out.println("---END isac.util.parser.TestXMLParserDigest");
    }
}