package isac.wsdialog;

import isac.TestConstants;
import isac.TestException;
import isac.session.DialogGuide;
import isac.session.Session;
import isac.session.SessionManager;
import isac.users.User;
import isac.users.UserManager;
import isac.util.Formalization;
import isac.util.ObjectManagerPaths;
import isac.util.formulae.KEStoreKey;
import java.rmi.RemoteException;

/**
 * This class serves initialization of tests for the WorksheetDialog (*RLB is parallel)
 * as seen from the GUI.
 * The architecture involves interdependencies with other objects
 * such that the initialization of the dialog requires ugly methods;
 * see deprecated methods of WorksheetDialog, SessionManager,
 * Session.
 * The connections to MathEngine and WindowApplication+Worksheet are
 * established by production code, thus invisible in the tests. So the
 * results of the tests are accessed by specific methods of a
 * WindowApplication+Worksheet (TODO).
 * An initialization MUST be followed by respective tests IMMEDIATELY; 
 * so the initialization comes in 2 versions, one for WorksheetDialog and
 * one for WorksheetDialogRLB.
 *
 * @author Markus Kienleitner MKI
 */
public class TestInitDialog {

  private Formalization fmz_;
  private WorksheetDialog ws_dialog_;
  private SessionManager session_man_;
  private Session session_;
  private MockWorksheetRMI ws_mock_;
  
  public TestInitDialog(String exp) throws DialogProtocolException, TestException, RemoteException {
    /* test setup:
     * we compare behavior of WorksheetDialog and WorksheetDialogRLB
     * from outside, triggering by notifyUserAction.
     * TODO.PRELIMINARY: We register !2! dialogs:
     * (1) WorksheetDialog for comparing behaviour
     *     (might be replaced by for comparing behaviour after tests) with rules
     * (2) SEPARATELY IN TestInitDialog: WorksheetDialogRLB for testing rules
     * (3) Mock GUI (..TODO MockWorksheetDialog) for accessing results.
     */

    // creating WorksheetDialog requires a Session for
    //   user_ = UserManager.getInstance().getUser(session.getId());
    //   user_settings_ = user_.getUserSettings();
    // However, UserManager cannot be mocked as a singleton.
    // UserManager and SessionManger are intermingled such that we need to
    // use SessionManger#addSession for creating a session.
    // For getSessionByID("0") we have session_man_.resetForTests()
    
    ws_mock_ = new MockWorksheetRMI();
    
    session_man_ = SessionManager.getInstance();
    session_man_.PRIVATE_TEST_resetForTests();
    User user = new User("x", ObjectManagerPaths.SESSION_MANAGER_USER_PATH);
    session_man_.addSession(user);
    UserManager.getInstance().PRIVATE_TEST_resetForTests(user);
    DialogGuide new_dialog = new DialogGuide();
    session_ = session_man_.getSessionByID("0"); //due to session_man_.resetForTests()

    //prepareExample
    MockExampleDialog example_dialog = new MockExampleDialog(session_, new_dialog);
    fmz_ = example_dialog.openExample(TestConstants.KESTORE.TEST_DATA.EXP_TESTS_1B);
    
    session_.addWorksheetWithExample("wsd_id", new KEStoreKey(exp));
        
    ws_dialog_ = new WorksheetDialog(
            ObjectManagerPaths.MATH_ENGINE_PATH, session_, "wsd_id", fmz_, session_.getWSDialogManager().PRIVATE_TEST_getDialogForTests());

    ws_dialog_.addDataChangeListener(ws_mock_);

    //mocks the WindowApplication: the WorksheetDialog sets buttons etc.
    ws_dialog_.registerUIControlListener(ws_mock_);
    ws_dialog_.startCalculate();

  }

  public WorksheetDialog getWorksheetDialog() {
    return this.ws_dialog_;
  }
  
  public MockWorksheetRMI getMockWorksheet() {
    return this.ws_mock_;
  }
}
