package isac.wsdialog;

import isac.TestConstants;
import isac.TestException;
import isac.bridge.MathEngine;
import isac.bridge.MockIToDialog;
import isac.interfaces.ICalcIterator;
import isac.interfaces.IToCalc;
import isac.interfaces.IToUser;
import isac.kestore.KEStoreServices;
import isac.useractions.EUIContext;
import isac.useractions.EUIElement;
import isac.useractions.IUserAction;
import isac.useractions.UserAction;
import isac.util.CalcChanged;
import isac.util.Formalization;
import isac.util.formulae.CalcFormula;
import isac.util.parser.FormalizationDigest;
import isac.util.tactics.Tactic;

import java.rmi.RemoteException;
import java.util.Vector;

import junit.framework.TestCase;

import org.drools.KnowledgeBase;
import org.drools.runtime.StatefulKnowledgeSession;

/**
 * tests the functionalty of the RLB System -- useful as an introduction.
 * requires TestNextBtn.drl in same directory.
 *
 * RLB is http://www.jboss.org/drools/drools-expert.html
 * The doku for the version implemented is
 * http://docs.jboss.org/drools/release/5.3.0.Final/drools-expert-docs/html/index.html
 *
 * @author Markus Kienleitner MKI
 */
public class TestSimpleRLB extends TestCase {

  private IToCalc calc_tree_;
  private CalcChanged calc_changed_;
  private IToUser user_;
  private ICalcIterator calc_iterator_;
  public EUIElement request_;
  private final String path_ = "exp_IsacCore_Tests_1b.xml";

  private void prepareMathEngine() throws RemoteException {
    KEStoreServices ks = new KEStoreServices(TestConstants.KESTORE.XMLDATA_PATH);
    Vector<String> vec = ks.loadContent("exp", path_);
    //parse out the Formalization
    //see TestFormalizationDigest#testParseExplMini
    String xmlstr = (String) vec.firstElement();
    FormalizationDigest fd = new FormalizationDigest();
    Formalization fmz = fd.parseFormalization((String) xmlstr);
    //System.out.println(".end step_a-----"+ "xmlstr:" + xmlstr);

    //start the calculation
    MathEngine.init("localhost");
    MathEngine me = MathEngine.getMathEngine();

    calc_tree_ = me.getCalcTree(fmz);

    user_ = new MockIToDialog();
    calc_tree_.addDataChangeListener(user_);

    calc_tree_.completeCalcHead();
    // sent to bridge: autoCalculate 1 CompleteCalcHead;
    calc_iterator_ = calc_tree_.getActiveFormula();

    //let calculate the first formula
    Tactic tac = calc_tree_.fetchProposedTactic();
    //sent to bridge: fetchProposedTactic 1;

    calc_tree_.setNextTactic(tac);
  }

  /**
   * tests the functionalty of the RLB System
   * 
   * (1) initalize the RLB
   * (2) simulate next 
   * (3) fire rules
   * 
   * @throws RemoteException
   * @throws DialogProtocolException
   * @throws TestException
   *
   * requires TestNextBtn.drl in same directory.
   */
  public void testRLB() throws RemoteException, DialogProtocolException, TestException {

    IUserAction action = new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION);
    prepareMathEngine();
    this.request_ = action.getActionID();


    RlbBuilder rlb = new RlbBuilder(TestSimpleRLB.class, "TestNextBtn.drl");
    KnowledgeBase kbase = rlb.getKnowledgeBase();
    StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession();

    ksession.insert(this);
    System.out.println("Fire all rules...");
    ksession.fireAllRules();


    this.request_ = null;

    calc_changed_ = ((MockIToDialog) user_).getCalcChangedEvent();
    System.out.println("Check calculation...");

    assertEquals("unchanged", calc_changed_.getLastUnchangedFormula().toSMLString(),
            "([],Met)");
    assertEquals("deleted", calc_changed_.getLastDeletedFormula().toSMLString(),
            "([],Met)");
    assertEquals("generated", calc_changed_.getLastGeneratedFormula().toSMLString(),
            "([1],Frm)");

	Vector<CalcFormula> elems = calc_iterator_.getFormulaeFromTo(calc_changed_.getLastGeneratedFormula(),
            new Integer(2), false);
    CalcFormula calc_formula = (CalcFormula) (elems.firstElement());
    System.out.println(".2nd form: " + calc_formula.toSMLString());
    assertEquals("mini-auto: 2nd form", calc_formula.toSMLString(), "x + 1 = 2");
  }

  public void next() throws RemoteException {
    System.out.println("Called from RLB...");
    calc_tree_.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);
  }
}
