package isac.wsdialog;

import isac.TestException;
import isac.interfaces.ICalcIterator;
import isac.useractions.EUIContext;
import isac.useractions.EUIElement;
import isac.useractions.UserAction;
import isac.useractions.UserActionOnCalcElement;
import isac.useractions.UserActionOnPosition;
import isac.util.CalcChanged;
import isac.util.CalcMessage;
import isac.util.formulae.CalcFormula;
import isac.util.formulae.Position;

import java.rmi.RemoteException;
import java.util.Vector;

import junit.framework.TestCase;

/**
 * Test for dialog (without RLB ) during the solve-phase.
 * The specify-phase is still outside the test.
 *
 * NOTE: The tests perfectly cover WorksheetDialog#notifyUserAction and
 * IToDialog#calcResponse,
 * but they only partially cover IToGui#doUIAction.
 *
 * We compare the behavior of WorksheetDialog and WorksheetDialogNOrlb
 * in each test below.
 *
 * Note WN170215: all tests herein are destroyed by @see isac.users.TestUserManager,
 * Thus @see isac.users.TestUserManager is outcommented in @see isac.users.Testall.
 * 
 * @author Kienleitner Markus
 */
public class TestSolve extends TestCase {

  /* get the formula at the position indicated by CalcChanged;
   * The formula is retrieved via WorksheetDialog (instead from MockGUI!),
   * because the latter would get the formula from WorksheetDialog, too.
   */
  private String formulaInTreeAt(CalcChanged position, WorksheetDialog dialog) throws RemoteException {
    ICalcIterator calc_iterator_ = dialog.getCalcTree().iterator();
	Vector<CalcFormula> elems = calc_iterator_.getFormulaeFromTo(position.getLastGeneratedFormula(), new Integer(2), false);
    CalcFormula calc_formula_ = (CalcFormula) (elems.lastElement());

    return calc_formula_.toSMLString();
  }

  /* identical to formulaInTreeAt except 2nd argument */
  private String formulaInTreeAtNOrlb(CalcChanged position, WorksheetDialogNOrlb dialog) throws RemoteException {
    ICalcIterator calc_iterator_ = dialog.getCalcTree().iterator();
    Vector<CalcFormula> elems = calc_iterator_.getFormulaeFromTo(position.getLastGeneratedFormula(), new Integer(2), false);
    CalcFormula calc_formula_ = (CalcFormula) (elems.lastElement());

    return calc_formula_.toSMLString();
  }

  /* tests the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprising UC
   * 'start auto calculation' \label{SPECIFY:START:auto}
   *
   * @see TestBridge#testMinisubpblAutoCalc
   */
  public void testAutoButton() throws DialogProtocolException, TestException, RemoteException {

    TestInitDialog wrapper = new TestInitDialog("exp_IsacCore_Tests_1b.xml");
    WorksheetDialog dialog = wrapper.getWorksheetDialog();
    MockWorksheetRMI ws_mock = wrapper.getMockWorksheet();

    //do the whole calculation for the example
    dialog.notifyUserAction(new UserAction(
            EUIElement.UI_SOLVE_CALCULATE_ALL, EUIContext.UI_CONTEXT_CALCULATION));
    //check
    String result = formulaInTreeAt(ws_mock.getCalcChanged(), dialog);
    assertEquals("finished calculation", result, "[x = 1]");
  }

  public void testAutoButtonNOrlb() throws DialogProtocolException, TestException, RemoteException {

    TestInitDialogNOrlb wrapper = new TestInitDialogNOrlb("exp_IsacCore_Tests_1b.xml");
    WorksheetDialogNOrlb dialog = wrapper.getWorksheetDialog();
    MockWorksheetRMI ws_mock = wrapper.getMockWorksheet();

    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_ALL, EUIContext.UI_CONTEXT_CALCULATION));

    String result = formulaInTreeAtNOrlb(ws_mock.getCalcChanged(), dialog);
    assertEquals("finished calculation", result, "[x = 1]");
  }

  /* tests the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprising UC
   * 'start interactive calculation' \label{SPECIFY:START:interactive} +
   * \label{SOLVE:AUTO:one}}
   *
   * @see TestBridge#testMinisubpblNextStep
   */
  public void testNextButton() throws DialogProtocolException, TestException, RemoteException {

    TestInitDialog wrapper = new TestInitDialog("exp_IsacCore_Tests_1b.xml");
    WorksheetDialog dialog = wrapper.getWorksheetDialog();
    MockWorksheetRMI ws_mock = wrapper.getMockWorksheet();

    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));

    String result = formulaInTreeAt(ws_mock.getCalcChanged(), dialog);
    assertEquals("finished calculation", result, "[x = 1]");
  }

  public void testNextButtonNOrlb() throws DialogProtocolException, TestException, RemoteException {

    TestInitDialogNOrlb wrapper = new TestInitDialogNOrlb("exp_IsacCore_Tests_1b.xml");
    WorksheetDialogNOrlb dialog = wrapper.getWorksheetDialog();
    MockWorksheetRMI ws_mock = wrapper.getMockWorksheet();

    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));

    String result = formulaInTreeAtNOrlb(ws_mock.getCalcChanged(), dialog);
    assertEquals("finished calculation", result, "[x = 1]");
  }

  /**
   * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
   * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
   * smltest/FE-interface/interface.sml "...replace} right"
   *
   * @see TestBridgeInput#testReplaceRight
   */
  public void testReplaceRight() throws DialogProtocolException, TestException, RemoteException {

    TestInitDialog wrapper = new TestInitDialog("exp_IsacCore_Tests_1b.xml");
    WorksheetDialog dialog = wrapper.getWorksheetDialog();
    MockWorksheetRMI ws_mock = wrapper.getMockWorksheet();

    dialog.notifyUserAction(new UserAction(
            EUIElement.UI_SOLVE_CALCULATE_ALL, EUIContext.UI_CONTEXT_CALCULATION));

    //cklick a formula (at a certain position) on the Worksheet: effect on mathengine;
    //effect on the Worksheet is not implemented (planned: set write permisseion etc)
    Position pos = new Position();//([2],Res)
    pos.addInt("2");
    pos.setKind("Res");
    dialog.notifyUserAction(new UserActionOnPosition(
            EUIElement.UI_SOLVE_MOVE_ACTIVE_FORMULA, EUIContext.UI_CONTEXT_ONEELEMENT, pos));

    //notify the WorksheetDialog that user wishes to edit at this position
    dialog.notifyUserAction(new UserAction(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA));
    //check
    assertEquals("got allowance: ", ws_mock.getDoUIAction().getAction(),
            EUIElement.UI_DO_EDIT_FORMULA);

    // the active fromula by the edited formula at this position
    CalcFormula edited_formula_ = new CalcFormula();
    edited_formula_.setText("-1 + x = 0");
    edited_formula_.setPosition(pos);
    dialog.notifyUserAction(new UserActionOnCalcElement(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE, edited_formula_));
    //check
    assertEquals("calculation not affected;", "formula not changed", ws_mock.getCalcMessage().getText());
  }

  public void testReplaceRightNOrlb() throws DialogProtocolException, TestException, RemoteException {

    TestInitDialogNOrlb wrapper = new TestInitDialogNOrlb("exp_IsacCore_Tests_1b.xml");
    WorksheetDialogNOrlb dialog = wrapper.getWorksheetDialog();
    MockWorksheetRMI ws_mock = wrapper.getMockWorksheet();

    dialog.notifyUserAction(new UserAction(
            EUIElement.UI_SOLVE_CALCULATE_ALL, EUIContext.UI_CONTEXT_CALCULATION));

    //cklick a formula on the Worksheet: this has effect on the mathengine;
    //effect on the Worksheet is not implemented (planned: set write permisseion etc)
    Position pos = new Position();//([2],Res)
    pos.addInt("2");
    pos.setKind("Res");

    dialog.notifyUserAction(new UserActionOnPosition(
            EUIElement.UI_SOLVE_MOVE_ACTIVE_FORMULA, EUIContext.UI_CONTEXT_ONEELEMENT, pos));

    //notify the WorksheetDialog that user wishes to edit at this position
    dialog.notifyUserAction(new UserAction(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA));
    //check
    assertEquals("got allowance: ", ws_mock.getDoUIAction().getAction(),
            EUIElement.UI_DO_EDIT_FORMULA);

    //replace the active fromula by the edited formula at this position
    CalcFormula edited_formula_ = new CalcFormula();
    edited_formula_.setText("-1 + x = 0");
    edited_formula_.setPosition(pos);
    dialog.notifyUserAction(new UserActionOnCalcElement(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE, edited_formula_));

    assertEquals("calculation not affected;", "formula not changed", ws_mock.getCalcMessage().getText());
  }

  /**
   * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
   * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
   * smltest/FE-interface/interface.sml "...replace} other"
   *
   * @see TestBridgeInput#testReplaceOther1
   */
public void testReplaceOther1() throws DialogProtocolException, TestException, RemoteException {

    TestInitDialog wrapper = new TestInitDialog("exp_IsacCore_Tests_1b.xml");
    WorksheetDialog dialog = wrapper.getWorksheetDialog();
    MockWorksheetRMI ws_mock = wrapper.getMockWorksheet();

    dialog.notifyUserAction(new UserAction(
            EUIElement.UI_SOLVE_CALCULATE_ALL, EUIContext.UI_CONTEXT_CALCULATION));

    //cklick a formula on the Worksheet: this has effect on the mathengine;
    //effect on the Worksheet is not implemented (planned: set write permisseion etc)
    Position pos = new Position();//([2],Res)
    pos.addInt("2");
    pos.setKind("Res");
    dialog.notifyUserAction(new UserActionOnPosition(
            EUIElement.UI_SOLVE_MOVE_ACTIVE_FORMULA, EUIContext.UI_CONTEXT_ONEELEMENT, pos));
    //No effects on the Worksheet

    //notify the WorksheetDialog that user wishes to edit at this position
    dialog.notifyUserAction(new UserAction(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA));
    //check
    assertEquals("got allowance: ", ws_mock.getDoUIAction().getAction(),
            EUIElement.UI_DO_EDIT_FORMULA);

    //replace with"x - 1 = 0" which needs 11 steps (on same level) to justify
    CalcFormula edited_formula_ = new CalcFormula();
    edited_formula_.setText("x - 1 = 0");
    edited_formula_.setPosition(pos);
    dialog.notifyUserAction(new UserActionOnCalcElement(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE, edited_formula_));

    //check (1):
    CalcChanged position = ws_mock.getCalcChanged();
    assertEquals("unchanged",
            position.getLastUnchangedFormula().toSMLString(), "([1],Res)");
    assertEquals("deleted",
            position.getLastDeletedFormula().toSMLString(), "([],Res)");
    assertEquals("generated",
            position.getLastGeneratedFormula().toSMLString(), "([2],Res)");

    //check (2): only 1 formula shown on same level
	Vector<CalcFormula> elems = position.getLastUnchangedFormula().getFormulaeFromTo(
            position.getLastGeneratedFormula(), new Integer(0), false);
    assertEquals("the input formula is returned:", ((CalcFormula) elems.firstElement()).toSMLString(), "x - 1 = 0");
    assertEquals("only _one_ formula returned", elems.size(), 1);

    //check (3): but there are 11 steps in between one level deeper
    elems = position.getLastUnchangedFormula().getFormulaeFromTo(
            position.getLastGeneratedFormula(), new Integer(2), false);
    assertEquals("elems could be displayed ", elems.size(), 11);
  }

public void testReplaceOther1NOrlb() throws DialogProtocolException, TestException, RemoteException {

    TestInitDialogNOrlb wrapper = new TestInitDialogNOrlb("exp_IsacCore_Tests_1b.xml");
    WorksheetDialogNOrlb dialog = wrapper.getWorksheetDialog();
    MockWorksheetRMI ws_mock = wrapper.getMockWorksheet();

    dialog.notifyUserAction(new UserAction(
            EUIElement.UI_SOLVE_CALCULATE_ALL, EUIContext.UI_CONTEXT_CALCULATION));

    //cklick a formula on the Worksheet: this has effect on the mathengine;
    //effect on the Worksheet is not implemented (planned: set write permisseion etc)
    Position pos = new Position();//([2],Res)
    pos.addInt("2");
    pos.setKind("Res");
    dialog.notifyUserAction(new UserActionOnPosition(
            EUIElement.UI_SOLVE_MOVE_ACTIVE_FORMULA, EUIContext.UI_CONTEXT_ONEELEMENT, pos));
    //No effects on the Worksheet

    //notify the WorksheetDialog that user wishes to edit at this position
    dialog.notifyUserAction(new UserAction(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA));
    //check
    assertEquals("got allowance: ", ws_mock.getDoUIAction().getAction(),
            EUIElement.UI_DO_EDIT_FORMULA);

    //replace with"x - 1 = 0" which needs 11 steps (on same level) to justify
    CalcFormula edited_formula_ = new CalcFormula();
    edited_formula_.setText("x - 1 = 0");
    edited_formula_.setPosition(pos);
    dialog.notifyUserAction(new UserActionOnCalcElement(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE, edited_formula_));
    //check (1):
    CalcChanged position = ws_mock.getCalcChanged();
    assertEquals("unchanged",
            position.getLastUnchangedFormula().toSMLString(), "([1],Res)");
    assertEquals("deleted",
            position.getLastDeletedFormula().toSMLString(), "([],Res)");
    assertEquals("generated",
            position.getLastGeneratedFormula().toSMLString(), "([2],Res)");
    //check (2): only 1 formula shown on same level
	Vector<CalcFormula> elems = position.getLastUnchangedFormula().getFormulaeFromTo(
            position.getLastGeneratedFormula(), new Integer(0), false);
    assertEquals("the input formula is returned:", ((CalcFormula) elems.firstElement()).toSMLString(), "x - 1 = 0");
    assertEquals("only _one_ formula returned", elems.size(), 1);
    //check (3): but there are 11 steps in between one level deeper
    elems = position.getLastUnchangedFormula().getFormulaeFromTo(
            position.getLastGeneratedFormula(), new Integer(2), false);
    assertEquals("elems could be displayed ", elems.size(), 11);
  }

  /**
   * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
   * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
   * smltest/FE-interface/interface.sml"...replace} other 2"
   *
   * @see TestBridgeInput#testReplaceOther2
   */
  public void testReplaceOther2() throws DialogProtocolException, TestException, RemoteException {

    TestInitDialog wrapper = new TestInitDialog("exp_IsacCore_Tests_1b.xml");
    WorksheetDialog dialog = wrapper.getWorksheetDialog();
    MockWorksheetRMI ws_mock = wrapper.getMockWorksheet();

    dialog.notifyUserAction(new UserAction(
            EUIElement.UI_SOLVE_CALCULATE_ALL, EUIContext.UI_CONTEXT_CALCULATION));

    //cklick a formula on the Worksheet: this has effect on the mathengine;
    //effect on the Worksheet is not implemented (planned: set write permisseion etc)
    Position pos = new Position();//([2],Res)
    pos.addInt("2");
    pos.setKind("Res");

    dialog.notifyUserAction(new UserActionOnPosition(
            EUIElement.UI_SOLVE_MOVE_ACTIVE_FORMULA, EUIContext.UI_CONTEXT_ONEELEMENT, pos));
    //No effects on the Worksheet

    //notify the WorksheetDialog that user wishes to edit at this position
    dialog.notifyUserAction(new UserAction(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA));
    //check
    assertEquals("got allowance: ", ws_mock.getDoUIAction().getAction(),
            EUIElement.UI_DO_EDIT_FORMULA);

    //replace "x = 1" which needs 8 steps (+ one level down) to justify
    CalcFormula edited_formula_ = new CalcFormula();
    edited_formula_.setText("x = 1");
    edited_formula_.setPosition(pos);
    dialog.notifyUserAction(new UserActionOnCalcElement(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE, edited_formula_));
    //check (1):
    CalcChanged position = ws_mock.getCalcChanged();
    assertEquals("unchanged",
            position.getLastUnchangedFormula().toSMLString(), "([1],Res)");
    assertEquals("deleted",
            position.getLastDeletedFormula().toSMLString(), "([],Res)");
    assertEquals("generated",
            position.getLastGeneratedFormula().toSMLString(), "([3,2],Res)");
    //check (2): the inserted formulae _MUST_ be displayed (but not on level 3 !):
	Vector<CalcFormula> elems = position.getLastUnchangedFormula().getFormulaeFromTo(
            position.getLastGeneratedFormula(), new Integer(3), false);
    assertEquals("CalcElements to be shown:", elems.size(), 8);
  }

  public void testReplaceOther2NOrlb() throws DialogProtocolException, TestException, RemoteException {

    TestInitDialogNOrlb wrapper = new TestInitDialogNOrlb("exp_IsacCore_Tests_1b.xml");
    WorksheetDialogNOrlb dialog = wrapper.getWorksheetDialog();
    MockWorksheetRMI ws_mock = wrapper.getMockWorksheet();

    dialog.notifyUserAction(new UserAction(
            EUIElement.UI_SOLVE_CALCULATE_ALL, EUIContext.UI_CONTEXT_CALCULATION));

    //cklick a formula on the Worksheet: this has effect on the mathengine;
    //effect on the Worksheet is not implemented (planned: set write permisseion etc)
    Position pos = new Position();//([2],Res)
    pos.addInt("2");
    pos.setKind("Res");
    dialog.notifyUserAction(new UserActionOnPosition(
            EUIElement.UI_SOLVE_MOVE_ACTIVE_FORMULA, EUIContext.UI_CONTEXT_ONEELEMENT, pos));
    //No effects on the Worksheet

    //notify the WorksheetDialog that user wishes to edit at this position
    dialog.notifyUserAction(new UserAction(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA));
    //check
    assertEquals("got allowance: ", ws_mock.getDoUIAction().getAction(),
            EUIElement.UI_DO_EDIT_FORMULA);

    //replace "x = 1" which needs 8 steps (+ one level down) to justify
    CalcFormula edited_formula_ = new CalcFormula();
    edited_formula_.setText("x = 1");
    edited_formula_.setPosition(pos);
    dialog.notifyUserAction(new UserActionOnCalcElement(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE, edited_formula_));
    //check (1):
    CalcChanged position = ws_mock.getCalcChanged();
    assertEquals("unchanged",
            position.getLastUnchangedFormula().toSMLString(), "([1],Res)");
    assertEquals("deleted",
            position.getLastDeletedFormula().toSMLString(), "([],Res)");
    assertEquals("generated",
            position.getLastGeneratedFormula().toSMLString(), "([3,2],Res)");
    //check (2): the inserted formulae _MUST_ be displayed (but not on level 3 !):
	Vector<CalcFormula> elems = position.getLastUnchangedFormula().getFormulaeFromTo(
            position.getLastGeneratedFormula(), new Integer(3), false);
    assertEquals("CalcElements to be shown:", elems.size(), 8);
  }

  /**
   * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
   * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
   * smltest/FE-interface/interface.sml "...replace} NOTok"
   *
   * @see TestSolve#testReplaceNotOk
   */
  public void testReplaceNotOk() throws DialogProtocolException, TestException, RemoteException {

    TestInitDialog wrapper = new TestInitDialog("exp_IsacCore_Tests_1b.xml");
    WorksheetDialog dialog = wrapper.getWorksheetDialog();
    MockWorksheetRMI ws_mock = wrapper.getMockWorksheet();

    dialog.notifyUserAction(new UserAction(
            EUIElement.UI_SOLVE_CALCULATE_ALL, EUIContext.UI_CONTEXT_CALCULATION));

    //cklick a formula on the Worksheet: this has effect on the mathengine;
    //effect on the Worksheet is not implemented (planned: set write permisseion etc)
    Position pos = new Position();//([2],Res)
    pos.addInt("2");
    pos.setKind("Res");
    dialog.notifyUserAction(new UserActionOnPosition(
            EUIElement.UI_SOLVE_MOVE_ACTIVE_FORMULA, EUIContext.UI_CONTEXT_ONEELEMENT, pos));
    //No effects on the Worksheet

    //notify the WorksheetDialog that user wishes to edit at this position
    dialog.notifyUserAction(new UserAction(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA));
    //check
    assertEquals("got allowance: ", ws_mock.getDoUIAction().getAction(),
            EUIElement.UI_DO_EDIT_FORMULA);

    //replace position with formula "x = 4711" which cannot be derived
    CalcFormula edited_formula_ = new CalcFormula();
    edited_formula_.setText("x = 4711");
    edited_formula_.setPosition(pos);
    dialog.notifyUserAction(new UserActionOnCalcElement(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE, edited_formula_));
    //check:
    CalcMessage msg = ws_mock.getCalcMessage();
    assertEquals("CalcMessage:", msg.getText(), "no derivation found");
  }

  public void testReplaceNotOkNOrlb() throws DialogProtocolException, TestException, RemoteException {

    TestInitDialogNOrlb wrapper = new TestInitDialogNOrlb("exp_IsacCore_Tests_1b.xml");
    WorksheetDialogNOrlb dialog = wrapper.getWorksheetDialog();
    MockWorksheetRMI ws_mock = wrapper.getMockWorksheet();

    dialog.notifyUserAction(new UserAction(
            EUIElement.UI_SOLVE_CALCULATE_ALL, EUIContext.UI_CONTEXT_CALCULATION));

    //cklick a formula on the Worksheet: this has effect on the mathengine;
    //effect on the Worksheet is not implemented (planned: set write permisseion etc)
    Position pos = new Position();//([2],Res)
    pos.addInt("2");
    pos.setKind("Res");
    dialog.notifyUserAction(new UserActionOnPosition(
            EUIElement.UI_SOLVE_MOVE_ACTIVE_FORMULA, EUIContext.UI_CONTEXT_ONEELEMENT, pos));
    //No effects on the Worksheet

    //notify the WorksheetDialog that user wishes to edit at this position
    dialog.notifyUserAction(new UserAction(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA));
    //check
    assertEquals("got allowance: ", ws_mock.getDoUIAction().getAction(),
            EUIElement.UI_DO_EDIT_FORMULA);

    //replace position with formula "x = 4711" which cannot be derived
    CalcFormula edited_formula_ = new CalcFormula();
    edited_formula_.setText("x = 4711");
    edited_formula_.setPosition(pos);
    dialog.notifyUserAction(new UserActionOnCalcElement(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE, edited_formula_));
    //check:
    CalcMessage msg = ws_mock.getCalcMessage();
    assertEquals("CalcMessage:", msg.getText(), "no derivation found");
  }

  /**
   * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
   * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
   * smltest/FE-interface/interface.sml "...enter} right"
   *
   * @see TestBridgeInput#testAppendRight()
   */
  public void testAppendRight() throws DialogProtocolException, TestException, RemoteException {

    TestInitDialog wrapper = new TestInitDialog("exp_IsacCore_Tests_1b.xml");
    WorksheetDialog dialog = wrapper.getWorksheetDialog();
    MockWorksheetRMI ws_mock = wrapper.getMockWorksheet();

    //do the first two steps
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    String formula2 = formulaInTreeAt(ws_mock.getCalcChanged(), dialog);
    assertEquals("2nd formula", formula2, "x + 1 + -1 * 2 = 0");

    //append "-1 + x = 0" which the kernel would have proposed, too
    CalcFormula edited_formula_ = new CalcFormula();
    edited_formula_.setText("-1 + x = 0");
    dialog.notifyUserAction(new UserActionOnCalcElement(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE, edited_formula_));
    //check (1): a formula has been appended
    CalcChanged position = ws_mock.getCalcChanged();
    assertEquals("unchanged= ",
            position.getLastUnchangedFormula().toSMLString(), "([1],Res)");
    assertEquals("deleted= ",
            position.getLastDeletedFormula().toSMLString(), "([1],Res)");
    assertEquals("generated= ",
            position.getLastGeneratedFormula().toSMLString(), "([2],Res)");
    //check (2)
    String result = formulaInTreeAt(ws_mock.getCalcChanged(), dialog);
    assertEquals("input formula is returned: ", result, "-1 + x = 0");
  }
  
   public void testAppendRightNOrlb() throws DialogProtocolException, TestException, RemoteException {

    TestInitDialogNOrlb wrapper = new TestInitDialogNOrlb("exp_IsacCore_Tests_1b.xml");
    WorksheetDialogNOrlb dialog = wrapper.getWorksheetDialog();
    MockWorksheetRMI ws_mock = wrapper.getMockWorksheet();

    //do the first two steps
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    String formula2 = formulaInTreeAtNOrlb(ws_mock.getCalcChanged(), dialog);
    assertEquals("2nd formula", formula2, "x + 1 + -1 * 2 = 0");

    //append "-1 + x = 0" which the kernel would have proposed, too
    CalcFormula edited_formula_ = new CalcFormula();
    edited_formula_.setText("-1 + x = 0");
    dialog.notifyUserAction(new UserActionOnCalcElement(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE, edited_formula_));
    //check (1): a formula has been appended
    CalcChanged position = ws_mock.getCalcChanged();
    assertEquals("unchanged= ",
            position.getLastUnchangedFormula().toSMLString(), "([1],Res)");
    assertEquals("deleted= ",
            position.getLastDeletedFormula().toSMLString(), "([1],Res)");
    assertEquals("generated= ",
            position.getLastGeneratedFormula().toSMLString(), "([2],Res)");
    //check (2)
    String result = formulaInTreeAtNOrlb(ws_mock.getCalcChanged(), dialog);
    assertEquals("input formula is returned: ", result, "-1 + x = 0");
  }

  /**
   * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
   * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
   * smltest/FE-interface/interface.sml "...enter} other"
   *
   * @see TestBridgeInput#testAppendOther1
   */
  public void testAppendOther1() throws DialogProtocolException, TestException, RemoteException {

    TestInitDialog wrapper = new TestInitDialog("exp_IsacCore_Tests_1b.xml");
    WorksheetDialog dialog = wrapper.getWorksheetDialog();
    MockWorksheetRMI ws_mock = wrapper.getMockWorksheet();

    //do the first two steps
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    String formula2 = formulaInTreeAt(ws_mock.getCalcChanged(), dialog);
    assertEquals("2nd formula", formula2, "x + 1 + -1 * 2 = 0");

    //append "x - 1 = 0" which needs 11 steps (on same level) to justify
    CalcFormula edited_formula_ = new CalcFormula();
    edited_formula_.setText("x - 1 = 0");
    //edited_formula_.setPosition(pos);
    dialog.notifyUserAction(new UserActionOnCalcElement(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE, edited_formula_));
    //check (1): a formula has been appended
    CalcChanged position = ws_mock.getCalcChanged();
    assertEquals("unchanged= ",
            position.getLastUnchangedFormula().toSMLString(), "([1],Res)");
    assertEquals("deleted= ",
            position.getLastDeletedFormula().toSMLString(), "([1],Res)");
    assertEquals("generated= ",
            position.getLastGeneratedFormula().toSMLString(), "([2],Res)");
    //check (2)
    String result = formulaInTreeAt(ws_mock.getCalcChanged(), dialog);
    assertEquals("input formula is returned: ", result, "x - 1 = 0");
    //check (3): only _one_ formula returned : see TestBridgeInput#testAppendOther1
    //check (4): there are 11 steps in between: see TestBridgeInput#testAppendOther1
  }
  
   public void testAppendOther1NOrlb() throws DialogProtocolException, TestException, RemoteException {

    TestInitDialogNOrlb wrapper = new TestInitDialogNOrlb("exp_IsacCore_Tests_1b.xml");
    WorksheetDialogNOrlb dialog = wrapper.getWorksheetDialog();
    MockWorksheetRMI ws_mock = wrapper.getMockWorksheet();

    //do the first two steps
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    String formula2 = formulaInTreeAtNOrlb(ws_mock.getCalcChanged(), dialog);
    assertEquals("2nd formula", formula2, "x + 1 + -1 * 2 = 0");

    //append "x - 1 = 0" which needs 11 steps (on same level) to justify
    CalcFormula edited_formula_ = new CalcFormula();
    edited_formula_.setText("x - 1 = 0");
    //edited_formula_.setPosition(pos);
    dialog.notifyUserAction(new UserActionOnCalcElement(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE, edited_formula_));
    //check (1): a formula has been appended
    CalcChanged position = ws_mock.getCalcChanged();
    assertEquals("unchanged= ",
            position.getLastUnchangedFormula().toSMLString(), "([1],Res)");
    assertEquals("deleted= ",
            position.getLastDeletedFormula().toSMLString(), "([1],Res)");
    assertEquals("generated= ",
            position.getLastGeneratedFormula().toSMLString(), "([2],Res)");
    //check (2)
    String result = formulaInTreeAtNOrlb(ws_mock.getCalcChanged(), dialog);
    assertEquals("input formula is returned: ", result, "x - 1 = 0");
    //check (3): only _one_ formula returned : see TestBridgeInput#testAppendOther1
    //check (4): there are 11 steps in between: see TestBridgeInput#testAppendOther1
  }

  /**
   * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
   * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
   * smltest/FE-interface/interface.sml "...enter} oth 2"
   *
   * @see TestBridgeInput#testAppendOther2
   */
  public void testAppendOther2() throws DialogProtocolException, TestException, RemoteException {

    TestInitDialog wrapper = new TestInitDialog("exp_IsacCore_Tests_1b.xml");
    WorksheetDialog dialog = wrapper.getWorksheetDialog();
    MockWorksheetRMI ws_mock = wrapper.getMockWorksheet();

    //do the first two steps
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    String formula2 = formulaInTreeAt(ws_mock.getCalcChanged(), dialog);
    assertEquals("2nd formula", formula2, "x + 1 + -1 * 2 = 0");

    //append "x = 1" which needs 6 steps (+ one level down) to justify
    CalcFormula edited_formula_ = new CalcFormula();
    edited_formula_.setText("x = 1");
    //edited_formula_.setPosition(pos);
    dialog.notifyUserAction(new UserActionOnCalcElement(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE, edited_formula_));
    //check (1): a formula has been appended
    CalcChanged position = ws_mock.getCalcChanged();
    assertEquals("unchanged= ",
            position.getLastUnchangedFormula().toSMLString(), "([1],Res)");
    assertEquals("deleted= ",
            position.getLastDeletedFormula().toSMLString(), "([1],Res)");
    assertEquals("generated= ",
            position.getLastGeneratedFormula().toSMLString(), "([3,2],Res)");
    //check (2)
    String result = formulaInTreeAt(ws_mock.getCalcChanged(), dialog);
    assertEquals("input formula is returned: ", result, "x = 1");
    //check (3): the inserted formulae _MUST_ be displayed (but not on level 3 !):
    //see TestBridgeInput#testAppendOther2
  }
  
  public void testAppendOther2NOrlb() throws DialogProtocolException, TestException, RemoteException {

    TestInitDialogNOrlb wrapper = new TestInitDialogNOrlb("exp_IsacCore_Tests_1b.xml");
    WorksheetDialogNOrlb dialog = wrapper.getWorksheetDialog();
    MockWorksheetRMI ws_mock = wrapper.getMockWorksheet();

    //do the first two steps
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    String formula2 = formulaInTreeAtNOrlb(ws_mock.getCalcChanged(), dialog);
    assertEquals("2nd formula", formula2, "x + 1 + -1 * 2 = 0");

    //append "x = 1" which needs 6 steps (+ one level down) to justify
    CalcFormula edited_formula_ = new CalcFormula();
    edited_formula_.setText("x = 1");
    //edited_formula_.setPosition(pos);
    dialog.notifyUserAction(new UserActionOnCalcElement(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE, edited_formula_));
    //check (1): a formula has been appended
    CalcChanged position = ws_mock.getCalcChanged();
    assertEquals("unchanged= ",
            position.getLastUnchangedFormula().toSMLString(), "([1],Res)");
    assertEquals("deleted= ",
            position.getLastDeletedFormula().toSMLString(), "([1],Res)");
    assertEquals("generated= ",
            position.getLastGeneratedFormula().toSMLString(), "([3,2],Res)");
    //check (2)
    String result = formulaInTreeAtNOrlb(ws_mock.getCalcChanged(), dialog);
    assertEquals("input formula is returned: ", result, "x = 1");
    //check (3): the inserted formulae _MUST_ be displayed (but not on level 3 !):
    //see TestBridgeInput#testAppendOther2
  }

  /**
   * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
   * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
   * smltest/FE-interface/interface.sml "...enter} NOTok"
   *
   * @see TestBridgeInput#testAppendNotOk
   */
  public void testAppendNotOk() throws DialogProtocolException, TestException, RemoteException {

    TestInitDialog wrapper = new TestInitDialog("exp_IsacCore_Tests_1b.xml");
    WorksheetDialog dialog = wrapper.getWorksheetDialog();
    MockWorksheetRMI ws_mock = wrapper.getMockWorksheet();

    //do the first two steps
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    String formula2 = formulaInTreeAt(ws_mock.getCalcChanged(), dialog);
    assertEquals("2nd formula", formula2, "x + 1 + -1 * 2 = 0");

    //append "x = 4711" which cannot be derived
    CalcFormula edited_formula_ = new CalcFormula();
    edited_formula_.setText("x = 4711");
    //edited_formula_.setPosition(pos);
    dialog.notifyUserAction(new UserActionOnCalcElement(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE, edited_formula_));
    //check
    CalcMessage msg = ws_mock.getCalcMessage();
    assertEquals("CalcMessage: ", msg.getText(), "no derivation found");
  }
  
  public void testAppendNotOkNOrlb() throws DialogProtocolException, TestException, RemoteException {

    TestInitDialogNOrlb wrapper = new TestInitDialogNOrlb("exp_IsacCore_Tests_1b.xml");
    WorksheetDialogNOrlb dialog = wrapper.getWorksheetDialog();
    MockWorksheetRMI ws_mock = wrapper.getMockWorksheet();

    //do the first two steps
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_CALCULATE_1, EUIContext.UI_CONTEXT_CALCULATION));
    String formula2 = formulaInTreeAtNOrlb(ws_mock.getCalcChanged(), dialog);
    assertEquals("2nd formula", formula2, "x + 1 + -1 * 2 = 0");

    //append "x = 4711" which cannot be derived
    CalcFormula edited_formula_ = new CalcFormula();
    edited_formula_.setText("x = 4711");
    //edited_formula_.setPosition(pos);
    dialog.notifyUserAction(new UserActionOnCalcElement(
            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE, edited_formula_));
    //check
    CalcMessage msg = ws_mock.getCalcMessage();
    assertEquals("CalcMessage: ", msg.getText(), "no derivation found");
  }
}
