package isac.wsdialog;

import isac.TestException;
import isac.interfaces.ICalcIterator;
import isac.useractions.EUIContext;
import isac.useractions.EUIElement;
import isac.useractions.UserAction;
import isac.useractions.UserActionOnPosition;
import isac.util.CalcChanged;
import isac.util.formulae.CalcFormula;

import java.rmi.RemoteException;
import java.util.Vector;

import junit.framework.TestCase;

/**
 * Further tests for the WorksheetDialog, TODO goon
 * @author neuper
 */
public class TestSolveEtc extends TestCase {
  
    /**
     * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl'; comprises the
     * usecase UC\label{SOLVE:INFO:intermediate-steps} see
     * smltest/FE-interface/interface.sml "interSteps: on 'miniscript with
     * mini-subpbl'"
     *
     * Note WN170215: all tests herein are destroyed by @see isac.users.TestUserManager,
     * Thus @see isac.users.TestUserManager is outcommented in @see isac.users.Testall.
     * 
     * @see TestBrdige#testIntermediateSteps
     */
   public void testIntermediateSteps() throws DialogProtocolException, TestException, RemoteException {

    TestInitDialog wrapper = new TestInitDialog("exp_IsacCore_Tests_1b.xml");
    WorksheetDialog dialog = wrapper.getWorksheetDialog();
    MockWorksheetRMI ws_mock = wrapper.getMockWorksheet();

    //STEP (1) do the whole calculation for the example
    dialog.notifyUserAction(new UserAction(
            EUIElement.UI_SOLVE_CALCULATE_ALL, EUIContext.UI_CONTEXT_CALCULATION));

    //STEP (2) request intermediate steps above the current position ([],Res)
    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_SHOW_INTERMEDIATE_STEPS,
            EUIContext.UI_CONTEXT_CALCELEMENTS));
    //check
    CalcChanged position = ws_mock.getCalcChanged();
    Vector<CalcFormula> elems = position.getLastUnchangedFormula().getFormulaeFromTo(
            position.getLastGeneratedFormula(), new Integer(1), false);
    assertEquals("these # of steps were inbetween: ", elems.size(), 6);
    System.out.print("elems ([],Res)= "); System.out.println(elems);

    //STEP (3) set the current position to ([2],Res) and request intermediate steps above
    //Position pos = new Position();//([2],Res)
    //pos.addInt("2");
    //pos.setKind("Res");
    //... intermediateSteps needs a CalcIterator (containing the CalcTree)
    position = ws_mock.getCalcChanged();
    ICalcIterator ci = position.getLastUnchangedFormula().cloneIterator();
    ci.moveDown(); //TODO: copy what is done in the Worksheet
    ci.moveDown();
    ci.moveDown(); //([2],Res), see BridgeLog
    dialog.notifyUserAction(new UserActionOnPosition(EUIElement.UI_SOLVE_CHANGE_CONTEXT_FORMULA,
            EUIContext.UI_CONTEXT_CALCELEMENTS));
	//    [
	//    (([], Frm), solve (x + 1 = 2, x)),
	//    (([1], Frm), x + 1 = 2),
	//    (([1], Res), x + 1 + -1 * 2 = 0),
	//    (([2], Res), -1 + x = 0),
	//    (([3], Pbl), solve (-1 + x = 0, x)),
	//    (([3,1], Frm), -1 + x = 0),
	//    (([3,1], Res), x = 0 + -1 * -1),
	//    (([3,2], Res), x = 1),
	//    (([3], Res), [x = 1]),
	//    (([4], Res), [x = 1]),
	//    (([], Res), [x = 1])]
    //
    //  sent to bridge: interSteps 1 ([4],Res); ...
    //  ... raised <ERROR> no Rewrite_Set... </ERROR> already in isabisac13-2,
    //  but the XML-Parser swallowed it without complaint, and there is no assert below.
    //
    //THE LINES BELOW WOULD BREAK AGAIN WHEN REPLACING "isabelle tty" WITH Pide,
    //  because in BridgeRMI.intermediateSteps IsaToJava.is_message is true 
    //  and thus there is still: CEvent return_val = null;
//    dialog.notifyUserAction(new UserAction(EUIElement.UI_SOLVE_SHOW_INTERMEDIATE_STEPS,
//            EUIContext.UI_CONTEXT_CALCELEMENTS));
//    //check
//    position = ws_mock.getCalcChanged(); //needed below
//    elems = position.getLastUnchangedFormula().getFormulaeFromTo(
//            position.getLastGeneratedFormula(), new Integer(1), false);
//    System.out.print("elems ([2],Res)= "); System.out.println(elems);
//    //TODO: still the same elems (in spite of other position)
   }
}
