/**
 * Created on Oct 13, 2003
 * @author Richard Gradischnegg RG
 */
package isac.bridge;

import isac.interfaces.ICalcElement;
import isac.interfaces.ICalcIterator;
import isac.util.Formalization;
import isac.util.formulae.Assumptions;
import isac.util.formulae.CalcHead;
import isac.util.formulae.Context;
import isac.util.formulae.CalcFormula;
import isac.util.formulae.ContextMethod;
import isac.util.formulae.ContextProblem;
import isac.util.formulae.ContextTheory;
import isac.util.formulae.Match;
import isac.util.formulae.MethodID;
import isac.util.formulae.Position;
import isac.util.formulae.ProblemID;
import isac.util.tactics.Tactic;
import isac.wsdialog.IContextProvider.ContextType;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Vector;

/**
 * @author Richard Gradischnegg
 * 
 * WN0422 here seems to be the key for simplifying the whole bridge.* drawbacks
 * of the present implementation: # more complicated than necessary #
 * JUnitTestCases require BridgeMain running # BridgeLogger cannot be made
 * serializable, thus not accessible for writing by JUnitTestCases # ...
 */
public interface IBridgeRMI extends Remote {

    public int getCalcTree(Formalization f) throws RemoteException;

    //for specify phase, see respective methods of IToCalc

    public CalcHead modelProblem(int calc_tree_id) throws RemoteException;

    public CalcHead nextSpecify(int calc_tree_id) throws RemoteException;

    public CalcHead checkCalcHead(int calc_tree_id, CalcHead calcHead)
            throws RemoteException;

    public CalcHead completeModel(int calc_tree_id) throws RemoteException;

    public CalcHead setTheory(int calc_tree_id, String thy_id) throws RemoteException;

    public CalcHead setProblem(int calc_tree_id, ProblemID id)
            throws RemoteException;

    public CalcHead setMethod(int calc_tree_id, MethodID id)
            throws RemoteException;

    public CalcHead completeCalcHead(int calc_tree_id) throws RemoteException;

    public CalcHead resetCalcHead(int calc_tree_id) throws RemoteException;

    //for solve phase

    public Tactic fetchProposedTactic(int calcTreeID) throws RemoteException;

    public int setNextTactic(int calcTreeID, Tactic tactic)
            throws RemoteException;

    public CEvent autoCalculate(int calcTreeID, int scope, int nSteps)
            throws RemoteException;

    public CEvent replaceFormula(int calcTreeID, CalcFormula f)
            throws RemoteException;

    public CEvent appendFormula(int id, CalcFormula f)
            throws RemoteException;

    public Vector getAppliableTactics(int calcTreeID, int scope, Position pos)
            throws RemoteException;

    public CEvent intermediateSteps(int id, ICalcIterator ci)
            throws RemoteException;

    public Vector getFormulaeFromTo(int id, ICalcIterator iterator_from,
            ICalcIterator iterator_to, Integer level,
            boolean result_includes_tactics) throws RemoteException;

    public int iterator(int calcTreeID) throws RemoteException;

    public Position moveRoot(int calcTreeID, int iteratorID)
            throws RemoteException;

    //WN041208 late decision for 'interpretSML(moveUp 1 ([1,2,3],"Frm"))'
    //WN041208 or 'interpretSML(moveActiveUp 1)' requires reduncancy here:
    //WN041208 these 2 args are redundant: (1)vvvvvvvvvvv (2)vvvvvvvv
    public Position moveUp(int calcTreeID, int iteratorID, Position p)
            throws RemoteException;

    public Position getActiveFormula(int calcTreeID) throws RemoteException;

    public Position moveActiveFormula(int calcTreeID, Position p)
            throws RemoteException;

    public Position moveDown(int calcTreeID, int iteratorID, Position p)
            throws RemoteException;

    public Position moveLevelUp(int calcTreeID, int iteratorID, Position p)
            throws RemoteException;

    public Position moveLevelDown(int calcTreeID, int iteratorID, Position p)
            throws RemoteException;

    public Position moveCalcHead(int calcTreeID, int iteratorID, Position p)
            throws RemoteException;

    public ICalcElement getElement(int calcTreeID, Position p)
            throws RemoteException;

    public Tactic getTactic(int calcTreeID, Position pos)
            throws RemoteException;

    public Assumptions getAssumptions(int calcTreeID, Position pos)
            throws RemoteException;

    public Assumptions getAccumulatedAssumptions(int calcTreeID, Position pos)
            throws RemoteException;

    public Match initMatchProblem(int calcTreeID)
    throws RemoteException;

    
    public Match tryMatchProblem(int calcTreeID, ProblemID problemID)
    throws RemoteException;

    public Match tryRefineProblem(int calcTreeID, ProblemID problemID)
            throws RemoteException;

    public Match initMatchMethod(int calcTreeID)
    throws RemoteException;

    public Match tryMatchMethod(int calcTreeID, MethodID problemID)
    throws RemoteException;

    /**
     * initialises the context
     * 
     * @param javaCalcID
     * @param type - the type of context
     * @param pos - the position
     * @return the context for the position
     * @throws RemoteException
     */
    public Context initContext(int javaCalcID, ContextType type, Position pos ) 
    throws RemoteException;
    
    /***
     * Checks whether the selected context is applicable to the formula with the 
     * given position 
     * 
     * @param javaCalcID
     * @param context
     * @param pos
     * @return new Context
     * @throws RemoteException
     */
    public Context checkContext(int javaCalcID, Context context, Position pos ) 
    throws RemoteException;
    
    
    /***
     * Refines the Context for Problems
     * 
     * @param javaCalcID
     * @param context
     * @param pos
     * @return new Context
     * @throws RemoteException
     */
    public Context refineProblem(int javaCalcID, Context context, Position pos ) throws RemoteException;
    
    /**
     * sets the context
     * 
     * @param javaCalcID
     * @param type - the type of context
     * @param pos - the position
     * @throws RemoteException
     */
    public CalcHead setContext(int javaCalcID, ContextProblem context, Position pos ) throws RemoteException;
    
    /**
     * sets the context
     * 
     * @param javaCalcID
     * @param type - the type of context
     * @param pos - the position
     * @throws RemoteException
     */
    public CalcHead setContext(int javaCalcID, ContextMethod context, Position pos ) throws RemoteException;
    
    /**
     * sets the context
     * 
     * @param javaCalcID
     * @param type - the type of context
     * @param pos - the position
     * @throws RemoteException
     */
    public CChanged setContext(int javaCalcID, ContextTheory context, Position pos ) throws RemoteException;


    public boolean destruct(int calcTreeID) throws RemoteException;

    public Vector saveCalcTree(int calcTreeID) throws RemoteException;

    public int loadCalcTree(Vector v) throws RemoteException;

    public CChanged startCalculate(int calcTreeID) throws RemoteException;

}