/*
 * @author Georg Kompacher, member of the ISAC-team, 
 * Copyright (c) 2006 by Georg Kompacher
 * created Jul 10, 2006 03:20:26 PM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.browserdialog;

import isac.users.IUserSettings;
//UL import isac.users.UserLogger;
import isac.users.UserLanguage;
import isac.users.UserManager;
import isac.users.LogItem;
import isac.users.User;
import isac.users.LogItem.DialogTypes;
import isac.gui.util.Hierarchy;
import isac.gui.util.HierarchyNode;
import isac.interfaces.IToGUI;
import isac.session.DialogGuide;
import isac.session.ILMService;
import isac.session.ILMService.SessionType;
import isac.session.Session;
import isac.useractions.EUIContext;
import isac.useractions.EUIElement;
import isac.useractions.IUserAction;
import isac.useractions.UIAction;
import isac.useractions.UIActionOnHierarchy;
import isac.useractions.UIActionOnLink;
import isac.useractions.UIActionOnMathContext;
import isac.useractions.UserActionOnMathContext;
import isac.util.KEStoreCommunication;
import isac.util.ObjectManagerPaths;
import isac.util.formulae.Context;
import isac.util.formulae.ContextUnknown;
import isac.util.formulae.HierarchyKey;
import isac.util.formulae.KEStoreKey;
import isac.util.parser.HierarchyKeyParser;
import isac.util.parser.XMLHierarchyParser;
import isac.wsdialog.DialogProtocolException;
import isac.wsdialog.IContextProvider;
import isac.wsdialog.IContextProvider.ContextType;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.log4j.Logger;
import org.apache.xmlrpc.XmlRpcClient;
import org.apache.xmlrpc.XmlRpcException;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;


/**
 * @author Georg Kompacher
 * 
 * Each BrowserDialog is responsible for the communication to the WSDialogs via
 * the ContextProvider-interface and also for the communication to the Browsers
 * via the UIActions. Each BrowserDialog gets information from the Browser by
 * triggered UserActions.
 */
public abstract class BrowserDialog extends UnicastRemoteObject implements
        IBrowserDialog, IContextPresenter {

    private static final Logger logger = Logger.getLogger(BrowserDialog.class
            .getName());

    /**
     * For serialization, see corresponding API Documentation; value generated
     * by eclipse to avoid a warning
     */
    private static final long serialVersionUID = 7780383307538687552L;

    protected IToGUI browser_frame_rmi_;

    private XmlRpcClient client_ = null;

    protected Hierarchy hierarchy_;

    /**
     * the context currently shown in the browser
     */
    protected Context current_context_;

    protected Session session_;

    protected ContextType context_type_;

    /** true if the Context On button is visible* */
    protected boolean context_on_visible_;

    protected boolean context_on_ = false;

    protected UserLanguage user_language_;

//UL     protected UserLogger user_logger_;

    protected ILMService compod_;
    
    protected DialogGuide dialog_;

    private LogItem log_Item_;
    
    /**
     * The constructor of this class which is given the session the
     * BrowserDialog is instanciated for. We need the session to get the correct
     * WSDialogManager (and also the active WSDialog) for this BrowserDialog.
     * 
     * @param session
     * @throws RemoteException
     */
    protected BrowserDialog(Session session, DialogGuide dialog) throws RemoteException {
        session_ = session;
        user_language_ = session_.getLanguage();        
//UL         user_logger_ = session_.getUserLogger();
        compod_ = session_.getCompod();
        dialog_ = dialog;
    }

    /***************************************************************************
     * Makes the corresponding BrowserFrame visible
     * 
     */
    public void showBrowserFrame() {
        System.out.println("get focus");
        drawButtons();
        if (context_on_)
            sendInitContextToBrowser();
        sendGetFocusToBrowser();
    }

    /**
     * Encapsulates the communication with KEStore. On each call, a new
     * XMLRPC-connection to the KESTore is established
     * 
     * @param function:
     *            the function from the KEStore to call
     * @param parameters:
     *            parameters needed by the called function
     */
    @SuppressWarnings("unchecked") //..type erasure, see Bakk Andreas Hujber
	private Vector<String> callKEStore(String function,
            Vector<String> parameters) {
        Vector<String> errvector = new Vector<String>();
        try {
            if (client_ == null)
                client_ = new XmlRpcClient(ObjectManagerPaths.KESTORE_PATH);
            return (Vector<String>) client_.execute(function, parameters);
            //^^^^ type erasure             ^^^ returns Object, not way to Vector<String>
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (XmlRpcException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return errvector;
    }

    /**
     * This method loads the hierarchy of a given type from the KEStore and
     * returns it. If the type is unknown, null will be the return value.
     */
    protected Hierarchy loadHierarchy(ContextType type) throws RemoteException {
        Vector<String> param = new Vector<String>();
        param.add(type.toString());
        Vector<String> keresult = callKEStore("KEStore.loadHierarchy", param);

        if (keresult.size() == 0)
            return null;

        String dummy = (String) keresult.elementAt(0);
        XMLHierarchyParser hierarchy_parser = new XMLHierarchyParser(dummy);

        Node root = hierarchy_parser.getRoot();
        Hierarchy hierarchy = hierarchy_parser.createNodesHierarchy(root);

        return hierarchy;
    }

    /**
     * @see isac.browserdialog.IBrowserDialog#registerBrowserFrame(IToGUI)
     * 
     * @param browser_frame
     *            the browser frame to register
     */
    public void registerBrowserFrame(IToGUI browser_frame)
            throws RemoteException {
        System.out.println("registerBrowserFrame init");
        browser_frame_rmi_ = browser_frame;

        UIActionOnHierarchy ui_action_on_hierarchy = new UIActionOnHierarchy(
                EUIElement.UI_BROWSER_HIERARCHY,
                EUIContext.UI_CONTEXT_HIERARCHY, user_language_, hierarchy_);
        System.out
                .println("registerBrowserFrame after 'new UIActionOnHierarchy'");
        try {
            browser_frame_rmi_.doUIAction(ui_action_on_hierarchy);
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }

    /**
     * To send a html link to the minibrowser
     * 
     * @param link
     * @throws RemoteException
     */
    protected void sendLinkToMiniBrowser(String link) {
        try {
            int user_status = UserManager.getInstance().getUser(
                    session_.getId()).getUserSettings().getValue(
                    IUserSettings.KEY3_STATUS);
            sendLinkToMiniBrowser(link, user_status);
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }

    /**
     * TODO: To show files "KEStoreKey.html" or "KEStoreKey_author.html" in
     * minibrowser depending on the user type. This should be rewritten as soon
     * as the UserModel is there.
     * 
     * @param link -
     *            link to send
     * @param user_setting -
     *            the user setting
     */
    protected void sendLinkToMiniBrowser(String link, int user_status)
            throws RemoteException {
        System.out.println("link is: " + link);

        switch (user_status) {
        case IUserSettings.VAL3_AUTHOR:
            link = link.replaceAll(".html", "_author.html");
            break;
        default:
            break;
        }

        System.out.println("status link is: " + link);
        browser_frame_rmi_.doUIAction(new UIActionOnLink(
                EUIElement.UI_MINIBROWSER_LINK, EUIContext.UI_CONTEXT_CONTENT,
                user_language_, link));
    }

    /**
     * interpret a link coming from the Browser in the following ways:
     * 
     * (1) if the url contains COMMAND_EXEC_EXAMPLE then do so
     * 
     * (2) if the url points outside isac with http://www... then do so
     * 
     * (3) if the url points into isac check if it is in the hierarchy, and if
     * so, then show the content in the respective browser.
     * 
     * @param url
     *            must contain a path with at least two directory-levels
     *            (../kbase/XXX/*.html) where XXX is exp | thy | pbl | met
     */
    protected void interpretLink(URL url) throws RemoteException {

        String to_interpret = url.toString();
        String[] link_array; // ..,proto2/www/kbase/thy/thy_xxx.html#COMMAND_EXEC_EXAMPLE#thy_xxx.xml
        String link = ""; // ...../proto2/www/kbase/thy/thy_xxx.html
        String[] path_array; // ..[proto2,www,kbase,thy,thy_xxx.html
        String filename = ""; // .......................^^^^^^^
        String target = ""; // ......................................targetInPage
        String command = ""; // .....................................COMMAND_EXEC_EXAMPLE
        String argument = ""; // ........................................thy_xxx.xml
        String type = ""; // taken from here............^^^ (mustbe same^^^)
        // filename and exp_key -- ".xxx" should be the same

        // WN080825 StringTokenizer better ?
        link_array = to_interpret.split("#");
        link = link_array[0];
        if (link_array.length == 2) {
            target = link_array[1];
        } // TODO.WN080422 push through filename_#target_
        if (link_array.length > 2) {
            command = link_array[1];
            argument = link_array[2];
            if (command.equals("COMMAND_EXEC_EXAMPLE")) {
                ExampleDialog example_dialog = session_.getExampleDialog();
                example_dialog.openWorksheetFromExample(argument);
                return;// ---------------------------------------(1)-->
            } else if (command.equals("COMMAND_START_ASSESSMENT")) {
                try {
                    compod_.setExamples(session_.getUser().getUsername(), // getUser: "Compod garbage"
                            argument);
                    compod_.startSession(session_.getUser().getUsername(), // getUser: "Compod garbage"
                            SessionType.ASSESMENT);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                // start the first example ...
                session_.getExampleDialog().notifyStartFirstExample();
                return;// --------------------------------------------->
            } else if (command.equals("COMMAND_STOP_ASSESSMENT")) {
                try {
                    compod_.stopSession(session_.getUser().getUsername(), // getUser: "Compod garbage"
                            false /* FIXME.WN071010 */);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                return;// --------------------------------------------->
            } else if (command.equals("COMMAND_START_COURSE")) {
                try {
                    compod_.setExamples(session_.getUser().getUsername(), // getUser: "Compod garbage"
                            argument);
                    compod_.startSession(session_.getUser().getUsername(), // getUser: "Compod garbage"
                            SessionType.COURSE);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                // start the first example ...
                session_.getExampleDialog().notifyStartFirstExample();
                return;// --------------------------------------------->
            } else if (command.equals("COMMAND_STOP_COURSE")) {
                try {
                    compod_
                            .stopSession(session_.getUser().getUsername(), // getUser: "Compod garbage"
                                    false /* FIXME.WN071010 */);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                return;// --------------------------------------------->
            }
        }
        path_array = link.split("/");
        filename = path_array[path_array.length - 1];
        if (filename.endsWith(".html"))
            filename = filename.replaceAll(".html", "");
        type = filename.substring(0, 3);

        BrowserDialog link_target = null;
        ContextType context_type = null;
        if (type.equals("pbl")) {
            link_target = session_.getProblemDialog();
            context_type = ContextType.CONTEXT_PROBLEMS;
        } else if (type.equals("exp")) {
            link_target = session_.getExampleDialog();
            context_type = ContextType.CONTEXT_EXAMPLES;
        } else if (type.equals("met")) {
            link_target = session_.getMethodDialog();
            context_type = ContextType.CONTEXT_METHODS;
        } else if (type.equals("thy")) {
            link_target = session_.getTheoryDialog();
            context_type = ContextType.CONTEXT_THEORIES;
        } else {
            System.out.println("Link is external");
            sendLinkToMiniBrowser(to_interpret);
            return;// -----------------------------http://www....-(2)-->
        }

        HierarchyNode node_to_select = link_target.getHierarchy(filename)
                .getNodeForKEStoreKey(new KEStoreKey(filename));
        if (node_to_select == null) {
            System.out.println("BrowserDialog: Sorry, file: '" + filename
                    + "' not found" + "in hierarchy");
            String link_to_send = ObjectManagerPaths.WEB_PATH + type.toString()
                    + "/blank.html";
            sendLinkToMiniBrowser(link_to_send);
            return;// ---------------------------NOT-in-hierarchy-(3)-->
        }
        HierarchyKey hierarchy_key = node_to_select.getHierarchyKey();
//        System.out.println("Node to select : " + node_to_select);

        Context link_context = new ContextUnknown();
        link_context.setHierarchyKey(hierarchy_key);
        link_context.setKEStoreKey(new KEStoreKey(filename));

        link_target.sendContextToBrowser(link_context);
        link_target.sendLinkToMiniBrowser(link_context, context_type, target);
        link_target.sendGetFocusToBrowser();

    }

    /**
     * reverses the sequence of path-elements of the HierarchyKey for technical
     * reasons.
     */

    protected HierarchyKey reverse(HierarchyKey key) {
        HierarchyKey result = new HierarchyKey();

        if (key == null)
            return result;

        Vector<String> input = key.getID();
        if (input.size() == 0)
            return result;
        for (int count = input.size() - 1; count >= 0; count--)
            result.addString((String) input.get(count));
        return result;
    }

    /**
     * This method belongs to the IContextPresenter interface. It is called by
     * the Worksheet Dialog, if context changes have to be shown in the browser.
     * 
     * @param context
     *            the new context to show in the browser
     * @see
     */
    public void presentContext(Context context) {
        if (!context_on_)
            return;

        System.out .println("In BrowserDialog::presentContext( context ): is " + context);
        updateState();
        drawButtons();
        if (context instanceof ContextUnknown) {
            context.setHierarchyKey(current_context_.getHierarchyKey());
            context.setKEStoreKey(current_context_.getKEStoreKey());
        }
        sendContextToBrowser(context);
        sendLinkToMiniBrowser(context, context_type_, "");
    }

    protected Vector<String> loadContent(String session, String type,
            String filename) {

        Vector<String> parameter = new Vector<String>();
        parameter.add(type);
        parameter.add(filename);
        Vector<String> result = callKEStore("KEStore.loadContent", parameter);

        // collecting the data for the error_log
        if ((result.firstElement()).toString().equals("ERROR")) {

            String file_name = result.get(1).toString();
            String problem_name = result.get(2).toString();
            result.removeAllElements();
            String error_information = "ERROR" + "FILE: " + file_name
                    + " =>  PROBLEM: " + problem_name;
            result.add(error_information);
            return result;

        }
        result.setElementAt(KEStoreCommunication.decodeUmlauts(
                result.firstElement()).toString(), 0);
        return filterContent(session, result);
    }

    /*
     * @param string @return
     */
    private String getErrorXML(String string) {
        return "<ERROR><DESCRIPTION> <h1> Error in file XBrowserDialog.java </h1>"
                + string + "</DESCRIPTION></ERROR>";
    }

    /**
     * Loads the requested KE-Object of the type <code>type</code> from the
     * KE-Store. <code>type</code> and <code>content_id</code> together give
     * the unique id. One and the same <code>content_id</code> might be used
     * in different types (<code>type</code> can be pbl, or met for the basic
     * items of the knowledge-base as well as an identifyer used for different
     * courses). If objects with equal <code>content_id</code> s reside in
     * different types, their only difference should reside within the
     * description. XML-RPC version.
     * 
     * @param session
     *            session to fetch the KE-Object for.
     * @param type
     *            requestet information-type to load the content for. e.g. pbl,
     *            met, or exp
     * @param content_id
     *            identifies the KE-Object within the <code>type</code>
     * @return requested KE-Object in its XML-representation. The result might
     *         be filtered according to the session given py parameter
     *         <code>session</code>
     */
    private Vector<String> filterContent(String session, Vector<String> result) {
        // if session == "", no filtering is needed (author mode)
        if (session.equals("")) // no filtering necessary
            return result;
        try {
            TransformerFactory tFactory = TransformerFactory.newInstance();
            Transformer transformer = tFactory.newTransformer();

            String content = result.firstElement();
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();
            StringReader sr = new StringReader(content);
            Document document = db.parse(new InputSource(sr));
            DOMSource source = new DOMSource(document);
            DOMResult dom_result = new DOMResult();

            transformer.transform(source, dom_result);
            Source filtered_source = filter(dom_result.getNode());
            StringWriter output = new StringWriter();
            StreamResult filterresult = new StreamResult(output);
            transformer.transform(filtered_source, filterresult);
            String resultxml = output.toString();
//            System.out.print(resultxml);

            result = new Vector<String>();
            result.add(resultxml);
        } catch (TransformerConfigurationException e) {
            result.removeAllElements();
            result.add(getErrorXML("TransformerConfigurationEception: "
                    + e.getMessage()));
            e.printStackTrace();
        } catch (ParserConfigurationException e) {
            result.removeAllElements();
            result.add(getErrorXML("ParserConfigurationEception: "
                    + e.getMessage()));
            e.printStackTrace();
        } catch (SAXException e) {
            result.removeAllElements();
            result.add(getErrorXML("SAXException: " + e.getMessage()));
            e.printStackTrace();
        } catch (IOException e) {
            result.removeAllElements();
            result.add(getErrorXML("IOEception: " + e.getMessage()));
            e.printStackTrace();
        } catch (TransformerException e) {
            result.removeAllElements();
            result.add(getErrorXML("TransformerEception: " + e.getMessage()));
            e.printStackTrace();
        }

        return result;
    }

    /*
     * @param node
     */
    private Source filter(Node node) {
        recFilter(node, "EVAL_PRECOND");
        recFilter(node, "CAS");
        recFilter(node, "RELATE");
        return new DOMSource(node);
    }

    private void recFilter(Node node, String removename) {
        NodeList nlist = node.getChildNodes();
//        System.out.println(nlist);
        for (int indx = 0; indx < nlist.getLength(); indx++) {
            Node childnode = nlist.item(indx);
            if (childnode == null) {
//                System.out.println("child is null !!!");
                return;
            }
            if (childnode.getLocalName() == null){
//                System.out.println("localname is null !");
            }else if (childnode.getLocalName().equals(removename)) {
//                System.out.println("i should have been deleted");
                node.removeChild(childnode);
            } else
                recFilter(childnode, removename);
        }
    }

    protected Hierarchy getHierarchy(String xml_content) {
        return hierarchy_;
    }

    protected HierarchyKey getHierarchyKey(String xml_content) {
        HierarchyKeyParser hkp = new HierarchyKeyParser("isac/src/xml/isac.dtd");
        hkp.setValidating(true);
        HierarchyKey hierarchy_key = hkp.parse(xml_content);
        return hierarchy_key;
    }

    /**
     * Sends a GET_FOCUS command to the browser
     */
    protected void sendGetFocusToBrowser() {
        UIAction ui_action = new UIAction(EUIElement.UI_BROWSER_GET_FOCUS,
                EUIContext.UI_CONTEXT_BROWSER, user_language_);
        try {
            browser_frame_rmi_.doUIAction(ui_action);
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }

    /**
     * This method sends a context to the browser. The sended context is the new
     * current_context_.
     * 
     * @param context
     *            the context to send and the new current_context_
     */
    protected void sendContextToBrowser(Context context) {
        KEStoreKey kestore_key = context.getKEStoreKey();
        if (kestore_key == null)
            return;
        HierarchyNode hierarchy_node = hierarchy_
                .getNodeForKEStoreKey(kestore_key);
//        System.out.println("browserdialog:" + hierarchy_node.getKEStoreKey());
//        System.out.println("browserdialog:" + hierarchy_node.getHierarchyKey());

        context.setHierarchyKey(reverse(hierarchy_node.getHierarchyKey()));
        UIActionOnMathContext action = new UIActionOnMathContext(
                EUIElement.UI_BROWSER_CHANGE_CONTEXT,
                EUIContext.UI_CONTEXT_BROWSER, user_language_, context);
        try {
            browser_frame_rmi_.doUIAction(action);
        } catch (RemoteException e) {
            e.printStackTrace();
        }
        current_context_ = context;
    }

    /**
     * Send the browser the information to display the new selected content.
     * This method is called if a new context has to be shown in the browser. A
     * link is created and this link is sent to the minibrowser.
     * 
     * @param context
     *            the context containing the link
     * @param type =
     *            "exp" | "met" | "pbl" | "thy"
     * @param target =
     *            occasional target like filename#targetInPage
     */
    protected void sendLinkToMiniBrowser(Context context, ContextType type,
            String target) {
        KEStoreKey ks_key = context.getKEStoreKey();
        if (ks_key == null)
            return;
        String link = "blank.html";
        // FIXXME RK 060720 in future all ke_store_keys are without
        // .xml
        if (ks_key.toString().endsWith(".xml"))
            link = ks_key.toString().replaceFirst(".xml", ".html");
        else
            link = ks_key.toString() + ".html";
        String link_to_send = "";
        if (target.equals("")) {
            link_to_send = ObjectManagerPaths.WEB_PATH + type.toString() + "/"
                    + link;
        } else {
            link_to_send = ObjectManagerPaths.WEB_PATH + type.toString() + "/"
                    + link + "#" + target;

        }
        sendLinkToMiniBrowser(link_to_send);
    }

    /**
     * To handle all actions that are same for all browsers; TODO WN070710
     * returnvalue should not be necessary
     * 
     * @see isac.browserdialog.IBrowserDialog#notifyUserAction(isac.useractions.IUserAction)
     */
    public boolean notifyUserAction(IUserAction action) throws RemoteException,
            DialogProtocolException {
        updateState();
        drawButtons();
        String temp_selected_path = null;
        String file_name =  null;
        
        // user_logger_.logUserAction(action);
                
        if (logger.isDebugEnabled())
            logger.debug("  BD: ActionID=" + action.getActionID());
        switch (action.getActionID()) {

        case UI_BROWSER_GOT_FOCUS:
            session_.setActiveBrowserDialog(this);
            drawButtons();
            if (context_on_)
                sendInitContextToBrowser();
            break;
        case UI_BROWSER_LOST_FOCUS:
            session_.setInactiveBrowserDialog(this);
            break;
        case UI_BROWSER_GET_FOCUS:
            showBrowserFrame();
            break;
        case UI_BROWSER_CHANGE_CONTEXT:            
            UserActionOnMathContext uaomc = (UserActionOnMathContext) action;
            Context context = uaomc.getMathContext();
            file_name = hierarchy_.getNodeForKEStoreKey(context.getKEStoreKey()).toString();
      
            if (!context_on_){
              sendLinkToMiniBrowser(context, context_type_, "");     
              temp_selected_path = generateSelectedPath(context); 
            }
            else {
              sendCheckedContextToBrowser(context);
              temp_selected_path = generateSelectedPath(context);
            }
            
            if(context_type_ == ContextType.CONTEXT_EXAMPLES)
              generateAndSaveLogItem(file_name, temp_selected_path, DialogTypes.EXAMPLE);
            else if(context_type_ == ContextType.CONTEXT_METHODS)
              generateAndSaveLogItem(file_name, temp_selected_path, DialogTypes.METHOD);            
            else if(context_type_ == ContextType.CONTEXT_PROBLEMS)              
              generateAndSaveLogItem(file_name, temp_selected_path, DialogTypes.PROBLEM);
            else if(context_type_ == ContextType.CONTEXT_THEORIES)
              generateAndSaveLogItem(file_name, temp_selected_path, DialogTypes.THEORY);

            break;
        default:
            return false;
        }
        return true;
    }

    /**
     * generates the button for the match = on
     * 
     * @throws RemoteException
     */
    protected void switchToMatchOff() throws RemoteException {
        browser_frame_rmi_.removeUIElement(new UIAction(
                EUIElement.UI_BROWSER_MATCH_OFF, EUIContext.UI_CONTEXT_BROWSER,
                user_language_));
        browser_frame_rmi_.addUIElement(new UIAction(
                EUIElement.UI_BROWSER_MATCH_ON, EUIContext.UI_CONTEXT_BROWSER,
                user_language_));
        context_on_ = false;
    }

    /**
     * generates the button for the match = off
     * 
     * @throws RemoteException
     * 
     */
    /*
     * protected void switchToMatchOn() throws RemoteException {
     * browser_frame_rmi_ .removeUIElement(new
     * UIAction(EUIElement.UI_BROWSER_MATCH_ON, EUIContext.UI_CONTEXT_BROWSER));
     * browser_frame_rmi_ .addUIElement(new
     * UIAction(EUIElement.UI_BROWSER_MATCH_OFF,
     * EUIContext.UI_CONTEXT_BROWSER)); context_on_ = true;
     * 
     * IContextProvider active_wsdialog = session_.getContextProvider();
     * 
     * current_context_ = active_wsdialog.initContext( context_type_ );
     * sendContextToBrowser( current_context_ ); sendLinkToMiniBrowser(
     * current_context_, context_type_ ); }
     */

    /**
     * This method asks the active Worksheet about the current context. This is
     * done over the IContextProvider interface. The returned context, has to be
     * displayed.
     */
    public void sendInitContextToBrowser() {
        // real implementation:
        System.out.println("send init context");
        Context new_context = null;
        try {
            IContextProvider context_provider = session_.getContextProvider();
            if (context_provider == null)
                new_context = new ContextUnknown();
            else
                new_context = context_provider.initContext(context_type_);
            sendContextToBrowser(new_context);
            if (!(new_context instanceof ContextUnknown))
                sendLinkToMiniBrowser(new_context, context_type_, "");
        } catch (RemoteException e) {
            e.printStackTrace();
        }

        // dummy implementation for test purposes while initContext() is not
        // implemented in WorksheetDialog:
        /*
         * HierarchyKey item_to_select = new HierarchyKey();
         * 
         * System.out.println(hierarchy_.toString());
         * 
         * item_to_select.addString("rat"); item_to_select.addString("sq");
         * item_to_select.addString("root");
         * item_to_select.addString("univariate");
         * item_to_select.addString("equation");
         * 
         * TreePath tree_path = hierarchy_.getPath(item_to_select.getID());
         * System.out.println("--------------------");
         * System.out.println("tree_path: " + tree_path.toString());
         * HierarchyNode node_to_select = (HierarchyNode) tree_path
         * .getLastPathComponent(); Random a = new Random(); Context context =
         * null; int rand = a.nextInt(5); System.out.println("rand: " + rand);
         * if (rand > 2) { context = new ContextUnknown(); } else { context =
         * new Context(); context.setHierarchyKey(item_to_select);
         * context.setHierarchyNode(node_to_select); }
         * sendContextToBrowser(context); if (!(context instanceof
         * ContextUnknown)) sendLinkToMiniBrowser(context,context_type_); //end
         * Experimantation:
         */
    }

    protected abstract void drawButtons();

    /**
     * This method causes the Context On button to be shown. If the button is
     * already shown, nothing happens. Otherwise the Button for Context Off is
     * removed and the Button for Context On is shown.
     */
    protected void drawButtonContextOn() {
        if (context_on_visible_)
            return;
        try {
            browser_frame_rmi_.removeUIElement(new UIAction(
                    EUIElement.UI_BROWSER_MATCH_OFF,
                    EUIContext.UI_CONTEXT_BROWSER, user_language_));

            browser_frame_rmi_.addUIElement(new UIAction(
                    EUIElement.UI_BROWSER_MATCH_ON,
                    EUIContext.UI_CONTEXT_BROWSER, user_language_));
            context_on_visible_ = true;
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }

    /**
     * This method causes the Context Off button to be shown. If the button is
     * already shown, nothing happens. Otherwise the Button for Context On is
     * removed and the Button for Context Off is shown.
     */
    protected void drawButtonContextOff() {
        if (!context_on_visible_)
            return;
        try {
            browser_frame_rmi_.removeUIElement(new UIAction(
                    EUIElement.UI_BROWSER_MATCH_ON,
                    EUIContext.UI_CONTEXT_BROWSER, user_language_));

            browser_frame_rmi_.addUIElement(new UIAction(
                    EUIElement.UI_BROWSER_MATCH_OFF,
                    EUIContext.UI_CONTEXT_BROWSER, user_language_));
            context_on_visible_ = false;
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }

    /**
     * This method takes the current context and asks the active Worksheet to
     * take it as new context for the worksheet. The <ToWorksheet> button is
     * only visible if there is an active Worksheet and if the calcHead is
     * opened.
     */
    protected void setCurrentContextToWorksheet() {
        try {
            IContextProvider context_provider = session_.getContextProvider();
            if (context_provider == null)
                return;
            context_provider.setContext(current_context_);
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }

    /**
     * This method checks the received context by use of the checkContext method
     * of the IContextProvider interface. This interface returns again a
     * context, which has to be displayed.
     * 
     * @param context
     *            the received context which has to be checked.
     */
    public void sendCheckedContextToBrowser(Context context) {
        // real implementation:

        Context new_context = null;
        try {
            IContextProvider context_provider = session_.getContextProvider();
            if (context_provider == null)
                new_context = new ContextUnknown();
            else
                new_context = context_provider.checkContext(context);
        } catch (RemoteException e) {
            e.printStackTrace();
        }
        if (new_context instanceof ContextUnknown) {
            new_context.setHierarchyKey(context.getHierarchyKey());
            new_context.setKEStoreKey(context.getKEStoreKey());
        }
        sendContextToBrowser(new_context);
        sendLinkToMiniBrowser(context, context_type_, "");

        // test implementation:
        /*
         * Context new_context = null; Random a = new Random(); int rand =
         * a.nextInt(5); if (rand > 2) { new_context = new ContextUnknown();
         * new_context.setHierarchyKey(context.getHierarchyKey());
         * new_context.setHierarchyNode(context.getHierarchyNode()); } else
         * new_context = context;
         * 
         * sendContextToBrowser(new_context);
         * sendLinkToMiniBrowser(context,context_type_);
         */
    }

    public void updateState() {
        // nothing to be done here, implementation is done
        // in derived classes if necessary
    }

    public ILMService getCompod() {
        return compod_;
    }
    
    /**
     * Generate a string from the selected Path in the chosen Browser(e.g.: ExampleBrowser)
     * @param context from the Browser
     * @return string of the selected Path
     */
     private String generateSelectedPath(Context context)
    {
      String selected_path = null;
              
      HierarchyNode hierarchy_node = hierarchy_.getNodeForKEStoreKey(context.getKEStoreKey());
      HierarchyNode node = hierarchy_node;
      while( node.getParent() != null ) 
      {
        node = node.getParent();
        if(selected_path == null)
          selected_path = node.toString();
        else
          selected_path = node.toString() + "\\"+ selected_path; 
      }
      return selected_path;
    }
   
    /**
      * Generate the LogItem and deliver to the DialogGuide
      * @param file_name name of the selected File in the browser
      * @param selected_path path to the selected File
      * @param dialog_type Type of the used Browser e.g.: Example, Method and so on
      */ 
    private void generateAndSaveLogItem(String file_name, String selected_path,
            DialogTypes dialog_type)throws RemoteException{
  
      User user = UserManager.getInstance().getUser(session_.getId());
      log_Item_ = new LogItem(user.getUsername(), session_.getId(),"", 
        dialog_type, file_name, selected_path, null,"", 
        EUIElement.UI_BROWSER_CHANGE_CONTEXT );
      
      dialog_.writeToDB(log_Item_);
    }
}
