/*
 * @author Robert Koenighofer, member of the ISAC-team, 
 * Copyright (c) 2006 by Robert Koenighofer
 * created Jul 11, 2006 10:09:13 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */

package isac.browserdialog;

import java.rmi.RemoteException;
import java.rmi.Remote;
import isac.wsdialog.IWSDialogManager;

/**
 * This is an interface to get the browser dialogs. It is implemented
 * by the Session, and returns the desired browser dialog of the session.
 * 
 * @see isac.session.Session
 * @author Robert Koenighofer
 */
public interface IBDialogManager extends Remote{

	/**
	 * returns an interface to the problem dialog of the session
	 * 
	 * @return an interface to the problem dialog of the session
	 * @throws RemoteException if remote object is not available
	 * @see isac.session.Session#getProblemDialog()
	 */
	public IBrowserDialog getProblemDialog() throws RemoteException;

	/**
	 * returns an interface to the method dialog of the session
	 * 
	 * @return an interface to the method dialog of the session
	 * @throws RemoteException if remote object is not available
	 * @see isac.session.Session#getMethodDialog()
	 */
	public IBrowserDialog getMethodDialog() throws RemoteException;

	/**
	 * returns an interface to the theory dialog of the session
	 * 
	 * @return an interface to the theory dialog of the session
	 * @throws RemoteException if remote object is not available
	 * @see isac.session.Session#getTheoryDialog()
	 */
	public IBrowserDialog getTheoryDialog() throws RemoteException;

	/**
	 * returns an interface to the example dialog of the session
	 * 
	 * @return an interface to the example dialog of the session
	 * @throws RemoteException if remote object is not available
	 * @see isac.session.Session#getExampleDialog()
	 */
	public IBrowserDialog getExampleDialog() throws RemoteException;
	
	/**
	 * returns an interface to the WSDialogManager of the session
	 * 
	 * @return an interface to the WSDialogManager of the session
	 * @throws RemoteException if remote object is not available
	 * @see isac.session.Session#getWSDialogManager()
	 */
	public IWSDialogManager getWSDialogManager() throws RemoteException;
	
	/**
	 * returns the id of the session
	 * 
	 * @return the id of the session
	 * @throws RemoteException if remote object is not available
	 * @see isac.session.Session#getId()
	 */
	public String getId() throws RemoteException;
    
     /**
     * Opens a worksheet from an example specified by the it's id. 
     * A new worksheet is opened by use of the WSDialogManager.
     * 
     * @param example the id of the example to open.
     * @return true if everything went ok, and false otherwise
     * @see ExampleDialog#openWorksheetFromExample(java.lang.String) 
     */
    public boolean openWorksheetFromExample(String example) throws RemoteException;
}	
