/*
 * @author Georg Kompacher, member of the ISAC-team, 
 * Copyright (c) 2006 by Georg Kompacher
 * created Jul 10, 2006 03:05:27 PM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.browserdialog;

import isac.interfaces.IToGUI;
import isac.useractions.IUserAction;
import isac.wsdialog.DialogProtocolException;

import java.io.Serializable;
import java.rmi.Remote;
import java.rmi.RemoteException;

/**
 * @author Andreas Griesmayer, JL, Georg Kompacher
 * 
 * Remote interface for methods of the BrowserDialog which are running on
 * different virtual machines and so they are called over RMI
 */
public interface IBrowserDialog extends Remote, Serializable {

	/**
	 * Notify the Dialog about events or requests triggered by the user
	 * 
	 * @param action
	 *            What's new from the UI
	 * @return true, if the event is being processed, false if processing is
	 *         denied or impossible
	 * @throws RemoteException
	 * @throws DialogProtocolException
	 */
	public boolean notifyUserAction(IUserAction action) throws RemoteException,
			DialogProtocolException;

	/**
	 * A BrowserDialog has to know his browser but the browser is made later
	 * than the dialog (in the WindowApplication). Therefore the browser has to
	 * be registered to the dialog.
	 * 
	 * @param browser_frame
	 *            browser to register
	 */
	public void registerBrowserFrame(IToGUI browser_frame)
			throws RemoteException;
}
