/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 					
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.gui;

import isac.gui.calcheadviews.CalcHeadView;
import isac.gui.calcheadviews.ModelPanel;
import isac.gui.calcheadviews.SpecificationPanel;
import isac.util.formulae.CalcHead;

import java.awt.BorderLayout;

import javax.swing.JPanel;


/**
 * Panel where the user can input/see a complete CalcHead with all its Elements.
 * 
 * @author Mario Hochreiter
 * @author Manuel Koschuch
 * @version $Revision: 1.44 $
 * 
 */
public class CalcHeadPanel extends JPanel {

    /**
     * Constant used for serialization (@see corresponding API Documentation)
     */
    private static final long serialVersionUID = 1L;

    public static final int TYPE_TRY_MATCH = 0;

    public static final int TYPE_TRY_REFINE = 1;

    /** changeable view to the data of the calchead */
    private CalcHeadView info_panel_;

    /** Panel holding the buttons */
    private JPanel button_panel_;

    /**
     * Creates a <code>CalcHeadPanel</code> with the given
     * <code>CalcHead</code>. The displayed detail level of the
     * <code>CalcHead</code> is set through viewStyle.
     * 
     * @param locale
     *            Locale that is used to display the CalcHeadPanel strings.
     * @param ch
     *            CalcHead that should be displayed.
     */
    public CalcHeadPanel(CalcHead ch) {
        super();
        this.init(ch);
    }

    private void init(CalcHead ch) {
        info_panel_ = new CalcHeadView(ch);
        button_panel_ = new JPanel();
        this.layoutComponents(ch);
    }

    private void layoutComponents(CalcHead ch) {
        this.setLayout(new BorderLayout());
        this.add(info_panel_, BorderLayout.CENTER);
        this.add(button_panel_, BorderLayout.SOUTH);

    }

    /**
     * Fills the textfields from the supplied CalcHead
     * 
     * @param ch
     *            CalcHead to fill textfields from
     * 
     */

    public void fillTextFieldsFromCalcHead(CalcHead ch) {
        info_panel_.fillTextFieldsFromCalcHead(ch);
        info_panel_.validate();
        this.validate();

    }

    /**
     * Fills the supplied CalcHead with the contens of the textfields
     * 
     * 
     * @param ch
     *            CalcHead to be filled
     */

    public void fillCalcHeadFromTextFields(CalcHead ch) {
        info_panel_.fillCalcHeadFromTextFields(ch);
    }

    /**
     * Set new CalcHead
     * 
     * @param ch
     *            new CalcHead for this CalcHeadPanel
     * 
     */

    public void setCalcHead(CalcHead ch) {
        info_panel_.fillTextFieldsFromCalcHead(ch);
        info_panel_.validate();
        this.validate();
    }

    /**
     * Clears all textfields
     */

    public void reset() {
        info_panel_.reset();
        info_panel_.validate();
        this.validate();
    }

    public JPanel getButtonPanel() {
        return button_panel_;
    }

    /**
     * Fills Textfields stepwise
     * 
     * @param ch
     *            CalcHead to fill TextFields from
     * @return false, when Textfields left to fill, true otherwise
     * @deprecated was misunderstanding
     */

    public boolean next(CalcHead ch) {
        return info_panel_.next(ch);
    }

    public ModelPanel getModelPanel() {
        return info_panel_.getModelPanel();
    }

    public SpecificationPanel getSpecificationPanel() {
        return info_panel_.getSpecificationPanel();
    }
}