/*
 * @author Walther Neuper, member of the ISAC-team, 
 * Copyright (c) 2007 by Walther Neuper
 * created Jul 18, 2007 3:34:54 PM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.gui.browser;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JInternalFrame;


import isac.browserdialog.IBrowserDialog;
import isac.interfaces.IToGUI;
import isac.useractions.IUIAction;
import isac.useractions.IUserAction;
import isac.util.formulae.Context;
import isac.util.formulae.ContextUnknown;
import isac.wsdialog.DialogProtocolException;

/**
 * 
 * This class represents the window of the Browser. It is the main dispatch
 * point for all Swing actions that happen in various sub-elements (MiniBrowser,
 * Buttons) as well as the main dispatch point where all UIActions land and get
 * dispatched according to thier context.
 * 
 * The class implements following interfaces
 * 
 * The class implements notifyBrowserDialog() method that sends the action to
 * its parent dialog
 * 
 * The class extends JInternalFrame as long as there is a Swing-front-end
 */
public class BrowserFrame extends JInternalFrame implements Serializable,
        IToGUI {

    static final long serialVersionUID = -1817863354068154971L;

    /** Link to the only appropriate dialog */
    protected IBrowserDialog browser_dialog_;

    /** Current MathContext to one worksheet */
    protected Context current_context_;

    /** Clients interested in change of the context to worksheet */
    protected Vector<IContextChangeListener> context_change_listeners_;

    /**
     * @param bdialog
     *            is ExampleDialog | TheoryDialog | Pbl | Met the only
     *            difference between the instances
     * @param title
     *            for BrowserFrame
     * 
     */
    public BrowserFrame(IBrowserDialog bdialog, String title) {
        super(title, true, true, false, false);
        browser_dialog_ = bdialog;
        // / setup the change listeners
        context_change_listeners_ = new Vector<IContextChangeListener>();
        current_context_ = new ContextUnknown();
        notifyContextChangeListeners();

    }

    /**
     * TODOWN070726
     * 
     * WARNING: hack for isacweb ...
     * @deprecated public use preliminary for isacweb: should be protected.
     * 
     * @see isac.stubs.Tab#Tab(String, BrowserFrameWeb, BrowserFrameWeb,
     *      BrowserFrameWeb, BrowserFrameWeb)
     * 
     * ... made PUBLIC for quick display of calculation (while creating
     * WorksheetSwing and not using it)
     * 
     * Description of the method in HTML format
     * @param <paramname>
     *            <paramdescription>
     * @return <return value>
     * @exception <exception>
     *                <description when it is thrown>
     * @see <fully.qualified.Classname#methodName(param-classes)>
     */
    /* FIXXME.WN070726 hack protected END hack.WN070726 */public IBrowserDialog getBrowserDialog() {
        return browser_dialog_;
    }

    /**
     * @param current_context_
     *            The current_context_ to set.
     */
    public void setCurrentContext_(Context current_context) {
        this.current_context_ = current_context;
    }

    /**
     * Method inherited from IToGUI interface
     * 
     * In the case of the BrowserFrame, the method should add the button
     * containing the UserAction to the main menu bar.
     * 
     * must be implemented accordig to GUI
     */
    public void addUIElement(IUIAction action) throws RemoteException {
    }

    /**
     * Method inherited from IToGUI interface
     * 
     * Performs the dispatching of the UIAction. This method is called by the
     * appropriate Dialog over RMI.
     * 
     * @param action -
     *            the action to perform. The BrowserFrame subclasses can
     *            implement following actions :
     * 
     * UIAction: UI_BROWSER_AUTO_MATCH_ON UI_BROWSER_AUTO_MATCH_OFF
     * 
     * UIActionOnMathContext: UI_BROWSER_CHANGE_CONTEXT
     * 
     * UIActionOnLink: UI_MINBROWSER_LINK
     * 
     * With the following contexts: UI_CONTEXT_HIERARCHY UI_CONTEXT_MINIBROWSER
     * UI_CONTEXT_FOCUS
     * 
     * The subclasses can call super.doUIAction( action ) to process the actions
     * with UI_CONTEXT_BROWSER context.
     * 
     * The following actions are implemented here:
     * 
     * UI_BROWSER_GET_FOCUS
     * 
     * must be implemented according to GUI
     * 
     */
    public void doUIAction(IUIAction action) throws RemoteException {
    }

    /**
     * Method inherited from IToGUI interface
     * 
     * ?deprecated in favour of isac.gui.MockBrowserFrame#doUIAction(IUIAction
     * action)
     * 
     * The UIActions and UserActions are different classes because we want to
     * easily distinguish the sender and receiver of the messages when looking
     * at the code. Otherwise there are no significant differences between them.
     * 
     * Dialog->Browser = UIAction; Browser->Dialog = UserAction
     * 
     */
    public boolean doUIAction(IUserAction action) throws RemoteException {
        return false;
    }

    /**
     * Method inherited from IToGUI interface
     * 
     * In the case of the BrowserFrame, the method should remove the button
     * containing the UserAction (if such button exists) from the main menu bar.
     * 
     * must be implemented according to GUI
     */
    public boolean removeUIElement(IUIAction action) throws RemoteException {
        return false;
    }

    public boolean isUIRunning() throws RemoteException {
        return true;
    }

    protected void notifyContextChangeListeners() {
        Iterator<IContextChangeListener> cclit = context_change_listeners_.iterator();
        while (cclit.hasNext()) {
            IContextChangeListener ccl = (IContextChangeListener) cclit.next();
            ccl.contextChanged(current_context_);
        }
    }

    /**
     * The method sends the UserAction to the parent BrowserDialog
     * 
     * @param action
     *            to send
     */
    public boolean notifyBrowserDialog(IUserAction action) {
        try {
            // / notify the parent
            return browser_dialog_.notifyUserAction(action);
        } catch (RemoteException remote_exception) {
            remote_exception.printStackTrace();
        } catch (DialogProtocolException dialog_protocol_exception) {
            dialog_protocol_exception.printStackTrace();
        }
        return false;
    }

    /**
     * Adds a context change listener
     */
    public void addContextChangeListener(IContextChangeListener client) {
        context_change_listeners_.add(client);
    }

    /**
     * Removes a context change listener
     */
    public void removeContextChangeListener(IContextChangeListener client) {
        context_change_listeners_.remove(client);
    }

    /**
     * @return Returns the current_context_.
     */
    public Context getCurrentContext() {
        return current_context_;
    }

}
