package isac.gui.browser.minibrowser;

import isac.gui.browser.BrowserFrameSwing;
import isac.gui.browser.IContextChangeListener;
import isac.useractions.EUIContext;
import isac.useractions.IUIAction;
import isac.useractions.IUserAction;
import isac.useractions.UIActionOnLink;
import isac.util.formulae.Context;

import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.rmi.RemoteException;

import javax.swing.JEditorPane;
import javax.swing.text.Document;
import org.apache.log4j.lf5.util.StreamUtils;

public class MiniBrowser extends JEditorPane implements IMiniBrowser, IContextChangeListener {

	static final long serialVersionUID = 6812817459482158572L;
	
	/**
	 * 
	 * The parent BrowserFrame is added as a HyperlinkListener to the 
	 * browser.
	 *  
	 */
	protected BrowserFrameSwing parent_;
	
	/**
	 * 
	 * The last page loaded by the browser
	 * 
	 */
	protected URL		   page_;
	
	/**
	 * The current math context
	 * 
	 */
	protected Context	   current_context_;
	
	/**
	 * 
	 * Shows whether the Browser has finished loading the page
	 * 
	 */
	protected boolean 		page_loaded_;
	
	/***
	 * 
	 * The MiniBrowserEditorKit used to present the Context tag
	 * 
	 */
	protected MiniBrowserEditorKit minibrowser_editor_kit_;
	
	/**
	 * Constructor sets up the Component and adds the parent as the HyperlinkListener
	 * 
	 * @param parent
	 */
	public MiniBrowser( BrowserFrameSwing parent ) {
		parent_ = parent;
		
		registerEditorKitForContentType( "text/html", "isac.gui.browser.minibrowser.MiniBrowserEditorKit" );
		
		minibrowser_editor_kit_ = new MiniBrowserEditorKit( this );
		setEditorKitForContentType( "text/html",  minibrowser_editor_kit_ ); 
		
		setEditorKit( minibrowser_editor_kit_ ); 
		
		setContentType( "text/html" );
        setEditable( false );
       
        addHyperlinkListener( parent_ );
        
        setVisible( true );
                
        parent_.addContextChangeListener( this );
	}
		
	/* (non-Javadoc)
	 * @see javax.swing.JEditorPane#setPage(java.lang.String)
	 */
	public void setPage(String arg0) {
		try {
			super.setPage(arg0);
		} catch (Exception e) {
			setText( "<h1><b>Error: Can not load the requested page </b></h1>" );
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.JEditorPane#setPage(java.net.URL)
	 */
	public void setPage(URL arg0) {
		try {
			if( getPage() != null && getPage().sameFile( arg0 ) )
				getDocument().putProperty(Document.StreamDescriptionProperty, null);
			super.setPage(arg0);
		} catch (Exception e) {
			setText( "<h1><b>Error: Can not load the requested page </b></h1>" );
		}
	}

	/**
	 * 
	 * Method inherited from IToGUI interface, but not really implemented here.
	 * 
	 */
	public void addUIElement(IUIAction action) throws RemoteException {
		System.out.println("In MiniBrowser::addUIElement(IUIAction a) : Action is " + action.getAction() + 
				", Context is : " + action.getContext() );
		System.exit(1);
	}

	/**
	 * 
	 * Method inherited from IToGUI interface.
	 * 
	 * Performs the UI actions that have UI_CONTEXT_CONTENT context.
	 * Mainly for seting the page URL.
	 * 
	 */		
	public void doUIAction(IUIAction action) throws RemoteException {
		System.out.println("In MiniBrowser::doUIAction(IUIAction a) : Action is " + action.getAction() + 
				", Context is : " + action.getContext() );
		
		if( action.getContext() != EUIContext.UI_CONTEXT_CONTENT )
			return;
		
		switch( action.getAction() ) {
		case UI_MINIBROWSER_LINK:
			UIActionOnLink uaol = (UIActionOnLink) action;
			System.out.println(", Link is : " + uaol.getLink() );
                        URLConnection ucon = null;
                        byte[] bytes = null;
                        try {
                            ucon = uaol.getLink().openConnection();
                            bytes = StreamUtils.getBytes(ucon.getInputStream());
                            new String(bytes);
                        } catch (IOException ex) {
                            System.out.println(ex.getMessage());
                        }
			setPage( uaol.getLink().toString() );
			break;
		default:
			System.out.println("Which witch watches which switch (in MiniBrowser::doUIAction()");
			break;
		}		

		return;		
	}

	/**
	 * 
	 * Method inherited from IToGUI interface, but not really implemented here.
	 * 
	 */
	public boolean doUIAction(IUserAction action) throws RemoteException {
		System.out.println("In MiniBrowser::doUIAction(IUserAction a) : Action is " + action.getActionID() + 
				", Context is : " + action.getContext() );
		System.exit(1);
		
		return false;
	}

	/**
	 * Method inherited from IToGUI interface, but not really implemented here.
	 */
	public boolean removeUIElement(IUIAction action) throws RemoteException {
		System.out.println("In MiniBrowser::removeUIElement(IUIAction a) : Action is " + action.getAction() + 
				", Context is : " + action.getContext() );
		System.exit(1);			
		return false;
	}

	/**
	 * The context has changed
	 */
	public void contextChanged(Context new_context) {
		current_context_ = new_context;
	}

	/**
	 * @return Returns the current_context_.
	 */
	public Context getCurrentContext() {
		return current_context_;
	}

	public boolean isUIRunning() throws RemoteException {
		return true;
	}

    /**
     * Copied from JEditorPane of jdk 1.5_22, since the code of jdk 1.6_025 didn't show the 
     * MiniBrowser, it blocked all AWT Events.
     * TODO: Change ISAC code to work with jdk 1.6 somehow
     * 
     * @param page
     * @return
     * @throws IOException 
     * @author thomas.zillinger
     */
    protected InputStream getStream(URL page) throws IOException {
	URLConnection conn = page.openConnection();
	if (conn instanceof HttpURLConnection) {
	    HttpURLConnection hconn = (HttpURLConnection) conn;
	    hconn.setInstanceFollowRedirects(false);
	    int response = hconn.getResponseCode();
	    boolean redirect = (response >= 300 && response <= 399);

	    /*
	     * In the case of a redirect, we want to actually change the URL
	     * that was input to the new, redirected URL
	     */
	    if (redirect) {
		String loc = conn.getHeaderField("Location");
		if (loc.startsWith("http", 0)) {
		    page = new URL(loc);
		} else {
		    page = new URL(page, loc);
		}
		return getStream(page);
	    }
	}

	String type = conn.getContentType();
	if (type != null) {
	    setContentType(type);
	    getDocument().putProperty("content-type", type);
	}
	getDocument().putProperty(Document.StreamDescriptionProperty, page);
	String enc = conn.getContentEncoding();
	if (enc != null) {
	    getDocument().putProperty("content-encoding", enc);
	}
	InputStream in = conn.getInputStream();
	return in;
    }   
    
}
