package isac.gui.mawen.editor

import java.awt.Rectangle
import java.awt.Graphics
import isac.gui.mawen.syntax.Ast
import isac.gui.mawen.syntax.Ast._

/**
 * Holds all data required to draw the <code>Ast</code>.
 * The "Empty*Box"es initalise respective data before calculation.
 */
object Box {

  sealed abstract class Box(val ast: Ast)
  class EmptyBox(val level: Int, val x0: Int, val y0: Int, override val ast: Ast) extends Box(ast) {
    def apply(level: Int, x0: Int, y0: Int, ast: Ast) = new EmptyBox(level, x0, y0, ast)
    def toEmptyBox() = new EmptyBox(level, x0, y0, ast)
    def toEmptyBox(ast: Ast) = new EmptyBox(level, x0, y0, ast)
    def toEmptyBox(x: Int, y: Int, ast: Ast) = new EmptyBox(level, x, y, ast)
  }
  case class EmptyLeafOpBox(override val level: Int, override val x0: Int, override val y0: Int, override val ast: Ast) extends EmptyBox(level, x0, y0, ast)
  case class EmptyUniOpBox(override val level: Int, override val x0: Int, override val y0: Int, param: Ast, override val ast: Ast) extends EmptyBox(level, x0, y0, ast)
  case class EmptyBinOpBox(override val level: Int, override val x0: Int, override val y0: Int, operator: Ast, paraml: Ast, paramr: Ast, override val ast: Ast) extends EmptyBox(level, x0, y0, ast)

  case class DrawBox(name: String, val level: Int, val x: Int, val y: Int, val width: Int, val height: Int, x0: Int, y0: Int, override val ast: Ast) extends Box(ast) {
    var children: List[DrawBox] = List.empty[DrawBox]
    var isBoxed: Boolean = false
    var colorCode: Int = 0
    var boxedElement: Int = 0
  }
  def isLeaf(b: DrawBox): Boolean = b.children.length == 0

  def Contains(parent: DrawBox, box: DrawBox): Boolean =
    box == parent || parent.children.exists(y => Contains(y, box))

  /*
   * draw a box around an <code>Ast</code>
   */
  def draw(b: DrawBox, g: Graphics): Unit = draw(b, g, true) //save a param.
  def draw(b: DrawBox, g: Graphics, withChildren: Boolean): Unit = {
    g.drawRect(b.x, b.y - b.height, b.width, b.height)
    if (withChildren) b.children.foreach(draw(_, g))
  }
  // draw a box around an <code>Ast</code>
  def drawFrame(b: DrawBox, g: Graphics) {
    if (b.isBoxed) {
      g.setColor(Settings.CollorMapping(b.colorCode))
      g.drawRect(b.x, b.y - b.height, b.width, b.height)
      g.setColor(java.awt.Color.BLACK)
    }
  }

  def BoxIn(b: DrawBox, rect: Rectangle): Boolean =
    rect.contains(b.x, b.y) || rect.contains(b.x + b.width, b.y) ||
      rect.contains(b.x, b.y + b.height) || rect.contains(b.x + b.width, b.y + b.height)
}