package isac.gui.mawen.editor

import isac.gui.mawen.editor.Box._
import isac.gui.mawen.syntax.Ast._
import isac.gui.mawen.syntax.Ast
import isac.gui.mawen.syntax.XSyntax
import java.awt.Graphics
import java.awt.Color
import java.awt.Dimension
import java.awt.Font
import java.awt.Graphics2D
import javax.swing.JButton
import javax.swing.JTextField

/**
 * <code>paint</code>s a <code>Box</code> (in the sense of LaTeX)
 * according to a <code>Box.name</code>, which refers to
 * specific layouts defined in <code>CalcUtil</code>.
 */
object BoxUtil {

  def draw(box : DrawBox, g: Graphics) : Unit = box.name match {
    case "GAP"                         => drawGap(g, box) 
    case "CURSOR"                      => drawCursor(g, box) 
    case "Fields.inverse_class.divide" => drawFraction(g, box) //TODOWN "/"
    case ""                            => {
      drawFrame(box, g)
      box.children.foreach( draw(_, g) )
    }
    case str if XSyntax.isa_ast(str) == "/" => drawFraction(g, box)
    case _ =>  {
      drawBoxContent(g, box)
      box.children.foreach( draw(_, g) )
    }
  }
  def drawGap(g: Graphics, box : DrawBox) {
    g.fillRoundRect(box.x, box.y- box.height, box.width, box.height, 15, 15)
    drawFrame(box, g)
  }
  def drawCursor(g: Graphics, box : DrawBox) = System.currentTimeMillis() match {
    case n if (n / 250) % 2 == 0  => box.ast match {
      case Appl(List(Constant("CURSOR"), a)) => {
        g.setColor(Color.RED)
        g.asInstanceOf[Graphics2D].drawLine(box.x, box.y, box.x + box.width , box.y)
        g.asInstanceOf[Graphics2D].drawLine(box.x, box.y, box.x, box.y - box.height)

        g.setColor(Color.BLACK)
        if (box.children.length >= 1) draw(box.children.head, g)
      }
      case Appl(List(a, Constant("CURSOR"))) => {
        g.setColor(Color.RED)
        g.asInstanceOf[Graphics2D].drawLine(box.x, box.y, box.x + box.width , box.y)
        g.asInstanceOf[Graphics2D].drawLine(box.x+ box.width, box.y, box.x + box.width, box.y - box.height)
        
        g.setColor(Color.BLACK)
        if (box.children.length >= 1)
          draw(box.children.head, g)
      }
    }
    case _ => {
      if (box.children.length >= 1)
          draw(box.children.head, g)
    }
  }
  // different from <code>drawBoxContent</code> because of additional <code>drawLine</code>
  def drawFraction(g: Graphics, box : DrawBox) {
    g.drawLine(box.x, box.y, box.x + box.width, box.y)
    drawFrame(box, g)
  }
  // draw a box, except <code>drawFraction</code>
  def drawBoxContent(g: Graphics, box : DrawBox) {
    // draw operators and parentheses
    XSyntax.ast_const.get(box.name) match {
      case Some((_,_,_,_, fontname, charIndex)) => {
        // Operator
        val font_var = new Font(fontname, Font.PLAIN,  CalcUtil.fontsizeOf(box.level))
        g.setFont(font_var)
        g.drawString(charIndex.toChar.toString(), box.x0, box.y0)
        drawFrame(box, g)
      }
      case None => {
        // draw numbers and identifiers
        if (box.name.forall(c => EditingUtil.isNumber(c)) ) {
          val font_var = new Font("AR PL KaitiM Big5", Font.PLAIN,  CalcUtil.fontsizeOf(box.level))
          g.setFont(font_var)
        } else if (box.name.forall(c => EditingUtil.isNumber(c) || EditingUtil.isIdentifier(c)) ) {
          val font_var = new Font("CMMI12", Font.PLAIN,  CalcUtil.fontsizeOf(box.level))
          g.setFont(font_var)
        }else if (box.name.forall(c => EditingUtil.isDelim(c))){
          val font_var = new Font("CMCSC8", Font.PLAIN,  CalcUtil.fontsizeOf(box.level))
          g.setFont(font_var)
        }else {
          val font_var = new Font("PTF55F", Font.PLAIN,  CalcUtil.fontsizeOf(box.level))
          g.setFont(font_var)
        }
        g.drawString(box.name, box.x0, box.y0)
        drawFrame(box, g)
      }
    }
  }
  
}