package isac.gui.mawen.editor

import isac.gui.mawen.syntax.XSyntax
import isac.gui.mawen.syntax.Ast._
import isac.util.WindowApplicationPaths

import java.awt.Font
import java.awt.Graphics2D
import java.awt.Graphics
import java.awt.Rectangle
import java.io.InputStream
import java.util.Properties
import scala.io.Source

/**
 * Call preset properties for editor and store LaTeX settings.
 */
object Settings {
  
  var g: Graphics = null
  var drawBoxesOnMouseMove = true;
  var (isdraggingAllowed:Boolean, isZoomAllowed:Boolean, zoom:Float,translateX:Int,translateY:Int) =
    try {
      val prop = new Properties()
      var url = Source.fromURL(getClass.getResource("/properties/Editor.properties"));
      if (url != null) {
        prop.load(url.bufferedReader())
        (
          prop.getProperty("IS_DRAG_ALLOWED").toBoolean,
          prop.getProperty("IS_ZOOM_ALLOWED").toBoolean,
          prop.getProperty("START_ZOOM_FACTOR").toFloat,
          prop.getProperty("START_X").toInt,
          prop.getProperty("START_Y").toInt
        )
      }
    } catch { case e: Exception => 
      println("Error loading Property file: " + e.getMessage())
    }


  def LARGESPACE = ((( 50f / 100f ) * INITIALFONTSIZE) ).round
  def MEDIUMSPACE = ((( 30f / 100f ) * INITIALFONTSIZE) ).round
  def FRACVERTIKALSPACE = (( 10f / 100f ) * INITIALFONTSIZE).round
  
  def SpaceOf(style: Char) = style match {
    case 'L' => LARGESPACE
    case 'M' => MEDIUMSPACE
    case 'F' => FRACVERTIKALSPACE
  }
  val INITIALFONTSIZE = 25
  val POWHORIZONTALSPACE = (INITIALFONTSIZE / 100f * 8f).round
  var ast_Stringbounds = Map.empty[String, (Rectangle, Rectangle, Rectangle)]
  
  def getStringBoundOf(key: String, level: Int) : Rectangle = {
    if (key == "" || key == "u"  || key == "bdv" ) {
      return new Rectangle(0,0,50,50);
    }
    ast_Stringbounds(key).productElement(Math.min(2, level)).asInstanceOf[Rectangle].clone().asInstanceOf[Rectangle]
  }
  def getStringBoundOf(key: String, level: Int, dx: Int, dy: Int) : Rectangle =  {
    val r = getStringBoundOf(key, level)
    r.translate(dx, dy)
    return r;
  }
  def fillStringbounds(ast: Ast, g: Graphics2D) {
    fillStringboundsRec(Variable(",x"), g)
    fillStringboundsRec(Variable(")"), g)
    fillStringboundsRec(Variable("i)"), g)
    fillStringboundsRec(Variable("(i"), g)
    fillStringboundsRec(Variable("xx"), g)
    fillStringboundsRec(ast, g)
  }
  def fillStringboundsRec(ast: Ast, g: Graphics2D) : Unit = ast match {
    case Constant(str) => {
      val op = XSyntax.isa_ast(str)
      g.setFont(new Font("CMCSC8", Font.PLAIN, CalcUtil.fontsizeOf(0)))
      val RectLv0 = getStringBounds(g, op, 0f,0f)
      g.setFont(new Font("CMCSC8", Font.PLAIN, CalcUtil.fontsizeOf(1)))
      val RectLv1 = getStringBounds(g, op, 0f,0f)
      g.setFont(new Font("CMCSC8", Font.PLAIN, CalcUtil.fontsizeOf(2)))
      val RectLv2 = getStringBounds(g, op, 0f,0f)
      ast_Stringbounds = ast_Stringbounds + (op -> (RectLv0, RectLv1, RectLv2))
    }
    case Variable(str) => {
     
      if (str.forall(c => EditingUtil.isNumber(c))){
        g.setFont(new Font("AR PL KaitiM Big5", Font.PLAIN, CalcUtil.fontsizeOf(0)))
        val RectLv0 = getStringBounds(g, str, 0f,0f)
        g.setFont(new Font("AR PL KaitiM Big5", Font.PLAIN, CalcUtil.fontsizeOf(1)))
        val RectLv1 = getStringBounds(g, str, 0f,0f)
        g.setFont(new Font("AR PL KaitiM Big5", Font.PLAIN, CalcUtil.fontsizeOf(2)))
        val RectLv2 = getStringBounds(g, str, 0f,0f)
        ast_Stringbounds = ast_Stringbounds + (str -> (RectLv0, RectLv1, RectLv2))
      } else if (str.forall(c => EditingUtil.isIdentifier(c))) {
        g.setFont(new Font("CMMI12", Font.PLAIN, CalcUtil.fontsizeOf(0)))
        val RectLv0 = getStringBounds(g, str, 0f,0f)
        g.setFont(new Font("CMMI12", Font.PLAIN, CalcUtil.fontsizeOf(1)))
        val RectLv1 = getStringBounds(g, str, 0f,0f)
        g.setFont(new Font("CMMI12", Font.PLAIN, CalcUtil.fontsizeOf(2)))
        val RectLv2 = getStringBounds(g, str, 0f,0f)
        ast_Stringbounds = ast_Stringbounds + (str -> (RectLv0, RectLv1, RectLv2))
      } else if (str.forall(c => EditingUtil.isDelim(c))) {
        g.setFont(new Font("CMCSC8", Font.PLAIN, CalcUtil.fontsizeOf(0)))
        val RectLv0 = getStringBounds(g, str, 0f,0f)
        g.setFont(new Font("CMCSC8", Font.PLAIN, CalcUtil.fontsizeOf(1)))
        val RectLv1 = getStringBounds(g, str, 0f,0f)
        g.setFont(new Font("CMCSC8", Font.PLAIN, CalcUtil.fontsizeOf(2)))
        val RectLv2 = getStringBounds(g, str, 0f,0f)
        ast_Stringbounds = ast_Stringbounds + (str -> (RectLv0, RectLv1, RectLv2))
      } else {
        g.setFont(new Font("PTF55F", Font.PLAIN, CalcUtil.fontsizeOf(0)))
        val RectLv0 = getStringBounds(g, str, 0f,0f)
        g.setFont(new Font("PTF55F", Font.PLAIN, CalcUtil.fontsizeOf(1)))
        val RectLv1 = getStringBounds(g, str, 0f,0f)
        g.setFont(new Font("PTF55F", Font.PLAIN, CalcUtil.fontsizeOf(2)))
        val RectLv2 = getStringBounds(g, str, 0f,0f)
        ast_Stringbounds = ast_Stringbounds + (str -> (RectLv0, RectLv1, RectLv2))
      }
    }
    case Appl(asts) => asts.foreach(x => fillStringbounds(x, g))
  }
  def getStringBounds (g2: Graphics2D, str: String , x: Float , y: Float ) : Rectangle = {
    val frc = g2.getFontRenderContext()
    val gv = g2.getFont().createGlyphVector(frc, str)
    val rect : Rectangle = gv.getPixelBounds(null, x, y)
    val norm_rect = new Rectangle((rect.x / Settings.zoom).round, (rect.y / Settings.zoom).round, (rect.width / Settings.zoom).round, (rect.height / Settings.zoom).round)
    new Rectangle(norm_rect.x, norm_rect.y, norm_rect.width, norm_rect.height)
  }
  // brighter colors are not easily visible
  val CollorMapping: Map[Int, java.awt.Color] =  Map(
    0 -> java.awt.Color.BLACK, 
    1 -> java.awt.Color.BLACK, 
    2 -> java.awt.Color.BLUE,
    3 -> java.awt.Color.MAGENTA,
    4 -> java.awt.Color.RED,
    5 -> java.awt.Color.DARK_GRAY,
    6 -> java.awt.Color.GREEN,
    7 -> java.awt.Color.LIGHT_GRAY,
    8 -> java.awt.Color.CYAN,
    9 -> java.awt.Color.ORANGE,
    10 -> java.awt.Color.PINK,
    11 -> java.awt.Color.GRAY,
    12 -> java.awt.Color.YELLOW,
    13 -> new java.awt.Color(128,64,0))
  val layout = Map (
    "=" -> ("binOp","L=L",   Appl(List(Constant("HOL.eq"),   Constant("GAP"), Constant("GAP")))),
    "+" -> ("binOp","M+M",   Appl(List(Constant("Groups.plus_class.plus"),   Constant("GAP"), Constant("GAP")))),
    "-" -> ("binOp","M-M",   Appl(List(Constant("Groups.minus_class.minus"),   Constant("GAP"), Constant("GAP")))),
    "*" -> ("binOp","M*M",   Appl(List(Constant("Groups.times_class.times"),   Constant("GAP"), Constant("GAP")))),
    "/" -> ("Frac","",       Appl(List(Constant("Fields.inverse_class.divide"),   Constant("GAP"), Constant("GAP")))),
    "^" -> ("Pow","",        Appl(List(Constant("Atools.pow"),   Constant("GAP"), Constant("GAP")))),
    "sin" -> ("UniOp","",    Appl(List(Constant("Transcendental.sin"), Constant("GAP")))),
    "cos" -> ("UniOp","",    Appl(List(Constant("Transcendental.cos"), Constant("GAP"), Constant("GAP")))),
    "tan" -> ("UniOp","",    Appl(List(Constant("Transcendental.tan"), Constant("GAP"), Constant("GAP")))),
    "Integral" -> ("Int","", Appl(List(Constant("Integrate.Integral"), Constant("GAP"), Constant("GAP"), Constant("GAP"), Constant("GAP"))))
  )
  def layoutName(str: String): String = { 
    layout.get(str) match {
      case Some((str, _,_)) => str 
      case None => str
    }
  }
  def layoutStyle(str: String): String = { 
    layout.get(str) match {
      case Some((_, str,_)) => str 
      case None => str
    }
  }
  def shortOpToIsaOp(o: String) : Ast = layout.get(o) match {
    case Some((_, _,a)) => a 
    case None => Appl(List(Constant(o), Constant("GAP")))
  }
}