package isac.gui.mawen.editor

import isac.gui.mawen.syntax.Ast._
import isac.gui.mawen.syntax.AstException

/**
 * Handle transformation on the <code>Ast</code> structure
 * triggered by input of operators (i.e. +, /, etc) and
 * by input of parentheses.
 */
object TransformAstUtil {
  
  def AstToCursor(ast : Ast) = ast match {
    case Constant("GAP")      =>  Appl(List(Constant("CURSOR"), ast))
    case Variable(str)        =>  Appl(List(ast, Constant("CURSOR")))
    case Constant(str)        =>  Appl(List(ast, Constant("CURSOR")))
    case Appl(asts)           =>  Appl(List(ast, Constant("CURSOR")))
  }
  def ReplaceCursor(newCursor: Ast)(ast: Ast) = ast match {
    case  Appl(List(Constant("CURSOR"), a)) => newCursor
    case  Appl(List(a, Constant("CURSOR"))) => newCursor
  }
  def CursorToAst(ast : Ast) =  ast match {
    case Appl(List(Constant("CURSOR"), a)) => a
    case Appl(List(a, Constant("CURSOR"))) => a
    case _ =>  throw new AstException("CursorToAst, Cursor has a false structure(required: Appl(List(Constant(\"CURSOR\"), ast), get: " + raw_string_of(ast) + ")")
  }
  def Delete(ast : Ast) = Constant("GAP")
  def Box(ast: Ast) = Appl(List(Constant("BOX.MAIN.0"), ast))
  
  def UnBox(ast: Ast) = ast match {
     case Appl(List(Constant(str), a)) if str.startsWith("BOX") => a
     case _ =>  throw new AstException("UnBox, Box has a false structure(required: Appl(List(Constant(\"BOX.1\"), ast), get: " + raw_string_of(ast) + ")")
  }
  def BoxNextChild(c: AstContainer) = {
     AstInfoUtil.FindBox(c.getAst()) match {
       case None => {}
       case Some(boxAst) => {// check if there is a box
         val boxAstElem  = TransformAstUtil.UnBox(boxAst) // get ast from Appl(Cursor, ast)
         val unboxedroot = TransformAstUtil.Update(c.getAst(), boxAst, TransformAstUtil.UnBox) // remove Box from Root Ast
         AstInfoUtil.Parent(unboxedroot, boxAstElem) match {
           case Some(parentAst) => c.setAst(TransformAstUtil.Update(unboxedroot, AstInfoUtil.NextChild(parentAst, boxAstElem), TransformAstUtil.Box))
           case None => {}
         }
       }
     }
  }
  def BoxPrevChild(c: AstContainer) =  AstInfoUtil.FindBox(c.getAst()) match {
    case None => {}
    case Some(boxAst) => {
       val boxAstElem  = TransformAstUtil.UnBox(boxAst)
       val unboxedAst = TransformAstUtil.Update(c.getAst(), boxAst, TransformAstUtil.UnBox)
       AstInfoUtil.Parent(unboxedAst, boxAstElem) match {
         case Some(parentAst) => c.setAst(TransformAstUtil.Update(unboxedAst, AstInfoUtil.PrevChild(parentAst, boxAstElem), TransformAstUtil.Box))
         case None => {}
       }
     }
  }
  def BoxParent(c: AstContainer) = AstInfoUtil.FindBox(c.getAst()) match{
    case None => {}
    case Some(boxAst) =>  {// check if there is a box
       val boxAstElem  = TransformAstUtil.UnBox(boxAst)
       val unboxedAst = TransformAstUtil.Update(c.getAst(), boxAst, TransformAstUtil.UnBox)
       AstInfoUtil.Parent(unboxedAst, boxAstElem) match {
         case Some(parentAst) => c.setAst(TransformAstUtil.Update(unboxedAst, parentAst, TransformAstUtil.Box))
         case None => {}
       }
     }
    
  }
  def BoxChild(c: AstContainer) = {
     AstInfoUtil.FindBox(c.getAst()) match {
       case None => {}
       case Some(boxAst) => {
         val boxAstElem  = TransformAstUtil.UnBox(boxAst)
         if (AstInfoUtil.FirstChild(boxAstElem) != null) {
           val unboxedAst = TransformAstUtil.Update(c.getAst(), boxAst, TransformAstUtil.UnBox)
            AstInfoUtil.FirstChild(boxAstElem) match {
             case Some(a) => c.setAst(TransformAstUtil.Update(unboxedAst, a, TransformAstUtil.Box))
             case None => {}
           }
           
         }
       }
     }
    
       
     
  }
  def CursorNextChild(c: AstContainer) = {
     val boxAst = AstInfoUtil.FindCursor(c.getAst())
     if (boxAst != null) {
       val boxAstElem  = TransformAstUtil.CursorToAst(boxAst) // get ast from Appl(Cursor, ast)
       val unboxedroot = TransformAstUtil.Update(c.getAst(), boxAst, TransformAstUtil.CursorToAst) // remove Box from Root Ast
       AstInfoUtil.Parent(unboxedroot, boxAstElem) match {
         case Some(parentAst) => c.setAst(TransformAstUtil.Update(unboxedroot, AstInfoUtil.NextChild(parentAst, boxAstElem), TransformAstUtil.AstToCursor))
         case None => {}
       }
       
     }
  }
  def CursorPrevChild(c: AstContainer) = {
     val boxAst = AstInfoUtil.FindCursor(c.getAst())
     if (boxAst != null) {// check if there is a box
       val boxAstElem  = TransformAstUtil.CursorToAst(boxAst)
       val unboxedAst = TransformAstUtil.Update(c.getAst(), boxAst, TransformAstUtil.CursorToAst)
       AstInfoUtil.Parent(unboxedAst, boxAstElem) match {
         case Some(parentAst) => c.setAst(TransformAstUtil.Update(unboxedAst, AstInfoUtil.PrevChild(parentAst, boxAstElem), TransformAstUtil.AstToCursor))
         case None => {}
       }
       
     }
  }
  def CursorParent(c: AstContainer) = {
     val boxAst = AstInfoUtil.FindCursor(c.getAst())
     if (boxAst != null) {// check if there is a box
       val boxAstElem  = TransformAstUtil.CursorToAst(boxAst)
       val unboxedAst = TransformAstUtil.Update(c.getAst(), boxAst, TransformAstUtil.CursorToAst)
       AstInfoUtil.Parent(unboxedAst, boxAstElem) match {
         case Some(parentAst) => c.setAst(TransformAstUtil.Update(unboxedAst, parentAst, TransformAstUtil.AstToCursor))
         case None => {}
       }
     }
  }
  def CursorChild(c: AstContainer) = {
     val boxAst = AstInfoUtil.FindCursor(c.getAst())
     if (boxAst != null) { // check if there is a box
       val boxAstElem  = TransformAstUtil.CursorToAst(boxAst)
       if (AstInfoUtil.FirstChild(boxAstElem) != null) {
         val unboxedAst = TransformAstUtil.Update(c.getAst(), boxAst, TransformAstUtil.CursorToAst)
         AstInfoUtil.FirstChild(boxAstElem) match {
           case Some(a) => c.setAst(TransformAstUtil.Update(unboxedAst, a, TransformAstUtil.AstToCursor))
           case None => {}
         }
         
       }
     }
  }
  /* replace a sub-Ast with another Ast.
   * <code>eq</code> could be replaced by carrying around
   * an integer-list as key in <code>Ast.add</code>
   * as shown in <code>TestAst.test_update</code>.
   */
  def Update(root : Ast, replace : Ast, updatefunc: Ast => Ast ) : Ast = root match {
    case Appl(List(a))                                             if  a eq replace                              => Appl(List(updatefunc(replace)))
    case Appl(a :: param)                                          if  a eq replace                              => Appl(List(updatefunc(replace)) :::  param)
    case Appl(param :: a :: Nil)                                   if  a eq replace                              => Appl(param :: List(updatefunc(replace)) )
    case Appl(param2 :: param1 :: a :: Nil)                        if  a eq replace                              => Appl(param2 :: param1 :: List(updatefunc(replace)) )
    case Appl(param3 :: param2 :: param1 :: a :: Nil)              if  a eq replace                              => Appl(param3 :: param2 :: param1 :: List(updatefunc(replace)) )
    case Appl(param4 :: param3 :: param2 :: param1 :: a :: Nil)    if  a eq replace                              => Appl(param4 :: param3 :: param2 :: param1 :: List(updatefunc(replace)) )
    case Appl(param1 :: a :: rparam)                               if  a eq replace                              => Appl(param1 :: List(updatefunc(replace)) ::: rparam )
    case Appl(param2 :: param1 :: a :: rparam)                     if  a eq replace                              => Appl(param2 :: param1 :: List(updatefunc(replace)) ::: rparam )
    case Appl(param3 :: param2 :: param1 :: a :: rparam)           if  a eq replace                              => Appl(param3 :: param2 :: param1 :: List(updatefunc(replace)) ::: rparam )
    case Appl(param4 :: param3 :: param2 :: param1 :: a :: rparam) if  a eq replace                              => Appl(param4 :: param3 :: param2 :: param1 :: List(updatefunc(replace)) ::: rparam )
    case a                                                         if  a eq replace                              => {
      return updatefunc(replace)
    }
    case Appl( a)                                                                                                => Appl(a.map(x => Update(x,replace, updatefunc) ))
    case a => {      
      return a
    }
  }
  
}