package isac.gui.mawen.termtree;
import java.util.Enumeration;
import java.util.concurrent.atomic.AtomicInteger;

import javax.sound.sampled.*;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;
/*
 * This class is needed to produce sounds for the acoustic model of mawen's termtree.
 * This class provides an auditive output of tree structure by pitches and repetition of sounds.
 * NOTE: Screen reader software is expected to be turned off (due to overlay).
 * 
 * @author NK 26072016
 * 
 */
public class SoundUtils {

  private static float SAMPLE_RATE = 8000f;
  
  //settings file
  public static final int SLEEP_INTERVAL = 20; //interval between nodes
  public static int SOUND_STEP = 30; //varying pitch of sound
  public static int BASE_SOUND = 280;
  public static int BEEP_PAUSE = 1; //breaks between node sounds (at repetition); is not affected when set too low  because of Hardware resolution
  /*
   * copied from StackOverflow
   * http://stackoverflow.com/questions/3780406/how-to-play-a-sound-alert-in-a-java-application
   */
  public static void tone(int hz, int msecs) 
     throws LineUnavailableException 
  {
     tone(hz, msecs, 1.0);
  }
 /*
  *copied from StackOverflow
  * http://stackoverflow.com/questions/3780406/how-to-play-a-sound-alert-in-a-java-application
  */
  public static void tone(int hz, int msecs, double vol)
      throws LineUnavailableException 
  {
    byte[] buf = new byte[1];
    AudioFormat af = 
        new AudioFormat(
            SAMPLE_RATE, // sampleRate
            8,           // sampleSizeInBits
            1,           // channels
            true,        // signed
            false);      // bigEndian
    SourceDataLine sdl = AudioSystem.getSourceDataLine(af);
    sdl.open(af);
    sdl.start();
    for (int i=0; i < msecs*8; i++) {
      double angle = i / (SAMPLE_RATE / hz) * 2.0 * Math.PI;
      buf[0] = (byte)(Math.sin(angle) * 127.0 * vol);
      sdl.write(buf,0,1);
    }
    sdl.drain();
    sdl.stop();
    sdl.close();
  }
  /*
   * copied from StackOverflow
   * http://stackoverflow.com/questions/3780406/how-to-play-a-sound-alert-in-a-java-application
   */
  public static void repeatTone(int hz, int msecs, int repeat) throws LineUnavailableException{
    for ( int i = repeat; i >0; i --){
      tone(hz, msecs); 
      try {
        Thread.sleep(BEEP_PAUSE);
      }catch (InterruptedException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }}
  }
  
  /*
   * term tree specific sound methods
   */
  
  //node structure downwards,getting higher ,count == depth, pitch == row
  public static void playNodeSound(DefaultMutableTreeNode n, JTree tree){
    if(n!=null && tree.isVisible(new TreePath(n.getPath()))){
      try {
      //find out depth n.getLevel();
      int row = tree.getRowForPath(new TreePath(n.getPath()));
      System.out.println("depth: " + n.getLevel() + " content: " + n.getUserObject() + " row:" + row);
        repeatTone(BASE_SOUND+(SOUND_STEP*row), 150, n.getLevel()+1);
        Thread.sleep(SLEEP_INTERVAL);
      //iterate through children
        Enumeration<DefaultMutableTreeNode>iter= n.children();
        while(iter.hasMoreElements()){
          DefaultMutableTreeNode current = iter.nextElement();
          if(current != null){
            playNodeSound(current,tree);
          }
        } 
      } catch (LineUnavailableException | InterruptedException e) {
        e.printStackTrace();
      }
    }
  }


  //node structure downwards, getting deeper, count == depth, pitch == row
  public static void playNodeSound2(DefaultMutableTreeNode n, JTree tree){
    playNodeSound2(n, tree, tree.getRowCount());
  }
  
  public static void playNodeSound2(DefaultMutableTreeNode n, JTree tree, int rowcount){
    if(n!=null && tree.isVisible(new TreePath(n.getPath()))){
      try {
      //find out depth n.getLevel();
      int row = tree.getRowForPath(new TreePath(n.getPath()));
      System.out.println("depth: " + n.getLevel() + " content: " + n.getUserObject() + " row:" + row + "/" +rowcount);
      System.out.println(BASE_SOUND+(SOUND_STEP*(rowcount - row)));
      repeatTone(BASE_SOUND+(SOUND_STEP*(rowcount - row)), 150, n.getLevel()+1);
      Thread.sleep(SLEEP_INTERVAL);
      //iterate through children
        Enumeration<DefaultMutableTreeNode>iter= n.children();
        while(iter.hasMoreElements()){
          DefaultMutableTreeNode current = iter.nextElement();
          if(current != null){
            playNodeSound2(current,tree,rowcount);
          }
        } 
      } catch (LineUnavailableException | InterruptedException e) {
        e.printStackTrace();
      }
    }
  }
  

  //play sound for current node, start high go deeper
  public static void playNodeSoundFixSingle(DefaultMutableTreeNode n, JTree tree){
    DefaultMutableTreeNode root =(DefaultMutableTreeNode) tree.getModel().getRoot(); 
    int maxRows = countNodes(root,tree,false);
    int curRow = getCurrentRowNumber(root, new AtomicInteger(0),n,tree).get();
    System.out.println(curRow  + "/"  + maxRows);
    
    try {
      //repeatTone(BASE_SOUND+(SOUND_STEP*(maxRows- curRow)), 150, n.getLevel()+1);
      //System.out.println(BASE_SOUND+(SOUND_STEP*(maxRows- curRow)));
      repeatTone(BASE_SOUND+(SOUND_STEP*curRow), 150, n.getLevel()+1);
      System.out.println(BASE_SOUND+(SOUND_STEP*curRow));
      
    } catch (LineUnavailableException e) {
      e.printStackTrace();
    }
    
  }
  //fix sound for each node
  public static void playNodeSoundfix(DefaultMutableTreeNode n, JTree tree){
    if(n!=null && tree.isVisible(new TreePath(n.getPath()))){
      playNodeSoundFixSingle(n, tree);
      try {
        Thread.sleep(SLEEP_INTERVAL);
      } catch (InterruptedException e) {
        e.printStackTrace();
      }
      //iterate through children
        Enumeration<DefaultMutableTreeNode>iter= n.children();
        while(iter.hasMoreElements()){
          DefaultMutableTreeNode current = iter.nextElement();
          if(current != null){
            playNodeSoundfix(current,tree);
          }
        } 
    }
  }  
  
  
  //usage of atomicInteger because int is passed by value everytime
  public static AtomicInteger getCurrentRowNumber (DefaultMutableTreeNode currentNode,AtomicInteger currentIndex, DefaultMutableTreeNode toSearch, JTree tree){
    if (currentNode == toSearch) return currentIndex;
    Enumeration<DefaultMutableTreeNode>iter= currentNode.children();
    while(iter.hasMoreElements()){
      DefaultMutableTreeNode current = iter.nextElement();
      if(current != null){
        currentIndex.incrementAndGet();
        AtomicInteger number = getCurrentRowNumber (current,currentIndex,toSearch,tree);
        if (number.get() >= 0) return number;
      }
    } 
    return new AtomicInteger(-1);
  }
  public static int countNodes(DefaultMutableTreeNode node,JTree tree,  boolean mustBeVisible){
    int count = 1;
    
    Enumeration<DefaultMutableTreeNode>iter= node.children();
    while(iter.hasMoreElements()){
      DefaultMutableTreeNode current = iter.nextElement();
      if(current != null && (!mustBeVisible || (mustBeVisible && tree.isVisible(new TreePath(current.getPath()))))){
        count += countNodes(current,tree,mustBeVisible);
      }
    } 
    return count;
  }

  //node structure downwards, getting deeper, count == how many children, pitch == depth, marker of location before clicks
  public static void playNodeSoundClicks(DefaultMutableTreeNode n, JTree tree){
    playNodeSoundClicks(n, tree, tree.getRowCount());
  }
  
  public static void playNodeSoundClicks(DefaultMutableTreeNode n, JTree tree, int rowcount){
    if(n!=null && tree.isVisible(new TreePath(n.getPath()))){
      try {
      TreePath selected = tree.getSelectionPath();
      if (selected != null && n == selected.getLastPathComponent()){
        //special tone
        repeatTone(BASE_SOUND+(SOUND_STEP*(n.getLevel())), 600,1);
      } else{
        repeatTone(BASE_SOUND+(SOUND_STEP*(n.getLevel())), 150,1);
      }
      int nodecount = countNodes(n, tree, false);
      System.out.println(nodecount);
      //if collapsed for each node a additional beep
      if (tree.isCollapsed(new TreePath(n.getPath()))){
        System.out.println("collpsed");
        repeatTone(BASE_SOUND+(SOUND_STEP*(n.getLevel())), 150,nodecount - 1);        
      }     
      Thread.sleep((long) (SLEEP_INTERVAL * 3));
      //iterate through children
        Enumeration<DefaultMutableTreeNode>iter= n.children();
        while(iter.hasMoreElements()){
          DefaultMutableTreeNode current = iter.nextElement();
          if(current != null){
            playNodeSoundClicks(current,tree,rowcount);
          }
        } 
      } catch (LineUnavailableException | InterruptedException e) {
        e.printStackTrace();
      }
    }
  }


  //node structure downwards, getting deeper, repetition == 1, pitch == depth, marker of location before clicks
  public static void playNodeSoundClicksBefore(DefaultMutableTreeNode n, JTree tree){
    playNodeSoundClicksBefore(n, tree, tree.getRowCount());
  }
  
  public static void playNodeSoundClicksBefore(DefaultMutableTreeNode n, JTree tree, int rowcount){
    if(n!=null && tree.isVisible(new TreePath(n.getPath()))){
      try {
      TreePath selected = tree.getSelectionPath();
      if (selected != null && n == selected.getLastPathComponent()){
        //special tone for marker of location
        repeatTone(BASE_SOUND+(SOUND_STEP*(n.getLevel())), 600,1);
      } else{
        repeatTone(BASE_SOUND+(SOUND_STEP*(n.getLevel())), 150,1);
      }
      int nodecount = countNodes(n, tree, false);
      //if collapsed for each node a additional beep
      if (tree.isCollapsed(new TreePath(n.getPath()))){
        repeatTone(BASE_SOUND+(SOUND_STEP*(n.getLevel())), 150,nodecount - 1);        
      } 
      Thread.sleep((long) (SLEEP_INTERVAL * 3));
      //iterate through children
        Enumeration<DefaultMutableTreeNode>iter= n.children();
        while(iter.hasMoreElements()){
          DefaultMutableTreeNode current = iter.nextElement();
          if(current != null){
            playNodeSoundClicksBefore(current,tree,rowcount);
          }
        } 
      } catch (LineUnavailableException | InterruptedException e) {
        e.printStackTrace();
      }
    }
  }
   
  
  //node structure downwards, getting deeper, count == how many children, pitch == depth, marker of location after clicks
  public static void playNodeSoundClicksAfter(DefaultMutableTreeNode n, JTree tree){
    playNodeSoundClicksAfter(n, tree, tree.getRowCount());
  }
  
  public static void playNodeSoundClicksAfter(DefaultMutableTreeNode n, JTree tree, int rowcount){
    if(n!=null && tree.isVisible(new TreePath(n.getPath()))){
      try {
        int nodecount = countNodes(n, tree, false);
        //if collapsed for each node a additional beep
        if (tree.isCollapsed(new TreePath(n.getPath()))){
          repeatTone(BASE_SOUND+(SOUND_STEP*(n.getLevel())), 150,nodecount - 1);        
        } 
      TreePath selected = tree.getSelectionPath();
      if (selected != null && n == selected.getLastPathComponent()){
        //special tone for marker of location
        repeatTone(BASE_SOUND+(SOUND_STEP*(n.getLevel())), 600,1);
      } else{
        repeatTone(BASE_SOUND+(SOUND_STEP*(n.getLevel())), 150,1);
      }    
      Thread.sleep((long) (SLEEP_INTERVAL * 3));
      //iterate through children
        Enumeration<DefaultMutableTreeNode>iter= n.children();
        while(iter.hasMoreElements()){
          DefaultMutableTreeNode current = iter.nextElement();
          if(current != null){
            playNodeSoundClicksAfter(current,tree,rowcount);
          }
        } 
      } catch (LineUnavailableException | InterruptedException e) {
        e.printStackTrace();
      }
    }
  }
  
}