package isac.gui.mawen.termtree;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

/**
 * The TermTreeEditPanel holds the tree-representation of the
 * active_formula(context_formula).
 * 
 * This panel is part of the context-menu of the <code>TermTreeView</code>. This
 * class is constructed under the model of <code>ModelPanel</code> from the
 * CalcHead context-menu.
 * 
 * @author NK 09052016
 */
public class TermTreeFormPanel extends JPanel implements ActionListener {
  private JTree term_tree_;

  private static final long serialVersionUID = 1L;

  public TermTreeFormPanel() {
    term_tree_ = new JTree();
    initComponents();
  }

  public TermTreeFormPanel(JTree tree) {
    term_tree_ = tree;
    initComponents();
  }

  private void initComponents() {
    this.setLayout(new BorderLayout());
    term_tree_.getSelectionModel().setSelectionMode(
        TreeSelectionModel.SINGLE_TREE_SELECTION);
    term_tree_.addTreeExpansionListener(new TermTreeExpansionListener());

    this.add(term_tree_);
    this.setVisible(true);
  }

  public JTree getTermTree() {
    return term_tree_;
  }

  public void setTermTree_(JTree term_tree) {
    this.term_tree_ = term_tree;
    this.add(term_tree_);
    repaint();
  }

  @Override
  public void actionPerformed(ActionEvent e) {
    // TODO Auto-generated method stub
  }
  
 protected class TermTreeExpansionListener implements TreeExpansionListener{

  @Override
  public void treeExpanded(TreeExpansionEvent event) {
    TreeSelectionEvent e = new TreeSelectionEvent(this,event.getPath(), true, event.getPath(), event.getPath());
    if (term_tree_.getTreeSelectionListeners().length > 0){
      term_tree_.getTreeSelectionListeners()[0].valueChanged(e);
    }
    TreePath selected = event.getPath();
    term_tree_.setSelectionPath(null);
    term_tree_.setSelectionPath(selected);
  }
  
  @Override
  public void treeCollapsed(TreeExpansionEvent event) {
    TreePath selected = event.getPath();
    term_tree_.setSelectionPath(null);
    term_tree_.setSelectionPath(selected);
  }
 }
}

