package isac.gui.mawen.termtree;

import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.Enumeration;

import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
/*
 * This class is needed for manipulation on mawen's termtree.
 * It was created to provide accessibility to visually impaired people on the termtree.
 * 
 * @author NK 23052016
 * 
 */
public class TermTreeKeyAdapter extends KeyAdapter{
  JTree main_tree_;
  DefaultMutableTreeNode selected_node_;
  int row_= 0;
  
 public TermTreeKeyAdapter(JTree tree){
    main_tree_=tree;
  }
  
  public void keyReleased(KeyEvent e) {

     int key = e.getKeyCode();
     
     if(main_tree_.getSelectionPath() == null){
       row_= 0;
       main_tree_.setSelectionRow(row_);
       selected_node_ = (DefaultMutableTreeNode) main_tree_.getModel().getRoot();
     }
     
     //termtree navigation - down
     if (key == KeyEvent.VK_NUMPAD5) {
        if(row_ < main_tree_.getRowCount() -1){
         row_=row_+1;
         main_tree_.setSelectionRow(row_);
        }else{
          //so a blind person knows the ends
          java.awt.Toolkit.getDefaultToolkit().beep();
        }
     }
     //termtree navigation - up
     if (key == KeyEvent.VK_NUMPAD8) {
        if(row_ > 0){
         row_=row_-1;
         main_tree_.setSelectionRow(row_);
        }else{
          //so a blind person knows the ends
          java.awt.Toolkit.getDefaultToolkit().beep();
        }
     }
     //termtree navigation - collapse
     if (key == KeyEvent.VK_NUMPAD4) {
         TreePath current = main_tree_.getSelectionPath();
         selected_node_ = ((DefaultMutableTreeNode) (current.getLastPathComponent())); 
         if(selected_node_ != null && !selected_node_.isLeaf()){
           //main_tree_.collapsePath(current);
           collapseRecursive(selected_node_);
         }
       }
     //termtree navigation - expand
     if(key == KeyEvent.VK_NUMPAD6) {
         TreePath current = main_tree_.getSelectionPath();
         selected_node_ = ((DefaultMutableTreeNode) (current.getLastPathComponent())); 
         if(selected_node_ != null && !selected_node_.isLeaf()){
           main_tree_.expandPath(current);
           setCollapsed(selected_node_,false);
           //refresh children
           Enumeration<DefaultMutableTreeNode> it = selected_node_.children();
           while (it.hasMoreElements()){
             DefaultMutableTreeNode child = it.nextElement();
             main_tree_.collapsePath(new TreePath(child.getPath()));
             setCollapsed(child, true);
           }    
         }
     }
     //acoustic termtree: marker of location before child clicks
     if(key == KeyEvent.VK_NUMPAD7){
         TreePath current = main_tree_.getSelectionPath();
         selected_node_ = ((DefaultMutableTreeNode) (current.getLastPathComponent()));
         SoundUtils.playNodeSoundClicksBefore((DefaultMutableTreeNode) main_tree_.getModel().getRoot(),main_tree_); 
     }      
     //acoustic termtree: marker of location after child clicks
     if(key == KeyEvent.VK_NUMPAD9){
       TreePath current = main_tree_.getSelectionPath();
       selected_node_ = ((DefaultMutableTreeNode) (current.getLastPathComponent()));
       SoundUtils.playNodeSoundClicksAfter((DefaultMutableTreeNode) main_tree_.getModel().getRoot(), main_tree_); 
     }

  }

  private void setCollapsed(DefaultMutableTreeNode node, boolean collapsed) {
    if (!node.isLeaf()) ((TreeNodeContent)node.getUserObject()).setCollapsed(collapsed);
    ((DefaultTreeModel)main_tree_.getModel()).nodeChanged(node);    
  }
  private void collapseRecursive(DefaultMutableTreeNode node){
    Enumeration<DefaultMutableTreeNode> it = node.children();
    while (it.hasMoreElements()){
      DefaultMutableTreeNode child = it.nextElement();
      collapseRecursive(child);
    }
    TreePath path = new TreePath(node.getPath());
    if (main_tree_.isExpanded((path))){
      main_tree_.collapsePath(path);
      setCollapsed(node, true);
    }
  }
}

