package isac.gui.treetable;

import isac.gui.Worksheet;
import isac.gui.mawen.editor.EditorListener;
import isac.gui.mawen.syntax.Ast;
import isac.interfaces.IEditor;
import isac.util.formulae.Formula;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.Iterator;

import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.UIManager;
import javax.swing.border.LineBorder;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.tree.TreeCellRenderer;

public class CalcTreeCellEditor implements javax.swing.tree.TreeCellEditor {

	protected ArrayList<CellEditorListener> listeners_;
	protected Worksheet worksheet_;
	
	protected Object editedNode_;
	protected TreeCellRenderer renderer_;
	
	protected IEditor editor_;
	protected JTextField tacticText_;
	
	
	protected JPanel editorPanel_;
	
	private int subCell_;
	
	public static final int SUBCELL_FORMULA = 1;
	public static final int SUBCELL_TACTIC = 2;
	
	public CalcTreeCellEditor( TreeCellRenderer renderer, Worksheet ws ) {
		listeners_ = new ArrayList<CellEditorListener>();
		editedNode_ = null;
		subCell_ = 0;
		renderer_ = renderer;
		
		editor_ = EditorFactory.getFormulaEditor();
		editor_.addEditorListener(new EditorListener() {
      
      @Override
      public void notifyLocalCheck() {
        
      }
      
      @Override
      public void notifyIsaCheck() {
/*err*/ editedNode_ = new Formula(editor_.getMathText(), editor_.getFormula());
        notifyWorksheet();
      }
/*err*/public void notifyBraille(Ast.Ast formula) {
			
      }

    });
		tacticText_ = new JTextField();

		
		editorPanel_ = new JPanel();
		
		editorPanel_.setOpaque(true);
		editorPanel_.setBackground(UIManager.getColor("Tree.background"));
	    
//		GridBagLayout gbl = new GridBagLayout();
		editorPanel_.setLayout(new BorderLayout());
	    
//    GridBagConstraints gbc = new GridBagConstraints();
//    gbc.gridwidth = GridBagConstraints.REMAINDER;
//    gbc.fill = GridBagConstraints.BOTH;
//    gbc.weightx = 1.0;
//    gbc.weighty = 1.0;
//        
//    gbc.anchor = GridBagConstraints.WEST;
    
    
    
//    gbl.setConstraints( ep, gbc );		    
    editorPanel_.add(editor_.getComponent(), BorderLayout.CENTER);	
    
//    gbc.anchor = GridBagConstraints.EAST;
//    gbc.insets = new Insets( 0, 30, 0, 0 );
//    gbl.setConstraints( tacticText_, gbc );
    editorPanel_.add(tacticText_, BorderLayout.SOUTH);
    
    worksheet_ = ws;
	}
	
	public void activateSubCell( int subcell ) {
		subCell_ = subcell;
	}
	
	public int getSubCell() {
		return subCell_;
	}
	
	public void addCellEditorListener(CellEditorListener listener) {
		if( !listeners_.contains( listener ) )
			listeners_.add( listener ); 
		Iterator<CellEditorListener> it = listeners_.iterator();
		while(it.hasNext())
			System.out.println( it.next().toString() );
	}
	
	public void notifyWorksheet() {
		if( worksheet_ != null )
			worksheet_.editingStopped( new ChangeEvent( this ) );
	}

	public Object getCellEditorValue() {
		return editedNode_;
	}

	public boolean isCellEditable(EventObject e) {
		return true;
	}

	public void removeCellEditorListener(CellEditorListener listener) {
		listeners_.remove( listener ); 
	}

	public boolean shouldSelectCell(EventObject e) {
		return true;
	}

	public boolean stopCellEditing() {
		for( int i = 0; i < listeners_.size(); i++ )
			((CellEditorListener)(listeners_.get(i))).editingStopped( new ChangeEvent( this ) );
		return true;
	}

	public void cancelCellEditing() {
		for( int i = 0; i < listeners_.size(); i++ )
			((CellEditorListener)(listeners_.get(i))).editingCanceled( new ChangeEvent( this ) );
	}	
	/**called by double-clicking TreeTextPanel (which includes formula and tactic textfield)*/	
        @Override
	public Component getTreeCellEditorComponent(JTree tree, Object value, 
			boolean isSelected, boolean expanded, boolean leaf, int row) {
          
          

		if( !isSelected ) {
			return renderer_.getTreeCellRendererComponent(tree, value, true, expanded, leaf, row, true);
		}
		
		editedNode_ = value;
		if( !(value instanceof CalcModelNode ) )
			return null; 
    
		CalcModelNode current_node = (CalcModelNode) value;
		
		//
		if (editor_ instanceof JTextField) {
		  editor_.setFormula( current_node.toString() );
		}else {
		  
		  if (current_node.getFormula() == null) {
		    // new Ast
/*err*/editor_.setFormula(new Ast.Constant("GAP"));
		    
		  }else {
/*err*/editor_.setFormula(current_node.getFormula().getFormula().getTerm()   );
		  }
		 

		}

		
		try {
			tacticText_.setText(((CalcModelNode)value).getSecondaryString() + "  ");
		} 
		catch (Exception e) {
			tacticText_.setText("");
		}
			    		
		if( subCell_ == SUBCELL_TACTIC ) {
		  if (editor_ instanceof JTextField) {
		    ((JTextField)editor_).setEditable( false );
		    ((JTextField)editor_).setEnabled( false );
		  }

		  
		  
			tacticText_.setBorder(new LineBorder( Color.BLACK  ) );			
			tacticText_.setEditable( true );
			tacticText_.setEnabled( true );
			tacticText_.requestFocusInWindow();
		}
		
		if( subCell_ == SUBCELL_FORMULA ) {
			tacticText_.setEditable( false );
			tacticText_.setEnabled( false );
			if (editor_ instanceof JTextField) {
  			((JTextField)editor_).setBorder(new LineBorder( Color.BLACK  ) );
  			((JTextField)editor_).setEditable( true );
  			((JTextField)editor_).setEnabled( true );
  			((JTextField)editor_).requestFocusInWindow();
			}
		}
		
		editorPanel_.revalidate();
		editorPanel_.setBackground(Color.RED);
		Rectangle worksheet_rect = this.worksheet_.getBounds();
		editorPanel_.setPreferredSize(new Dimension(worksheet_rect.width - 50, editor_.getExpressionSize().height + 25));
		editorPanel_.revalidate();
		tree.revalidate();
		return editorPanel_;
	}



}
