/**
 * @author Andreas Griesmayer, Martin Lang Created on Jul 5, 2003
 */
package isac.kestore;

import isac.util.KEStorePaths;
import isac.util.parser.XMLParserDigest;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.StringWriter;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class KEStoreServices {

    private String path_;

    // WN040830 public for use in java-tests/../bridge
    public KEStoreServices(String path) {
        path_ = path;
        System.out.println("KEStoreServices default constructor called");

    }

    public Vector<String> testMessage() {
        Vector<String> result = new Vector<String>();
        result.add("testMethod from KEStoreServices");
        return result;
    }

    /**
     * Loads the hierarchy_ for type <code>type</code> from the filesystem.
     * 
     * @param type
     * @return Vector with the XML-representation of the hierarchy_ in its first
     *         item
     */
    public Vector<String> loadHierarchy(String type) {
        Vector<String> return_value = new Vector<String>();
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document document = null;
            System.out.println("type:" + type);
            if (type.equals("pbl")) {
                document = db.parse(path_ + "/pbl/"
                        + KEStorePaths.PROBLEM_HIERARCHY_FILE_NAME);
            } else if (type.equals("met")) {
                document = db.parse(path_ + "/met/"
                        + KEStorePaths.METHOD_HIERARCHY_FILE_NAME);
            } else if (type.equals("exp")) {
                document = db.parse(path_ + "/exp/"
                        + KEStorePaths.EXAMPLE_HIERARCHY_FILE_NAME);
            } else if (type.equals("thy")) {
                document = db.parse(path_ + "/thy/"
                        + KEStorePaths.THEORY_HIERARCHY_FILE_NAME);
            }

            DOMSource source = new DOMSource(document);
            StringWriter output = new StringWriter();
            StreamResult result = new StreamResult(output);

            TransformerFactory tFactory = TransformerFactory.newInstance();
            Transformer transformer = tFactory.newTransformer();
            transformer.transform(source, result);
            System.out.println(output.toString());
            return_value.add(output.toString());
        } catch (java.io.IOException exc) {
            System.out.println(exc.getMessage());
        } catch (Exception exc) {
            System.out.println(exc.getMessage());
        }
        return return_value;
    }

    /**
     * loads the binary-file <code>file</code> for type <code>type</code>.
     * 
     * @param type
     *            type of the file to load - eg. exp
     * @param file
     *            to load - has to be in the directory of the type
     * @return a vector with a binary array of the file to load in its first
     *         element or empty array if the file was not found
     */
    public Vector<byte[]> loadBinary(String type, String file) {
        System.out.println("binaryfile:" + file);
        Vector<byte[]> return_value = new Vector<byte[]>();
        File ifile = new File(path_ + "/" + type + "/" + file);
        try(FileInputStream fis = new FileInputStream(ifile)){
            
            int size = fis.available();
            byte[] data = new byte[size];
            fis.read(data);
            return_value.add(data);

        } catch (FileNotFoundException exc) {
            return_value.removeAllElements();
            return_value.add(new byte[0]);
            return return_value;
        } catch (IOException e) {
            return_value.removeAllElements();
            return_value.add(new byte[0]);
            return return_value;
        }
        return return_value;
    }

    /**
     * selects and loads the requested file from the KEStore using the hiearchy
     */
    public Vector<String> loadContent(String type, Vector id) {
        System.out.println("id in KESTore: " + id);
        String filename = null;
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document document = null;
            System.out.println("type:" + type);

            if (type.trim().equals("pbl")) {
                document = db.parse(path_ + "/pbl/"
                        + KEStorePaths.PROBLEM_HIERARCHY_FILE_NAME);
            } else if (type.trim().equals("exp")) {
                document = db.parse(path_ + "/exp/"
                        + KEStorePaths.EXAMPLE_HIERARCHY_FILE_NAME);
            } else if (type.trim().equals("met")) {
                document = db.parse(path_ + "/met/"
                        + KEStorePaths.METHOD_HIERARCHY_FILE_NAME);
            } else if (type.trim().equals("thy")) {
                document = db.parse(path_ + "/thy/"
                        + KEStorePaths.THEORY_HIERARCHY_FILE_NAME);
            }

            NodeList node_list = document.getElementsByTagName("ID");

            // TODOO: in the future problems will occure when different
            // examples, problems...
            // have the same id in one hierarchy
            for (int i = 1; i < node_list.getLength(); i++) {
                Node node = node_list.item(i);
                if (node.getNodeType() == Node.ELEMENT_NODE) {
                    Node text_element = node.getFirstChild();
                    if (text_element.getNodeType() == Node.TEXT_NODE) {
                        String element_content = text_element.getNodeValue();
                        String node_value = id.firstElement().toString();
                        if (element_content.trim().equals(node_value)) {
                            Node result = node.getParentNode();
                            result = result.getChildNodes().item(3);
                            result = result.getFirstChild();
                            filename = result.getNodeValue().toString().trim();
                            break;
                        }
                    } else {
                        System.out.println("no txt node - error in hierarchy file");
                    }
                } else {
                    System.out.println("no element node - error in hierarchy file");
                }
            }

            System.out.println("FILE " + filename);
            Vector<String> contentvect = loadContent(type, filename);
            contentvect.add(filename);
            return contentvect;
        } catch (Exception exc) {
            exc.printStackTrace();
            System.out.println(exc.getMessage());
        }
        return new Vector<String>();
    }

    /**
     * loads an KE-Object in its XML-representation. uses an XML-DocumentBuilder
     * to validate if the file is in proper format future implementation shall
     * create the XML-representations out of smaller chunks - a final
     * data-format is not found yet.
     * 
     * WN071016 The xml-data already encode < to &lt; and > to ^gt; thus calling
     * encodeEntities is superfluous here.
     * 
     * @param type
     *            of the KE-Object (like a collection) ::= "exp" | "pbl" | "met"
     * @param file
     *            id of the KE-Object to load (filename in the current
     *            implementation)
     * @return Vector with the KE-Object in its XML-representation in its first
     *         element or empty Vector if the file was not found or if 'parse'
     *         failed
     */
    public Vector<String> loadContent(String type, String file) {

        System.out.println("File:" + file);
        Vector<String> return_value = new Vector<String>();
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();

            String path = path_ + "/" + type + "/" + file;
            String temp_file_path = path_ + "/temp_file";

            // creating a temp file for replacing the special characters
            // otherwise the parser gets problems while parsing
            String content_xml_file = readFileAsString(path);

            // replacing the special characters in the xml data file
            // WN071016 The xml-data already encode < to &lt; and > to ^gt;
            // thus calling encodeEntities is superfluous here
            content_xml_file = XMLParserDigest.encodeEntities(content_xml_file);

            OutputStreamWriter writer = new OutputStreamWriter(
                    new FileOutputStream(temp_file_path), "UTF-8");
            writer.write(content_xml_file);
            writer.flush();
            writer.close();
            Document document = db.parse(temp_file_path);
            // here for xerxes......^^^^^ the content MUST be encoded,
            // e.g. "<" as &lt;"

            DOMSource source = new DOMSource(document);
            StringWriter output = new StringWriter();
            StreamResult result = new StreamResult(output);
            TransformerFactory tFactory = TransformerFactory.newInstance();
            Transformer transformer = tFactory.newTransformer();
            transformer.transform(source, result);
            String final_result = encodeUmlauts(output.toString());
            return_value.add(final_result);
            // WN040830 why vector for one
            // element ? AG: (very early) design decision for later expansion
        } catch (Exception exc) {
            System.out.println("KEStore:Exception " + exc.getMessage());
            return_value.addElement("ERROR");
            return_value.addElement(file);
            return_value.addElement(exc.getMessage());

            return return_value;
        }
        return return_value;
    }

    /**
     * this method reads a file into a string by use of the iso8859-15 encoding
     * to get the 'Umlaute' as well
     * 
     * @param filePath
     *            the path of the file
     * @return the content of the file as string
     */
    private static String readFileAsString(String filePath)
            throws java.io.IOException {
        StringBuffer fileData = new StringBuffer(1000);
        InputStreamReader reader = new InputStreamReader(new FileInputStream(
                filePath), "iso8859-15");
        char[] buf = new char[1024];
        int numRead = 0;
        while ((numRead = reader.read(buf)) != -1) {
            String readData = String.valueOf(buf, 0, numRead);
            fileData.append(readData);
            buf = new char[1024];
        }
        reader.close();
        return fileData.toString();
    }

    /**
     * encodes the german 'Umlaute' \u00D6 is the unicode character for the
     * german Umlaut 'Oe' \u00F6 is the unicode character for the german Umlaut
     * 'oe' \u00DC is the unicode character for the german Umlaut 'Ue' \u00FC is
     * the unicode character for the german Umlaut 'ue' \u00C4 is the unicode
     * character for the german Umlaut 'Ae' \u00E4 is the unicode character for
     * the german Umlaut 'ae' \u00DF is the unicode character for the german
     * 'sz'
     * 
     * @param the
     *            input string
     * 
     * @return the output string
     */
    private String encodeUmlauts(String input) {
        input = input.replaceAll("\u00D6", "&Ouml;");
        input = input.replaceAll("\u00F6", "&ouml;");
        input = input.replaceAll("\u00DC", "&Uuml;");
        input = input.replaceAll("\u00FC", "&uuml;");
        input = input.replaceAll("\u00C4", "&Auml;");
        input = input.replaceAll("\u00E4", "&auml;");
        input = input.replaceAll("\u00DF", "&szlig;");
        return input;
    }
}
