/**
 * @author Andreas Griesmayer
 * Created on Oct 6, 2003
 */
package isac.session;

import isac.browserdialog.IBrowserDialog;
import isac.users.IUserManager;

import java.rmi.Remote;
import java.rmi.RemoteException;

/**
 * @author Andreas Griesmayer
 */
public interface IObjectManager extends Remote {

    /**
     * fetch the XSessionManager of the system - multiple calls to this method
     * result in one and the same XSessionManager.
     */
    public abstract ISessionManager getSessionDialog() throws RemoteException;
    
    /**
     * fetch the UserManager of the system - multiple calls to this method
     * result in one and the same UserManager.
     */
    public abstract IUserManager getUserManager() throws RemoteException;

    /**
     * fetch the XBrowserDialog identified by <code>bdialog_id</code>
     */
    public abstract IBrowserDialog getBrowserDialog(String id) throws RemoteException;

    /**
     * Access from a web-browser by an url;
     * the url contains all data required for starting one example:
     *
     * example: the unique identifier in the KEStore
     * institution+user: identifier for a user unique within ISAC;
     *    access in this case is given by a password for the institution
     * mode: ="assessment" is handled specifically: only _one_ session
     *    is possible; sessions with other mode are closed.
     *
     * Since webAccess contains no means to address a certain session,
     * the first one found
     * {@link SessionManager#getFirstSession(username)} is taken.
     *
     * Starting a WindowApplication without webAcess allows several
     * sessions in parallel --
     * -- @TODO reactivate this feature by cookies or the like;
     * probably out-source identification from ISAC completely.
     */
    WebAccessReponse webAccess(String username, String password, String mode, String example) throws AuthenticationException, RemoteException;

    /**
     * fetch the WorksheetDialog identified by <code>wdialog_id</code>
     */
    //public abstract IWorksheetDialog getWorksheetDialog(String id) throws RemoteException;
}