/*
 * @author Robert Koenighofer, member of the ISAC-team, 
 * Copyright (c) 2006 by Robert Koenighofer
 * created Jul 11, 2006 10:09:13 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */

package isac.session;

import isac.users.User;

import java.rmi.Remote;
import java.rmi.RemoteException;

/**
 * This is an interface for the SessionManager. The SessionManager administrates
 * all sessions.
 * 
 * @see isac.session.SessionManager
 * @author Robert Koenighofer
 */
public interface ISessionManager extends Remote {
	/**
	 * This method creates a new session for the user with the
	 * name 'user_name'. It returns the created session.
	 * 
	 * @param user_name the name of the user of the new session
	 * @return the created session
	 * @throws RemoteException if remote object is not available
	 * @see isac.session.SessionManager#addSession(String)
	 */
	ISession addSession(User user) throws RemoteException;

	/**
	 * returns the session with the desired session_id. If the session
	 * does not exist, null is returned. 
	 * 
	 * @param session_id the unique id of the desired session
	 * @return the desired session or null if it doesn't exist
	 * @throws RemoteException if remote object is not available
	 * @see isac.session.SessionManager#getSessionByID(String)
	 */
	ISession getSessionByID(String session_id) throws RemoteException;

	/**
	 * returns the name of the user, which owns a specific session as String
	 * If the session does not exist, null is returned. 
	 * 
	 * @param session_id the unique id of the desired session
	 * @return the desired user name or null if the session does not exist
	 * @throws RemoteException if remote object is not available
	 * @see isac.session.SessionManager#getUserNameByID(String)
	 */
	String getUserNameByID(String session_id) throws RemoteException;

    /**
     * Returns the session for the specified user.
     * @param username the username of the user
     * @return the session.
     */
    ISession getFirstSession(String username) throws RemoteException;
}