/**
 * @author Andreas Griesmayer
 * Created on Oct 6, 2003
 */
package isac.session;

import isac.browserdialog.IBrowserDialog;
import isac.util.FixedPortRMISocketFactory;
import isac.util.ObjectManagerPaths;
import isac.users.UserManager;
import isac.users.IUserManager;
import isac.users.UserLogger;

import java.io.IOException;
import java.rmi.Naming;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.server.RMISocketFactory;
import java.rmi.server.UnicastRemoteObject;


/**
 * @author Andreas Griesmayer
 * 
 * 
 */
@SuppressWarnings("deprecation") // RMISecurityManager : RMI reform will come
public class ObjectManager extends UnicastRemoteObject implements
        IObjectManager {

    static final long serialVersionUID = -7823336348890642600L;

    SessionManager sdialog_ = SessionManager.getInstance();

    UserManager user_manager_ = UserManager.getInstance();
    
    UserLogger user_logger_ = UserLogger.getInstance();

    WatchdogThread watchdog_;

    /**
     * Access from a web-browser by an url;
     * the url contains all data required for starting one example:
     * 
     * example: the unique identifier in the KEStore
     * institution+user: identifier for a user unique within ISAC;
     *    access in this case is given by a password for the institution
     * mode: ="assessment" is handled specifically: only _one_ session
     *    is possible; sessions with other mode are closed.
     *
     * Since webAccess contains no means to address a certain session,
     * the first one found 
     * {@link SessionManager#getFirstSession(username)} is taken.
     * 
     * Starting a WindowApplication without webAcess allows several
     * sessions in parallel --
     * -- @TODO reactivate this feature by cookies or the like;
     * probably out-source identification from ISAC completely.
     */
    public WebAccessReponse webAccess(String username, String password, String mode, String example) 
    throws AuthenticationException, RemoteException {
        WebAccessReponse webAccessReponse = new WebAccessReponse();
Session session = sdialog_.getFirstSession(username);
		if (session == null) {
		    webAccessReponse.setHasSessionToOpenExample(false);
		} else {
		    if (LearningMode.ASSESSMENT.toString().equalsIgnoreCase(mode)) {
		        if (session.getLearningMode() == LearningMode.ASSESSMENT) {
		            openWorksheet(session, example);
		            webAccessReponse.setHasSessionToOpenExample(true);
		        } else {
		            closeAllOpenSessions(username);
		            webAccessReponse.setHasSessionToOpenExample(false);
		        }
		    } else { // mode != ASSESSMENT
		        if (session.getLearningMode() == LearningMode.ASSESSMENT) {
		            closeAllOpenSessions(username);
		            webAccessReponse.setHasSessionToOpenExample(false);
		        } else { //session != ASSESSMENT
		            openWorksheet(session, example);
		            webAccessReponse.setHasSessionToOpenExample(true);
		        }
		    }
		}
        return webAccessReponse;
    }
    
    private void openWorksheet(Session session, String example) throws RemoteException {
        session.getExampleDialog().openWorksheetFromExample(example + ".xml");
    }
    
    private void closeAllOpenSessions(String username) {
        // ATTENTION: Close all Sessions for this User and each Session closes its own WindowApplication.
        //sdialog_.closeAllOpenSessionsFor(String username);
        throw new UnsupportedOperationException("Not yet implemented");
    }
    
    /**
     * fetch the UserManager of the system - multiple calls to this method
     * result in one and the same UserManager.
     */
    public IUserManager getUserManager() throws RemoteException {
        if (user_manager_ == null)
            user_manager_ = UserManager.getInstance();
        return user_manager_;
    }

    /**
     * instantiates the Objectmanager which is used to simulate a function of
     * dinopolis The ObjectManager provides functions to fetch objects which are
     * used by other modules of the system using RMI
     * 
     * @throws RemoteException
     */
    protected ObjectManager() throws RemoteException {
        super();
        rmiBind();
        // RK for RMI-problems 07htl: watchdog_ = new WatchdogThread();
        // watchdog_.start();

//      Just testing webAccess: Wait 20 secs after ObjectManager started, then display an example  
//        new Thread(new Runnable() {
//
//            public void run() {
//                try {
//                    Thread.sleep(20000);
//                    webAccess("ae/tomzi", "ae", "assessment", "exp_IsacCore_Tests_1a");
//                } catch (Exception ex) {
//                    ex.printStackTrace();
//                }
//            }
//        }).start();
    }

    /**
     * fetch the XBrowserDialog identified by <code>session</code>
     */
    public IBrowserDialog getBrowserDialog(String session)
            throws RemoteException {
        return (IBrowserDialog) sdialog_.getSessionByID(session)
                .getExampleDialog();
    }

    /**
     * fetch the WorksheetDialog identified by <code>wdialog_id</code>
     */
    /*
     * public IWorksheetDialog getWorksheetDialog(String wdialog_id) throws
     * RemoteException { return (IWorksheetDialog)
     * sdialog_.getWorksheetDialog(wdialog_id); }
     */

    /**
     * fetch the XSessionManager of the system - multiple calls to this method
     * result in one and the same XSessionManager.
     */
    public ISessionManager getSessionDialog() throws RemoteException {
        if (sdialog_ == null)
            sdialog_ = SessionManager.getInstance();
        return sdialog_;
    }

    /*
     * instantiates the ObjectManager and registers it on the RMIRegistry on the
     * local machine as //localhost/isac-ObjectManager . If no RMIRegistry is
     * running, a new one is started.
     */
    public static void main(String[] args) {
        System.out.println("Starting ObjectManager...");
        try {
            String ini_load_result = ObjectManagerPaths.loadFromFile(args[0]);

            if (ini_load_result != null)
                ini_load_result += ObjectManagerPaths.loadFromResource(args[0]);

            if (ini_load_result != null) {
                System.err.println(ini_load_result);
                System.exit(1);
            }

            try {
                // Use ObjectManagerRMISocketFactory 0704
                RMISocketFactory
                        .setSocketFactory(new FixedPortRMISocketFactory(
                                ObjectManagerPaths.OBJECT_MANAGER_RMI_PORT));
                // new
                // ObjectManagerRMISocketFactory(3113,ObjectManagerPaths.OBJECT_MANAGER_RMI_PORT));
            } catch (IOException e) {
                System.out.println("Exception: " + e.getMessage());
                e.printStackTrace();
            }

            System.out.println(new ObjectManager());

        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }
    
	private void rmiBind() {
        if (System.getSecurityManager() == null) {
            System.setSecurityManager(new RMISecurityManager());
        }

        String name = ObjectManagerPaths.OBJECT_MANAGER_PATH;
        try {
            System.out.println("try to bind as " + name);
            Naming.rebind(name, this);
            System.out.println("Object Manager bound to " + name);
        } catch (java.rmi.ConnectException e) {
            System.err.println("failed to contact as " + name
                    + " (creating RMI-Server on localhost:1099)");
            try {
                LocateRegistry
                        .createRegistry(ObjectManagerPaths.OBJECT_MANAGER_PORT);
                rmiBind();
            } catch (java.rmi.RemoteException exc2) {
                System.err.println("can not create registry:"
                        + exc2.getMessage());
            }
        } catch (Exception exc) {
            System.err.println("failed to connect to rmiregistry");
            System.err.println(exc);
        }
   }
}
