/*
 * @author Walther Neuper, member of the ISAC-team,
 * Copyright (c) 2005 by Walther Neuper
 * created Mar 12, 2005 8:09:42 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 *
 * Use is subject to PGPL license terms.
 */
package isac.users;

import isac.util.FileUtils;
import java.util.Properties;

/**
 * @author Walther Neuper
 *
 * Contains passwords looking forward to handle security issues elsewhere
 * (outside isac)
 */
public class Accounts extends Properties {

    static final long serialVersionUID = 6145685806086624884L;
    public static final String ACCOUNTS_FILE_NAME = "accounts.txt";
    private static Properties DUMMY_ACCOUNTS = new Properties();

    static {
        DUMMY_ACCOUNTS.setProperty("x", "x");
        DUMMY_ACCOUNTS.setProperty("y", "y");
        DUMMY_ACCOUNTS.setProperty("anonymous", "anonymous");
    }

    public Accounts(String directoryOfAccountsFile) {
        createAccountsFileIfItDoesNotExist(directoryOfAccountsFile + ACCOUNTS_FILE_NAME);
        loadAccountsFromFile(directoryOfAccountsFile + ACCOUNTS_FILE_NAME);
    }

    private void loadAccountsFromFile(String file) {
        FileUtils.loadPropertiesFromFile(file, this, "user accounts");
    }

    private static void createAccountsFileIfItDoesNotExist(String file) {
        FileUtils.createPropertiesFileIfItDoesNotExist(file, DUMMY_ACCOUNTS, "user accounts", 
            "Use this as follows: username=pwd");
    }

    public boolean authenticationValid(String username, String password) {
        return containsKey(username) && getProperty(username).equals(password);
    }

    public boolean userExists(String username) {
        return containsKey(username);
    }
}
