/*
 * @author Thomas Zillinger
 *
 * Use is subject to PGPL license terms.
 */
package isac.users;

import isac.util.FileUtils;
import java.util.Properties;

/**
 * @author Thomas Zillinger
 *
 * Contains domain names / passwords pairs.
 */
public class Domains extends Properties {
    
    private static final long serialVersionUID = 6472978261427053312L;
    
    public static final String DOMAINS_FILE_NAME = "domains.txt";
    private static Properties DUMMY_DOMAINS = new Properties();

    static {
        DUMMY_DOMAINS.setProperty("dummyDomain", "dummyPassword");
    }

    public Domains(String directoryOfDomainsFile) {
        createDomainsFileIfItDoesNotExist(directoryOfDomainsFile + DOMAINS_FILE_NAME);
        loadDomainsFromFile(directoryOfDomainsFile + DOMAINS_FILE_NAME);
    }

    private void loadDomainsFromFile(String file) {
        FileUtils.loadPropertiesFromFile(file, this, "domains");
    }

    private static void createDomainsFileIfItDoesNotExist(String file) {
        FileUtils.createPropertiesFileIfItDoesNotExist(file, DUMMY_DOMAINS, "domains", 
            "Use this as follows: domainame = domainpwd");
    }

    public boolean authenticationValid(String domainName, String domainPassword) {
        return containsKey(domainName) && getProperty(domainName).equals(domainPassword);
    }

    public boolean domainExists(String domainName) {
        return containsKey(domainName);
    }
}
