/*
 * Created during correction of UserLogger Feb. 2012
 *
 * @author Kober Franz
 */
package isac.users;

import isac.useractions.EUIElement;
import isac.util.formulae.Position;
import java.sql.Timestamp;

/**
 * Holds all data to be logged in data base by UserLogger.
 *
 * LogItem is created at carefully selected locations in the source, where all arguments are available without much ado.
 * completeWithChanged, completeWithError. LogItem is completed when the response of the math-engine is returned:
 *
 * Thus NO set*, get* !
 */
public class LogItem {

    public enum DialogTypes {

        WORKSHEET,
        EXAMPLE,
        THEORY,
        PROBLEM,
        METHOD,
        SESSION,
        NONE
    }
    private String username_;
    private String session_id_;
    private DialogTypes dialog_type_;// DialogTypes DialogTypes
    private String worksheet_id_; // worksheet the dialog_type_ is in "Context" with
    private Timestamp time_;      // time of writing to data base
    private Position position_;
    private String formula_from_; // the formula the interaction is set on
    private EUIElement step_;     // see EUIElement
    private String step_arg_1_;   // filename for start example, formula for UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE
    private String step_arg_2_;   // 2nd argument if required, "" otherwise
    private Boolean error_;       // TRUE <=> error_message_ != ""
    // FALSE <=> formula_to_ != ""
    private String formula_to_;   // the formula resulting fromthe interaction
    private String error_message_;

    //general case
    public LogItem(String username, String session, String worksheet, DialogTypes dialog_type,
            String step_arg_1, String step_arg_2, Position position, String formula_from,
            EUIElement step) {

        username_ = username;
        session_id_ = session;
        worksheet_id_ = worksheet;
        dialog_type_ = dialog_type;
        time_ = new java.sql.Timestamp(new java.util.Date().getTime());

        position_ = position;
        step_arg_1_ = step_arg_1;
        step_arg_2_ = step_arg_2;
        formula_from_ = formula_from;
        formula_to_ = " ";
        step_ = step;

        error_ = false;
        error_message_ = "";
    }

    //used for appendFormula, replaceFormula: formula_to !
    public LogItem(String username, String session, String worksheet, DialogTypes dialog_type,
            String step_arg_1, String step_arg_2, Position position, String formula_from,
            String formula_to, EUIElement step) {

        username_ = username;
        session_id_ = session;
        worksheet_id_ = worksheet;
        dialog_type_ = dialog_type;
        time_ = new java.sql.Timestamp(new java.util.Date().getTime());

        position_ = position;
        step_arg_1_ = step_arg_1;
        step_arg_2_ = step_arg_2;
        formula_from_ = formula_from;
        formula_to_ = formula_to;
        step_ = step;

        error_ = false;
        error_message_ = "";
    }

    /**
     * complete LogItem in case an error has been found by the math-engine
     *
     */
    public void completeWithError(String error_message) {
        this.error_ = true;
        this.error_message_ = error_message;
        time_ = new java.sql.Timestamp(new java.util.Date().getTime());
    }

    /**
     * complete LogItem in case the math-engine accepted the interaction and returned a result.
     *
     */
    public void completeWithChanged(String formula_to) {
        formula_to_ = formula_to;
        time_ = new java.sql.Timestamp(new java.util.Date().getTime());
        this.error_ = false;
    }

    public void printLogItem() {
        System.out.println("LogItem");
        System.out.println("<--------------------LogItem--------------------->");
        System.out.println("Username: " + username_);
        System.out.println("Session ID: " + session_id_);
        System.out.println("Example Name: " + step_arg_1_);
        System.out.println("Worksheet_ID: " + worksheet_id_);
        if (position_ != null) {
            System.out.println("Position: " + position_);
        }
        System.out.println("Time: " + time_.toString());
        System.out.println("Formula From: " + formula_from_);
        System.out.println("Formula To: " + formula_to_);
        System.out.println("Step: " + step_);
        System.out.println("Error: " + error_);
        System.out.println("Error Message: " + error_message_);
        System.out.println("<----------------------------------------------->");
    }

    public String getUsername() {
        return username_;
    }

    public String getSessionId() {
        return session_id_;
    }

    public DialogTypes getDialogType() {
        return dialog_type_;
    }

    public String getWorksheetId() {
        return worksheet_id_;
    }

    public Timestamp getTime() {
        return time_;
    }

    public Position getPosition() {
        return position_;
    }

    public String getFormulaFrom() {
        return formula_from_;
    }

    public EUIElement getStep() {
        return step_;
    }

    public String getStepArg1() {
        return step_arg_1_;
    }

    public String getStepArg2() {
        return step_arg_2_;
    }

    public Boolean getError() {
        return error_;
    }

    public String getFormulaTo() {
        return formula_to_;
    }

    public String getErrorMessage() {
        return error_message_;
    }
}
