/****************************************************************** 
 * @author Alois Kirchsteiger member of the isac team
 * Copyright (c) 2004 by Alois Kirchsteiger
 * create 02.09.2004, 09:22:43
 * Insitute for Softwaretechnology, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/
package isac.users;

import isac.users.UserLanguage.LanguageName;

import java.io.Serializable;

/**
 * Represents the logged in user. The user has a username. A username can be split into a domain and a 
 * usernameWitoutDomain. eg <br/><br/>
 * <table border="1">
 * <tr><th>getUsername()</th><th>getDomain()</th><th>getUsernameWithoutDomain()</th></tr>
 * <tr><td>testuser</td><td></td><td>testuser</td></tr>
 * <tr><td>domain/testuser</td><td>domain</td><td>testuser</td></tr>
 * <tr><td>domain\testuser</td><td>domain</td><td>testuser</td></tr>
 * </table>
 * @author Alois Kirchsteiger 02.09.2004 09:22:43
 */
public class User implements Serializable {

    static final long serialVersionUID = 1124381392757950882L;
    private String username;
    private String domain;
    private String usernameWithoutDomain;
    private String userpath;

    public User(String username, String userpath) {
        this.username = username;
        this.userpath = userpath;
        parseDomainAndUsername();
    }

    private void parseDomainAndUsername() {
        String[] pieces = splitUsernameToForwardOrBackSlash();
        if (pieces.length >= 2) {
            domain = pieces[0];
            usernameWithoutDomain = pieces[1];
        } else {
            domain = "";
            usernameWithoutDomain = pieces[0];
        }
    }

    private String[] splitUsernameToForwardOrBackSlash() {
        String[] pieces;
        if (username.contains("/")) {
            pieces = username.split("/");
        } else {
            pieces = username.split("\\\\");
        }
        return pieces;
    }

    public UserAdmin getUserAdmin() {
        if (this.hasDomain()) {
            return new UserAdmin(userpath, domain);
        } else {
            return new UserAdmin(userpath, username);
        }

    }

    public UserModel getUserModel() {
        if (this.hasDomain()) {
            return new UserModel(userpath, domain);
        } else {
            return new UserModel(userpath, username);
        }
    }

    public UserSettings getUserSettings() {
        if (this.hasDomain()) {
            return new UserSettings(userpath, domain);
        } else {
            return new UserSettings(userpath, username);
        }
    }

    public String getUsername() {
        return username;
    }

    public String getDomain() {
        return domain;
    }

    public boolean hasDomain() {
        return domain != null && !domain.isEmpty();
    }

    public String getUsernameWithoutDomain() {
        return usernameWithoutDomain;
    }

    public String getUserpath() {
        return userpath;
    }

    public UserLanguage getLanguage() {
        UserSettings us = getUserSettings();
        LanguageName language = LanguageName.ENGLISH;
        switch (us.getValue("KEY4_LANGUAGE")) {
            case IUserSettings.VAL4_ENGLISH:
                language = LanguageName.ENGLISH;
                break;
            case IUserSettings.VAL4_DEUTSCH:
                language = LanguageName.DEUTSCH;
                break;
        }
        return new UserLanguage(userpath, language);
    }
}