/*
 * @author Walther Neuper, member of the ISAC-team, 
 * Copyright (c) 2005 by Walther Neuper 
 * created Mar 12, 2005 8:55:37 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.users;

import isac.util.FileUtils;
import java.util.Properties;

/**
 * @author Walther Neuper
 * @author thomas.zillinger
 * 
 * contains administrative data (and nothing about the dialog !!!) looking
 * forward to be handled elsewhere outside isac
 */
public class UserAdmin extends Properties {

    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private static final String DEFAULT_FIRSTNAME_SUFFIX = "-firstname";
    private static final String DEFAULT_LASTNAME_SUFFIX = "-lastname";
    private String USERADMIN_FILESUFFIX = "_admin.txt";
    public UserAdmin(String directoryOfUserAdminFile, String username) {
        final String userAdminFilePath = directoryOfUserAdminFile + getUserAdminFileName(username);
        createUserAdminFileIfItDoesNotExist(userAdminFilePath, username);
        loadUserAdminFromFile(userAdminFilePath);
    }

    private void loadUserAdminFromFile(String file) {
        FileUtils.loadPropertiesFromFile(file, this, "user admin");
    }

    private static void createUserAdminFileIfItDoesNotExist(String file, String username) {
        Properties dummyUserAdminData = new Properties();

        dummyUserAdminData.setProperty("firstName", username + DEFAULT_FIRSTNAME_SUFFIX);
        dummyUserAdminData.setProperty("lastName", username + DEFAULT_LASTNAME_SUFFIX);
        
        FileUtils.createPropertiesFileIfItDoesNotExist(file, dummyUserAdminData, "user admin", 
            "Administrative data of user '" + username + "'");
    }

    private String getUserAdminFileName(String username) {
        return username + USERADMIN_FILESUFFIX;
    }
}