package isac.users;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import isac.util.ObjectManagerPaths;

/**
 * Created during correction of UserLogger Feb. 2012
 *
 * UserLogger creates a table 'userlogger' if not yet existing.
 *
 * @author Kober Franz
 */
public class UserLogger {

    private static UserLogger instance_ = null;
    public static final String STMT_INSERT = "INSERT INTO userlogger ("
            + "username, session_id, dialog_type, worksheet_id, time, position, formula_from, step, step_arg_1, "
            + "step_arg_2, formula_to, error, error_message )"
            + "VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?)";
    public static final String STMT_CREATE_TABLE = "CREATE TABLE IF NOT EXISTS userlogger ("
            + "userlogger_id INT NOT NULL AUTO_INCREMENT,"
            + "username VARCHAR(25),"
            + "session_id VARCHAR(50),"
            + "dialog_type VARCHAR(15),"
            + "worksheet_id VARCHAR(50),"
            + "time TIMESTAMP,"
            + "position VARCHAR(50),"
            + "formula_from VARCHAR(150),"
            + "step VARCHAR(50), "
            + "step_arg_1 VARCHAR(60),"
            + "step_arg_2 VARCHAR(255),"
            + "formula_to VARCHAR(150),"
            + "error TINYINT(1),"
            + "error_message VARCHAR(255),"
            + "PRIMARY KEY(userlogger_id))";
    public static final String STMT_CREATE_DATABASE = "CREATE DATABASE IF NOT EXISTS userlogger";
    public static final String STMT_DROP_TABLE = "DROP TABLE IF EXISTS userlogger";
    private Connection connection_ = null;

    private UserLogger() {
        connection_ = startDBConnection("");
        createDatabase();
        close(connection_);
        connection_ = startDBConnection(ObjectManagerPaths.LOGGER_DATABASE_NAME);
        createTable();
        if (connection_ == null) {
            System.err.println("ERROR: Could not create connection to database "
                    + getDBUrl(ObjectManagerPaths.LOGGER_DATABASE_NAME)
                    + ", check if it is running!");
        }
    }

    public static UserLogger getInstance() {
        if (instance_ == null && ObjectManagerPaths.LOGGER_DATABASE_ENABLED) {
            return instance_ = new UserLogger();
        } else {
            return instance_;
        }
    }

    private Connection startDBConnection(String database) {
        Connection con = null;
        String db_path = getDBUrl(database);
        try {
            Class.forName("com.mysql.jdbc.Driver");
            try {
                con = DriverManager.getConnection(db_path, ObjectManagerPaths.LOGGER_DATABASE_USER,
                        ObjectManagerPaths.LOGGER_DATABASE_PASSWORD);
            } catch (SQLException ex) {
                printSQLException(ex);
            }
        } catch (ClassNotFoundException ex) {
            printSQLException(ex);
        }
        return con;
    }

    public void saveLogItem(LogItem log_item) {
        if (connection_ != null) {
            PreparedStatement pstmt = null;
            try {
                pstmt = connection_.prepareStatement(STMT_INSERT);

                pstmt.setString(1, log_item.getUsername());
                pstmt.setString(2, log_item.getSessionId());
                pstmt.setString(3, log_item.getDialogType() == null ? "null" : log_item.getDialogType().toString());
                pstmt.setString(4, log_item.getWorksheetId());
                pstmt.setTimestamp(5, log_item.getTime());
                pstmt.setString(6, log_item.getPosition() == null ? "null" : log_item.getPosition().toString());
                pstmt.setString(7, log_item.getFormulaFrom());
                pstmt.setString(8, log_item.getStep() == null ? "null" : log_item.getStep().toString());
                pstmt.setString(9, log_item.getStepArg1());
                pstmt.setString(10, log_item.getStepArg2());
                pstmt.setString(11, log_item.getFormulaTo());
                pstmt.setBoolean(12, log_item.getError());
                pstmt.setString(13, log_item.getErrorMessage());
                pstmt.executeUpdate();
            } catch (SQLException ex) {
                printSQLException(ex);
            } finally {
                close(pstmt);
            }
        }
    }

    //until now this has been done in repos/src/sql/UserLogger.sql
    //what are the respective advantages ?
    private void createTable() {
        if (connection_ != null) {
            Statement stmt = null;
            try {
                stmt = connection_.createStatement();
                stmt.executeUpdate(STMT_CREATE_TABLE);
            } catch (SQLException ex) {
                printSQLException(ex);
            } finally {
                close(stmt);
            }
        }
    }

    private void createDatabase() {
        if (connection_ != null) {
            Statement stmt = null;
            try {
                stmt = connection_.createStatement();
                stmt.executeUpdate(STMT_CREATE_DATABASE);
            } catch (SQLException ex) {
                printSQLException(ex);
            } finally {
                close(stmt);
            }
        }
    }

    private static String getDBUrl(String database) {
        return "jdbc:mysql://" + ObjectManagerPaths.LOGGER_DATABASE_PATH + ":"
                + ObjectManagerPaths.LOGGER_DATABASE_PORT + "/" + database;
    }

    private static void close(Statement pstmt) {
        try {
            pstmt.close();
        } catch (SQLException ex) {
            printSQLException(ex);
        }
    }

    private static void close(Connection con) {
        try {
            if (con != null) {
                con.close();
            }
        } catch (SQLException ex) {
            printSQLException(ex);
        }
    }

    private static void printSQLException(Throwable ex) {
        System.err.println("SQLException: " + ex.getMessage());
    }
}
