/*
 * @author Alan Krempler
 * Created on Feb 22, 2005
 */
package isac.users;

import isac.util.FileUtils;
import isac.util.formulae.CalcElement;

import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.TreeMap;

/**
 * @author Alan Krempler
 * 
 * The UserModel is a class to gather statistics about one user's interaction
 * with the system. Statistis are gathered per user and per interactional
 * context.
 */
public class UserModel extends Properties {

    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	/** Value for retrieving the statistics regardless of the dialog atom used in the interaction. */
    public static final int DIALOG_ATOM_ANY = 0;
    private static int nbOfPendingInteractions = 1;
    private static final String USERMODEL_FILESUFFIX = "_model.txt";
    private static final Properties DUMMY_USERMODEL_DATA = new Properties();

    static {
        DUMMY_USERMODEL_DATA.setProperty("todo-key", "todo-value");
    }
    /**
     * successInfos stores the successInfos for CalcElements. that means the key
     * for this TreeMap is a calcElement, the value is the relating SuccessInfo
     */
    private TreeMap<CalcElement, SuccessInfo> successInfos;

    public UserModel(String directoryOfUserModelFile, String username) {
        createUserModelFileIfItDoesNotExist(directoryOfUserModelFile + getUserModelFileName(username), username);
        loadUserModelFromFile(directoryOfUserModelFile + getUserModelFileName(username));
    }

    private String getUserModelFileName(String username) {
        return username + USERMODEL_FILESUFFIX;
    }

    private void loadUserModelFromFile(String file) {
        FileUtils.loadPropertiesFromFile(file, this, "user model");
    }

    private static void createUserModelFileIfItDoesNotExist(String file, String username) {
        FileUtils.createPropertiesFileIfItDoesNotExist(file, DUMMY_USERMODEL_DATA, "user model",
            "Dynamic dialog data data of user '" + username + "'");
    }

    /**
     * @param item_id
     *            ID of the item being queried, a tactic for example
     * @param dialog_atom
     *            ID of the interactional context use DIALOG_ATOM_ANY if asking
     *            for statistics for all dialog atoms
     * @return count of previously recorded interactions with this item
     */
    public int getTouchedCount(int item_id, int dialog_atom) {
        // TODO.AK0502: returning dummy value in absence of valid statistics
        return 0;
    }

    /**
     * @param item_id
     *            ID of the item being queried, a tactic for example
     * @param dialog_atom
     *            ID of the interactional context use DIALOG_ATOM_ANY if asking
     *            for statistics for all dialog atoms
     * @return count of previously recorded successful interactions with this
     *         item
     */
    public int getSuccessCount(int item_id, int dialog_atom) {
        // TODO.AK0502: returning dummy value in absence of valid statistics
        return 0;
    }

    /**
     * @param item_id
     *            ID of the item being queried, a tactic for example
     * @param dialog_atom
     *            ID of the interactional context use DIALOG_ATOM_ANY if asking
     *            for statistics for all dialog atoms
     * @return time of last recorded interaction with this item
     */
    public Date getTouchedLast(int item_id, int dialog_atom) {
        // TODO.AK0502: returning dummy value in absence of valid statistics
        return new java.util.Date();
    }

    /**
     * @param item_id
     *            ID of the item being queried, a tactic for example
     * @param dialog_atom
     *            ID of the interactional context use DIALOG_ATOM_ANY if asking
     *            for statistics for all dialog atoms
     * @return time of last recorded successful interaction with this item
     */
    public Date getSuccessLast(int item_id, int dialog_atom) {
        // TODO.AK0502: returning dummy value in absence of valid statistics
        return new java.util.Date();
    }

    /**
     * @param item_id
     *            ID of the item being queried, a tactic for example
     * @param dialog_atom
     *            ID of the interactional context use DIALOG_ATOM_ANY if asking
     *            for statistics for all dialog atoms
     * @return average time (seconds) spent in interactions with this item
     */
    public int getTimeSpentAvg(int item_id, int dialog_atom) {
        // TODO.AK0502: returning dummy value in absence of valid statistics
        return 0;
    }

    /**
     * @param item_id
     *            ID of the item being queried, a tactic for example
     * @param dialog_atom
     *            ID of the interactional context use DIALOG_ATOM_ANY if asking
     *            for statistics for all dialog atoms
     * @return maximum time (seconds) spent in interactions with this item
     */
    public int getTimeSpentMax(int item_id, int dialog_atom) {
        // TODO.AK0502: returning dummy value in absence of valid statistics
        return 0;
    }

    /**
     * @param item_id
     *            ID of the item being queried, a tactic for example
     * @param dialog_atom
     *            ID of the interactional context use DIALOG_ATOM_ANY if asking
     *            for statistics for all dialog atoms
     * @return standard deviation of time spent in interactions with this item
     */
    public float getTimeSpentStddev(int item_id, int dialog_atom) {
        // TODO.AK0502: returning dummy value in absence of valid statistics
        return 0;
    }

    /**
     * Mark the beginning of an interaction with an item.
     * 
     * @param item_id
     *            ID of the item being acted upon, a tactic for example
     * @param dialog_atom
     *            ID of the interactional context
     * @return unique number to indentify the end of the transaction
     */
    public int startInteraction(int item_id, int dialog_atom) {
        // TODO.AK0502: insert code to record statistics

        // record timestamp
        // new java.util.Date(); ...

        // save in static vector of currently open interactions
        // ...

        return nbOfPendingInteractions++;
    }

    /**
     * Mark the end of an interaction with an item.
     * 
     * @param interaction_id
     *            ID of the interaction returned from the call to
     *            {@link # startInteraction() }
     * @param success
     *            information about the success of the interaction
     * @return false in case of failure - no matching start could be found, true
     *         in case of success
     */
    public boolean endInteraction(int interaction_id, int success) {
        // TODO.AK0502: insert code to record statistics

        // record end timestamp
        // new java.util.Date(); ...

        // add to statistics

        // delete from static vector of currently open interactions
        // ...

        return true;
    }

    /**
     * gets the info for each CalcElement of the parameter calcElements
     * 
     * @param calcElements a List of {@link CalcElement}
     * @return a TreeMap with the SuccessInfo for each CalcElement in the
     *         calcElements List. Those CalcElements are the index in the
     *         HashMap, the relating SuccessInfos are the values
     */
    public Map<CalcElement, SuccessInfo> getSuccessInfoFor(List<CalcElement> calcElements) {
        TreeMap<CalcElement, SuccessInfo> result = new TreeMap<CalcElement, SuccessInfo>();
        Iterator<CalcElement> iterator = calcElements.iterator();
        while (iterator.hasNext()) {
            CalcElement calcElement = iterator.next();
            if (successInfos.containsKey(calcElement)) {
                result.put(calcElement, successInfos.get(calcElement));
            } else {
                // no successInfo available
                result.put(calcElement, null);
            }
        }
        //TODO
        return result;
    }

    /**
     * @param calcElement
     * @return SuccessInfo or null if no SuccessInfo available
     */
    public SuccessInfo getSuccessInfoFor(CalcElement calcElement) {
        return successInfos.get(calcElement);
    }

    /**
     * @param calcElementsSuccesses,
     *            a TreeMap with a CalcElement as key and the relating success
     *            (Float between 0 and 1) as value
     */
    public void addSuccessInfoFor(Map<CalcElement, Float> calcElementsSuccesses)
        throws ClassCastException {
        Set<CalcElement> set = calcElementsSuccesses.keySet();
        Iterator<CalcElement> iterator = set.iterator();
        while (iterator.hasNext()) {
            try {
                CalcElement calcElement = iterator.next();
                Float success = calcElementsSuccesses.get(calcElement);
                addSuccessInfoFor(calcElement, success.floatValue());
            } catch (ClassCastException e) {
                throw new ClassCastException(
                    "the parameter for the method "
                    + "void addSuccessInfoFor(TreeMap calcElementsSuccesses) "
                    + "must have CalcElements as keys and Floats as value!!");
            }
        }
    }

    public void addSuccessInfoFor(CalcElement calcElement, float success) {
        SuccessInfo successInfo;
        if (successInfos.containsKey(calcElement)) {
            successInfo = successInfos.get(calcElement);
            successInfo.addSuccess(success);
            successInfos.put(calcElement, successInfo);
        } else {
            successInfo = new SuccessInfo(success);
            successInfos.put(calcElement, successInfo);
        }
    }
}