package isac.util;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

public class BridgeMainPaths {

    public static String HOST;
    public static int BRIDGE_MAIN_PORT;
    public static int BRIDGE_RMI_PORT;
    public static String DTD_PATH;
    public static boolean SHOW_GUI_LOGGER;
    public static String LOG_PATH;
    public static boolean LOG_TO_FILE;
    public static String ISABELLE_HOME;

    public static String loadFromResource(String resname) {
        InputStream is = ObjectManagerPaths.class.getResourceAsStream(resname);

        if (is == null) {
            return "BridgeMain: Initialisation file not found";
        }

        return loadFromStream(is);
    }

    public static String loadFromFile(String filename) {
        FileInputStream fis = null;

        try {
            fis = new FileInputStream(filename);
        } catch (FileNotFoundException fnfex) {
            System.out.println(fnfex.getMessage());
            return "BridgeMain: Initialisation file not found";
        }

        return loadFromStream(fis);
    }

    private static String loadFromStream(InputStream is) {
        Properties p = new Properties();

        try {
            p.load(is);
        } catch (IOException ioex) {
            System.out.println(ioex.getMessage());
            return "BridgeMain: Initialisation file is not a Property file";
        }

        if ((HOST = p.getProperty("HOST")) == null) {
            return "BridgeMain: HOST is required parameter";
        }

        if (p.getProperty("BRIDGE_MAIN_PORT") == null) {
            return "BridgeMain: BRIDGE_MAIN_PORT is required parameter";
        }

        try {
            BRIDGE_MAIN_PORT = Integer.parseInt(p.getProperty("BRIDGE_MAIN_PORT"));
        } catch (NumberFormatException nfe) {
            System.out.println(nfe.getMessage());
            return "BridgeMain: BRIDGE_MAIN_PORT is not a number";
        }


        if (p.getProperty("BRIDGE_RMI_PORT") == null) {
            return "BridgeMain: BRIDGE_RMI_PORT is required parameter";
        }

        try {
            BRIDGE_RMI_PORT = Integer.parseInt(p.getProperty("BRIDGE_RMI_PORT"));
        } catch (NumberFormatException nfe) {
            System.out.println(nfe.getMessage());
            return "BridgeMain: BRIDGE_RMI_PORT is not a number";
        }


        if ((DTD_PATH = p.getProperty("DTD_PATH")) == null) {
            return "BridgeMain: DTD_PATH is required parameter";
        }

        if (p.getProperty("SHOW_GUI_LOGGER") == null) {
            return "BridgeMain: SHOW_GUI_LOGGER is required parameter";
        } else {
            SHOW_GUI_LOGGER = Boolean.parseBoolean(p.getProperty("SHOW_GUI_LOGGER"));
        }

        if ((p.getProperty("LOG_PATH")) == null) {
            return "BridgeMain: LOG_PATH is required parameter";
        } else {
        	LOG_PATH = p.getProperty("LOG_PATH");
        }


        if (p.getProperty("LOG_TO_FILE") == null) {
            return "BridgeMain: LOG_TO_FILE is required parameter";
        } else {
            LOG_TO_FILE = Boolean.parseBoolean(p.getProperty("LOG_TO_FILE"));
        }

        if (p.getProperty("ISABELLE_HOME") == null) {
            return "BridgeMain: ISABELLE_HOME is required parameter";
        } else {
        	ISABELLE_HOME = p.getProperty("ISABELLE_HOME");
        }


        return null;
    }
}
