/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.util;

import isac.util.formulae.Specification;
import isac.bridge.xml.DataTypes; //.scala

import info.hupel.isabelle.api.XML; // from libisabelle's setup-assembly-*.jar

import java.io.Serializable;
import java.util.ArrayList;

/**
 * Describes a way how a specific calculation will be calculated (which method
 * will be used to solve this calculation).
 * 
 * @author Mario Hochreiter
 * @version 0.1
 *  
 */
public class Variant implements Serializable {

    static final long serialVersionUID = 4983770207782569120L;

	/** strings betweend the isa tags */
    protected ArrayList<String> isa_strings_;

    protected Specification spec_;

    public Variant() {
        isa_strings_ = new ArrayList<String>();
    }

    /**
     * Add a new isa string
     * 
     * @param isa
     *            The isa string
     */
    public void addNewIsaString(String isa) {
        isa_strings_.add(isa);
    }

    public String toString() {
        String content = "<TERMLIST>";
        for (int index = 0; index < isa_strings_.size(); index++)
            content += "<MATHML>\n<ISA>\n" + isa_strings_.get(index)
                    + "\n</ISA>\n</MATHML>\n";

        if (spec_ != null) {
            content += spec_;
        }
        content += "</TERMLIST>\n";
        return content;
    }

    /**
     * Format the object in a way that the bridge can handle it
     * 
     * @return SML representation of this Variant
     */
    public String toSMLString() {
        StringBuffer sb = new StringBuffer();

        int size = isa_strings_.size();
        sb.append("[");
        for (int i = 0; i < size; i++) {
            sb.append("\"" + (String) isa_strings_.get(i) + "\"");
            if (i < size - 1) sb.append(", ");
        }
        sb.append("],");
        sb.append("(" + spec_.toSMLString() + ")");
        return sb.toString();
    }
    /**
     * Format the object in a way that libisabelle/PIDE can handle it
     * @return XML representation of this Variant
     */
    // conversion is done in Scala, because it is much simpler here
    public XML.Tree toXML() {
    	return DataTypes.xml_of_Variant(this);
    }

    public void setSpecification(Specification spec) {
        this.spec_ = spec;
    }

    public Specification getSpecification() {
        return spec_;
    }
    public ArrayList<String> getIsaStrings() {
        return isa_strings_;
    }

}