/*
 * @author Robert Koenighofer, member of the ISAC-team, 
 * Copyright (c) 2006 by Robert Koenighofer
 * created Jul 11, 2006 10:09:13 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */

package isac.util.formulae;

import java.io.Serializable;



/**
 * This is a base class for ContextTheory, ContextProblem and
 * ContextMethod.
 * 
 * @author Robert Koenighofer
 */
public class Context implements Serializable {

	static final long serialVersionUID = 9114793278420556793L;
	
	private HierarchyKey hierarchy_key_;
	
	private KEStoreKey kestore_key_;
	
    /**
     * Constructor 
     * 
     */
	public Context() {
		hierarchy_key_ = null;
		kestore_key_ = null;
	}
	public Context(HierarchyKey hkey, KEStoreKey kkey) {
		hierarchy_key_ = hkey;
		kestore_key_ = kkey;
	}
	
    /**
     * sets the KEStoreKey 
     * 
     *  @param kestore_key the KEStoreKey to store.
     *  @see isac.util.formulae.KEStoreKey
     */
	public void setKEStoreKey(KEStoreKey kestore_key){
		kestore_key_ = kestore_key;
	}
	
    /**
     * returns the KEStoreKey of the context.
     * 
     *  @return the KEStoreKey of the context.
     *  @see isac.util.formulae.KEStoreKey
     */
	public KEStoreKey getKEStoreKey(){
		return kestore_key_;
	}
	
    /**
     * sets the HierarchyKey
     * 
     *  @param hierarchy_key the HierarchyKey to store.
     *  @see isac.gui.util.formulae.HierarchyKey
     */
	public void setHierarchyKey(HierarchyKey hierarchy_key){
		hierarchy_key_ = hierarchy_key;
	}
	
    /**
     * returns the HierarchyKey of the context.
     * 
     *  @return the HierarchyKey of the context.
     *  @see isac.gui.util.formulae.HierarchyKey
     */
	public HierarchyKey getHierarchyKey(){
		return hierarchy_key_;
	}
	   	
    /**
     * Transforms the content of the object into a string representation.
     *  @return the string representation of the Context
     */
	/*
	public String toString() {
    	// TODO RK060710 implement me
		System.out.println("TODO: isac.util.formulae.Context#toString(): implement me!");
		System.exit(-1);
		return null;
	}*/
	
    /**
     * Transforms the content of the object into a HTML representation.
     *  @return the HTML representation of the Context
     */
	public String toHTML() {
    	// TODO RK060710 implement me
		System.out.println("TODO: isac.util.formulae.Context#toHTML(): implement me!");
		//System.exit(-1);
		return "abc";
	}
}
