/*
 * @author isac-team
 * Created on Sep 19, 2003
 */
package isac.util.formulae;

import java.io.Serializable;
import java.util.Iterator;
import java.util.Vector;

/**
 * string lists pointing into hierarchies of KEStore-data (same kind of hierarchy
 * for exp, met, pbl, thy). This key is _not_ used for uniquely identifying
 * KEStore-data over time, because they may be moved around in the hierarhy.
 * 
 * @see <isac.util.formulae.KEStoreKey> is the _unique_identifier_ for KEStore
 *  
 * @see <isac.util.formulae.CalcHead>, <isac.gui.calcheadviews.CalcHeadView>
 */
public class HierarchyKey implements Serializable {

    /**
     * For serialization, see corresponding API documentation
     */
    private static final long serialVersionUID =504L;

    protected Vector<String> id_ = new Vector<String>();

    public HierarchyKey () {}

    /**
     * @param key_str
     *    ATTENTION: do not insert blanks around the ",", e.g.
     *    "Examples,IsacCore,Calculus,Differentiation,Introduction,chained functions"
     */
    public HierarchyKey (String key_str) {
      String[] key_arr = null;
      key_arr = key_str.split(",");
      for(int i = 0; i < key_arr.length; i++) {
        //System.out.println("key_arr[" + i + "] = " + key_arr[i]);
        id_.add(key_arr[i]);
      }
    }

    public Vector<String> getID() {
        return id_;
    }

    public void setID(Vector<String> v) {
        id_ = v;
    }

    public void addString(String id) {
        this.id_.add(id.trim());
    }

    /**
     * Format the object in a way that the bridge can handle it
     * 
     * @return SML representation (is NOT a Java-String)
     */
    public String toSMLString() {
        if (id_.isEmpty()) {
            return "[]";
        } else {
            StringBuffer sb = new StringBuffer();
            Iterator<String> it = id_.iterator();
            sb.append("[");
            while (it.hasNext()) {
                sb.append("\"");
                sb.append(it.next());
                sb.append("\",");
            }
            sb.deleteCharAt(sb.length() - 1); // remove last ","
            sb.append("]");
            return sb.toString();
        }
    }

    public String toString() {
        String content = "<KEY>\n";// TODO.WN041001 drop that XML
        Iterator<String> vectit = id_.iterator();
        while (vectit.hasNext()) {
            content += "<ID>" + vectit.next() + "</ID>\n";// TODO.WN041001
        }
        content += "</KEY>\n";// TODO.WN041001 drop that XML

        return content;
    }

    public boolean equals(Object o) {
        if (o instanceof HierarchyKey)
            return this.getID().equals(((HierarchyKey) o).getID());
        else
            return false;
    }

    public int size() {
      return id_.size();
    }

}