/*
 * @author Walther Neuper, member of the ISAC-team, 
 * Copyright (c) 2005 by Walther Neuper
 * created Jun 17, 2005 6:47:05 PM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.util.formulae;

/**
 * @author Walther Neuper
 * @author Manuel Koschuch
 * 
 */
public class Model extends CalcElement {

	/**
	 * For serialization, see corresponding API documentation
	 */
	private static final long serialVersionUID = 1L;

	private ModelItemList given_, where_, find_, relate_;

	public Model() {
		given_ = new ModelItemList();
		where_ = new ModelItemList();
		find_ = new ModelItemList();
		relate_ = new ModelItemList();
	}
	public Model(ModelItemList given, ModelItemList where, 
		ModelItemList find, ModelItemList relate) {
		given_ = given;
		where_ = where;
		find_ = find;
		relate_ = relate;
	}

	/**
	 * @see isac.util.CalcElement#getMathML()
	 * @deprecated because it does NOT make sense for a whole Model
	 */
	public String getMathML() {
		return null;
	}

	/**
	 * @see isac.util.CalcElement#getType()
	 */
	public int getType() {
		return CALCEL_MODEL;
	}

	// get*-----------------------------------------------------------
	public ModelItemList getFind() {
		return find_;
	}

	public ModelItemList getWhere() {
		return where_;
	}

	public ModelItemList getGiven() {
		return given_;
	}

	public ModelItemList getRelate() {
		return relate_;
	}

	// set*-----------------------------------------------------------
	public void setFind(ModelItemList el) {
		find_ = el;
	}

	public void setWhere(ModelItemList el) {
		where_ = el;
	}

	public void setGiven(ModelItemList el) {
		given_ = el;
	}

	public void setRelate(ModelItemList el) {
		relate_ = el;
	}

	/**
	 * @return String including all data, i.e 'Where' and item_status_, too,
	 *         which is too much for SMK-input
	 * 
	 * @see isac.interfaces.ICalcElement#toSMLString()
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("\nModel:");
		sb.append("\nGiven:\n");
		sb.append(given_.toString());
		sb.append("\nWhere:\n");
		sb.append(where_.toString());
		sb.append("\nFind:\n");
		sb.append(find_.toString());
		sb.append("\nRelate:\n");
		sb.append(relate_.toString());
		return sb.toString();
	}

	/**
	 * @return String to be sent to the stdin of SML running the math-engine
	 *         CAUTION: regard the delimiters if used for CalcHead
	 * 
	 * @see isac.interfaces.ICalcElement#toSMLString()
	 */
	public String toSMLString() {
		StringBuffer sb = new StringBuffer();
		sb.append("");
		sb.append("Given ");
		sb.append(given_.toSMLString());
		sb.append(", Find ");
		sb.append(find_.toSMLString());
		sb.append(", Relate ");
		sb.append(relate_.toSMLString());
		sb.append("");
		return sb.toString();
	}

	public int getNumberOfItems() {
		return given_.getNumberOfItems() + where_.getNumberOfItems()
				+ find_.getNumberOfItems() + relate_.getNumberOfItems();
	}
}