/*
 * Created on Oct 22, 2003
 */
package isac.util.formulae;

import java.io.Serializable;

/**
 * @author Richard Gradischnegg
 */
//WN150720 ? extends CalcElement  ...like Formula?
public class ModelItem implements Serializable {

	/**
	 * For serialization, see corresponding API Documentation
	 */
	private static final long serialVersionUID = 505L;

	protected String item_status_;

	protected Formula formula_;

	/**
	 * Labels a model item as "correct"
	 */
	public static final int STATUS_CORRECT = 1;

	/**
	 * Labels a model item with a syntax error
	 */
	public static final int STATUS_SYNTAXERROR = 2;

	/**
	 * Labels a model item with a type error
	 */
	public static final int STATUS_TYPEERROR = 3;

	/**
	 * Labels a model item, which belongs to 'Where', as "false"
	 */
	public static final int STATUS_FALSE = 5;

	/**
	 * Labels a model item, which is a list, as "incomplete"
	 */
	public static final int STATUS_INCOMPLETE = 4;

	/**
	 * Labels a model item as 'superfluous'; all items mistakenly input are
	 * recognized as 'superfluous' by the math-engine, unless they have a
	 * syntaxerror or a typeerror
	 */
	public static final int STATUS_SUPERFLUOUS = 7;

	/**
	 * Labels a model item as missing
	 */
	public static final int STATUS_MISSING = 8;

	/**
	 * Labels a model item as to be filled in by the math-engine; this is the
	 * only status used by the front-end to inform the math-engine; all others
	 * are vice versa.
	 */
	public static final int STATUS_HELP_ME = 6;

	public ModelItem() {
		formula_ = new Formula();
		item_status_ = "";
	}
	public ModelItem(String status, String formula) {
		formula_ = new Formula(formula);
		this.setItemStatus(status);
	}

	/**
	 * @see isac.util.parser.XMLParserDigest uses this method, only
	 */
	public void setText(String mathText) {
		this.formula_.setText(mathText);
	}

	/**
	 * Format the object in a way that the bridge can handle it
	 * 
	 * @return SML representation
	 */
	public String toSMLString() {
		return formula_.toSMLString();
	}

	/**
	 * Show all data contained (item_status_ too much for SML input)
	 * 
	 * @see isac.util.formulae.ModelItem#toSMLString()
	 * @return SML representation
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(item_status_);
		sb.append(" (");
		sb.append(formula_.toString());
		sb.append(")");
		return sb.toString();
	}

	/**
	 * @return item_status_ in Integer-representation for use in Java
	 * 
	 * @see CalcHead
	 * @see CalcHead.MODEL_*
	 */
	// this is the SML function generating the XML-status for item_status_
	// fun itm_2xml j (Cor (dts,_))=
	// (indt j ^"<ITEM status=\"correct\">\n"^
	// term2xml (j) (comp_dts' dts)^"\n"^
	// indt j ^"</ITEM>\n"):xml
	// | itm_2xml j (Syn c) =
	// indt j ^"<ITEM status=\"syntaxerror\">\n"^
	// indt (j) ^c^
	// indt j ^"</ITEM>\n"
	// | itm_2xml j (Typ c) =
	// indt j ^"<ITEM status=\"typeerror\">\n"^
	// indt (j) ^c^
	// indt j ^"</ITEM>\n"
	// (*type item also has 'False of cterm' set in preconds2xml WN 050618*)
	// | itm_2xml j (Inc (dts,_)) =
	// indt j ^"<ITEM status=\"incomplete\">\n"^
	// term2xml (j) (comp_dts' dts)^"\n"^
	// indt j ^"</ITEM>\n"
	// | itm_2xml j (Sup dts) =
	// indt j ^"<ITEM status=\"superfluous\">\n"^
	// term2xml (j) (comp_dts' dts)^"\n"^
	// indt j ^"</ITEM>\n"
	// | itm_2xml j (Mis (d,pid)) =
	// indt j ^"<ITEM status=\"missing\">\n"^
	// term2xml (j) (d $ pid)^"\n"^
	// indt j ^"</ITEM>\n";
	public int getStatus() {
		if (item_status_.equals("correct")) {
			return (STATUS_CORRECT);
		} else if (item_status_.equals("syntaxerror")) {
			return (STATUS_SYNTAXERROR);
		} else if (item_status_.equals("typeerror")) {
			return (STATUS_TYPEERROR);
		} else if (item_status_.equals("false")) {
			return (STATUS_FALSE);
		} else if (item_status_.equals("incomplete")) {
			return (STATUS_INCOMPLETE);
		} else if (item_status_.equals("superfluous")) {
			return (STATUS_SUPERFLUOUS);
		} else if (item_status_.equals("missing")) {
			return (STATUS_MISSING);
		}
		return -1;
	}

	/**
	 * translate item_status_ to SML contructors for input to math-engine
	 */
	// This is the respective definition in the SML-kernel 0506dd
	// datatype item =
	// Correct of cterm'
	// | SyntaxE of string
	// | TypeE of string
	// | False of cterm'
	// | Incompl of cterm'
	// | Superfl of string
	// | Missing of cterm';
	// STATUS_HELP_ME !!!!!!!!!!!!!!!!!!!!!!!!!!!TODO.WN050617
	public String statusToSMLconstructor() {
		StringBuffer sb = new StringBuffer();
		if (item_status_.equals("correct")) {
			return "Correct";
		} else if (item_status_.equals("syntaxerror")) {
			return "SyntaxE";
		} else if (item_status_.equals("typeerror")) {
			return "TypeE";
		} else if (item_status_.equals("false")) {
			return "False";
		} else if (item_status_.equals("incomplete")) {
			return "Incompl";
		} else if (item_status_.equals("superfluous")) {
			return "Superfl";
		} else if (item_status_.equals("missing")) {
			return "Missing";
		} else if (item_status_.equals("helpme")) {
			return "HelpMe";
		}
		return sb.toString();
	}

	public String getItemStatus() {
		return item_status_;
	}

	public void setItemStatus(String string) {
		item_status_ = string;
	}

	public Formula getFormula() {
		return formula_;
	}
}