/*
 * @author Andreas Griesmayer, Martin Lang
 * Created on Jun 24, 2003
 *  
 */
package isac.util.genhtml;

import isac.util.KEStoreCommunication;
import isac.util.KEStorePaths;

import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * @author Andreas Griesmayer
 * 
 */
public class GenHTML {

    static final int PBLTREE_XSL = 1;

    static final int METTREE_XSL = 2;

    static final int EXPTREE_XSL = 3;

    // WN080824 why is there no THYTREE_XSL ???

    static final int SIMPLE_HIERARCHY_XSL = 4;

    static final int SIMPLE_PBL_NODE = 5;

    // made an argument of loadContent for usage by GenHTMLsingle
    //private static FileWriter error_log;

    public static void main(String[] args) {

        if (args.length != 1) {
            System.out.println("Only the path to the config file is allowed " +
                               "as one and only parameter");
            System.exit(1);
        }
        KEStorePaths.setConstants(args[0]);
        
        
        // storage-destination of the problems occuring in the xml files
        FileWriter error_log = null;
        try {
            error_log = new FileWriter(KEStorePaths.GENHTML_DESTINATON_PATH
                    + "GENHTML_error_log.txt");
        } catch (IOException e) {
            e.printStackTrace();
        }

        String destination = "";
        if (args.length > 0)
            destination = KEStorePaths.GENHTML_DESTINATON_PATH;
        GenHTML genhtml = new GenHTML();

        genhtml.buildHierarchy(destination, "met/code.html");
        genhtml.buildHierarchy(destination, "met/index.html");
        genhtml.buildHierarchy(destination, "met/heading.html");
        genhtml.buildHierarchy(destination, "met/menu_empty.html");
        genhtml.buildHierarchy(destination, "met/main.html");

        genhtml.buildHierarchy(destination, "pbl/code.html");
        genhtml.buildHierarchy(destination, "pbl/index.html");
        genhtml.buildHierarchy(destination, "pbl/heading.html");
        genhtml.buildHierarchy(destination, "pbl/menu_empty.html");
        genhtml.buildHierarchy(destination, "pbl/main.html");

        genhtml.buildHierarchy(destination, "thy/code.html");
        genhtml.buildHierarchy(destination, "thy/index.html");
        genhtml.buildHierarchy(destination, "thy/heading.html");
        genhtml.buildHierarchy(destination, "thy/menu_empty.html");
        genhtml.buildHierarchy(destination, "thy/main.html");

        genhtml.buildHierarchy(destination, "exp/code.html");
        genhtml.buildHierarchy(destination, "exp/index.html");
        genhtml.buildHierarchy(destination, "exp/heading.html");
        genhtml.buildHierarchy(destination, "exp/menu_empty.html");
        genhtml.buildHierarchy(destination, "exp/main.html");

        Set<String> exp_set = genhtml.loadFileList("exp");
        Iterator<?> exp_it = exp_set.iterator();
        exp_set.size();
        while (exp_it.hasNext()) {
            //System.out.println("exp to generate:" + (index--));
            genhtml.loadContent(error_log, destination, "exp/" + exp_it.next());
        }

        Set<String> pbl_set = genhtml.loadFileList("pbl");
        pbl_set.size();
        Iterator<?> pbl_it = pbl_set.iterator();
        while (pbl_it.hasNext()) {
            //System.out.println("pbl to generate:" + (index--));
            genhtml.loadContent(error_log, destination, "pbl/" + pbl_it.next());
        }

        Set<String> thy_set = genhtml.loadFileList("thy");
        thy_set.size();
        Iterator<?> thy_it = thy_set.iterator();
        while (thy_it.hasNext()) {
            //System.out.println("thy to generate:" + (index--));
            genhtml.loadContent(error_log, destination, "thy/" + thy_it.next());
        }

        Set<String> met_set = genhtml.loadFileList("met");
        met_set.size();
        Iterator<?> met_it = met_set.iterator();
        while (met_it.hasNext()) {
            //System.out.println("met to generate:" + (index--));
            genhtml.loadContent(error_log, destination, "met/" + met_it.next());
        }

        try {
            error_log.close();
        } catch (IOException e1) {
            e1.printStackTrace();
        }
    }

    Set<String> loadFileList(String type) {
        Set<String> return_set = new HashSet<String>();
        try {

            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();

            InputSource inputsource = new InputSource(new StringReader(
            // fixed path ???
                    KEStoreCommunication.loadHierarchy("", type)));
            Document document = db.parse(inputsource);

            Element doc_elem = document.getDocumentElement();
            NodeList nodelist = doc_elem.getElementsByTagName("CONTENTREF");
            for (int index = 0; index < nodelist.getLength(); index++) {
                return_set.add(nodelist.item(index).getFirstChild()
                        .getNodeValue().trim());
            }

            nodelist = doc_elem.getElementsByTagName("EXAMPLE");
            for (int index = 0; index < nodelist.getLength(); index++) {
                return_set.add(nodelist.item(index).getFirstChild()
                        .getNodeValue().trim());
            }
        } catch (ParserConfigurationException e) {
            e.printStackTrace();
        } catch (SAXException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return return_set;
    }

    /**
     * Loads all hierarchies which are displayed on the web-browser
     * 
     * @param destination
     *            path to kbase
     * @param path
     *            is exp/exp_filname.xxx | thy/thy_filname.xxx | ...
     */
    protected void buildHierarchy(String destination, String path) {

        String session = ""; // /FIXME why is here an empty
        // session ID??? -> Problem with
        // BrwoserDialog
        StringTokenizer tokens = new StringTokenizer(path, "/");
        path = "";
        String type = tokens.nextToken();

        if (!tokens.hasMoreTokens())
            path = "/index.html";
        else
            while (tokens.hasMoreTokens())
                path += "/" + tokens.nextToken();
        try {
            if (path.endsWith("code.html")) {
                Vector<String> parameter = new Vector<>();
                parameter.add(session);
                parameter.add(type);
                Source xslSource = null;
                String hierarchy_string = KEStoreCommunication.loadHierarchy(
                        session, type);
                hierarchy_string = hierarchy_string.replaceAll(
                        ".xml </CONTENTREF>", " </CONTENTREF>");
                Source xmlSource = new StreamSource(new StringReader(
                        hierarchy_string));

                if (type.equals("pbl"))
                    xslSource = new StreamSource(
                            // fixed path
                            // /isac/util/genhtml/templates/pbl_hierarchy_jmenu.xsl
                            getClass()
                                    .getResourceAsStream(
                                            "/isac/util/genhtml/templates/pbl_hierarchy_jmenu.xsl"));
                else if (type.equals("met"))
                    xslSource = new StreamSource(
                            // fixed path
                            // /isac/util/genhtml/templates/met_hierarchy_jmenu.xsl
                            getClass()
                                    .getResourceAsStream(
                                            "/isac/util/genhtml/templates/met_hierarchy_jmenu.xsl"));

                else if (type.equals("thy"))
                    xslSource = new StreamSource(
                            // fixed path
                            // /isac/util/genhtml/templates/thy_hierarchy_jmenu.xsl
                            getClass()
                                    .getResourceAsStream(
                                            "/isac/util/genhtml/templates/thy_hierarchy_jmenu.xsl"));

                else
                    xslSource = new StreamSource(
                            // fixed path
                            // /isac/util/genhtml/templates/exp_hierarchy_jmenu.xsl
                            getClass()
                                    .getResourceAsStream(
                                            "/isac/util/genhtml/templates/exp_hierarchy_jmenu.xsl"));
                // fixed path /isac/util/genhtml/templates/codeheader.html
                String header = toString(getClass().getResourceAsStream(
                        "/isac/util/genhtml/templates/codeheader.html"));
                // fixed path /isac/util/genhtml/templates/codefooter.html
                String footer = toString(getClass().getResourceAsStream(
                        "/isac/util/genhtml/templates/codefooter.html"));

                // Perform the transformation, sending the output to the
                // response.
                // java.io.PrintWriter out = response.getWriter();
                FileWriter out = new FileWriter(destination + type + "/" + path);
                out.write(header);
                Encoder.encodeInformation(xslSource, xmlSource, out);
                out.write(footer);
                out.close();
            } else {
                if (path.trim().endsWith("index.html")) {
                    if (type.equals("pbl")) {
                        path = "index_pbl.html";
                    } else if (type.equals("met"))
                        path = "index_met.html";
                    else if (type.equals("thy"))
                        path = "index_thy.html";
                    else
                        path = "index_exp.html";
                }
                System.out.println(path);
                // fixed path /isac/util/genhtml/jscript/
                InputStream istream = getClass().getResourceAsStream(
                        "/isac/util/genhtml/jscript/" + path);
                FileWriter out = new FileWriter(destination + type + "/" + path);
                int input;
                while ((input = istream.read()) != -1) {
                    out.write(input);
                }
                out.close();
            }

        } catch (IOException exc) {
            System.out.println("IOException:" + exc.getMessage());
        }
    }

    public String toString(InputStream istream) {
        StringBuffer buf = new StringBuffer(10000);
        int input;
        try {
            while ((input = istream.read()) != -1) {
                buf.append((char) input);
            }
        } catch (IOException e) {
            return getErrorXML("Sorry, IOException while accessing the InputStream "
                    + e.getMessage());
        }
        return buf.toString();
    }

    private static String getErrorXML(String string) {
        return "<ERROR><DESCRIPTION> <h1> Error in file LoadHierarchy.java </h1>"
                + string + "</DESCRIPTION></ERROR>";
    }

    protected void loadFile(String destination, String path) {
        StringTokenizer tokens = new StringTokenizer(path, "/");
        // (String)request.getSession().getAttribute("session_id");
        // if(session==null)
        // session="";
        String session_id = "";
        path = "";
        String type = (String) tokens.nextToken();

        if (!tokens.hasMoreTokens())
            path = "/index.html";
        else
            while (tokens.hasMoreTokens())
                path += "/" + tokens.nextToken();
        try {
            if (path.endsWith("code.html")) {
                Vector<String> parameter = new Vector<>();
                parameter.add(session_id);
                parameter.add(type);
                Source xmlSource = new StreamSource(new StringReader(
                        KEStoreCommunication.loadHierarchy(session_id, type)));

                // Perform the transformation, sending the output to the
                // response.
                java.io.FileWriter out = new FileWriter(destination + type
                        + "/" + path);
                Encoder.encodeHierarchy(type, xmlSource, out);
                return;
            } else if (path.trim().endsWith("heading.html")) {
                java.io.FileWriter out = new FileWriter(destination + type
                        + "/" + path);
                out.write(getHeader(session_id));
                out.close();
                return;
            } else if (path.trim().endsWith("treecontent.html")) {
                Vector<String> parameter = new Vector<>();
                parameter.add(session_id);
                parameter.add(type);
                Source xmlSource = new StreamSource(new StringReader(
                        KEStoreCommunication.loadHierarchy(session_id, type)));

                // Perform the transformation, sending the output to the
                // response.
                java.io.FileWriter out = new FileWriter(destination + type
                        + "/" + path);

                Encoder.encodeHContent(type, xmlSource, out);
                return;
            }
            // TODO this type should not result in different files but in an
            // autogeneration of a fitting file
            if (path.trim().endsWith("index.html")) {
                if (type.equals("pbl")) {
                    path = "index_pbl.html";
                } else if (type.equals("met"))
                    path = "index_met.html";
                else if (type.equals("thy"))
                    path = "index_thy.html";
                else
                    path = "index_exp.html";
            }
            InputStream istream = getClass().getResourceAsStream(
            // fixed path /isac/BrowserServlet/jscript/ OUT OF DATE
                    "/isac/BrowserServlet/jscript/" + path);
            java.io.FileWriter out = new FileWriter(destination + type + "/"
                    + path);
            int input;
            while ((input = istream.read()) != -1) {
                out.write(input);
            }
            out.close();

        } catch (IOException exc) {
            System.out.println("bla");
        }
    }

    private String getHeader(String session_id) {
        String headercontent = "<html>"
                + "<body>"
                + "<a href=\"../../isac.BrowserServlet.Login\" target=\"text\">"
                + " login </a>"
                + "&nbsp;&nbsp;"
                + "<a href=\"../../isac.BrowserServlet.Logout\" target=\"text\">"
                + " logout </a>"
                + "<br> <a href=\"../../isac.BrowserServlet.LoadFile/pbl/index.html\" target=\"_top\">PBLHierarchy</a>"
                + "&nbsp;&nbsp;<a href=\"../../isac.BrowserServlet.LoadFile/met/index.html\" target=\"_top\">METHierarchy</a>"
                + "&nbsp;&nbsp;<a href=\"../../isac.BrowserServlet.LoadFile/thy/index.html\" target=\"_top\">THYHierarchy</a>"
                + "&nbsp;&nbsp;<a href=\"../../isac.BrowserServlet.LoadFile/exp/index.html\" target=\"_top\">ExampleHierarchy</a>"
                + "&nbsp;&nbsp;";

        String username = "";
        /*
         * if ((username = InformationProcessor.getUsername(session_id)) ==
         * null) headercontent += "not logged in"; else
         */
        headercontent += " welcome <b>" + username + "</b>";
        headercontent += "</body> </html>";
        return headercontent;
    }

    /**
     * process one xml-file: read from KEStore, translate into .html, update
     * error_log
     * 
     * @param error_log
     *            required for usage from other classes
     * @param destination
     *            required for usage from other classes
     * @param path
     *            the end of, following destination
     */
    protected void loadContent(FileWriter error_log, String destination,
            String path) {

        if (!path.endsWith(".xml"))
            path += ".xml";
        StringTokenizer tokens = new StringTokenizer(path, "/");
        String session = "";

        Vector<String> parameter = new Vector<>();
        parameter.add(session);
        try {
            Source xslSource = null;
            Source xslSource_author = null;
            Source xslSource_nomodel = null;
            Source xmlSource = null;
            Source xmlSource_author = null;
            Source xmlSource_nomodel = null;

            String command = tokens.nextToken();

            String header = "<HTML xmlns=\"http://www.w3.org/1999/xhtml\"\n"
                    + "xmlns:math=\"http://www.w3.org/1998/Math/MathML\"\n"
                    + "xmlns:xlink=\"http://www.w3.org/1999/xlink\">\n";

            FileWriter out;
            FileWriter out_author;
            FileWriter out_nomodel;

            // removing the ".xml" from the PathString
            String file_name = path.substring(0, path.length() - 4);

            parameter.add(command);
            if (!tokens.hasMoreElements()) {
                out = new FileWriter(destination + file_name + ".html");
                out.write(header);
                out.write("no details");
                out.close();
                return;
            }
            String contentfile = tokens.nextToken();
            parameter.add(contentfile);

            if (!contentfile.trim().endsWith(".xml")) {
                return;
            }

            // System.out.println("destination_: '" + destination + "'");
            out = new FileWriter(destination + file_name + ".html");
            out_author = new FileWriter(destination + file_name
                    + "_author.html");
            out_nomodel = new FileWriter(destination + file_name
                    + "_nomodel.html");

            // loading the content from the xml
            String result = KEStoreCommunication.loadContent(session, command,
                    contentfile);

            if (result.startsWith("ERROR")) {
                result = result.substring(5, result.length());
                try {
                    error_log.write(result + '\n');
                    error_log.flush();
                    out.close();
                    out_author.close();
                    out_nomodel.close();
                    return;

                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }

            xmlSource = new StreamSource(new StringReader(result));
            xmlSource_author = new StreamSource(new StringReader(result));
            xmlSource_nomodel = new StreamSource(new StringReader(result));

            // deciding which stylesheet to use
            if (contentfile.trim().startsWith("pbl")) {
                xslSource = new StreamSource(getClass().getResourceAsStream(
                // fixed path /isac/util/genhtml/templates/pbl_node_content.xsl
                        "/isac/util/genhtml/templates/pbl_node_content.xsl"));
                xslSource_author = new StreamSource(
                        getClass()
                                .getResourceAsStream(
                                // fixed path
                                        // /isac/util/genhtml/templates/pbl_node_content_author.xsl
                                        "/isac/util/genhtml/templates/pbl_node_content_author.xsl"));
                xslSource_nomodel = new StreamSource(
                        getClass()
                                .getResourceAsStream(
                                // fixed path
                                        // /isac/util/genhtml/templates/pbl_node_content_author.xsl
                                        "/isac/util/genhtml/templates/pbl_node_content_nomodel.xsl"));

                // generating the nomodel html from the pbl
                out_nomodel.write(header);
                Encoder.encodeInformation(xslSource_nomodel, xmlSource_nomodel,
                        out_nomodel);
                out_nomodel.close();

            }

            else if (contentfile.trim().startsWith("met")) {
                xslSource = new StreamSource(getClass().getResourceAsStream(
                // fixed path /isac/util/genhtml/templates/met_node_content.xsl
                        "/isac/util/genhtml/templates/met_node_content.xsl"));
                xslSource_author = new StreamSource(
                        getClass()
                                .getResourceAsStream(
                                // fixed path
                                        // /isac/util/genhtml/templates/met_node_content_author.xsl
                                        "/isac/util/genhtml/templates/met_node_content_author.xsl"));
                xslSource_nomodel = new StreamSource(
                        getClass()
                                .getResourceAsStream(
                                // fixed path
                                        // /isac/util/genhtml/templates/pbl_node_content_author.xsl
                                        "/isac/util/genhtml/templates/met_node_content_nomodel.xsl"));

                // generating the nomodel html from the met
                out_nomodel.write(header);
                Encoder.encodeInformation(xslSource_nomodel, xmlSource_nomodel,
                        out_nomodel);
                out_nomodel.close();
            } else if (contentfile.trim().startsWith("thy")) {
                xslSource = new StreamSource(getClass().getResourceAsStream(
                // fixed path /isac/util/genhtml/templates/thy_node_content.xsl
                        "/isac/util/genhtml/templates/thy_node_content.xsl"));
                xslSource_author = new StreamSource(
                        getClass()
                                .getResourceAsStream(
                                // fixed path
                                        // /isac/util/genhtml/templates/thy_node_content_author.xsl
                                        "/isac/util/genhtml/templates/thy_node_content_author.xsl"));
                xslSource_nomodel = new StreamSource(
                        getClass()
                                .getResourceAsStream(
                                // fixed path
                                        // /isac/util/genhtml/templates/thy_node_content_author.xsl
                                        "/isac/util/genhtml/templates/thy_node_content_nomodel.xsl"));

                // generating the nomodel html from the thy
                out_nomodel.write(header);
                Encoder.encodeInformation(xslSource_nomodel, xmlSource_nomodel,
                        out_nomodel);
                out_nomodel.close();
            }

            else if (contentfile.trim().startsWith("exp")) {
                xslSource = new StreamSource(getClass().getResourceAsStream(
                // fixed path /isac/util/genhtml/templates/exp_node_content.xsl
                        "/isac/util/genhtml/templates/exp_node_content.xsl"));
                xslSource_author = new StreamSource(
                        getClass()
                                .getResourceAsStream(
                                // fixed path
                                        // /isac/util/genhtml/templates/exp_node_content_author.xsl
                                        "/isac/util/genhtml/templates/exp_node_content_author.xsl"));
            }

            // Perform the transformation, sending the output to the response.
            out.write(header);
            out_author.write(header);
            Encoder.encodeInformation(xslSource, xmlSource, out);
            out.close();
            Encoder.encodeInformation(xslSource_author, xmlSource_author,
                    out_author);
            out_author.close();

        } catch (IOException exc) {
            exc.printStackTrace();
        }
        // arg1.getWriter().print("pathinfo: "+arg0.getPathInfo()+"\n
        // transl:"+arg0.getPathTranslated());
    }

}