<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0">


<!--
  This is a template which inserts unknown tags (such as html
  tags inside the EXPLANATIONS field) by simply writing:
 <xsl:apply-templates/>
  If no other template matches better, this one is taken
-->
<xsl:template match="*">
 <xsl:copy>
  <xsl:copy-of select="@*"/>
 <xsl:apply-templates/>
</xsl:copy>
</xsl:template>

<!--
  Call the template below to insert the content of a tag with
  replacement of newlines by <br> as following: 
  <xsl:call-template name="insert_with_line_breaks">
    <xsl:with-param name="text" select="TAG_TO_SELECT"/>
  </xsl:call-template>
-->
<xsl:template name="insert_with_line_breaks">
  <xsl:param name="text" select="."/>
  <xsl:choose>
  <xsl:when test="contains($text, '&#xa;')">
  
  
    <xsl:variable name="token-before" select="substring-before($text,'&#xa;')"/>
    <xsl:choose>
  	<xsl:when test="string-length($token-before) > 0">
  
  	<xsl:value-of select="substring-before($text,'&#xa;')"/>

  	<br/> <xsl:text>&#xa;</xsl:text>
  	</xsl:when>   
	</xsl:choose>
  	<xsl:call-template name="insert_with_line_breaks">
  		<xsl:with-param name="text" select="substring-after($text,'&#xa;')"/>
  	</xsl:call-template>
  </xsl:when>
  <xsl:otherwise>
  	<xsl:value-of select="$text"/>
  </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!--
  Call the template below to insert a link. The template expects
  that there is a <STRING> tag and a <GUH> tag inside of the 
  selected element. The content of the <STRING> tag is used as
  test of the link, the link is created out of the content 
  of the <GUH> tag. Call the template as following:
  <xsl:call-template name="insert_link">
    <xsl:with-param name="text" select="TAG_TO_SELECT"/>
  </xsl:call-template>
-->
<xsl:template name="insert_link">
  <xsl:param name="text" select="."/>
  <xsl:variable name="guh" select="normalize-space(GUH)"/>
	<xsl:element name="a">
		<xsl:attribute name="href">
			<xsl:text>../</xsl:text>
			<xsl:text><xsl:value-of select="substring-before($guh, '_')"/></xsl:text>
			<xsl:text>/</xsl:text>
			<xsl:text><xsl:value-of select="normalize-space(GUH)"/></xsl:text>
			<xsl:text>.html</xsl:text>
		</xsl:attribute>
		<xsl:value-of select="STRING"/>
	</xsl:element>
</xsl:template>


<xsl:template match="/">
<html>

<body>

<xsl:for-each select="RULESETDATA">

<table style="text-align: left; width: 100%;" border="0" cellspacing="2"
 cellpadding="2">
  <tbody>
<tr>
<td width="112"> <strong> Identifier: </strong> </td>
<td>
<xsl:value-of select="RULESET/ID"/>
</td>
</tr>
</tbody>
</table>
<hr></hr>
<div class="context">
</div>
<hr></hr>

<xsl:for-each select="RULESET/RULES">
<strong>Rules:</strong>
<table style="text-align: left; width: 100%;" border="0" cellspacing="2"
 cellpadding="2">
  <tbody>
<xsl:for-each select="RULE">
<tr>

<td  width="112">
<xsl:value-of select="TAG"/>
</td>
<td>
  <xsl:call-template name="insert_link">
    <xsl:with-param name="text" select="."/>
  </xsl:call-template>
</td>
</tr>
</xsl:for-each> 

</tbody>
</table>
</xsl:for-each> 

<xsl:for-each select="RULESET/ORDER">
<br/>
<strong>For applying rules:</strong>
<table style="text-align: left; width: 100%;" border="0" cellspacing="2"
 cellpadding="2">
  <tbody>
<tr>

<td  width="112">
Rewriteorder
</td>
<td>
<xsl:value-of select="STRING"/>
</td>
</tr>
</tbody>
</table>
</xsl:for-each>

<xsl:for-each select="RULESET/ERLS">
<br/>
<strong>For evaluating conditions in rules:</strong>
<table style="text-align: left; width: 100%;" border="0" cellspacing="2"
 cellpadding="2">
  <tbody>
<tr>

<td  width="112">
<xsl:value-of select="TAG"/>
</td>
<td>
  <xsl:call-template name="insert_link">
    <xsl:with-param name="text" select="."/>
  </xsl:call-template>
</td>
</tr>
</tbody>
</table>
</xsl:for-each>


<hr></hr>
<xsl:for-each select="EXPLANATIONS">
<xsl:apply-templates/>
</xsl:for-each>

<hr></hr>
<xsl:for-each select="MATHAUTHORS">
<xsl:for-each select="STRING">
(c) <xsl:value-of select="."/> (math-autor) <br/>
</xsl:for-each>
</xsl:for-each>
<xsl:for-each select="COURSEDESIGNS/STRING">
(c) <xsl:value-of select="."/> (course-designer) <br/>
</xsl:for-each>
</xsl:for-each> 

<xsl:for-each select="HTMLDATA">
<xsl:for-each select="EXPLANATIONS">
<xsl:apply-templates/>
</xsl:for-each>
<hr></hr>
<xsl:for-each select="MATHAUTHORS">
<xsl:for-each select="STRING">
(c) <xsl:value-of select="."/> (math-autor) <br/>
</xsl:for-each>
</xsl:for-each>
<xsl:for-each select="COURSEDESIGNS/STRING">
(c) <xsl:value-of select="."/> (course-designer) <br/>
</xsl:for-each>
</xsl:for-each> 



<xsl:for-each select="THEOREMDATA">
<xsl:for-each select="THEOREM">

<table style="width: 100%;">
  <tbody>
    <tr>
      <td width="112"><strong>Theorem:<br></br>
      </strong></td>
      <td width="609">
      

<xsl:value-of select="ID"/>

    </td>
    </tr>
    <tr>
      <td></td>
      <td><em>
      
<xsl:value-of select="MATHML/ISA"/>  

  </em></td>
    </tr>
  </tbody>
</table>
</xsl:for-each> 

<hr></hr>
<div class="context">
</div>
<hr></hr>

<xsl:for-each select="PROOF">

<xsl:element name="a">
<xsl:attribute name="href">
<xsl:value-of select="normalize-space(EXTREF/URL)"/>

</xsl:attribute>
<xsl:value-of select="normalize-space(EXTREF/TEXT)"/>
</xsl:element>
</xsl:for-each>

<hr></hr>

<xsl:for-each select="EXPLANATIONS">
<xsl:apply-templates/>
</xsl:for-each>
<hr></hr>

<xsl:for-each select="MATHAUTHORS">
<xsl:for-each select="STRING">
(c) <xsl:value-of select="."/> (math-autor) <br/>
</xsl:for-each>
</xsl:for-each>

<xsl:for-each select="COURSEDESIGNS/STRING">
(c) <xsl:value-of select="."/> (course-designer) <br/>
</xsl:for-each>
</xsl:for-each> 



</body>

</html>

</xsl:template>

</xsl:stylesheet>
