/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.util.parser;

import isac.gui.util.Hierarchy;
import isac.gui.util.ProblemNode;
import isac.util.formulae.KEStoreKey;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

/**
 * Parses the XML representation of the problem hierarchy_ and converts it to a
 * object representation. Each NODE Tag is converted to a
 * <code>ProblemNode</code> object. The problem nodes are managed by the
 * <code>Hierarchy</code>.
 * 
 * @author Mario Hochreiter, JL
 * @version 1.0
 *  
 */
public class XMLProblemParser extends XMLHierarchyParser {

    public XMLProblemParser(String xml) {
        super(xml);
        System.out.println("XMLProblemParser::DON'T USE ME!!!");
        System.exit(-1);
    }

    private void createProblemNodesHierarchy(Node node, ProblemNode root,
            String txtType) {
        ProblemNode pblNode = null;
        String elemName = null;
        // is there anything to do?
        if (node == null) { return; }

        int type = node.getNodeType();
        switch (type) {
        case Node.DOCUMENT_NODE:
            {
                Document document = (Document) node;
                createNodesHierarchy(document.getDocumentElement());
                break;
            }

        case Node.ELEMENT_NODE:
            {
                String nodeName = node.getNodeName().trim();
                if (nodeName.equals(TAG_NODE_)) {
                    pblNode = new ProblemNode();
                    hierarchy_.addNode(root, pblNode);
                } else if (nodeName.equals(TYPE_ID_)) {
                    elemName = TYPE_ID_;
                } else if (nodeName.equals(TYPE_CONTENTREF_)) {
                    elemName = TYPE_CONTENTREF_;
                }

                // drop through to entity reference
            }

        case Node.ENTITY_REFERENCE_NODE:
            {
                Node child = node.getFirstChild();
                while (child != null) {
                    // elemName is set -> next node is of type text
                    if (elemName != null)
                            createProblemNodesHierarchy(child, root, elemName);
                    createProblemNodesHierarchy(child, pblNode, txtType);
                    child = child.getNextSibling();
                }
                break;

            }

        case Node.CDATA_SECTION_NODE:
            {
                break;
            }

        case Node.TEXT_NODE:
            {
                // unknown elemType or whitespace is ignored
                if (txtType == null) return;

                Text text = (Text) node;
                String txt = text.getNodeValue().trim();

                if (txtType.equals(TYPE_ID_)) {
                    root.setId(txt);
                } else if (txtType.equals(TYPE_CONTENTREF_)) {
                    root.setKEStoreKey( new KEStoreKey(txt) );
                }
                break;
            }
        }
    }

    public Hierarchy createNodesHierarchy(Node node) {
        this.createProblemNodesHierarchy(node, null, null);
        return hierarchy_;
    }
}