/*
 * Build rule-based system (RLB) with path to rules.
 *
 * RLB is http://www.jboss.org/drools/drools-expert.html
 * The doku for the version implemented is
 * http://docs.jboss.org/drools/release/5.3.0.Final/drools-expert-docs/html/index.html
 *
 */
package isac.wsdialog;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderConfiguration;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;

/**
 *
 * @author Kienleitner Markus
 */
public class RlbBuilder {

    private KnowledgeBase kbase_;
        
    public RlbBuilder(Class<?> factoryclass, String rulepath) {
        try {
            KnowledgeBuilderConfiguration conf = KnowledgeBuilderFactory.newKnowledgeBuilderConfiguration();
            KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder(conf);

            kbuilder.add(ResourceFactory.newClassPathResource(rulepath,
                    factoryclass), ResourceType.DRL);

            if (kbuilder.hasErrors()) {
                System.out.println(kbuilder.getErrors().toString());
                throw new RuntimeException("Unable to compile \""+ rulepath +"\".");
            }

            kbase_ = KnowledgeBaseFactory.newKnowledgeBase();
            kbase_.addKnowledgePackages(kbuilder.getKnowledgePackages());

        } catch (RuntimeException e) {
            System.err.println(e.getMessage());
        }
    }
    
    public KnowledgeBase getKnowledgeBase()
    {
        return this.kbase_;
    }
}
