package isac.wsdialog;

import isac.bridge.*;
import isac.browserdialog.ExampleDialog;
import isac.browserdialog.IContextPresenter;
import isac.browserdialog.MethodDialog;
import isac.browserdialog.ProblemDialog;
import isac.browserdialog.TheoryDialog;
import isac.interfaces.*;
import isac.session.DialogGuide;
import isac.useractions.EUIContext;
import isac.useractions.EUIElement;
import isac.useractions.IUIAction;
import isac.useractions.IUserAction;
import isac.useractions.UIAction;
import isac.useractions.UIActionOnCalcElement;
import isac.users.IUserSettings;
//UL import isac.users.UserLogger;
import isac.users.User;
import isac.users.UserLanguage;
import isac.users.UserSettings;
import isac.util.*;
import isac.util.formulae.*;
import isac.util.tactics.*;
import isac.session.Session;
import isac.users.LogItem;
import isac.users.LogItem.DialogTypes;
import isac.users.UserManager;




import java.io.Serializable;
import java.net.MalformedURLException;
import java.rmi.*;
import java.rmi.registry.LocateRegistry;
import java.rmi.server.UnicastRemoteObject;
import java.util.Iterator;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.drools.KnowledgeBase;
import org.drools.runtime.StatefulKnowledgeSession;

/**
 * This is an adaption from WorksheetDialog for replacement of code in
 * WorksheetDialog#notifyUserAction by a rule based system (RLB).
 *
 * NOTES:
 * * rules are as similar as possible to the Java code
 * * rules do not contain an if (with few exceptions): instead they are
 *   split into two rules.
 * * similarity forced to use setGlobal("request", request) +
 *   + global EUIElement request in RlbLearn.drl
 * * RLB can only use public methods; so private methods of WorksheetDialog
 *   had to be make public !!! (marked as @deprecated)
 *
 * @author Markus Kienleitner 2011
 */
public class WorksheetDialog extends UnicastRemoteObject implements
        IWorksheetDialog, IToUser, Serializable, IContextProvider {

    static final long serialVersionUID = 4952921581668118572L;

    private static final Logger logger = Logger.getLogger(WorksheetDialog.class.getName());

    protected MathEngine math_engine_;

    protected CalcTree calc_tree_;

    /**
     * The active formula in the mathematics engine
     */
    public ICalcIterator active_formula_;

    /**
     * Represents the formula on which the data-context depends. It is the last
     * selected formula in the worksheet or the active formula from the
     * mathematics engine
     */
    protected ICalcIterator context_formula_;

    protected Session session_;

    /* determines if the calculation on the worksheet is ready */
    protected boolean worksheet_ready_;

    protected int phase_; // DIALOGPHASE_SPECIFY or DIALOGPHASE_SOLVE

    /**
     * A calculation is finished successfully if a result (this is a specific
     * position on the Worksheet) has been obtained. This variable is set once;
     * i.e. trials on the calculation after the first result are <em>not</em>
     * recorded.
     */
    private boolean calculation_finished_successfully_ = false;

    private Vector<IToUser> datachange_listeners_;

    private IToGUI ui_control_listener_;

    private LogItem log_item_;

    protected String worksheet_id_;

    protected UserSettings user_settings_;// hold whole hashmap locally

    protected UserLanguage user_language_;

    private User user_;

    private DialogGuide dialog_;

    /* all specifica for rule-based system */
    private RlbBuilder rlb_;
    private KnowledgeBase kbase_;
    private StatefulKnowledgeSession ksession_;

//UL    protected UserLogger user_logger_; KEEP until all deleted TOGETHER

    /**
     * @param ME_path
     *            URL of the math engine to be used
     * @param user_path
     *            path(without URL) to UserSettings, UserModel
     * @param session_dialog
     * @param session_id
     *            which create the WorksheetDialog and Worksheet
     * @throws RemoteException
     */
    public WorksheetDialog(String ME_path, Session session,
            String worksheet_id, Formalization f, DialogGuide dialog) throws RemoteException {
        initialise(ME_path, session, worksheet_id, f, dialog);
    }

    public WorksheetDialog(String ME_path, Session session, String worksheet_id, DialogGuide dialog)
            throws RemoteException {
        initialise(ME_path, session, worksheet_id, new Formalization(), dialog);
    }

 void initialise(String ME_path, Session session, String worksheet_id,
            Formalization f, DialogGuide dialog) throws RemoteException {

        this.rmiBind();
        datachange_listeners_ = new Vector<IToUser>();

        MathEngine.init(ME_path);
        math_engine_ = MathEngine.getMathEngine();

        phase_ = DIALOGPHASE_IDLE;

        session_ = session;

        user_ = UserManager.getInstance().getUser(session.getId());

        user_settings_ = user_.getUserSettings();
        user_language_ = user_.getLanguage();

        worksheet_id_ = worksheet_id;

//UL        user_logger_ = session_.getUserLogger();

        getCalcTree(f);
        dialog_ = dialog;

        String exp_name = "";
        if(session_.getExampleOfWorksheet(worksheet_id_) != null)
          exp_name = session_.getExampleOfWorksheet(worksheet_id_).toString();

        log_item_ = new LogItem(user_.getUsername(), session_.getId() ,worksheet_id_,
          LogItem.DialogTypes.WORKSHEET, exp_name," ",null," ", EUIElement.UI_OPEN_WORKSHEET);
        dialog_.writeToDB(log_item_);
    }

    public String getWorksheetID() {
        return worksheet_id_;
    }

    public IToCalc getCalcTree() {
        return calc_tree_;
    }

    /**
     * start a calculation
     */
    public void getCalcTree(Formalization f) {
        // do administrative stuff
        calc_tree_ = math_engine_.getCalcTree(f);

        //calc_tree_.addDataChangeListener(this);
        ((CalcTree)calc_tree_).addDataChangeListener(this);

        switch (user_settings_.getValue(IUserSettings.KEY1_START_SPECIFY)) {
        case IUserSettings.VAL1_SKIP_SPECIFY_TO_START_SOLVE:
            phase_ = DIALOGPHASE_SOLVE;
            break;
        case IUserSettings.VAL1_POP_CALCHEAD_WITH_DESCRIPTIONS:
            phase_ = DIALOGPHASE_SPECIFY;
            break;
        }

        return;
    }

    /**
     * <b>Use only privately within WorksheetDialog.</b><br></br>
     * enter the specifying-phase; 2 cases: (1) at the beginning of a
     * calculation as well as of a subproblem; sets the Descriptions in the
     * Model (to support user-input) and in CAS-commands completes the Model.
     * (2) for inspection of a "correct" CalcHead somewhere above the
     * activeFormula; in this case the CalcHead is not updated, it is just the
     * starting point into the math-knowledge.
     *
     * Is public because this method is necessary for RLB (MKI 10022012).
     *
     * @return CalcHead eg. for providing CalcHead#VIEWSTYLE_*
     *
     * @see isac.wsdialog.IWorksheetDialog#startSpecifying()
     */
 
    public void startSpecifying(CalcHead calc_head) {
        phase_ = DIALOGPHASE_SPECIFY;
        try {
            // remove solving buttons
            removeNextAndAutoButtonForWorksheet();
            if (calc_head.getStatusString().equals("incorrect")) {
                // case (1)
                // -----------------------------------------------------------
                // if you think, modelProblem should only set the descriptions
                // in the Model (and NOT complete it for CAS-commands)
                // search the sml-code for WN060225-modelProblem
                calc_head = calc_tree_.modelProblem();
                // -----------------------------------------------------------
                doUIAction(new UIActionOnCalcElement(
                        EUIElement.UI_SPECIFY_OPEN_CALCHEAD,
                        EUIContext.UI_CONTEXT_SWITCH_PHASES, user_language_,
                        calc_head));
                addCalcHeadPanelButtons();
                addNextAndAutoButtonsForCalchead();

            } else {// case (2)
                // only Button Close
                doUIAction(new UIActionOnCalcElement(
                        EUIElement.UI_SPECIFY_OPEN_CALCHEAD,
                        EUIContext.UI_CONTEXT_SWITCH_PHASES, user_language_,
                        calc_head));
                ui_control_listener_.addUIElement(new UIAction(
                        EUIElement.UI_SPECIFY_CLOSE_CALCHEAD,
                        EUIContext.UI_CONTEXT_CALCHEAD, user_language_));
                addNextAndAutoButtonForWorksheet();
            }

        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }

    /**
     * @see isac.wsdialog.IWorksheetDialog#startCalculate()
     */
    public void startCalculate() {
        ((CalcTree)calc_tree_).startCalculate();
    }

    /**
     * <b>Use only private to WorksheetDialog.</b><br></br>
     * Dismiss the CalcHeadPanel and start WorksheetDialog#DIALOGPHASE_SOLVE
     *
     * @see isac.interfaces.IToCalc#startSolving()
     *
     * Is public because this method is necessary for RLB (MKI 10022012).
     */
    public void startSolving() {
        try {
            removeCalcHeadPanelButtons();
            removeNextAndAutoButtonsForCalchead();
            addNextAndAutoButtonForWorksheet();

        } catch (RemoteException e) {
            e.printStackTrace();
        }
        phase_ = DIALOGPHASE_SOLVE;
    }

    private void buildUIElementsContextCalcElements() {
        try {
            //entries in the context-menu
            ui_control_listener_.addUIElement(new UIAction(
                    EUIElement.UI_SOLVE_SHOW_ASSUMPTIONS,
                    EUIContext.UI_CONTEXT_CALCELEMENTS, user_language_));
            ui_control_listener_.addUIElement(new UIAction(
                    EUIElement.UI_SOLVE_TACTIC_APPLIED,
                    EUIContext.UI_CONTEXT_CALCELEMENTS, user_language_));
            ui_control_listener_.addUIElement(new UIAction(
                    EUIElement.UI_SOLVE_SHOW_INTERMEDIATE_STEPS,
                    EUIContext.UI_CONTEXT_CALCELEMENTS, user_language_));
            ui_control_listener_.addUIElement(new UIAction(
                    EUIElement.UI_SOLVE_TERM_TREE,
                    EUIContext.UI_CONTEXT_CALCELEMENTS, user_language_));
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }

    private void addNextAndAutoButtonForWorksheet() throws RemoteException {
        // Button next
        ui_control_listener_.addUIElement(new UIAction(
                EUIElement.UI_SOLVE_CALCULATE_1,
                EUIContext.UI_CONTEXT_CALCULATION, user_language_));
        // Button auto
        ui_control_listener_.addUIElement(new UIAction(
                EUIElement.UI_SOLVE_CALCULATE_ALL,
                EUIContext.UI_CONTEXT_CALCULATION, user_language_));
    }

    /**
     * <b>Use only private to WorksheetDialog.</b><br></br>
     * Is public because this method is necessary for RLB (MKI 10022012).
     * @throws RemoteException
     */
    public void removeNextAndAutoButtonForWorksheet() throws RemoteException {
        // Button next
        ui_control_listener_.removeUIElement(new UIAction(
                EUIElement.UI_SOLVE_CALCULATE_1,
                EUIContext.UI_CONTEXT_CALCULATION, user_language_));
        // Button auto
        ui_control_listener_.removeUIElement(new UIAction(
                EUIElement.UI_SOLVE_CALCULATE_ALL,
                EUIContext.UI_CONTEXT_CALCULATION, user_language_));

    }

    private void addCalcHeadPanelButtons() throws RemoteException {
        // Button Complete
        ui_control_listener_.addUIElement(new UIAction(
                EUIElement.UI_SPECIFY_COMPLETE_CALCHEAD,
                EUIContext.UI_CONTEXT_CALCHEAD, user_language_));
        // // Button Check
        // ui_control_listener_.addUIElement(new UIAction(
        // UI_SPECIFY_CHECK_CALCHEAD, EUIContext.UI_CONTEXT_CALCHEAD));
        // Button reset
        ui_control_listener_.addUIElement(new UIAction(
                EUIElement.UI_SPECIFY_RESET, EUIContext.UI_CONTEXT_CALCHEAD,
                user_language_));
        // Button Start Solving
        ui_control_listener_.addUIElement(new UIAction(
                EUIElement.UI_SPECIFY_TO_SOLVE, EUIContext.UI_CONTEXT_CALCHEAD,
                user_language_));

    }

    private void removeCalcHeadPanelButtons() throws RemoteException {
        // Button Complete
        ui_control_listener_.removeUIElement(new UIAction(
                EUIElement.UI_SPECIFY_COMPLETE_CALCHEAD,
                EUIContext.UI_CONTEXT_CALCHEAD, user_language_));
        // Button Start Solving
        ui_control_listener_.removeUIElement(new UIAction(
                EUIElement.UI_SPECIFY_TO_SOLVE, EUIContext.UI_CONTEXT_CALCHEAD,
                user_language_));
        // // Button Check
        // ui_control_listener_.removeUIElement(new UIAction(
        // UI_SPECIFY_CHECK_CALCHEAD, EUIContext.UI_CONTEXT_CALCHEAD));
        // Button reset
        ui_control_listener_.removeUIElement(new UIAction(
                EUIElement.UI_SPECIFY_RESET, EUIContext.UI_CONTEXT_CALCHEAD,
                user_language_));
    }

    /**
     * add next and auto Button to WindowApplication
     */
    // Context Calculation is used instead of context calchead,
    // because buttons have to be added to windowapplication and not to
    // calcheadpanel
    private void addNextAndAutoButtonsForCalchead() {
        try {
            // add next Button
            ui_control_listener_.addUIElement(new UIAction(
                    EUIElement.UI_SPECIFY_COMPLETE_STEPWISE,
                    EUIContext.UI_CONTEXT_CALCULATION, user_language_));
            // add auto Button
            ui_control_listener_.addUIElement(new UIAction(
                    EUIElement.UI_SPECIFY_CALCULATE_ALL,
                    EUIContext.UI_CONTEXT_CALCULATION, user_language_));
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }

    /**
     *
     */
    private void removeNextAndAutoButtonsForCalchead() {

        // Context Calculation is used instead of context calchead,
        // because buttons have to be added to windowapplication and not to
        // calcheadpanel
        try {
            // add next Button
            ui_control_listener_.removeUIElement(new UIAction(
                    EUIElement.UI_SPECIFY_COMPLETE_STEPWISE,
                    EUIContext.UI_CONTEXT_CALCULATION, user_language_));
            // add auto Button
            ui_control_listener_.removeUIElement(new UIAction(
                    EUIElement.UI_SPECIFY_CALCULATE_ALL,
                    EUIContext.UI_CONTEXT_CALCULATION, user_language_));
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }

    /**
     * accept the response of the bridge on an operation on a calculation; this
     * can be a <code>CalcChanged</code>-information or a
     * <code>CalcMessage</code>; pass them to the Worksheet (on different
     * ways: the <code>CalcMessage</code> is passed as a <code>UIAction</code>
     * TODO.WN0710)
     *
     * @see isac.wsdialog.IToUser#calcResponse(isac.wsdialog.CalcChanged
     *
     */
    public void calcResponse(CalcEvent event) throws RemoteException {
        if (event instanceof CalcChanged) {//========================CalcChanged
            CalcChanged ccevent = (CalcChanged) event;
            worksheet_ready_ = false;
            if (logger.isInfoEnabled())
                logger.info(" WS<-. . : calcChanged (unc="
                        + ccevent.getLastUnchangedFormula().toSMLString()
                        + ", del="
                        + ccevent.getLastDeletedFormula().toSMLString()
                        + ", gen="
                        + ccevent.getLastGeneratedFormula().toSMLString());
            try {
                for (int i = 0; i < datachange_listeners_.size(); i++) {
                    CalcChanged wrappedEvent = new CalcChanged(event
                            .getTransactionId(), ccevent.isCompleted(),
                            new DialogIterator((CalcIterator) ccevent
                                    .getLastUnchangedFormula(), this),
                            new DialogIterator((CalcIterator) ccevent
                                    .getLastDeletedFormula(), this),
                            new DialogIterator((CalcIterator) ccevent
                                    .getLastGeneratedFormula(), this));
                    ((IToUser) datachange_listeners_.elementAt(i))
                            .calcResponse(wrappedEvent);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            addUIElementsContextOneElement(ccevent);
            active_formula_ = ccevent.getLastGeneratedFormula();
            if (!ccevent.getLastGeneratedFormula().onCalcHead()
            // ..a calculation cannot finish with a CalcHead
                    && !calculation_finished_successfully_)
            { // ..calculation_finished_successfully_ = true: only once
                calculation_finished_successfully_ = ((CalcFormula) (active_formula_
                        .getElement())).getPosition().onEndOfCalculation();
              log_item_.completeWithChanged(((CalcFormula)(active_formula_).getElement()).toString());
              dialog_.writeToDB(log_item_);
              if(calculation_finished_successfully_)
              {
                buildLogItem(active_formula_.onCalcHead(), user_.getUsername(), session_.getId(), worksheet_id_,
                  DialogTypes.WORKSHEET, "", "", active_formula_.getPosition(), EUIElement.LO_EXAMPLE_FINISHED_SUCCESSFUL);
                dialog_.writeToDB(log_item_);
              }
            }
            context_formula_ = (ICalcIterator) ccevent
                    .getLastGeneratedFormula().cloneIterator();
//UL        user_logger_.logCalcChanged(worksheet_id_, ccevent);
            updateContextFormulaOnWorksheet(context_formula_);
            notifyContextPresenter();

            if (active_formula_.onCalcHead()) {
                switch (user_settings_.getValue("KEY1_START_SPECIFY")) {
                case IUserSettings.VAL1_SKIP_SPECIFY_TO_START_SOLVE:
                    startSolving();
                    break;
                case IUserSettings.VAL1_POP_CALCHEAD_WITH_DESCRIPTIONS:
                    startSpecifying((CalcHead) active_formula_.getElement());
                    break;
                default:
                    try {
                        throw new Exception(
                                "WorksheetDialog#calcChanged:switch"
                                        + " (user_settings_.getValue(KEY1_START_SPECIFY)");
                    } catch (Exception e1) {
                        e1.printStackTrace();
                    }
                }
            }
        } else { //==================================================CalcMessage
          CalcMessage cmsg = (CalcMessage)event;
          for (int i = 0; i < datachange_listeners_.size(); i++)
            ((IToUser) datachange_listeners_.elementAt(i)).calcResponse(event);
          logger.fatal("calcResponse : CalcMessage -> " + cmsg.getText());
          log_item_.completeWithError(cmsg.getText());
          dialog_.writeToDB(log_item_);
//UL          user_logger_.logCalcMessage(worksheet_id_, active_formula_
//UL                    .toSMLString(), ((CalcMessage) event).getText());

          String err_patt = cmsg.checkForErropattern(cmsg);
          if (! err_patt.equals("#nothing found#")) {
            showHintPage (err_patt);

            // or enter specific dialog sequence
          }
        }
    }

    public void showHintPage (String err_patt) {
      // DialogGuide dg = new DialogGuide(); //NO import REQUESTED in TestDialogGuide?!?
      String key = "";
      String hier = "";
      // key = dg.getHintPage("KEStoreKey_" + "err_patt");
      // hier = dg.getHintPage("HierarchyKey_" + "err_patt");
      //catch an exception here and skip the rest of the method.
      
      //fixed for chain rule:
      key = "exp_IsacCore_CDi_chain";
      hier = "Examples,IsacCore,Calculus,Differentiation,Introduction,chained functions";
      //fixed for cancel:
      key = "exp_Etc_Frac_Hint_EP3";
      hier = "Examples,Etc,Fractions,Hints,Hint_3";

      KEStoreKey hint_key = new KEStoreKey(key);
      HierarchyKey hint_hier = new HierarchyKey(hier);

      Context current_context = new ContextUnknown();
      current_context = initContext(ContextType.CONTEXT_EXAMPLES);
      current_context.setHierarchyKey(hint_hier);
      current_context.setKEStoreKey(hint_key);

      IContextPresenter ctxtp = session_.getContextPresenter();
      ctxtp.presentContext(current_context);
    }

    /**
     * construct the context-menu on one specific calcelement (e.g. with
     * 'Calchead' and without)
     *
     */
    private void addUIElementsContextOneElement(CalcChanged event) {
        ICalcIterator from = event.getLastUnchangedFormula();
        ICalcIterator to = event.getLastGeneratedFormula();

        Vector<CalcFormula> calc_formulae = null;

        try {
            calc_formulae = from.getFormulaeFromTo(to, new Integer(0), false);
        } catch (RemoteException e1) {
            e1.printStackTrace();
        }

        if (calc_formulae == null)
            return;

        Iterator<?> vector_it = calc_formulae.iterator();

        while (vector_it.hasNext()) {
            CalcFormula cf = (CalcFormula) vector_it.next();

            if (cf.getPosition().getKind().equals("Pbl")
                    || cf.getPosition().getKind().equals("Met")) {
                try {
                    ui_control_listener_.addUIElement(new UIAction(
                            EUIElement.UI_SOLVE_SHOW_CALCHEAD,
                            EUIContext.UI_CONTEXT_ONEELEMENT, cf.getPosition(),
                            user_language_));
                } catch (RemoteException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * <b>Use only private to WorksheetDialog.</b><br></br>
     * This method sends the applicable tactics to the specific item in the
     * worksheet determined by the Position of the iterator
     *
     * @param it
     *
     * Is public because this method is necessary for RLB (MKI 10022012).
     */
    public void sendApplicableTacticsToWorksheet(ICalcIterator it) {
        try {
            Vector<Tactic> tactics = it.getApplicableTactics(ICalcIterator.TACTICS_ALL);

            if (tactics != null) {
                for (int i = 0; i < tactics.size(); i++)
                    ui_control_listener_
                            .addUIElement(new UIActionOnCalcElement(
                                    EUIElement.UI_SOLVE_SET_NEXT_TACTIC,
                                    EUIContext.UI_CONTEXT_APPLICABLE_TACTIC,
                                    user_language_, it.getPosition(),
                                    (Tactic) tactics.get(i)));
            }
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }

    private void updateContextFormulaOnWorksheet(ICalcIterator it) {
        if (ui_control_listener_ != null)
            try {
                ui_control_listener_.doUIAction(new UIAction(
                        EUIElement.UI_SOLVE_CHANGE_CONTEXT_FORMULA,
                        EUIContext.UI_CONTEXT_ONEELEMENT, it.getPosition(),
                        user_language_));
            } catch (RemoteException e) {
                e.printStackTrace();
            }
    }

    /**
     * @see isac.wsdialog.IWorksheetDialog#iterator(CalcIterator,
     *      IWorksheetDialog)
     * @see isac.interfaces.IToCalc#iterator()
     */
    public ICalcIterator iterator(CalcIterator it, IWorksheetDialog dg) {
        try {
            return new DialogIterator(it, dg);
        } catch (RemoteException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * @see isac.wsdialog.IToCalc#addListener(isac.wsdialog.IToUser)
     */
    public boolean addDataChangeListener(IToUser listener)
            throws RemoteException {
        if (datachange_listeners_.contains(listener)) {
            return false;
        } else {
            datachange_listeners_.add(listener);
        }
        return true;
    }

    public boolean registerUIControlListener(IToGUI listener)
            throws RemoteException {
        ui_control_listener_ = listener;
        buildUIElementsContextCalcElements();
        return true;
    }

    /**
     * Notify the Dialog Guide about interaction from the User
     * 
     * NOTE: the whole switch case translated into rules (MKI 02132012)!
     *
     * @param action
     * @return true, if the action has been processed, false if the processing
     *         the action has been denied
     * @throws RemoteException
     */
    public boolean notifyUserAction(IUserAction action)
            throws DialogProtocolException, RemoteException {

        EUIElement request = action.getActionID();
        CalcHead calc_head = new CalcHead();

        if (logger.isInfoEnabled())
            logger.info(" WS->DG: notifyUserAction request= " + request);

        /* ATTENTION use e.g. the right class 'WorksheetDialogRLB'
           and ruleset 'RlbLearnRLB' */
        rlb_ = new RlbBuilder(WorksheetDialog.class, "RlbLearn.drl");
        //here serveral rule sets could be invoked (according to many
        // errors, to timeout, etc.)
        kbase_ = rlb_.getKnowledgeBase();
        ksession_ = kbase_.newStatefulKnowledgeSession();

        /* LOGGING RULE APPLICATION */
        //ksession_.addEventListener(new DebugAgendaEventListener());
        //ksession_.addEventListener(new DebugWorkingMemoryEventListener());
        //KnowledgeRuntimeLogger logger =
        //         KnowledgeRuntimeLoggerFactory.newFileLogger(ksession_, "./LOGFILE_Worksheetdialog");
        //+ dont forget 'logger.close();' below

        //application could be restricted by priorities etc.

        //This allows to have the same code in rules as in original Java object.
        //setGlobal makes thes variables accessible over all rules.
        ksession_.setGlobal("request", request);
        ksession_.setGlobal("context_formula", context_formula_);
        ksession_.setGlobal("session", session_);
        ksession_.setGlobal("action",action);
        ksession_.setGlobal("user_language", user_language_);
        ksession_.setGlobal("calc_head", calc_head);
        ksession_.setGlobal("user", user_);
        ksession_.setGlobal("worksheet_id", worksheet_id_);

        ksession_.insert(this);
        ksession_.insert((CalcTree)calc_tree_);
        //===== this tries to employ all applicable rules !
        ksession_.fireAllRules();
        //application could be restricted by priorities etc.

        /* LOGGING RULE APPLICATION: required to flush the buffer */
        //logger.close();
        ksession_.dispose();

        return true;
    }

   /**
     * <b>Do not use.</b><br></br>
     * Is public because this method is necessary for RLB (MKI 10022012).
     */
    public int getPhase()
    {
      return phase_;
    }

    /**
     * <b>Do not use.</b><br></br>
     * Is public because this method is necessary for RLB (MKI 10022012).
     */
    public IToGUI getUi_control_listener()
    {
      return ui_control_listener_;
    }

    @SuppressWarnings("deprecation") // RMISecurityManager : RMI reform will come
	private void rmiBind() {
        if (System.getSecurityManager() == null) {
            System.setSecurityManager(new RMISecurityManager());
        }

        try {
            LocateRegistry
                    .createRegistry(ObjectManagerPaths.OBJECT_MANAGER_PORT);
        } catch (java.rmi.RemoteException exc2) {
            System.err.println("can not create registry: " + exc2.getMessage());
        }

        // fixed path name =
        // //localhost/isac-WorksheetDialog...Naming.rebind(name, this);
        String name = ObjectManagerPaths.WORKSHEET_DIALOG_PATH; // "//localhost/isac-WorksheetDialog";
        try {
            System.out.println("try to bind as " + name);
            Naming.rebind(name, this);
            System.out.println("Dialog Guide bound to " + name);
            // WN040906 was Object Manager ...
        } catch (java.rmi.ConnectException e) {
            System.err.println("failed to contact as " + name
                    + " (creating RMI-Server on localhost: 1099)");
        } catch (RemoteException e) {
            e.printStackTrace();
        } catch (MalformedURLException e) {
            e.printStackTrace();
        }
    }

    /**
     * @see isac.wsdialog.IWorksheetDialog#notifyGUIReady()
     */
    public void notifyGUIReady() throws RemoteException {
        worksheet_ready_ = true;
    }

    /**
     * @see isac.interfaces.IToGUI#doUIAction(isac.wsdialog.EUIContext) LK
     *      introduced worksheet_ready_ which required new Runnable
     */
    public synchronized void doUIAction(final IUIAction action)
            throws RemoteException {
        Thread t = new Thread(new Runnable() {

            public void run() {
                try {
                    while (worksheet_ready_ == false) {
                        try {
                            Thread.sleep(1000);
                        } catch (InterruptedException e1) {
                            e1.printStackTrace();
                        }
                    }
                    ui_control_listener_.doUIAction(action);
                } catch (RemoteException e) {
                    e.printStackTrace();
                }
            }
        });
        t.run();
    }

    /**
     * request the gui to add an interactive element
     *
     * @see isac.interfaces.IToGUI#addUIElement(isac.wsdialog.IUIAction)
     */
    public synchronized void addUIElement(final IUIAction action)
            throws RemoteException {
        Thread t = new Thread(new Runnable() {

            public void run() {
                try {
                    while (worksheet_ready_ == false) {
                        try {
                            Thread.sleep(1000);
                        } catch (InterruptedException e1) {
                            e1.printStackTrace();
                        }
                    }
                    ui_control_listener_.addUIElement(action);
                } catch (RemoteException e) {
                    e.printStackTrace();
                }
            }
        });
        t.run();
    }

    /**
     * request the gui to remove an interactive element
     *
     * @see isac.interfaces.IToGUI#removeUIElement(isac.wsdialog.IUIAction)
     */
    public synchronized void removeUIElement(final IUIAction action)
            throws RemoteException {
        Thread t = new Thread(new Runnable() {

            public void run() {
                try {
                    while (worksheet_ready_ == false) {
                        try {
                            Thread.sleep(1000);
                        } catch (InterruptedException e1) {
                            e1.printStackTrace();
                        }
                    }
                    ui_control_listener_.removeUIElement(action);
                    // ???worksheet_ready_ = false;
                } catch (RemoteException e) {
                    e.printStackTrace();
                }
            }
        });
        t.run();
    }

    /**
     * <b>Use only private to WorksheetDialog.</b><br></br>
     * TODO: notify the active context presenter that something happened
     *
     * Is public because this method is necessary for RLB (MKI 10022012).
     */
    public void notifyContextPresenter() {
        System.out.println("In WorksheetDialog::notifyContextPresenter !!");

        Context current_context = new ContextUnknown();
        IContextPresenter bdg = session_.getContextPresenter();

        if (bdg == null) {
            System.out.println("WSDialop#notifyContextPresenter: No dialog");
            return;
        }

        if (bdg instanceof ExampleDialog) {
            System.out.println("WSDialop#notifyContextPresenter: Example dialog");
            current_context = initContext(ContextType.CONTEXT_EXAMPLES);
        } else if (bdg instanceof ProblemDialog) {
            System.out.println("WSDialop#notifyContextPresenter: Problem dialog");
            current_context = initContext(ContextType.CONTEXT_PROBLEMS);
        } else if (bdg instanceof MethodDialog) {
            System.out.println("WSDialop#notifyContextPresenter: Method dialog");
            current_context = initContext(ContextType.CONTEXT_METHODS);
        } else if (bdg instanceof TheoryDialog) {
            System.out.println("WSDialop#notifyContextPresenter: Theory dialog");
            current_context = initContext(ContextType.CONTEXT_THEORIES);
        }

        bdg.presentContext(current_context);
    }

    /**
     * @see isac.wsdialog.IContextProvider#initContext(ContextMethod)
     *
     * @throws RemoteException
     *
     */
    public Context initContext(ContextType type) {
//        System.out.println("In WorksheetDialog::initContext() : type is "
//                + type);

        Context current_context = new ContextUnknown();

        switch (type) {
        case CONTEXT_EXAMPLES:
            current_context = new ContextExample();
            current_context.setKEStoreKey(session_
                    .getExampleOfWorksheet(worksheet_id_));
            break;
        case CONTEXT_METHODS:
        case CONTEXT_PROBLEMS:
        case CONTEXT_THEORIES:
            try {
                current_context = context_formula_.initContext(type);
            } catch (RemoteException e) {
                e.printStackTrace();
            }
            break;
        }

//        System.out.println("current context:" + current_context);

        if (current_context == null)
            return new ContextUnknown();
        else
            return current_context;
    }

    /*
     * (non-Javadoc)
     *
     * @see isac.wsdialog.IContextProvider#refineContext(isac.util.formulae.Context)
     */
    public Context refineContext(Context context) {

        Context current_context = null;

        try {
            current_context = context_formula_.refineProblem(context);
        } catch (RemoteException e) {
            e.printStackTrace();
        }

        if (current_context == null)
            return new ContextUnknown();
        else
            return current_context;
    }

    /**
     * <b>Do not use.</b><br></br>
     * Is public because this method is necessary for RLB (MKI 10022012).
     */
    public void setPhase(int phase)
    {
      this.phase_ = phase;
    }

    /**
     * <b>Do not use.</b><br></br>
     * Is public because this method is necessary for RLB (MKI 10022012).
     */
    public void setActiveFormula(ICalcIterator active_formula)
    {
      this.active_formula_ = active_formula;
    }

    /**
     * <b>Do not use.</b><br></br>
     * Is public because this method is necessary for RLB (MKI 10022012).
     */
    public void setContextFormula(ICalcIterator context_formula)
    {
      this.context_formula_ = context_formula;
    }


    public void setContext(Context context) {
        CalcHead ch = null;

        try {
            if (context instanceof ContextTheory) {
                System.out.println("In WSDialog::setContext() : Theory");
                context_formula_.setContext((ContextTheory) context);
                return;
            } else if (context instanceof ContextMethod) {
                System.out.println("In WSDialog::setContext() : Method");
                ch = context_formula_.setContext((ContextMethod) context);
            } else if (context instanceof ContextProblem) {
                System.out.println("In WSDialog::setContext() : Problem");
                ch = context_formula_.setContext((ContextProblem) context);
            } else {
                System.out
                        .println("Can not set this context (Basis class, ContextExample or ContextUnknown)");
                return;
            }

            if (ch != null) {
                doUIAction(new UIActionOnCalcElement(
                        EUIElement.UI_SPECIFY_CHECK_CALCHEAD,
                        EUIContext.UI_CONTEXT_CALCHEAD, user_language_, ch));
            }

        } catch (RemoteException e) {
            e.printStackTrace();
        }

        return;
    }

    /*
     * (non-Javadoc)
     *
     * @see isac.wsdialog.IContextProvider#checkContext(isac.util.formulae.Context)
     */
    public Context checkContext(Context context) {
        Context current_context = null;

        try {
            current_context = context_formula_.checkContext(context);
        } catch (RemoteException e) {
            e.printStackTrace();
        }

        if (current_context == null)
            return new ContextUnknown();
        else
            return current_context;
    }

    /**
     * @see isac.wsdialog.IContextProvider#getWorksheetState()
     */
    public WorksheetState getWorksheetState() {
        // try {
        // WorksheetState wsstate = new WorksheetState(
        // context_formula_.onCalcHead() );
        // return wsstate;
        // } catch (RemoteException e) {
        // e.printStackTrace();
        // }

        return new WorksheetState(phase_ == DIALOGPHASE_SPECIFY);
    }

    /**
     * <b>Use only privately within WorksheetDialog.</b><br></br>
     * It is public because this method is necessary for RLB.
     */
    public void buildLogItem(Boolean on_calc_head, String user_name, String session_id,
                              String ws_id, DialogTypes dialog_type, String step_arg_1,
                              String step_arg_2, Position pos, EUIElement chosen_step )
                              throws RemoteException {

      String formula_from;

      if(on_calc_head)
        formula_from = ((CalcHead) (active_formula_).getElement()).getHeadLine().toSMLString();
      else
        formula_from = ((CalcFormula)(active_formula_).getElement()).toString();

      log_item_ = new LogItem(user_name, session_id, ws_id, dialog_type, step_arg_1,
                              step_arg_2, pos, formula_from, chosen_step);
    }

    /**
     * <b>Use only private to WorksheetDialog.</b><br></br>
     * Is public because this method is necessary for RLB (MKI 10022012).
     */
    public void buildLogItem(Boolean on_calc_head, String user_name, String session_id,
                        String ws_id, DialogTypes dialog_type, String step_arg_1,
                        String step_arg_2, Position pos, EUIElement chosen_step,
                        String formula_from, String formula_to)
                        throws RemoteException {

    log_item_ = new LogItem(user_name, session_id, ws_id, dialog_type, step_arg_1,
                        step_arg_2, pos, formula_from,formula_to, chosen_step);
    }
    /**
     * <b>Use only privately within WorksheetDialog.</b><br></br>
     * This Method is for passing the current CalcFormula into the context-menu.<br></br>
     * 
     * COPIED FROM wsd.startSpecifying.
     * Its public because this method is necessary for RLB
     * @author Natalie Karl 160509
     */
    public void switchToTermTree(CalcFormula cf) {
      try {
          // remove solving buttons
          removeNextAndAutoButtonForWorksheet();
          // case (2)
          // only Button Close
          doUIAction(new UIActionOnCalcElement(
            EUIElement.UI_SOLVE_TERM_TREE,
            EUIContext.UI_CONTEXT_ONEELEMENT, user_language_,
                      cf));
          //TODO still something wrong with this one?
          //ui_control_listener_.addUIElement(new UIAction(
          //EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE,
          //EUIContext.UI_CONTEXT_ONEELEMENT, user_language_));
          addNextAndAutoButtonForWorksheet();
          

      } catch (RemoteException e) {
          e.printStackTrace();
      }
  }
}
